/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_com.c
**  13 Jan 2004 : Hydra II communication module.
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "hydra_com_private.h"


/*  globals 
------------------------------------------------------------------------------*/
char CL [HC_BUF_SIZE];
bool VERBOSE = true;

static Device device [HC_DEVICES_MAX];
static unsigned short device_count;
static unsigned char MODE;
static char msg [HC_BUF_SIZE];
static Log_Unit log_unit;


/*==============================================================================
**  broadcast
**
*/
int broadcast (char *cl, char *reply[])
{
    int rt_val = OK;
    unsigned short i;

    for (i = 0; i < device_count; ++i)
    {
        if (device[i].global_member && device[i].original)
        {
            sprintf (msg, "[%s] broadcast on %s\n", cl, device[i].port);
            hc_log (VERBOSE ? 2 : 0, msg);

            if (MODE & REAL_COMMUNICATION)
            {
                if (talk (&device[i], cl, reply) == ERR)
                {
                    sprintf (msg, "ERROR: broadcast to device %hu\n", device[i].id);
                    hc_log (2, msg);
                    rt_val = ERR;
                }
            }
        }
    }

    return rt_val;
}


/*==============================================================================
**  com_end
**
*/
int com_end (void)
{
    return ports_close();
}


/*==============================================================================
**  com_init
**
*/
int com_init (const char *cfg_file, const char *end_char, unsigned short ms_timeout, unsigned char mode)
{
    unsigned short i;

    strcpy (log_unit.tag, "Hydra_com");

    if (log_init (&log_unit) == ERR)
    {
        fprintf (stderr, "%s\n", "hydra_com: FATAL: could not initiate logging");
        exit (ERR);
    }

    hc_log (VERBOSE ? 2 : 0, version());
    sprintf (msg, "PID: %d", getpid());
    hc_log (0, msg);

    MODE = mode; 
    rd_cfg (cfg_file);
    ports_open();

    for (i = 0; i < device_count; ++i)
    {
        strcpy (device[i].end_char, end_char);
        device[i].ms_timeout = ms_timeout;

            sprintf (msg, "\n[%4hu] [%25s]  fd[%2d]  sa[%10lu]  original[%s]  global[%s]  strip[%s]\n",
                    device[i].id, device[i].port, device[i].fd, device[i].s_addr,
                    device[i].original ? "Y" : "N",
                    device[i].global_member ? "Y" : "N",
                    device[i].strip_id ? "Y" : "N");
            hc_log (VERBOSE ? 2 : 0, msg);
    }

    return OK;
}


/*==============================================================================
**  device_lock
**
*/
static int device_lock (Device *device)
{
    if (0 == device->s_addr)
    {
        char *base = strrchr (device->port, '/');
        sprintf (device->lock, "/tmp/%s.LCK", base ? base + 1 : device->port);
    }
    else
    {
        sprintf (device->lock, "/tmp/%lu.LCK", device->s_addr);
    }

    sprintf (msg, "creating lock [%s]\n", device->lock);
    hc_log (VERBOSE ? 2 : 0, msg);

    if (open (device->lock, O_RDWR | O_CREAT | O_EXCL , 0444) == ERR)
    {
        perror (device->lock);
        exit (ERR);
    }

    return OK;
} 


/*==============================================================================
**  device_unlock
**
*/
static int device_unlock (Device *device)
{
    sprintf (msg, "removing lock [%s]\n", device->lock);
    hc_log (VERBOSE ? 2 : 0, msg);

    if (access (device->lock, F_OK) == OK)
    {
        if (unlink (device->lock) == ERR) 
        {
            perror (device->lock);
            return ERR;
        }
    }
    else
    {
        perror (device->lock);
        return ERR;
    }

  return OK;
}


/*==============================================================================
**  get_cl
**
*/
int get_cl (const char *prompt)
{
    char *cl = (NULL == prompt) ?  readline ("> ") : readline (prompt);

    if (cl == NULL) return ERR;

    // account for null and optional end char
    if (strlen (cl) >= sizeof (CL) - 2)
    {
        hc_log (2, "ERROR: command too big: ignored\n");
        return ERR;
    }

    bzero (CL, sizeof (CL));
    strcpy (CL, cl);
    free (cl);

    // get rid of precdeing blanks
    cl = CL;
    while (isspace (*cl)) cl++;
    strcpy (CL, cl);

    if (*CL) add_history (CL);
    
    return OK;
}


/*==============================================================================
**  hc_log
**
*/
int hc_log (int pipe, char *msg)
{
    return log_it (&log_unit, pipe, msg);
}


/*==============================================================================
**  open_serial
**
*/
static int open_serial (Device *device)
{
    struct termios s_t;
    //int status = 0xeeeeeeee;

    if ((device->fd = open (device->port, O_RDWR)) == ERR)
    {
        perror (device->port);
        exit (ERR);
    }

    // RS485 communication would fail after Hydra II and
    // computer were power recycled (commands would transmit
    // but no responses were received). The DEACTIVATED code
    // segments below were various remedies that were tried
    // and failed.
    //
    // DEACTIVATED
    /* 
    if (ioctl (device->fd, TIOCEXCL, &status) == ERR)
    {
       perror (device->port);
       exit (ERR);
    }
    */

    // DEACTIVATED
    /*
    if (ioctl (device->fd, TIOCMGET, &status) == ERR)
    {
       perror (device->port);
       exit (ERR);
    }

    sprintf (msg, "serial line status: 0x%x", status);
    hc_log (VERBOSE ? 2 : 0, msg); 

    if ( ! (status & (TIOCM_DTR | TIOCM_RTS)) )
    {
        hc_log (2, "ERROR: serial port in bad state");
        hc_log (2, "resetting serial port...");

        status = (TIOCM_DTR | TIOCM_RTS);

        if (ioctl (device->fd, TIOCMSET, &status) == ERR)
        {
           perror (device->port);
           exit (ERR);
        }
    }
    */

    // This part had to be deactivated to fix the RS485
    // communications after power recycle. But note that
    // this code only works if we compile using -O2 flag!
    // See also below.
    //
    // DEACTIVATED
    /*
    if (tcgetattr (device->fd, &s_t) == ERR)
    {
        perror (device->port);
        exit (ERR);
    }
    */

    if (cfsetispeed (&s_t, B38400) == ERR)
    {
        perror (device->port);
        exit (ERR);
    }

    if (cfsetospeed (&s_t, B38400) == ERR)
    {
        perror (device->port);
        exit (ERR);
    }

    s_t.c_cflag |= (CS8 | CLOCAL | HUPCL | CREAD);
    s_t.c_iflag |= IGNCR;

	if (tcsetattr (device->fd, TCSANOW, &s_t) == ERR)
    {
        perror (device->port);
		exit (ERR);
    }

    // Essential to make sure we capture all the characters
    // when reading responses. Note that if we compile with
    // -O0 flag this setting gets ignored!!!
	s_t.c_cc [VMIN]  = 16;
	s_t.c_cc [VTIME] = 1;
	// s_t.c_cc [VMIN]  = 0;
	// s_t.c_cc [VTIME] = 0;

	if (tcsetattr (device->fd, TCSANOW, &s_t) == ERR)
    {
        perror (device->port);
		exit (ERR);
    }

	if (tcflush (device->fd, TCIOFLUSH) == ERR)
    {
        perror (device->port);
        exit (ERR);
    }

	return OK;
}


/*==============================================================================
**  open_socket 
**
*/
static int open_socket (Device *device)
{
    size_t adr_len;
    int fd = socket (AF_INET, SOCK_STREAM, 0);
    struct sockaddr_in adr;

    adr.sin_addr.s_addr = device->s_addr;
    adr.sin_family = AF_INET;
    adr.sin_port = htons (HC_ISERVER_PORT);
    adr_len = sizeof(adr);

    if (connect (fd, (struct sockaddr *)&adr, adr_len) == ERR)
    {
        perror (device->port);
        exit (ERR);
    }

    device->fd = fd;

    return OK;
}


/*==============================================================================
**  ports_close
**
*/
static int ports_close (void)
{
    unsigned short i;

    for (i = 0; i < device_count; ++i)
    {
        if (device[i].original)
        {
            if (close (device[i].fd) == ERR)
            {
                perror (device[i].port);
            }
            else
            {
                sprintf (msg, "closing %s\n", device[i].port);
                hc_log (VERBOSE ? 2 : 0, msg);
            }

            if ( ! (MODE & IGNORE_LOCK) )
            {
                device_unlock (&device[i]);
            }
        }
    }

    return OK;
}


/*==============================================================================
**  ports_open
**
*/
static int ports_open (void)
{
    unsigned short i;

    for (i = 0; i < device_count; ++i)
    {
        struct stat buf; 

        if ((stat (device[i].port, &buf)) == 0)
        {
            device[i].s_addr = 0;
        }
        else
        {
            struct hostent *host;

            if ((host = gethostbyname (device[i].port)) == NULL)
            {
                perror (device[i].port);
                exit (ERR);
            }
            
            device[i].s_addr = *((unsigned long *) host->h_addr_list[0]);
        }
    }

    for (i = 0; i < device_count; ++i)
    {
        int j;

        // if device open already
        for (j = i - 1; j >= 0; --j)
        {
            if (strcmp (device[i].port, device[j].port) == 0 || (device[i].s_addr != 0 && device[i].s_addr == device[j].s_addr))
            {
                sprintf (msg, "port %s already open\n", device[i].port);
                hc_log (VERBOSE ? 2 : 0, msg);

                device[i].fd = device[j].fd;
                device[i].original = false;
                break;
            }
        }

        // if device was not open already
        if (-1 == j) 
        {
            sprintf (msg, "opening port %s\n", device[i].port);
            hc_log (VERBOSE ? 2 : 0, msg);

            device[i].original = true;

            if (0 == device[i].s_addr)
            {
                open_serial (&device[i]);
            }
            else
            {
                open_socket (&device[i]);
            }

            if ( ! (MODE & IGNORE_LOCK) )
            {
                device_lock (&device[i]);
            }
        }
    }

    return OK;
}


/*==============================================================================
**  rd_cfg
**
*/
static int rd_cfg (const char *file_name)
{
    char line [256];
    int line_len = sizeof (line);

    FILE *cfg = fopen (file_name, "r");

    if ( ! cfg )
    {
        perror (file_name);
        exit (ERR);
    }

    while (fgets (line, line_len, cfg) != NULL)
    {
        int i;
        int id, gm, si;
        char extraneous[2];
        char *cp = line;

        // skip white space
        while (isspace (*cp))
        {
            cp++;
        }

        // skip blank lines and comment lines
        if ( ( ! (*cp) ) || '#' == *cp)
        {
            continue;
        }
        
        // don't exceed max no. of devices
        if (HC_DEVICES_MAX == device_count)
        {
            sprintf (msg, "FATAL: no. of devices (%d) exceeds limit of %d\n", device_count + 1, HC_DEVICES_MAX);
            hc_log (2, msg);
            exit (ERR);
        }

        // each device line has four fields, and an optional inline comment
        i = sscanf (line, "%d %s %d %d %1s", &id, device[device_count].port, &gm, &si, extraneous);

        if ( ! ((4 == i) || (5 == i && '#' == extraneous[0])) )
        {
            sprintf (msg, "FATAL: invalid line for device %d\n", device_count + 1);
            hc_log (2, msg);
            exit (ERR);
        }

        if (id >= 0 && id <= USHRT_MAX)
        {
            device[device_count].id = id;
        }
        else
        {
            sprintf (msg, "FATAL: invalid id for device %d\n", device_count + 1);
            hc_log (2, msg);
            exit (ERR);
        }

        if (1 == gm || 0 == gm)
        {
            device[device_count].global_member = (1 == gm) ? true : false;
        }
        else
        {
            sprintf (msg, "FATAL: global member flag should be 1 or 0 (device %d)\n", device_count + 1);
            hc_log (2, msg);
            exit (ERR);
        }

        if (1 == si || 0 == si)
        {
            device[device_count].strip_id = (1 == si) ? true : false;
        }
        else
        {
            sprintf (msg, "FATAL: strip id flag should be 1 or 0 (device %d)\n", device_count + 1);
            hc_log (2, msg);
            exit (ERR);
        }

        // make sure device id is unique
        for (i = device_count -1; i > -1; --i)
        {
            if (device[device_count].id == device[i].id)
            {
                sprintf (msg, "FATAL: devices %d and %d have the same id\n", i + 1, device_count +1);
                hc_log (2, msg);
                exit (ERR);
            }
        }

        device_count++;
    }

    // catch file without any devices
    if (0 == device_count)
    {
        sprintf (msg, "FATAL: %s had no devices\n", file_name);
        hc_log (2, msg);
        exit (ERR);
    }

    return OK;
}


/*==============================================================================
**  singlecast
**
*/
int singlecast (char *cl, char *reply[])
{
    unsigned short i;
    unsigned short id = atoi (cl);
    unsigned short id_len = 0;
    char *save = cl;

    // find no. of digits in id
    while (isdigit (*cl))
    {
        id_len++;
        cl++;
    }

    // get rid of space between id and command
    while (isspace (*cl))
    {
        id_len++;
        cl++;
    }

    // besides the id, is there a command?
    if ( ! (*cl) )
    {
        hc_log (2, "ERROR: empty command\n");
        return ERR;
    }

    for (i = 0; i < device_count; ++i)
    {
        if (id == device[i].id)
        {
            char cmd [HC_BUF_SIZE];

            if (device[i].strip_id)
            {
                strcpy (cmd, cl);
            }
            else
            {
                cmd[0] = id + 128;
                cmd[1] = '\0';
                strcat (cmd, cl);
            }

            sprintf (msg, "[%s] sent to device %hu\n", save, device[i].id);
            hc_log (VERBOSE ? 2 : 0, msg);

            if (MODE & REAL_COMMUNICATION)
            {
                if (talk (&device[i], cmd, reply) == ERR)
                {
                    sprintf (msg, "ERROR: singlecast to device %hu\n", device[i].id);
                    hc_log (2, msg);
                    return ERR;
                }

                return OK;
            }
            else return OK;
        }
    }

    sprintf (msg, "ERROR: invalid device id: %hu\n", id);
    hc_log (2, msg);
    return ERR;
}


/*==============================================================================
**  talk
**
*/
static int talk (Device *device, char *cmd, char *reply[])
{
    static char buf [HC_BUF_SIZE];
	int num_fds = device->fd + 1;
	fd_set io_set;
	struct timeval timeout;

    // Crucial for how we communicate: we do not do a read
    // when we send commands that are not supposed to have a 
    // response; so the flush is a critical safe guard. Also,
    // this will quash any unsolicited messages on the line
    // (such messages should not occur, but we play it safe).
    tcflush (device->fd, TCIFLUSH);

	timeout.tv_sec  = device->ms_timeout / 1000;
	timeout.tv_usec =  (device->ms_timeout % 1000)  * 1000;

	FD_ZERO (&io_set);
	FD_SET  (device->fd, &io_set);

	// <send command>
    if (cmd && *cmd)
    {
        if (isascii (device->end_char[0]))
        {
            strcat (cmd, device->end_char);
        }

        if (select (num_fds, NULL, &io_set, NULL, &timeout) < 0)
        {
            printf ("WHAT?! 0\n");
            return ERR; 
        }

        if (FD_ISSET (device->fd, &io_set))
        {
            int cmd_len = strlen (cmd);

            if (write (device->fd, cmd, cmd_len) != cmd_len)
            {
                return ERR; 
            }
        }
        else
        {
            printf ("WHAT?! 1\n");
            return ERR;
        }
    }
	// </send command>

	// <get reply>
    //
    // Doing this speeds things up for commands that do not generate a response,
    // and we safe guard by doing a flush earlier (see above).
    if ( ! reply ) return OK;

    bzero (buf, HC_BUF_SIZE);
    *reply = NULL;

	FD_ZERO (&io_set);
	FD_SET  (device->fd, &io_set);

	if (select (num_fds, &io_set, NULL, NULL, &timeout) < 0)
    {
		return ERR; 
    }

	if (FD_ISSET (device->fd, &io_set))
	{
        // iserver rs-485 glitch
        //char *cr;

		ssize_t bytes_read = read (device->fd, buf, HC_BUF_SIZE - 1);

		if (bytes_read < 0)
        {
			return ERR;
        }

        //clear line (e.g., when motors are powered up)
        /*
        if ( ! (isprint (buf[0]) || (isascii (buf[0]))) )
        {
            int len = strlen (device->end_char);

            if (write (device->fd, device->end_char, len) != len)
            {
                return ERR;
            }
        }
        */

        // iserver rs-485 glitch
        //if ((cr = strrchr (buf, '\r'))) *cr = '\0';

        *reply = buf;
        sprintf (msg, "{%s}\n", buf);
        hc_log (0, msg);
	}
	// </get reply>

	return OK;
}
