/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_mc.c
**  9 Aug 2004 : Hydra II 8051 microcontroller module.
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "hydra_mc_private.h"


/*  globals 
------------------------------------------------------------------------------*/
Set is_8051 = {_is_8051_, sizeof (_is_8051_) / sizeof (Instruct), instruct_cmp};

static char msg [HYDRA_MC_BUF_SIZE];
static Log_Unit log_unit;
static Micro_Controller mc_8051;


/*==============================================================================
**  instruct_cmp
**
*/
static int instruct_cmp (const void *a, const void *b)
{
    return strcmp (((const pInst) a)->name, ((const pInst) b)->name);
}


/*==============================================================================
**  instruct_find
**
*/
static pInst instruct_find (char *name)
{
    Set *sp = &is_8051;
    Instruct target;
    target.name = name;
    return ((pInst) bsearch (&target, sp->arr, sp->arr_sz, sizeof (Instruct), sp->fnc_cmp));
}


/*==============================================================================
**  mc_exe_cmd
**
*/
int mc_exe_cmd (char *name)
{
    pInst pinst;

    if ((pinst = instruct_find (name)))
    {
        pinst->fnc (pinst);

        if (false == pinst->failed)
        {
            return pinst->state.reg_int;
        }
        else
        {
            sprintf (msg, "ERROR: command failed [%s]", name);
            mc_log (2, msg);
            return ERR;
        }
    }
    else
    {
        sprintf (msg, "ERROR: could not find command [%s]", name);
        mc_log (2, msg);
        return ERR;
    }
}


/*==============================================================================
**  mc_init
**
*/
int mc_init (void)
{
    strcpy (mc_8051.name, "MC_8051");
    strcpy (mc_8051.CMD, "6");      // 8051 ID (prorammed)
    mc_8051.cmd = mc_8051.CMD + strlen (mc_8051.CMD);

    strcpy (log_unit.tag, "MC_8051");

    if (log_init (&log_unit) == ERR)
    {
        fprintf (stderr, "%s", "MC_8051: FATAL: could not initiate logging\n");
        exit (ERR);
    }

    // sort as safegaurd
    qsort (is_8051.arr, is_8051.arr_sz, sizeof (Instruct), is_8051.fnc_cmp);

    return OK;
}


/*==============================================================================
**  mc_log
**
*/
int mc_log (int pipe, char *msg)
{
    return log_it (&log_unit, pipe, msg);
}


/*==============================================================================
**  __operate__
**
*/
static int __operate__ (pInst pinst, Hydra_Time *tp)
{
    send_mc_cmd (pinst->map);

    if (tp)
    {
        hydra_sleep (tp);
    }

    if (pinst->status_cmd)
    {
        return (mc_exe_cmd (pinst->status_cmd));
    }
    else
    {
        return ERR;
    }
}


/*==============================================================================
**  operate_generic
**
*/
static void operate_generic (pInst self)
{
    pInst s;

    self->failed = false;
    send_mc_cmd (self->map);

    // update status
    if ((s = instruct_find (self->status_cmd)))
    {
        s->state.reg_int = self->mask;
    }
    else
    {
        self->failed = true;
        sprintf (msg, "ERROR: status update failed [%s]", self->name);
        mc_log (2, msg);
    }

    return;
}


/*==============================================================================
**  operate_gripper
**
*/
static void operate_gripper (pInst self)
{
    pInst b;
    int state;
    Hydra_Time delay = {1, 500};

    self->failed = false;

    // execute cmd, wait, and check status
    if ((state = __operate__ (self, &delay)) == ERR)
    {
        self->failed = true;
        sprintf (msg, "ERROR: status report failed [%s]", self->name);
        mc_log (2, msg);
        return;
    }

    switch (self->mask)
    {
        // we should neither be wide open or fully closed or partly closed (with button)
        case JAWS_OPEN :
        case JAWS_RELAX :
            if (state & (JAWS_OPENWIDE | JAWS_CLOSE))
            {
                self->failed = true;
            }
            break;
        // we should be either fully or partly closed, and not wide open
        case JAWS_CLOSE :
            if (( ! (state & JAWS_CLOSE) ) || state & JAWS_OPENWIDE)
            {
                self->failed = true;
            }
            else
            {
                // update button status (only meaningful after closing gripper jaws)
                if ((b = instruct_find ("button status")))
                {
                    b->state.reg_int = state == JAWS_HOLD_BUTTON ? 1 : 0;
                }
                else
                {
                    self->failed = true;
                    mc_log (2, "ERROR: failed to update button status");
                }
            }
            break;
        // we should be wide open and not fully or partly closed
        case JAWS_OPENWIDE :
            if (( ! (state & JAWS_OPENWIDE) ) || state & JAWS_CLOSE)
            {
                self->failed = true;
            }
            break;
    }

    return;
}


/*==============================================================================
**  operate_lamps
**
*/
static void operate_lamps (pInst self)
{
    int state;
    Hydra_Time delay = {1, 500};

    self->failed = false;

    // execute cmd, wait, and check status
    if ((state = __operate__ (self, &delay)) == ERR)
    {
        self->failed = true;
        sprintf (msg, "ERROR: status report failed [%s]", self->name);
        mc_log (2, msg);
        return;
    }

    switch (self->mask)
    {
        case _LAMPS_ON_ :
            if (state != _LAMPS_ON_)
            {
                self->failed = true;
            }
            break;
        case _LAMPS_OFF_ :
            if (state != _LAMPS_OFF_)
            {
                self->failed = true;
            }
            break;
        case _LAMPS_CU_ :
            if (( ! (state & _LAMPS_CU_) ) || state & _LAMPS_TH_)
            {
                self->failed = true;
            }
            break;
        case _LAMPS_TH_ :
            if (( ! (state & _LAMPS_TH_) ) || state & _LAMPS_CU_)
            {
                self->failed = true;
            }
            break;
    }

    return;
}


/*==============================================================================
**  operate_power
**
*/
static void operate_power (pInst self)
{
    int state;
    Hydra_Time delay = {5, 0};

    self->failed = false;

    if (_ON_42V_ == self->mask)
    {
        // ON is special
        send_mc_cmd (self->map);
        hydra_sleep (&delay);
        // first one will not work
        mc_exe_cmd (self->status_cmd);
        return (mc_exe_cmd (self->status_cmd));
    }
    else
    {
        // execute cmd, wait, and check status
        if ((state = __operate__ (self, &delay)) == ERR)
        {
            self->failed = true;
            sprintf (msg, "ERROR: status report failed [%s]", self->name);
            mc_log (2, msg);
            return;
        }
    }

    switch (self->mask)
    {
        case _ON_42V_ :
            if (state & _OFF_42V_)
            {
                self->failed = true;
            }
            break;
        case _OFF_42V_ :
            if ( ! (state & _OFF_42V_) )
            {
                self->failed = true;
            }
            break;
    }

    return;
}


/*==============================================================================
**  operate_screen
**
*/
static void operate_screen (pInst self)
{
    int state;
    Hydra_Time delay = {1, 500};

    self->failed = false;

    // execute cmd, wait, and check status
    if ((state = __operate__ (self, &delay)) == ERR)
    {
        self->failed = true;
        sprintf (msg, "ERROR: status report failed [%s]", self->name);
        mc_log (2, msg);
        return;
    }

    switch (self->mask)
    {
        case _SCREEN_IN_:
            if (( ! (state & _SCREEN_IN_) ) || state & _SCREEN_OUT_)
            {
                self->failed = true;
            }
            break;
        case _SCREEN_OUT_:
            if (( ! (state & _SCREEN_OUT_) ) || state & _SCREEN_IN_)
            {
                self->failed = true;
            }
            break;
    }

    return;
}


/*==============================================================================
**  operate_vacuum
**
*/
static void operate_vacuum (pInst self)
{
    int state;
    Hydra_Time delay = {1, 500};

    self->failed = false;

    // execute cmd, wait, and check status
    if ((state = __operate__ (self, &delay)) == ERR)
    {
        self->failed = true;
        sprintf (msg, "ERROR: status report failed [%s]", self->name);
        mc_log (2, msg);
        return;
    }

    switch (self->mask)
    {
        case _VAC_CLOSE_:
            //if (( ! (state & _VAC_CLOSE_) ) || state & _VAC_OPEN_)
            if (state !=  _VAC_CLOSE_)
            {
                self->failed = true;
            }
            break;
        case _VAC_OPEN_:
            //if (( ! (state & _VAC_OPEN_) ) || state & _VAC_CLOSE_)
            if (state !=  _VAC_OPEN_)
            {
                self->failed = true;
            }
            break;
    }

    return;
}


/*==============================================================================
**  report_generic
**
*/
static void report_generic (pInst self)
{
    return;
}


/*==============================================================================
**  send_mc_cmd
**
*/
static char* send_mc_cmd (char *cmd)
{
    char *reply = 0;
    strcpy (mc_8051.cmd, cmd);
    singlecast (mc_8051.CMD, &reply);
    return reply;
}


/*==============================================================================
**  status_generic
**
*/
static void status_generic (pInst self)
{
    char *reply = send_mc_cmd (self->map);
        
    self->failed = false;

    if (reply)
    {
        self->state.reg_int = atol (reply);
    }
    else
    {
        self->failed = true;
    }

    return;
}
