/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_mc_private.h
**  30 Jul 2004 : Hydra 8051 microcontroller private interface
**
*/


#ifndef HYDRA_MC_PRIVATE_H
#define HYDRA_MC_PRIVATE_H

#ifdef __cplusplus
    extern "C" {
#endif


/*  defines
------------------------------------------------------------------------------*/
#define HYDRA_MC_BUF_SIZE (1024)

// 8051 commands
#define GRIPPER_CLOSE      ("W gripper_close 1")
#define GRIPPER_DISABLE    ("W gripper_enable 0")
#define GRIPPER_ENABLE     ("W gripper_enable 1")
#define GRIPPER_OPEN       ("W gripper_open 1")
#define GRIPPER_OPENWIDE   ("W gripper_openwide 1")
#define GRIPPER_RELAX      ("W gripper_relax 1")
#define GRIPPER_STATUS     ("R gripper_screen")
#define IR_SWITCH_OFF      ("W gripper_fly_led_on_off 0")
#define IR_SWITCH_ON       ("W gripper_fly_led_on_off 1")
#define LAMPS_POWER_HC_OFF ("W lamp_power_on_off 0")
#define LAMPS_POWER_HC_ON  ("W lamp_power_on_off 1")
#define LAMPS_POWER_QZ_OFF ("W lamp_quartz_power 0")
#define LAMPS_POWER_QZ_ON  ("W lamp_quartz_power 1")
#define LAMPS_OFF          ("W lamp_on_off 0")
#define LAMPS_ON           ("W lamp_on_off 1")
#define LAMPS_REPORT_TYPE  ("R lamp_select_status")
#define LAMPS_SELECT_CU    ("W lamp_select 0")
#define LAMPS_SELECT_TH    ("W lamp_select 1")
#define LAMPS_STATUS       ("R lamp_indiv")
#define LEDS_OFF           ("W gripper_led_on_off 0")
#define LEDS_ON            ("W gripper_led_on_off 1")
#define MOTORS_DISABLE     ("W system_enable 0")
#define MOTORS_ENABLE      ("W system_enable 1")
#define ORTHO_STATUS       ("R ortho")
#define POWER_STATUS       ("R power_panic")
#define SCREEN_IN          ("W screen_in 1")
#define SCREEN_OFF         ("W screen_power_on_off 0")
#define SCREEN_ON          ("W screen_power_on_off 1")
#define SCREEN_OUT         ("W screen_out 1")
#define SCREEN_STATUS      ("R gripper_screen")
#define VACUUM_CLOSE       ("W vacuum_FSA_open 0")
#define VACUUM_OFF         ("W vacuum_pump_on_off 0")
#define VACUUM_ON          ("W vacuum_pump_on_off 1")
#define VACUUM_OPEN        ("W vacuum_FSA_open 1")
#define VACUUM_STATUS      ("R vacuum_status")

/*  headers
------------------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "hydra_common.h"
#include "hydra_log.h"
#include "hydra_com.h"
#include "hydra_sleep.h"
#include "hydra_mc.h"


/*  typedefs
------------------------------------------------------------------------------*/
typedef struct {
    char name [16];
    char CMD [256];
    char *cmd; 
} Micro_Controller;


/*  prototypes
------------------------------------------------------------------------------*/
static int instruct_cmp (const void *a, const void *b);
static pInst instruct_find (char *name);
static int __operate__ (pInst pinst, Hydra_Time *tp);
static void operate_generic (pInst self);
static void operate_gripper (pInst self);
static void operate_lamps (pInst self);
static void operate_power (pInst self);
static void operate_screen (pInst self);
static void operate_vacuum (pInst self);
static void report_generic (pInst self);
static char* send_mc_cmd (char *cmd);
static void status_generic (pInst pinst);


/*  static globals
------------------------------------------------------------------------------*/
enum {
    // first 3 refer to bits 0-2 of RE-4
    JAWS_OPENWIDE    = 0x01,
    JAWS_HOLD_BUTTON = 0x04,
    JAWS_CLOSE       = 0x06,    // OR bits 1 and 2 
    JAWS_OPEN,
    JAWS_RELAX,
    // next two are bits 6-7 of RE-4 
    _SCREEN_IN_      = 0x40,
    _SCREEN_OUT_     = 0x80
};

enum {
    _LAMPS_OFF_ = 0x00,
    _LAMPS_TH_  = 0x02,
    _LAMPS_CU_  = 0x04,
    _LAMPS_QZ_  = 0x08,
    _LAMPS_ON_  = 0xFF
};

/*
enum {
    _VAC_PIER_  = 0x01,
    _VAC_FSA_   = 0x02,
    _VAC_OPEN_  = 0x04,
    _VAC_CLOSE_ = 0x08
};
*/
// RE-1 : bit 0 should always be ON (Pier Vacuum)
//        bit 1 is ON if valve is open and OFF when valve is closed
//        bit 2 is 0x04 | bit 1 | bit 0
//        bit 3 is 0x08 | bit 0
enum {
    _VAC_OPEN_  = 0x07,
    _VAC_CLOSE_ = 0x09
};

// Commands are listed alphabetically by first field,
// but in mc_init() qsort is applied as a defence.
//
// 1. high level command
// 2. 8051 command it maps to
// 3. high level command it uses to manage its state
// 4. mask
// 5. error flag
// 6. status
// 7. function pointer
//
static Instruct _is_8051_[] = {
    {"button status",      NULL,               NULL,                0x00,             false, 0, report_generic},
    {"gripper close",      GRIPPER_CLOSE,      "gripper status",    JAWS_CLOSE,       false, 0, operate_gripper},
    {"gripper disable",    GRIPPER_DISABLE,    "gripper powered",   0x00,             false, 0, operate_generic},
    {"gripper enable",     GRIPPER_ENABLE,     "gripper powered",   0x00,             false, 0, operate_generic},
    {"gripper open",       GRIPPER_OPEN,       "gripper status",    JAWS_OPEN,        false, 0, operate_gripper},
    {"gripper openwide",   GRIPPER_OPENWIDE,   "gripper status",    JAWS_OPENWIDE,    false, 0, operate_gripper},
    {"gripper relax",      GRIPPER_RELAX,      "gripper status",    JAWS_RELAX,       false, 0, operate_gripper},
    {"gripper powered",    NULL,               NULL,                0x00,             false, 0, report_generic},
    {"gripper status",     GRIPPER_STATUS,     NULL,                0x00,             false, 0, status_generic},
    {"ir switch off",      IR_SWITCH_OFF,      "ir switch status",  0x00,             false, 0, operate_generic},
    {"ir switch on",       IR_SWITCH_ON,       "ir switch status",  0x00,             false, 0, operate_generic},
    {"ir switch status",   NULL,               NULL,                0x00,             false, 0, report_generic},
    {"lamps hc status",    NULL,               NULL,                0x00,             false, 0, report_generic},
    {"lamps off",          LAMPS_OFF,          "lamps status",      _LAMPS_OFF_,      false, 0, operate_lamps},
    {"lamps on",           LAMPS_ON,           "lamps status",      _LAMPS_ON_,       false, 0, operate_lamps},
    {"lamps power hc off", LAMPS_POWER_HC_OFF, "lamps hc status",   0x00,             false, 0, operate_generic},
    {"lamps power hc on",  LAMPS_POWER_HC_ON,  "lamps hc status",   0x00,             false, 0, operate_generic},
    {"lamps power qz off", LAMPS_POWER_QZ_OFF, "lamps qz status",   0x00,             false, 0, operate_generic},
    {"lamps power qz on",  LAMPS_POWER_QZ_ON,  "lamps qz status",   0x00,             false, 0, operate_generic},
    {"lamps qz status",    NULL,               NULL,                0x00,             false, 0, report_generic},
    {"lamps report type",  LAMPS_REPORT_TYPE,  NULL,                0x00,             false, 0, status_generic},
    {"lamps select cu",    LAMPS_SELECT_CU,    "lamps report type", _LAMPS_CU_,       false, 0, operate_lamps},
    {"lamps select th",    LAMPS_SELECT_TH,    "lamps report type", _LAMPS_TH_,       false, 0, operate_lamps},
    {"lamps status",       LAMPS_STATUS,       NULL,                0x00,             false, 0, status_generic},
    {"leds off",           LEDS_OFF,           "leds status",       0x00,             false, 0, operate_generic},
    {"leds on",            LEDS_ON,            "leds status",       0x00,             false, 0, operate_generic},
    {"leds status",        NULL,               NULL,                0x00,             false, 0, report_generic},
    {"motors disable",     MOTORS_DISABLE,     "power status",      _OFF_42V_,        false, 0, operate_power},
    {"motors enable",      MOTORS_ENABLE,      "power status",      _ON_42V_,         false, 0, operate_power},
    {"ortho status",       ORTHO_STATUS,       NULL,                0x00,             false, 0, status_generic},
    {"power status",       POWER_STATUS,       NULL,                0x00,             false, 0, status_generic},
    {"screen in",          SCREEN_IN,          "screen status",     _SCREEN_IN_,      false, 0, operate_screen},
    {"screen off",         SCREEN_OFF,         "screen powered",    0x00,             false, 0, operate_generic},
    {"screen on",          SCREEN_ON,          "screen powered",    0x00,             false, 0, operate_generic},
    {"screen out",         SCREEN_OUT,         "screen status",     _SCREEN_OUT_,     false, 0, operate_screen},
    {"screen powered",     NULL,               NULL,                0x00,             false, 0, report_generic},
    {"screen status",      SCREEN_STATUS,      NULL,                0x00,             false, 0, status_generic},
    {"vacuum close",       VACUUM_CLOSE,       "vacuum status",     _VAC_CLOSE_,      false, 0, operate_vacuum},
    {"vacuum off",         VACUUM_OFF,         "vacuum powered",    0x00,             false, 0, operate_generic},
    {"vacuum on",          VACUUM_ON,          "vacuum powered",    0x00,             false, 0, operate_generic},
    {"vacuum open",        VACUUM_OPEN,        "vacuum status",     _VAC_OPEN_,       false, 0, operate_vacuum},
    {"vacuum powered",     NULL,               NULL,                0x00,             false, 0, report_generic},
    {"vacuum status",      VACUUM_STATUS,      NULL,                0x00,             false, 0, status_generic},
};


/* 
------------------------------------------------------------------------------*/
#ifdef HYDRA_MC_PRIVATE_H_TEST
int main (void)
{
    return 0;
}
#endif

#ifdef __cplusplus
    } 
#endif

#endif /* HYDRA_MC_PRIVATE_H */
