/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_sm.c
**  3 Feb 2004 : Hydra II Smart Motor module.
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "hydra_sm_private.h"


/*  globals 
------------------------------------------------------------------------------*/
Instrument hydra;
bool POS_ABSOLUTE = true;
bool XYZ_NEEDS_INIT = true;
bool EXPERT_USER = false;
bool HYDRA_SAFE_MODE = false;
// flag to reduce speed for fops to 1/3 of regular fibers.
bool THIS_FIBER_IS_FOP = false;

static char msg [HYDRA_SM_BUF_SIZE];
static short DEFAULT_ENC_DIR = -1;          // move to left (x) or down (y)
static bool MODEL_GEOMETRY = false;
static bool MODEL_TEMPERATURE = false;
static double CURRENT_TEMPERATURE = 78.0;
static int SYSTEM_IN_MOTION = 0;
static Log_Unit log_unit;


/*==============================================================================
**  abs_enc_state_machine
**
*/
static int abs_enc_state_machine (int state)
{
    char *reply;
    int i, j;
    static long stop_1, stop_2;
    static short leader;
    unsigned t;
    static time_t init_t;

    const int ENC_DMZ = 500;
    const int ENC_TRAVEL = 14000;
    // note negative sign to correct in opposite direction of last move
    const int ENC_BACKUP = -6000;   // about half ENC_TRAVEL to put index in mid next travel

    const long TRAVEL_A = 100;
    const long TRAVEL_V = 32212;    // 1/5 rev/sec (2000 counts/sec)

    // add 3 sec for settling
    Hydra_Time travel_t = {7 + 3, 0};
    Hydra_Time backup_t = {3 + 3, 0};

    switch (state)
    {
        case 0: // initial state (point of entry to state machine)
            init_t = time (NULL);

            // we must use external encoder for this to work
            axes_reset (EXCEPT_Z);
            
            // enable ISR on port G (Heidenhain index signal)
            singlecast (CMD_G_ENABLE_ISR, 0);

            for (i = 0; i != Z; ++i)
            {
                send_axis_val (&AXIS(i), CMD_A, TRAVEL_A);
                send_axis_val (&AXIS(i), CMD_V, TRAVEL_V);
                AXIS(i).enc_dir = DEFAULT_ENC_DIR; // by default move to left (x) or down (y)
            }

            if (axes_update (EXCEPT_Z) == ERR && which_direction() == ERR)
            {
                return ERR;
            }

            // fall thru
        case 1: // recoverable errors cause a recursive call with this state
            stop_1 = stop_2 = -88888;
            leader = -99;

            // crucial for knowing where ae are and maintaining relative position of X0 and X1
            singlecast (CMD_G_SET_ORIGIN, 0);
            
            for (i = 0; i != Z; ++i)
            {
                AXIS(i).P = AXIS(i).enc_dir * ENC_TRAVEL;
                send_axis_val (&AXIS(i), CMD_P_ABS, AXIS(i).P);
            }

            servo_xy();
            singlecast ("0a=-99999", 0);    // set a to bogus value so we can check if a good index was captured
            hydra_sleep (&travel_t);

            if (axes_update (EXCEPT_Z) == ERR)
            {
                if (which_direction() == OK)
                {
                    return abs_enc_state_machine (1);
                }
                else
                {
                    return ERR;
                }
            }

            for (i = 0; i != Z; ++i)
            {
                for (j = 0; j != AXIS(i).dim; ++j)
                {
                    if ((reply = send_sm_cmd (&MOTOR(i, j), "Ra")))
                    {
                        MOTOR(i, j).enc_index[0] = atoi (reply);

                        sprintf (msg, "index 1 = %ld\n", MOTOR(i, j).enc_index[0]);
                        sm_log (VERBOSE ? 2 : 0, msg);

                        if (MOTOR(i, j).enc_index[0] == -99999)
                        {
                            sm_log (2, "ERROR: no index!\n");
                            return ERR;
                        }
                        else if (abs (MOTOR(i, j).enc_index[0] - AXIS(i).P) < ENC_DMZ)
                        {
                            // too close to index; move in OPPOSITE direction a bit and repeat
                            send_axis_val (&AXIS(i), CMD_P_ABS, AXIS(i).P + ENC_BACKUP * AXIS(i).enc_dir);
                            servo_xy();
                            hydra_sleep (&backup_t);

                            if (axes_update (EXCEPT_Z) == ERR)
                            {
                                if (which_direction() == OK)
                                {
                                    return abs_enc_state_machine (1);
                                }
                                else
                                {
                                    return ERR;
                                }
                            }

                            return abs_enc_state_machine (1);
                        }
                    }
                    else
                    {
                        return ERR;
                    }
                }
            }

            if (AXIS(X).enc_dir * AXIS(X).sm[0].enc_index[0] < AXIS(X).enc_dir * AXIS(X).sm[1].enc_index[0])
            {
                stop_1 = AXIS(X).sm[0].enc_index[0];
                stop_2 = AXIS(X).sm[1].enc_index[0] - stop_1;
                leader = 1;
            }
            else
            {
                stop_1 = AXIS(X).sm[1].enc_index[0];
                stop_2 = AXIS(X).sm[0].enc_index[0] - stop_1;
                leader = 0;
            }

            if (0 == stop_2) leader = -1;   // neither of X motors leading

            AXIS(X).P += stop_1;
            AXIS(Y).P += AXIS(Y).sm[0].enc_index[0];

            for (i = 0; i != Z; ++i)
            {
                send_axis_val (&AXIS(i), CMD_P_ABS, AXIS(i).P);
            }

            servo_xy();
            hydra_sleep (&travel_t);

            if (axes_update (EXCEPT_Z) == ERR)
            {
                if (which_direction() == OK)
                {
                    return abs_enc_state_machine (1);
                }
                else
                {
                    return ERR;
                }
            }
            
            state = 3;  // we hope that with one move we get both X0 and X1 indices

            for (i = 0; i != Z; ++i)
            {
                for (j = 0; j != AXIS(i).dim; ++j)
                {
                    if ((reply = send_sm_cmd (&MOTOR(i, j), "Ra")))
                    {
                        MOTOR(i, j).enc_index[1] = atoi (reply);

                        sprintf (msg, "index 2 = %ld\n", MOTOR(i, j).enc_index[1]);
                        sm_log (VERBOSE ? 2 : 0, msg);

                        if (MOTOR(i, j).enc_index[1] == MOTOR(i, j).enc_index[0])
                        {
                            if (i != X || leader == -1 || j != leader)
                            {
                                sm_log (2, "ERROR: no index!\n");
                                return ERR;
                            }
                            else
                            {
                                state = 2;
                            }
                        }
                        else if (abs (MOTOR(i, j).enc_index[1] - AXIS(i).P) < ENC_DMZ)
                        {
                            if (i != X || leader == -1 || j != leader)
                            {
                                // too close to index; move in OPPOSITE direction a bit and repeat
                                send_axis_val (&AXIS(i), CMD_P_ABS, AXIS(i).P + ENC_BACKUP * AXIS(i).enc_dir);
                                servo_xy();
                                hydra_sleep (&backup_t);

                                if (axes_update (EXCEPT_Z) == ERR)
                                {
                                    if (which_direction() == OK)
                                    {
                                        return abs_enc_state_machine (1);
                                    }
                                    else
                                    {
                                        return ERR;
                                    }
                                }

                                return abs_enc_state_machine (1);
                            }
                            else
                            {
                                state = 2;
                            }
                        }
                    }
                    else
                    {
                        return ERR;
                    }
                }
            }

            return abs_enc_state_machine (state);
            break;
        case 2: // if leading X motor still does not have second index, do proper move and read it
            // calculate travel time
            t = (unsigned) (ceil (abs (stop_2) * ((double)(travel_t.s + 1)) / ((double) (ENC_TRAVEL))));

            AXIS(X).P += stop_2;

            send_axis_val (&AXIS(X), CMD_P_ABS, AXIS(X).P);
            servo_xy();
            sleep_s_ms (t, 0);

            if (axes_update (EXCEPT_Z) == ERR)
            {
                if (which_direction() == OK)
                {
                    return abs_enc_state_machine (1);
                }
                else
                {
                    return ERR;
                }
            }

            if ((reply = send_sm_cmd (&AXIS(X).sm[leader], "Ra")))
            {
                AXIS(X).sm[leader].enc_index[1] = atoi (reply);

                sprintf (msg, "index 2 = %ld\n", AXIS(X).sm[leader].enc_index[1]);
                sm_log (VERBOSE ? 2 : 0, msg);

                if (AXIS(X).sm[leader].enc_index[1] == AXIS(X).sm[leader].enc_index[0])
                {
                        sm_log (2, "ERROR: no index!\n");
                        return ERR;
                }
                else if (abs (AXIS(X).sm[leader].enc_index[1] - AXIS(X).P) < ENC_DMZ)
                {
                    // too close to index; move in OPPOSITE direction a bit and repeat
                    send_axis_val (&AXIS(X), CMD_P_ABS, AXIS(X).P + ENC_BACKUP * AXIS(X).enc_dir);
                    servo_xy();
                    hydra_sleep (&backup_t);
                    
                    if (axes_update (EXCEPT_Z) == ERR)
                    {
                        if (which_direction() == OK)
                        {
                            return abs_enc_state_machine (1);
                        }
                        else
                        {
                            return ERR;
                        }
                    }

                    return abs_enc_state_machine (1);
                }
            }
            else
            {
                return ERR;
            }

            // fall thru
        case 3: // done with moves; calculate position using Heidenhain formula
            // disable ISR on port G (Heidenhain index signal)
            singlecast (CMD_G_DISABLE_ISR, 0);

            for (i = 0; i !=Z ; ++i)
            {
                for (j = 0; j != AXIS(i).dim; ++j)
                {
                    int MRR = (int) (round (abs (MOTOR(i, j).enc_index[1] - MOTOR(i, j).enc_index[0]) / 20.0));
                    int B = 2 * MRR - 1000;
                    short sgn = B < 0 ? -1 : 1;
                    MOTOR(i, j).enc_p = (abs(B) - sgn - 1) * 10000 + 
                                          (sgn - MOTOR(i, j).enc_sense * AXIS(i).enc_dir) * MRR * 10;
                    sprintf (msg, "MRR= %d B=%d p=%ld <%ld>\n", MRR, B, MOTOR(i, j).enc_p, MOTOR(i, j).enc_index[0]);
                    sm_log (VERBOSE ? 2 : 0, msg);
                }
            }

            AXIS(X).alignment_err = AXIS(X).sm[0].enc_p + AXIS(X).sm[1].enc_p + 
                                    AXIS(X).sm[1].enc_index[0] - AXIS(X).sm[0].enc_index[0];

            sprintf (msg, "Validation: X0 + X1 - dx = %ld\n", AXIS(X).alignment_err);
            sm_log (VERBOSE ? 2 : 0, msg);

            send_sm_val (&AXIS(X).sm[0], "O", 
                         AXIS(X).sm[0].enc_p + (AXIS(X).P - AXIS(X).sm[0].enc_index[0]) - AXIS(X).sm[0].origin);
            send_sm_val (&AXIS(X).sm[1], "O", 
                         (640000 - AXIS(X).sm[1].enc_p) + (AXIS(X).P - AXIS(X).sm[1].enc_index[0]) - AXIS(X).sm[1].origin);
            send_axis_val (&AXIS(Y), "O", 
                           AXIS(Y).sm[0].enc_p + (AXIS(Y).P - AXIS(Y).sm[0].enc_index[0]) - AXIS(Y).sm[0].origin);


            break;
        default:
            sprintf (msg, "*** case %d ***\n", state);
            sm_log (2, msg);
            return ERR;
            break;
    }

    sprintf (msg, "init took %ld sec\n", time (NULL) - init_t);
    sm_log (VERBOSE ? 2 : 0, msg);

    return OK;
}


/*==============================================================================
**  axes_pv
**
*/
static int axes_pv (bool except_z)
{
    int i;
    int lim = except_z ? Z : hydra.dim;
    int ret_val = OK;

    for (i = 0; i != lim; ++i)
    {
        ret_val += axis_pv (&AXIS(i)); 
    }

    return (OK == ret_val) ? OK : ERR;
}


/*==============================================================================
**  axes_reset
**
*/
int axes_reset (bool except_z)
{
    int i;
    int lim = except_z ? Z : hydra.dim;

    if (except_z)
    {
        sm_log (VERBOSE ? 2 : 0, "Reseting XY axes...\n");
    }
    else
    {
        sm_log (VERBOSE ? 2 : 0, "Reseting all axes...\n");
    }

    for (i = 0; i != lim; ++i)
        AXIS(i).A = AXIS(i).P = AXIS(i).V = 0;
    
    if (except_z)
    {
        // we try twice, in case Z is already asleep
        (wakeup_toggle (&AXIS(Z)) == ASLEEP || wakeup_toggle (&AXIS(Z)) == ASLEEP);
    }

    singlecast (CMD_G_RESET, 0);
    sleep_s_ms (2, 0);
    set_defaults(); 

    if (except_z)
    {
        // wakeup Z
        if (wakeup_toggle (&AXIS(Z)) == ASLEEP)
        {
            return ERR;
        }
    }

    POS_ABSOLUTE = true;
    XYZ_NEEDS_INIT = true;

    return OK;
}


/*==============================================================================
**  axes_update
**
*/
static int axes_update (bool except_z)
{
    int i;
    int lim = except_z ? Z : hydra.dim;
    int ret_val = OK;

    SYSTEM_IN_MOTION = 0;

    for (i = 0; i != lim; ++i)
    {
        AXIS(i).moving = 0;
        ret_val += axis_status (&AXIS(i)); 
        SYSTEM_IN_MOTION |= AXIS(i).moving;
    }

    if (except_z)
    {
        AXIS(Z).moving = 0;
        axis_status (&AXIS(Z)); 
        SYSTEM_IN_MOTION |= AXIS(Z).moving;
    }

    return (OK == ret_val) ? OK : ERR;
}


/*==============================================================================
**  init_xyz
**
*/
int init_xyz (void)
{
    if ( ! VERBOSE ) fprintf (stderr, "Please wait...\n");

    sm_log (2, "initializing XY...");
    if (init_xy() == ERR)
    {
        return ERR;
    }

    sm_log (2, "initializing Z...");
    if (init_z() == ERR)
    {
        return ERR;
    }

    XYZ_NEEDS_INIT = false;

    sm_log (2, "Hydra is ready\n");
    return OK;
}


/*==============================================================================
**  __init_xy__
**  We introduce an error of around 20 microns when the init procedure is done
**  with X and/or Y near the negative limit, which means that we run into the
**  limit and reverse direction. We could not figure out why this error occurs,
**  so we simply init twice (the second init will be bound to happen when we 
**  are at center (0, 0)), which is the reason for wrapping __init_xy__ in
**  init_xy 
**
*/
static int __init_xy__ (void)
{
    int i;

    // find out where we are
    if (abs_enc_state_machine (0) == ERR)
    {
        axes_reset (EXCEPT_Z); 
        sm_log (2, "ERROR: failed to find absolute position");
        return ERR;
    }

    // set software limits and get ready to go to origin
    for (i = 0; i != Z; ++i)
    {
        send_axis_val (&AXIS(i), "SLN", AXIS(i).P_MIN - SW_LIM_OFFSET);
        send_axis_val (&AXIS(i), "SLP", AXIS(i).P_MAX + SW_LIM_OFFSET);
        send_axis_val (&AXIS(i), "h", 0xA5);    // so SM knows it has been initialized
    }

    {
        Point p = {0, 0};

        if (_move_xy_ (&p, true) == ERR)
        {
            axes_reset (EXCEPT_Z); 
            sm_log (2, "ERROR: failed to go to the roigin");
            return ERR;
        }
        
        // settling time
        sleep_s_ms (2, 0);
    }

    // zero P, V and A to avoid runawys
    for (i = 0; i != Z; ++i)
    {
        AXIS(i).V = AXIS(i).A = 0;
        AXIS(i).P = 0;
        send_axis_val (&AXIS(i), CMD_P_ABS, AXIS(i).P);
        send_axis_val (&AXIS(i), CMD_V, AXIS(i).V);
        send_axis_val (&AXIS(i), CMD_A, AXIS(i).A);
        send_axis_cmd (&AXIS(i), "MF0");    // set internal encoder to zero
    }

    if (axes_update (EXCEPT_Z) == ERR)
    {
        axes_reset (EXCEPT_Z); 
        sm_log (2, "ERROR: failed to move to origin");
        return ERR;
    }

    // we enable software limits here, after going to position zero; 
    // otherwise we might be outside boundaries
    singlecast (CMD_G_ENABLE_LIM_SOFT, 0);

    if (axes_update (EXCEPT_Z) == ERR)
    {
        axes_reset (EXCEPT_Z); 
        sm_log (2, "ERROR: failed to enable software limits");
        return ERR;
    }

    return OK;
}
// Wrapper
int init_xy (void)
{
    sm_log (2, "initializing XY: pass 1");
    if (__init_xy__() == ERR) return ERR;
    sm_log (2, "initializing XY: pass 2");
    if (__init_xy__() == ERR) return ERR;

    sm_log (2, "XY is ready\n");
    return OK;
}


/*==============================================================================
**  init_z
**
*/
int init_z (void)
{
    send_axis_cmd (&AXIS(Z), "Z");
    sleep_s_ms (2, 0);

    if (axis_status (&AXIS(Z)) == ERR)
    {
        unsigned fault = AXIS(Z).sm[0].status;
        unsigned ignore = ~(SERVO | WRAPAROUND | INDEX_AVAILABLE | 0xFFFFFE00);

        // only proceed if error is due to limit switches
        fault &= ignore;

        switch (fault)
        {
            case LIM_NEG:
                break;
            case LIM_POS:
                // This could be the IR LED, or the one further up from
                // it (they are ORed). We move down enough to clear both.
                sm_log (2, "Z: backing out of positive limit...");
                
                send_axis_val (&AXIS(Z), CMD_P_REL, AXIS(Z).P_MIN * 2);
                send_axis_val (&AXIS(Z), CMD_V, AXIS(Z).V_SAFE);
                send_axis_val (&AXIS(Z), CMD_A, AXIS(Z).A_SAFE);
                send_axis_cmd (&AXIS(Z), "G");
                sleep_s_ms (2, 0);
                send_axis_cmd (&AXIS(Z), "Zr");
                
                if (axis_status (&AXIS(Z)) == ERR &&
                    ((AXIS(Z).sm[0].status & ignore) != LIM_NEG)) // in case we went too far down
                {
                    sm_log (2, "ERROR: Z: backing out of positive limit failed");
                    return ERR;
                }

                break;
            default:
                sm_log (2, "ERROR: unresolvable error");
                return ERR;
                break;
        }
    }

    sm_log (2, "homing Z...\n");

    // drive into positive limit
    send_axis_val (&AXIS(Z), CMD_V, AXIS(Z).V_SAFE);
    send_axis_val (&AXIS(Z), CMD_A, AXIS(Z).A_SAFE);
    send_axis_val (&AXIS(Z), CMD_P_REL, AXIS(Z).P_MIN * (-5));
    send_axis_cmd (&AXIS(Z), "G");
    sleep_s_ms (2, 0);
    send_axis_cmd (&AXIS(Z), "Zl");

    // positive limit must be flagged by now
    if (axis_status (&AXIS(Z)) == OK)
    {
        sm_log (2, "ERROR: Z: driving into positive limit switch failed");
        return ERR;
    }

    // now clear positive limit flag and back out a tad
    sm_log (2, "Z: getting out of positive limit...\n");

    send_axis_val (&AXIS(Z), CMD_P_REL, -120);
    send_axis_cmd (&AXIS(Z), "G");
    sleep_s_ms (1, 0);
    send_axis_cmd (&AXIS(Z), "Zr");

    if (axis_status (&AXIS(Z)) == ERR)
    {
        sm_log (2, "ERROR: Z: moving to origin failed");
        return ERR;
    }

    AXIS(Z).A = AXIS(Z).P = AXIS(Z).V = 0;

    send_axis_val (&AXIS(Z), "O", 0);
    send_axis_val (&AXIS(Z), "V", 0);
    send_axis_val (&AXIS(Z), "A", 0);
    send_axis_val (&AXIS(Z), "SLN", AXIS(Z).P_MIN - 40);
    send_axis_val (&AXIS(Z), "SLP", AXIS(Z).P_MAX + 40);
    send_axis_val (&AXIS(Z), "h", 0xA5);    // so SM knows it has been initialized
    send_axis_cmd (&AXIS(Z), "SLE");

    if (axis_status (&AXIS(Z)) == ERR)
    {
        sm_log (2, "ERROR: Z: initialization attempt failed");
        return ERR;
    }

    sm_log (2, "Z is ready\n");
    return OK;
}


/*==============================================================================
**  move_z
**
*/
static int __move_z__ (long p, bool safe_mode)
{
    long v;
    unsigned long a;

    if (safe_mode)
    {
        v = AXIS(Z).V_SAFE;
        a = AXIS(Z).A_SAFE;
    }
    else
    {
        v = AXIS(Z).V_MAX;
        a = AXIS(Z).A_MAX;
    }

    if (set_p (&AXIS(Z), p, true) == ERR || 
        set_v (&AXIS(Z), v) == ERR || 
        set_a (&AXIS(Z), a) == ERR)
    {
        return ERR;
    }

    if (servo_z() == ERR)
    {
        return ERR;   
    }

    return OK;
}
// Wrapper:
int _move_z_ (long p, bool safe_mode)
{
    char *reply;

    if (__move_z__ (p, safe_mode) == ERR)
    {
        sm_log (2, "ERROR: Z move failed");
        return ERR;
    }

    // might need a delay here
    sleep_s_ms (1, 0);

    // make sure we are where we are supposed to be
    if ((reply = send_sm_cmd (&MOTOR(Z, 0), CMD_REPORT_P)))
    {
        if (abs (AXIS(Z).P - (MOTOR(Z, 0).p = atoi (reply))) > ERR_MARGIN_Z)
        {
            sm_log (2, "ERROR: mismatching target and current Z positions");
            return ERR;
        }
    }
    else
    {
        sm_log (2, "ERROR: could not verify final Z position");
        return ERR;
    }

    return OK;
}


/*==============================================================================
**  park
**
*/
int park (void)
{
    int i;

    if (XYZ_NEEDS_INIT)
    {
        sm_log (2, "ERROR: cannot park unitialized system");
        return ERR;
    }

    if (axes_update (ALL_AXES) == ERR)
    {
        sm_log (2, "ERROR: cannot park system in fault mode");
        return ERR;
    }

    if ( ! VERBOSE ) fprintf (stderr, "Parking...\n");

    {
        Point p = {AXIS(X).P_MAX, AXIS(Y).P_MAX};
        POS_ABSOLUTE = true;

        if (_move_xy_ (&p, true) == ERR)
        {
            sm_log (2, "ERROR: failed to go to park position");
            return ERR;
        }

        sleep_s_ms (1, 0); // settle before engaging brakes
    }

    singlecast (CMD_G_OFF, 0);

    if (axes_update (ALL_AXES) == ERR)
    {
        sm_log (2, "ERROR: attempt to park failed");
        return ERR;
    }

    for (i = 0; i != Z; ++i)
    {
        AXIS(i).V = AXIS(i).A = 0;
        send_axis_val (&AXIS(i), CMD_V, AXIS(i).V);
        send_axis_val (&AXIS(i), CMD_A, AXIS(i).A);
    }
    
    if ( ! VERBOSE ) fprintf (stderr, "Done.\n");

    return OK;
}


/*==============================================================================
**  servo
**
*/
int servo (void)
{
    bool err = false;

    if (axes_update (ALL_AXES) == ERR) 
    {
        int i;

        for (i = 0; i != hydra.dim; ++i)
        {
            if (AXIS(i).fault)
            {
                sprintf (msg, "WARNING: axis [%s] is in fault mode\n", AXIS(i).name);
                sm_log (2, msg);
                err = true;
            }
        }
    }

    if (SYSTEM_IN_MOTION)
    {
        sm_log (2, "ERROR: system is in motion");
        return ERR;
    }

    if (err)
    {
        get_cl ("Are you sure you want to proceed? (y/n): ");

        while (strncasecmp (CL, "y", 1) && strncasecmp (CL, "n", 1))
        {
            get_cl ("Please enter y or n: ");
        }

        if (strncasecmp (CL, "y", 1) == 0)
        {
            err = false;
        }
    }

    if ( ! err ) singlecast (CMD_G_GO, 0);

    return err ? ERR : OK;
}


/*==============================================================================
**  servo_xy
**
*/
int servo_xy (void)
{
    int i;
    char *reply = 0;
    bool all_axes = XYZ_NEEDS_INIT ? EXCEPT_Z : ALL_AXES;

    if (axes_update (all_axes) == ERR) 
    {
        for (i = 0; i != hydra.dim; ++i)
        {
            if (AXIS(i).fault)
            {
                sprintf (msg, "ERROR: axis [%s] is in fault mode\n", AXIS(i).name);
                sm_log (2, msg);
            }
        }

        return ERR;
    }

    if (SYSTEM_IN_MOTION)
    {
        sm_log (2, "ERROR: system is in motion");
        return ERR;
    }

    // make sure Z is at psoition 0 (cruise height)
    if ((reply = send_sm_cmd (&AXIS(Z).sm[0], CMD_REPORT_P)))
    {
        if (0 != (AXIS(Z).sm[0].p = atoi (reply)))
        {
            sm_log (2, "ERROR: Z is not at cruise height");
            return ERR;
        }
    }
    else
    {
        sm_log (2, "ERROR: could not verify if Z is at cruise height");
        return ERR;
    }

    // we try twice, in case Z is already asleep
    if (wakeup_toggle (&AXIS(Z)) == ASLEEP || wakeup_toggle (&AXIS(Z)) == ASLEEP)
    {
        singlecast (CMD_G_GO, 0);
    }

    // wakeup Z
    if (wakeup_toggle (&AXIS(Z)) == ASLEEP)
    {
        return ERR;
    }

    return OK;
}


/*==============================================================================
**  servo_z
**
*/
int servo_z (void)
{
    if (axes_update (ALL_AXES) == ERR) 
    {
        int i;

        for (i = 0; i != hydra.dim; ++i)
        {
            if (AXIS(i).fault)
            {
                sprintf (msg, "ERROR: axis [%s] is in fault mode\n", AXIS(i).name);
                sm_log (2, msg);
            }
        }

        return ERR;
    }

    if (SYSTEM_IN_MOTION)
    {
        sm_log (2, "ERROR: system is in motion");
        return ERR;
    }

    // we try twice, in case X or Y is already asleep
    if ((wakeup_toggle (&AXIS(X)) == ASLEEP || wakeup_toggle (&AXIS(X)) == ASLEEP) &&
        (wakeup_toggle (&AXIS(Y)) == ASLEEP || wakeup_toggle (&AXIS(Y)) == ASLEEP))
    {
        singlecast (CMD_G_GO, 0);
    }

    // wakeup X & Y
    if (wakeup_toggle (&AXIS(X)) == ASLEEP || wakeup_toggle (&AXIS(Y)) == ASLEEP)
    {
        return ERR;
    }


    return OK;
}


/*==============================================================================
**  set_a
**
*/
static int set_a (Axis *ax, unsigned long a)
{
    if (validate_a (ax, &a) == ERR) return ERR;

    ax->A = a;
    send_axis_val (ax, CMD_A, ax->A);
    return OK;
}


/*==============================================================================
**  set_defaults
**
*/
static void set_defaults (void)
{
    singlecast (CMD_G_ZERO_P_REL, 0);
    singlecast (CMD_G_ZERO_P_ABS, 0);
    singlecast (CMD_G_ZERO_A, 0);
    singlecast (CMD_G_ZERO_V, 0);
    singlecast (CMD_G_P_MODE, 0);
    singlecast (CMD_G_BRKSRV, 0);           // no effect on Z
    singlecast (CMD_G_P_ERR, 0);
    singlecast (CMD_G_AMPS, 0);
    singlecast (CMD_G_ENC_EXTERNAL, 0);     // no effect on Z
}


/*==============================================================================
**  set_p
**
*/
static int set_p (Axis *ax, long p, bool validate)
{
    if (validate && (validate_p (ax, &p, POS_ABSOLUTE) == ERR)) return ERR;

    ax->P_VIRTUAL = ax->P_VIRTUAL_NEW;  // transaction complete
    ax->P = p;
    send_axis_val (ax, CMD_P_ABS, ax->P);
    return OK;
}


/*==============================================================================
**  set_v
**
*/
static int set_v (Axis *ax, long v)
{
    if (validate_v (ax, &v) == ERR) return ERR;

    ax->V = v;
    send_axis_val (ax, CMD_V, ax->V);
    return OK;
}


/*==============================================================================
**  sm_init
**
*/
int sm_init (void)
{
    int ret_val;
    int i, j;
    char *user;

    // 3 axes: 0=X, 1=Y, 2=Z
    static Axis axis[3]; 
    // X has two motors
    static Smart_Motor sm_x [2], sm_y, sm_z;

    hydra.axis = axis;
    hydra.dim = sizeof (axis) / sizeof (axis[0]);

    // X axis
    strcpy (axis[0].name, "X axis");
    axis[0].sm = sm_x;
    axis[0].dim = sizeof (sm_x) / sizeof (sm_x[0]);
    strcpy (sm_x[0].name, "X0");
    strcpy (sm_x[1].name, "X1");
    strcpy (sm_x[0].CMD, "2");      // smart motor id (programmed into EEPROM)
    strcpy (sm_x[1].CMD, "3");

    // Y axis
    strcpy (axis[1].name, "Y axis");
    axis[1].sm = &sm_y;
    axis[1].dim = 1;
    strcpy (sm_y.name, "Y");
    strcpy (sm_y.CMD, "4");

    // Z axis
    strcpy (axis[2].name, "Z axis");
    axis[2].sm = &sm_z;
    axis[2].dim = 1;
    strcpy (sm_z.name, "Z");
    strcpy (sm_z.CMD, "5");

    for (i = 0; i < hydra.dim; ++i)
        for (j = 0; j < AXIS(i).dim; ++ j)
            MOTOR(i, j).cmd = MOTOR(i, j).CMD + strlen (MOTOR(i, j).CMD);

    strcpy (log_unit.tag, "Smart_motor");

    if (log_init (&log_unit) == ERR)
    {
        fprintf (stderr, "%s\n", "smart_motor: FATAL: could not initiate logging");
        exit (ERR);
    }

    sm_log (VERBOSE ? 2 : 0, version());
    sprintf (msg, "PID: %d", getpid());
    sm_log (0, msg);

    user = getenv ("USER");

    if (user && strcmp (user, "behzad") == 0)
    {
        EXPERT_USER = true;
    }
    else
    {
        EXPERT_USER = false;
    }
    
    set_defaults();

    if ((ret_val = axes_update (ALL_AXES)) == ERR)
    {
        for (i = 0; i != hydra.dim; ++i)
        {
            if (AXIS(i).fault)
            {
                sprintf (msg, "WARNING: axis [%s] is in fault mode\n", AXIS(i).name);
                sm_log (2, msg);
            }
        }
    }
    else
    {
        int j;
        XYZ_NEEDS_INIT = false;

        for (i = 0; i != hydra.dim; ++i)
            for (j = 0; j != AXIS(i).dim; ++j)
            {
                // BUG!
                //if (atoi (send_sm_cmd (&AXIS(i).sm[j], "Rh")) != 0xA5) XYZ_NEEDS_INIT = true;
                
                // <Fix>
                char *reply = NULL;

                if ((reply = send_sm_cmd (&MOTOR(i, j), "Rh")))
                {
                    if (strtol (reply, 0, 0) != 0xA5)
                    {
                        XYZ_NEEDS_INIT = true;
                    }
                }
                // </FIX>
            }
    }

    AXIS(X).CMD_SLEEP  = AXIS(Y).CMD_SLEEP  = "SLEEP1";   
    AXIS(X).CMD_WAKEUP = AXIS(Y).CMD_WAKEUP = "WAKE1";   
    AXIS(Z).CMD_SLEEP  = "SLEEP";   
    AXIS(Z).CMD_WAKEUP = "WAKE";   
    AXIS(X).asleep = AXIS(Y).asleep = AXIS(Z).asleep = false;

    AXIS(X).ENC_CONST = ENC_CONST_X;
    AXIS(Y).ENC_CONST = ENC_CONST_Y;
    AXIS(Z).ENC_CONST = ENC_CONST_Z;

    AXIS(X).sm[0].enc_sense = ENC_SENSE_X0;
    AXIS(X).sm[1].enc_sense = ENC_SENSE_X1;   // X1 encoder is backwards
    AXIS(Y).sm[0].enc_sense = ENC_SENSE_Y;
    AXIS(Z).sm[0].enc_sense = ENC_SENSE_Z;

    AXIS(X).P_CONST = AXIS(Y).P_CONST = P_CONST_XY;
    AXIS(X).A_CONST = AXIS(Y).A_CONST = A_CONST_XY;
    AXIS(X).V_CONST = AXIS(Y).V_CONST = V_CONST_XY;
    AXIS(Z).P_CONST = P_CONST_Z;
    AXIS(Z).A_CONST = A_CONST_Z;
    AXIS(Z).V_CONST = V_CONST_Z;

    AXIS(X).CENTER = CENTER_X;

    AXIS(X).sm[0].calib_dist_model = (long) MODEL_CENTER_TO_RHS_X0;
    AXIS(X).sm[1].calib_dist_model = (long) MODEL_CENTER_TO_RHS_X1;

    AXIS(X).sm[0].calib_dist_real = (long) REAL_CENTER_TO_RHS_X0;
    AXIS(X).sm[1].calib_dist_real = (long) REAL_CENTER_TO_RHS_X1;

    AXIS(X).sm[0].origin = AXIS(X).CENTER - AXIS(X).sm[0].calib_dist_model + AXIS(X).sm[0].calib_dist_real;
    AXIS(X).sm[1].origin = AXIS(X).CENTER - AXIS(X).sm[1].calib_dist_model + AXIS(X).sm[1].calib_dist_real;

    AXIS(Y).CENTER = CENTER_Y;
    AXIS(Y).sm[0].calib_dist_model = (long) MODEL_CENTER_TO_BOTTOM_Y;
    AXIS(Y).sm[0].calib_dist_real = (long) REAL_CENTER_TO_BOTTOM_Y;
    AXIS(Y).sm[0].origin = AXIS(Y).CENTER - AXIS(Y).sm[0].calib_dist_model + AXIS(Y).sm[0].calib_dist_real;

    AXIS(X).P_MAX = (long) P_MAX_X;
    AXIS(Y).P_MAX = (long) P_MAX_Y;
    AXIS(Z).P_MAX = (long) P_MAX_Z;

    AXIS(X).P_MIN = (long) P_MIN_X;
    AXIS(Y).P_MIN = (long) P_MIN_Y;
    AXIS(Z).P_MIN = (long) P_MIN_Z;

    AXIS(X).V_MAX = (long) (AXIS(X).V_CONST * AXIS(X).ENC_CONST * V_MAX_XY);
    AXIS(Y).V_MAX = (long) (AXIS(Y).V_CONST * AXIS(Y).ENC_CONST * V_MAX_XY);
    AXIS(Z).V_MAX = (long) (AXIS(Z).V_CONST * AXIS(Z).ENC_CONST * V_MAX_Z);

    AXIS(X).A_MAX = (unsigned long) (AXIS(X).A_CONST * AXIS(X).ENC_CONST * A_MAX_XY);
    AXIS(Y).A_MAX = (unsigned long) (AXIS(Y).A_CONST * AXIS(Y).ENC_CONST * A_MAX_XY);
    AXIS(Z).A_MAX = (unsigned long) (AXIS(Z).A_CONST * AXIS(Z).ENC_CONST * A_MAX_Z);
   
    AXIS(X).V_SAFE = (long) (AXIS(X).V_CONST * AXIS(X).ENC_CONST * V_SAFE_XY); // with fiber
    AXIS(Y).V_SAFE = (long) (AXIS(Y).V_CONST * AXIS(Y).ENC_CONST * V_SAFE_XY); // with fiber
    AXIS(Z).V_SAFE = (long) (AXIS(Z).V_CONST * AXIS(Z).ENC_CONST * V_SAFE_Z);

    AXIS(X).A_SAFE = (unsigned long) (AXIS(X).A_CONST * AXIS(X).ENC_CONST * A_SAFE_XY); // with fiber
    AXIS(Y).A_SAFE = (unsigned long) (AXIS(Y).A_CONST * AXIS(Y).ENC_CONST * A_SAFE_XY); // with fiber
    AXIS(Z).A_SAFE = (unsigned long) (AXIS(Z).A_CONST * AXIS(Z).ENC_CONST * A_SAFE_Z);

    AXIS(X).V_MIN = (long) (AXIS(X).V_CONST * AXIS(X).ENC_CONST * V_MIN_XY);
    AXIS(Y).V_MIN = (long) (AXIS(Y).V_CONST * AXIS(Y).ENC_CONST * V_MIN_XY);
    AXIS(Z).V_MIN = (long) (AXIS(Z).V_CONST * AXIS(Z).ENC_CONST * V_MIN_Z);

    AXIS(X).A_MIN = (unsigned long) (AXIS(X).A_CONST * AXIS(X).ENC_CONST * A_MIN_XY);
    AXIS(Y).A_MIN = (unsigned long) (AXIS(Y).A_CONST * AXIS(Y).ENC_CONST * A_MIN_XY);
    AXIS(Z).A_MIN = (unsigned long) (AXIS(Z).A_CONST * AXIS(Z).ENC_CONST * A_MIN_Z);

    if (HYDRA_SAFE_MODE)
    {
        AXIS(X).V_MAX = AXIS(Y).V_MAX = AXIS(X).V_SAFE;
        AXIS(X).A_MAX = AXIS(Y).A_MAX = AXIS(X).A_SAFE;
        AXIS(Z).V_MAX = AXIS(Z).V_SAFE;
        AXIS(Z).A_MAX = AXIS(Z).A_SAFE;
    }

    if (false == XYZ_NEEDS_INIT)
    {
        for (i = 0; i != hydra.dim; ++i)
        {
            axis_pv (&AXIS(i));
            AXIS(i).P_VIRTUAL = AXIS(i).P_VIRTUAL_NEW = AXIS(i).P = AXIS(i).sm[0].p;
            send_axis_val (&AXIS(i), CMD_P_ABS, AXIS(i).P);
        }
    }

    return ret_val;
}


/*==============================================================================
**  sm_log
**
*/
int sm_log (int pipe, char *msg)
{
    return log_it (&log_unit, pipe, msg);
}


/*==============================================================================
**  test_run
**
*/
void test_run (char mode)
{
    int i;
    bool safe_mode = false;
    Point p;

    if (HYDRA_SAFE_MODE || 'T' == mode)
    {
        safe_mode = true;
    }

    if (XYZ_NEEDS_INIT)
    {
        sm_log (2, "ERROR: cannot test unitialized system");
        return;
    }

    POS_ABSOLUTE = true;

    for (i = 0; i != 5; ++i)
    {
        p.x = AXIS(X).P_MIN;
        p.y = AXIS(Y).P_MIN;
        if (_move_xy_ (&p, safe_mode) == ERR) return;  

        p.x = AXIS(X).P_MAX;
        p.y = AXIS(Y).P_MAX;
        if (_move_xy_ (&p, safe_mode) == ERR) return;  
    }

    p.x = p.y = 0;
    if (_move_xy_ (&p, safe_mode) == ERR) return;  
}


/*==============================================================================
**  _move_xy_
**
*/
static int __move_xy__ (Point *p, bool safe_mode, Hydra_Time *tp)
{
    unsigned long a, a_x, a_y;
    long v, v_x, v_y;
    Point p_now, p_model;
    unsigned long dist_x, dist_y;
    double theta;
    unsigned long D;
    double t, d, dbl_v, dbl_a;
    int axis;

    if (XYZ_NEEDS_INIT)
    {
        p_now.x = AXIS(X).P;
        p_now.y = AXIS(Y).P;
    }
    else
    {
        p_now.x = MOTOR(X, 0).p;
        p_now.y = MOTOR(Y, 0).p;
    }

    if (validate_p (&AXIS(X), &p->x, POS_ABSOLUTE) == ERR || validate_p (&AXIS(Y), &p->y, POS_ABSOLUTE) == ERR) return ERR;

    AXIS(X).P_VIRTUAL_NEW = p->x;
    AXIS(Y).P_VIRTUAL_NEW = p->y;

    if (MODEL_GEOMETRY)
    {
        p_model = model_geometry (p);

        if (validate_p (&AXIS(X), &p_model.x, true) == ERR || validate_p (&AXIS(Y), &p_model.y, true) == ERR) return ERR;

        fprintf (stderr, "\t\tPx: %ld\t Py: %ld\n", p->x, p->y);
        fprintf (stderr, "G Model:\tPx: %ld\t Py: %ld\n", p_model.x, p_model.y);
    }
    
    if (MODEL_TEMPERATURE)
    {
        p_model = model_temperature (p, CURRENT_TEMPERATURE);

        if (validate_p (&AXIS(X), &p_model.x, true) == ERR || validate_p (&AXIS(Y), &p_model.y, true) == ERR) return ERR;

        fprintf (stderr, "\t\tPx: %ld\t Py: %ld\n", p->x, p->y);
        fprintf (stderr, "T Model:\tPx: %ld\t Py: %ld\n", p_model.x, p_model.y);

        p = &p_model;
    }

    dist_x = abs (p->x - p_now.x);
    dist_y = abs (p->y - p_now.y);
    theta = atan2 (dist_y, dist_x);

    if (safe_mode)
    {
        // the vector should be equal to the safe value
        v = AXIS(X).V_SAFE;
        a = AXIS(X).A_SAFE;

        // global used to reduce fiber speed by 1/3 for fops
        if (THIS_FIBER_IS_FOP)
        {
            v = (long) (v / 3.0);
            a = (unsigned long) (v / 3.0);
        }
    }
    else
    {
        // but here the largest component of the vector 
        // should be equal to the max value, which means the
        // vector could be larger.
        
        if (theta <= M_PI / 2.0 && theta >= M_PI / 4.0)
        {
            v = (long) (AXIS(X).V_MAX / sin (theta));
            a = (unsigned long) (AXIS(X).A_MAX / sin (theta));
        }
        else
        {
            v = (long) (AXIS(X).V_MAX / cos (theta));
            a = (unsigned long) (AXIS(X).A_MAX / cos (theta));
        }
    }

    v_x = (long) (v * cos (theta));
    v_y = (long) (v * sin (theta));
  
    a_x = (unsigned long) (a * cos (theta));
    a_y = (unsigned long) (a * sin (theta));

    /*
    if (validate_v (&AXIS(X), &v_x) == ERR || validate_v (&AXIS(Y), &v_y) == ERR) return ERR;
    if (validate_a (&AXIS(X), &a_x) == ERR || validate_a (&AXIS(Y), &a_y) == ERR) return ERR;
    */
    
    if (set_p (&AXIS(X), p->x, false) == ERR || set_p (&AXIS(Y), p->y, false) == ERR) return ERR;

    if (dist_x > 0)
    {
        if (set_v (&AXIS(X), v_x) == ERR || set_a (&AXIS(X), a_x) == ERR) return ERR;
    }

    if (dist_y > 0)
    {
        if (set_v (&AXIS(Y), v_y) == ERR || set_a (&AXIS(Y), a_y) == ERR) return ERR;
    }

    if (servo_xy() == ERR) return ERR;   

    fprintf (stderr, "virtual: %ld %ld\n", AXIS(X).P_VIRTUAL, AXIS(Y).P_VIRTUAL);

    if (dist_x > dist_y + 250)
    {
        axis = X;
        D = dist_x;
    }
    else if (dist_y > 250)
    {
        axis = Y;
        D = dist_y;
    }
    else
    {
        sm_log (VERBOSE ? 2 : 0, "Move <= 250 u");
        tp->s = tp->ms = 0;
        return OK;
    }

    dbl_v = AXIS(axis).V / AXIS(axis).ENC_CONST / AXIS(axis).V_CONST;
    dbl_a = AXIS(axis).A / AXIS(axis).ENC_CONST / AXIS(axis).A_CONST;
    d = dbl_v * dbl_v * 10000.0 / dbl_a;

    if (d <= 250.0)
    {
        sm_log (2, "********** WARNING: move <= 250 (should not be; possible bug)");
        tp->s = tp->ms = 0;
        return OK;
    }

    if (D >= d - 250)
    {
        t = (d + D) / 10000.0 / dbl_v;
    }
    else
    {
        t = 2.0 * sqrt (D / 10000.0 / dbl_a);
    }

    tp->s = (unsigned long) (floor (t));
    tp->ms = ((unsigned long) (t * 1000.0)) % 1000;

    return OK;
}
// Wrapper:
int _move_xy_ (Point *p, bool safe_mode)
{
    int i;
    Hydra_Time tp;

    if (__move_xy__ (p, safe_mode, &tp) == ERR)
    {
        sm_log (2, "ERROR: XY trajectory failed");
        return ERR;
    }

    sprintf (msg, "Trajectory will take %lu s and %lu ms", tp.s, tp.ms);
    sm_log (2, msg);
    hydra_sleep (&tp);

    // make sure we are where we are supposed to be
    if ( ! XYZ_NEEDS_INIT )
    {
    for (i = 0; i != Z; ++i)
    {
        char *reply = 0;
        int j;

        for (j = 0; j != AXIS(i).dim; ++j)
        {
            if ((reply = send_sm_cmd (&MOTOR(i, j), CMD_REPORT_P)))
            {
                if (abs (AXIS(i).P - (MOTOR(i, j).p = atoi (reply))) > ERR_MARGIN_XY)
                {
                    sm_log (2, "ERROR: mismatching target and current XY positions");
                    return ERR;
                }
            }
            else
            {
                sm_log (2, "ERROR: could not verify final XY position");
                return ERR;
            }
        }
    }
    }

    return OK;
}


/*==============================================================================
**  validate_a
**
*/
static int validate_a (Axis *ax, unsigned long *accel)
{
    unsigned long a = *accel;

    if (ULONG_MAX == a || a > LONG_MAX)
    {
        sprintf (msg, "ERROR: invalid acceleration value for %s\n", ax->name);
        sm_log (2, msg);
        return ERR;
    }
    
    if (a > ax->A_MAX)
    {
        a = ax->A_MAX;
        sprintf (msg, "WARNING: A set to maximum for %s", ax->name);
        sm_log (VERBOSE ? 2 : 0, msg);
    }
    else if (a < ax->A_MIN)
    {
        a = ax->A_MIN;
        sprintf (msg, "WARNING: A set to minimum for %s", ax->name);
        sm_log (VERBOSE ? 2 : 0, msg);
    }

    *accel = a;

    return OK;
}


/*==============================================================================
**  validate_p
**
*/
static int validate_p (Axis *ax, long *pos, bool is_absolute)
{
    long p = *pos;

    p += is_absolute ? 0 : ax->P_VIRTUAL;

    if (LONG_MAX == p || LONG_MIN == p)
    {
        sprintf (msg, "ERROR: invalid %s position\n", ax->name);
        sm_log (2, msg);
        return ERR;
    }

    if ((false == XYZ_NEEDS_INIT) && (p > ax->P_MAX || p < ax->P_MIN))
    {
        sprintf (msg, "ERROR: %s position should be between %ld and %ld", ax->name, ax->P_MIN, ax->P_MAX);
        sm_log (2, msg);
        return ERR;
    }

    *pos = p;

    return OK;
}


/*==============================================================================
**  validate_v
**
*/
static int validate_v (Axis *ax, long *vel)
{
    long v = *vel;

    if (LONG_MAX == v || v < 0)
    {
        sprintf (msg, "ERROR: invalid velocity value for %s\n", ax->name);
        sm_log (2, msg);
        return ERR;
    }

    if (v > ax->V_MAX)
    {
        v = ax->V_MAX;
        sprintf (msg, "WARNING: V set to maximum for %s", ax->name);
        sm_log (VERBOSE ? 2 : 0, msg);
    }
    else if (v < ax->V_MIN)
    {
        v = ax->V_MIN;
        sprintf (msg, "WARNING: V set to minimum for %s", ax->name);
        sm_log (VERBOSE ? 2 : 0, msg);
    }

    *vel = v;

    return OK;
}


/*==============================================================================
**  which_direction
**
**  should be called only if axes_update (EXCEPT_Z) returns ERR
*/
static int which_direction (void)
{
    int i;

    for (i = 0; i != Z; ++i)
    {
        if (AXIS(i).fault)
        {
            int j;
            unsigned int fault = 0;

            for (j = 0; j != AXIS(i).dim; ++j)
            {
                fault |= MOTOR(i, j).status;
            }

            // only proceed if error is due to limit switches
            fault &= ~(SERVO | WRAPAROUND | INDEX_AVAILABLE | 0xFFFFFE00);

            switch (fault)
            {
                case LIM_NEG:
                    send_axis_cmd (&AXIS(i), CMD_CLR_LIM_NEG);
                    send_axis_cmd (&AXIS(i), CMD_DISABLE_LIM_SOFT);
                    AXIS(i).enc_dir = 1;
                    break;
                case LIM_POS:
                    send_axis_cmd (&AXIS(i), CMD_CLR_LIM_POS);
                    send_axis_cmd (&AXIS(i), CMD_DISABLE_LIM_SOFT);
                    AXIS(i).enc_dir = -1;
                    break;
                default:
                    sprintf (msg, "i=%d, j=%d [%u]\n", i, j, fault);
                    sm_log (2, msg);
                    sm_log (2, "ERROR: unresolvable error");
                    //return ERR;
                    exit (ERR);
                    break;
            }
        }
    }

    if (axes_update (EXCEPT_Z) == ERR)
    {
        sm_log (2, "ERROR: unexpected error");
        return ERR;
    }

    return OK;
}
