/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_system.c
**  11 Aug 2004 : Hydra II integration of Smart Motors and 8051.
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "hydra_system_private.h"


/*  globals & statics
------------------------------------------------------------------------------*/
// this gets updated each time we successfully pick up or drop off
// a button; we rely on this to decide if we should move fast (no button)
// or slow (with button).
static int GRIPPER_HAS_BUTTON = FALSE;

// turn off safe (slow) mode in moves
#define MOVE_FAST (false)
#define MOVE_SLOW (true)


/*==============================================================================
**  hydra_shutdown
**
*/
void hydra_shutdown (void)
{
    int status;
    int warnings = 0;

    // turn vacuum off
    if (mc_exe_cmd ("vacuum off") == ERR)
    {
        fprintf (stderr, "WARNING: could not turn off vacuum\n");
        ++warnings;
    }
    
    // turn off lamp power
    if (mc_exe_cmd ("lamps power hc off") == ERR)
    {
        fprintf (stderr, "WARNING: could not turn off lamp power\n");
        ++warnings;
    }

    // turn off screen power
    if (mc_exe_cmd ("screen off") == ERR)
    {
        fprintf (stderr, "WARNING: could not turn off screen power\n");
        ++warnings;
    }

    // turn off leds
    if (mc_exe_cmd ("leds off") == ERR)
    {
        fprintf (stderr, "WARNING: could not turn off leds\n");
        ++warnings;
    }

    // turn off IR limit switch (Z positive limit)
    if (mc_exe_cmd ("ir switch off") == ERR)
    {
        fprintf (stderr, "WARNING: could not turn off IR limit switch\n");
        ++warnings;
    }

    // check power status
    if ((status = mc_exe_cmd ("power status")) == ERR)
    {
        fprintf (stderr, "WARNING: either serial link is down or main power is off\n");
        ++warnings;
    }
    else
    {
        // check motor power (42 V) and turn it off if on
        if ( ! (status & _OFF_42V_) )
        {
            fprintf (stderr, "turnning off motor power (42 V)\n");

            if (mc_exe_cmd ("motors disable") == ERR)
            {
                fprintf (stderr, "WARNING: could not turn off motor power (42 V)\n");
                ++warnings;
            }
        }
        else
        {
            fprintf (stderr, "motor power (42 V) is already off\n");
        }
    }

    // make sure gripper is not holding a button
    if ((status = gripper_has_button()) == TRUE)
    {
        fprintf (stderr, "WARNING: gripper has button\n");
        ++warnings;
    }
    else if (ERR == status)
    {
        fprintf (stderr, "WARNING: gripper check for button failed\n");
        ++warnings;
    }

    // disable gripper
    if (mc_exe_cmd ("gripper disable") == ERR)
    {
        fprintf (stderr, "WARNING: could not disable gripper\n");
        ++warnings;
    }

    // end serial communication
    com_end();

    fprintf (stderr, "Hydra II shutdwon complete with %d warnings\n", warnings);

    return;
}


/*==============================================================================
**  hydra_startup
**
*/
void hydra_startup (void)
{
    int status;

    // serial communication
    com_init (HYDRA_COM_CONFIG, HYDRA_COM_END_CHAR, HYDRA_COM_WAIT, REAL_COMMUNICATION | IGNORE_LOCK);
    
    // 8051 microcontroller data
    mc_init();

    // check power status
    if ((status = mc_exe_cmd ("power status")) == ERR)
    {
        fprintf (stderr, "ERROR: either serial link is down or main power is off\n");
        exit (ERR);
    }

    // check panic switches
    if (status & (_PANIC_SW1_ | _PANIC_SW2_))
    {
        fprintf (stderr, "ERROR: panic switch is active\n");
        exit (ERR);
    }

    // check motor power (42 V) and turn it on if off
    if (status & _OFF_42V_)
    {
        fprintf (stderr, "turnning on motor power (42 V)\n");

        if (mc_exe_cmd ("motors enable") == ERR)
        {
            fprintf (stderr, "ERROR: could not turn on motor power (42 V)\n");
            exit (ERR);
        }
    }
    else
    {
        fprintf (stderr, "motor power (42 V) is already on\n");
    }

    // enable gripper
    if (mc_exe_cmd ("gripper enable") == ERR)
    {
        fprintf (stderr, "ERROR: could not enable gripper\n");
        exit (ERR);
    }

    // make sure gripper is not holding a button
    if ((status = gripper_has_button()) == TRUE)
    {
        fprintf (stderr, "ERROR: gripper has button\n");
        exit (ERR);
    }
    else if (ERR == status)
    {
        fprintf (stderr, "ERROR: gripper check for button failed\n");
        exit (ERR);
    }

    // turn on IR limit switch (Z positive limit)
    if (mc_exe_cmd ("ir switch on") == ERR)
    {
        fprintf (stderr, "ERROR: could not turn on IR limit switch\n");
        exit (ERR);
    }

    // turn on lamp power
    if (mc_exe_cmd ("lamps power hc on") == ERR)
    {
        fprintf (stderr, "ERROR: could not turn on lamp power\n");
        exit (ERR);
    }

    // turn on screen power
    if (mc_exe_cmd ("screen on") == ERR)
    {
        fprintf (stderr, "ERROR: could not turn on screen power\n");
        exit (ERR);
    }

    // turn vacuum on
    if (mc_exe_cmd ("vacuum on") == ERR)
    {
        fprintf (stderr, "ERROR: could not turn on vacuum\n");
        exit (ERR);
    }

    // Smart Motor data
    sm_init();

    // initialize the XYZ stage if needed
    if (XYZ_NEEDS_INIT)
    {
        if (init_xyz() == ERR)
        {
            fprintf (stderr, "ERROR: XYZ initialization failed");
            exit (ERR);
        }
    }

    fprintf (stderr, "Hydra II is ready\n");

    return;
}


/*==============================================================================
**  gripper_has_button
**
*/
static int gripper_has_button (void)
{
    if (mc_exe_cmd ("gripper close") == ERR)
    {
        return ERR;
    }

    return (mc_exe_cmd ("button status"));
}


/*==============================================================================
**  move_xy
**
*/
int move_xy (long x, long y, System system)
{
    Point p = {x, y};
    bool mode;

    if (HYDRA_ONE == system)
    {
        p.x = (long) (HYDRA_ONE_SCALE * p.x);
        p.y = (long) (HYDRA_ONE_SCALE * p.y);
    }

    switch (GRIPPER_HAS_BUTTON)
    {
        case TRUE :
            mode = MOVE_SLOW;
            break;
        case FALSE :
            mode = MOVE_FAST;
            break;
        default :
            return ERR;
    }

    return _move_xy_ (&p, mode);
}


/*==============================================================================
**  move_z
**
*/
int move_z (long z)
{
    return _move_z_ (z, MOVE_FAST);
}


/*==============================================================================
**  pick_drop
**
*/
int pick_n_drop (unsigned mode)
{
    long z; 

    if (PICK_PARK == mode || DROP_PARK == mode)
    {
        z = Z_PARK;
    }
    else
    {
        z = Z_PLATE;
    }

    switch (mode)
    {
        case DROP_PARK :
        case DROP_PLATE :
            // 1. go down
            if (move_z (z) == ERR)
            {
                return ERR;
            }

            // delay here ?
            
            // 2. open (button will drop)
            if (mc_exe_cmd ("gripper open") == ERR)
            {
                return ERR;
            }

            // delay here ?

            // 3. go up
            if (move_z (Z_CRUISE) == ERR)
            {
                return ERR;
            }

            // 4. close
            if (mc_exe_cmd ("gripper close") == ERR)
            {
                return ERR;
            }
#ifndef HYDRA_II_TEST_MODE
            // 5. we should NOT have a button
            if ((GRIPPER_HAS_BUTTON = mc_exe_cmd ("button status")) == TRUE)
            {
                return ERR;
            }
#endif
// HYDRA_II_TEST_MODE
            break;
        case PICK_PARK :
        case PICK_PLATE :
            // 1. open wide
            if (mc_exe_cmd ("gripper openwide") == ERR)
            {
                return ERR;
            }

            // 2. go down
            if (move_z (z) == ERR)
            {
                return ERR;
            }

            // delay here ?

            // 3. close (to grab button)
            if (mc_exe_cmd ("gripper close") == ERR)
            {
                return ERR;
            }
            
            // delay here ?
            
            // 4. go up
            if (move_z (Z_CRUISE) == ERR)
            {
                return ERR;
            }

#ifndef HYDRA_II_TEST_MODE
            // 5. we should have a button
            if ((GRIPPER_HAS_BUTTON = mc_exe_cmd ("button status")) == TRUE)
            {
                return ERR;
            }
#endif
// HYDRA_II_TEST_MODE
            break;
    }

    return OK;
}
