/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_tools.c
**  01 Jun 2004 : Hydra II test tools.
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "hydra_tools_private.h"


/*  globals 
------------------------------------------------------------------------------*/
Hydra_Time SETTLING_TIME = {0, 500};

static char msg [TOOLS_BUF_SIZE];
static FILE *map_fp = NULL;
static FILE *out_fp = NULL;
static FILE *profile_fp = NULL;
static bool STOP_MAPPING;


/*==============================================================================
**  log_profile
**
*/
int log_profile (Point *p, Hydra_Time *t)
{
    static char profile [] = "profile.log";
    unsigned long t_ms = t->s * 1000 + t->ms;
    Hydra_Time s = {0, 100};
    int n;

    if (NULL == profile_fp)
    {
        profile_fp = fopen (profile, "a");

        if ( ! profile_fp )
        {
            perror (profile);
            return ERR;
        }

        fprintf (profile_fp, "# Key: Target X Y, Current X0 Y X1 (all in microns)\n");
    }

    fprintf (profile_fp, "%-10ld %-10ld\n", p->x, p->y);

    for (n = t_ms / 100; n != -1; --n)
    {
        fprintf (profile_fp, "\t\t\t%-10s",  send_sm_cmd (&MOTOR(X, 0), "RP"));
        fprintf (profile_fp, " %-10s",   send_sm_cmd (&MOTOR(Y, 0), "RP"));
        fprintf (profile_fp, " %-10s\n", send_sm_cmd (&MOTOR(X, 1), "RP"));
        hydra_sleep (&s);
    }

    if (t_ms % 100)
    {
        fprintf (profile_fp, "\t\t\t%-10s",  send_sm_cmd (&MOTOR(X, 0), "RP"));
        fprintf (profile_fp, " %-10s",   send_sm_cmd (&MOTOR(Y, 0), "RP"));
        fprintf (profile_fp, " %-10s\n", send_sm_cmd (&MOTOR(X, 1), "RP"));
    }

    return OK;
}


/*==============================================================================
**  mapper
**
*/
int mapper (char *cl, char mode)
{
    static bool WAS_STOPPED;
    static bool DRY_RUN;
    static unsigned long record;
    static Point p;
    static char id [192];
    static char map_file [128] = "hydra.map";
    static char out_file [] = "map.out";
    char line [256];
    int line_len = sizeof (line);
    static long max_v;
    static unsigned long max_a; 

    if (XYZ_NEEDS_INIT)
    {
        sm_log (2, "ERROR: HYDRA is not initialized");
        return ERR;
    }

    POS_ABSOLUTE = true;

    if (STOP_MAPPING)
    {
        STOP_MAPPING = false;
        WAS_STOPPED = true;
    }

    if (NULL == map_fp)
    {
        record = 0;
        max_v = AXIS(X).V_MAX;
        max_a = AXIS(X).A_MAX;

        while (isspace (*cl)) cl++;

        if (*cl)
        {
            sscanf (cl, "%127s", map_file);
        }

        map_fp = fopen (map_file, "r");

        if ( ! map_fp )
        {
            perror (map_file);
            return ERR;
        }

        out_fp = fopen (out_file, "a");

        if ( ! out_fp )
        {
            perror (out_file);
            return ERR;
        }

        fprintf (out_fp, "# Last 6 fields are X0, X1, and Y internal encoder readouts in\n"
                         "# microns, with corresponding external encoder values in braces.\n"
                         "# Note that internal encoder resolution is 2.5 microns, and\n"
                         "# the accuracy is 3.8 microns.\n");

        DRY_RUN = (EXPERT_USER && 'n' == mode) ? true : false;
    }

    while (false == STOP_MAPPING)
    {
        int i;
        double x, y, v_p, a_p;
        char *cp = line;
        char cmd [320];
        int fits;

        if (WAS_STOPPED)
        {
            WAS_STOPPED = false;
            --record;
        }
        else
        {
            if (fgets (line, line_len, map_fp) == NULL) break;

            // skip white space
            while (isspace (*cp))
            {
                cp++;
            }

            // skip blank lines and comment lines
            if ( ( ! (*cp) ) || '#' == *cp)
            {
                continue;
            }
            
            // each line has 6 fields: id, x, y, and optional V and A
            // as percentage of max V and A
            i = sscanf (line, "%s %lf %lf %lf %lf", id, &x, &y, &v_p, &a_p);

            if ( ! ((3 == i) || (5 == i)) )
            {
                sprintf (msg, "ERROR: invalid record in map file; skipping record %lu\n", ++record);
                sm_log (2, msg);
                continue;
            }

            if ( ! HYDRA_SAFE_MODE )
            {
                if (5 == i)
                {
                    if (v_p >= 1.0 && v_p <= 100.0 && a_p >= 1.0 && a_p <= 100.0)
                    {
                        AXIS(X).V_MAX = AXIS(Y).V_MAX = (long)(max_v * v_p / 100.0);
                        AXIS(X).A_MAX = AXIS(Y).A_MAX = (unsigned long) (max_a * a_p / 100.0);
                    }
                    else
                    {
                        sprintf (msg, "ERROR: invalid V (%f) and/or A (%f) percentage; skipping record %lu\n", v_p, a_p, ++record);
                        sm_log (2, msg);
                        continue;
                    }
                }
                else
                {
                    AXIS(X).V_MAX = AXIS(Y).V_MAX = max_v;
                    AXIS(X).A_MAX = AXIS(Y).A_MAX = max_a;
                }
            }

            p.x = (long) (round (x));
            p.y = (long) (round (y));
        }

        sprintf (msg, "Mapping rec %-6lu [ID] %-12s [X] %-10ld [Y] %-10ld\n", record + 1, id, p.x, p.y);
        sm_log (2, msg);

        if (_move_xy_ (&p, false) == ERR)
        {
            sprintf (msg, "skipping record %lu (ID: %s)", ++record, id);
            sm_log (2, msg);
            continue;
        }

        sm_log (2, "settling...");
        hydra_sleep (&SETTLING_TIME);

        // <read out internal and external encoder values and write them to map.out>
        {
            int i, j;

            fprintf (out_fp, "%-12s %-10ld %-10ld", id, p.x, p.y);

            for (i = 0; i != Z; ++i)
            {
                for (j = 0; j != AXIS(i).dim; ++j)
                {
                    fprintf (out_fp, " %-10s", send_sm_cmd (&AXIS(i).sm[j], "RCTR"));
                    fprintf (out_fp, " (%-10s)", send_sm_cmd (&AXIS(i).sm[j], "RP"));
                }
            }

            fprintf (out_fp, "%s", "\n"); 
        }
        
        sprintf (cmd, "tellguider snapshot %s", id);
        sprintf (msg, "creating fits image <%s> ...", cmd);
        sm_log (2, msg);

        if (false == DRY_RUN)
        {
            fits = system (cmd);

            if (WIFSIGNALED (fits) && WTERMSIG (fits) == SIGINT)
            {
                STOP_MAPPING = true;
            }

            if (ERR == fits)
            {
                sprintf (msg, "ERROR: problem with FITS image (ID: %s)", id);
                sm_log (2, msg);
            }
        }

        record++;
    }

    if ( ! HYDRA_SAFE_MODE )
    {
        AXIS(X).V_MAX = AXIS(Y).V_MAX = max_v;
        AXIS(X).A_MAX = AXIS(Y).A_MAX = max_a;
    }

    if (STOP_MAPPING)
    {
        sm_log (2, "Mapping paused");
        return OK;
    }

    sprintf (msg, "Mapping done; %lu records processed.", record);
    sm_log (2, msg);

    if (fclose (map_fp) == EOF)
    {
        perror (map_file);
        return ERR;
    }
    
    if (fclose (out_fp) == EOF)
    {
        perror (out_file);
        return ERR;
    }

    map_fp = NULL;
    out_fp = NULL;

    return OK;
}


/*==============================================================================
**  mapper_isr
**
*/
void mapper_signal_handler (int sig)
{
    STOP_MAPPING = true;
}
