/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  smart_motor.c
**  3 Feb 2004 : Instrument-neutral Smart Motor module.
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "smart_motor_private.h"


/*  globals 
------------------------------------------------------------------------------*/


/*==============================================================================
**  axis_pv
**
*/
int axis_pv (Axis *ax)
{
    int i;
    ax->fault = false;

    for (i = 0; i != ax->dim; ++i)
    {
        motor_pv (&ax->sm[i]);
        ax->fault |= (ax->sm[i].fault | ax->sm[i].com_fail);
    }

    return ax->fault ? ERR : OK;
}


/*==============================================================================
**  axis_status
**
*/
int axis_status (Axis *ax)
{
    int i;
    ax->fault = false;

    for (i = 0; i != ax->dim; ++i)
    {
        motor_status (&ax->sm[i]);
        ax->fault |= (ax->sm[i].fault | ax->sm[i].com_fail);
        ax->moving |= (ax->sm[i].moving);
    }

    return ax->fault ? ERR : OK;
}


/*==============================================================================
**  motor_pv
**
*/
static int motor_pv (Smart_Motor *sm)
{
    char *reply = 0;
    sm->com_fail = false;
    
    if ((reply = send_sm_cmd (sm, CMD_REPORT_P))) sm->p = atoi (reply);
    else sm->com_fail = true;

    if ((reply = send_sm_cmd (sm, CMD_REPORT_P_ERR))) sm->p_err = atoi (reply);
    else sm->com_fail = true;

    if ((reply = send_sm_cmd (sm, CMD_REPORT_V))) sm->v = atoi (reply);
    else sm->com_fail = true;

    return sm->com_fail ? ERR : OK;
}


/*==============================================================================
**  motor_status
**
*/
static int motor_status (Smart_Motor *sm)
{
    char *reply = send_sm_cmd (sm, CMD_REPORT_STATUS);

    if ( ! reply )   
    {
        sm->com_fail = true;
        return ERR;
    }

    sm->status = atoi (reply);

    sm->excess_current = sm->status & EXCESS_CURRENT ? true : false;
    sm->servo = sm->status & SERVO ? false : true;
    sm->excess_heat = sm->status & EXCESS_HEAT? true : false;
    sm->excess_p_err = sm->status & EXCESS_P_ERR ? true : false;
    sm->wraparound = sm->status & WRAPAROUND ? true : false;
    sm->index_available = sm->status & INDEX_AVAILABLE ? true : false;
    sm->lim_neg = sm->status & LIM_NEG ? true : false;
    sm->lim_pos = sm->status & LIM_POS ? true : false;
    sm->moving = sm->status & MOVING ? true : false;

    sm->fault = sm->status & (EXCESS_CURRENT | EXCESS_HEAT | EXCESS_P_ERR | WRAPAROUND | LIM_NEG | LIM_POS) ? true : false;
    sm->com_fail = false;

    return OK;
}


/*==============================================================================
**  report_long
**
*/
int report_long (Axis *ax)
{
    int i;

    fprintf (stderr, "\n%29s%12s%12s%12s%12s\n", "motor", "moving", "servo", "index", "fault");
    fprintf (stderr, "%s\n", "                        -----------------------------------------------------");

    axis_pv (ax);
    report_short (ax);

    fprintf (stderr, "%s", "\n");

    for (i = 0; i != ax->dim; ++i)
    {
        if (ax->sm[i].com_fail)
        {
            continue;
        }

        if (ax->sm[i].fault)
        {
            if (ax->sm[i].excess_current) fprintf (stderr, "%10s FAULT: excessive current\n", ax->sm[i].name); 
            if (ax->sm[i].excess_heat) fprintf (stderr, "%10s FAULT: excessive heat\n", ax->sm[i].name);
            if (ax->sm[i].excess_p_err) fprintf (stderr, "%10s FAULT: excessive position error\n", ax->sm[i].name); 
            if (ax->sm[i].wraparound) fprintf (stderr, "%10s FAULT: wraparound\n", ax->sm[i].name); 
            if (ax->sm[i].lim_neg) fprintf (stderr, "%10s FAULT: negative limit\n", ax->sm[i].name); 
            if (ax->sm[i].lim_pos) fprintf (stderr, "%10s FAULT: positive limit\n", ax->sm[i].name); 
            fprintf (stderr, "%s", "\n");
        }
    }

    fprintf (stderr, "%5s%12s%12s%12s%12s%12s%12s\n", "motor", "P (target)", "P now", "P err", "V (max)", "V now", "A");
    fprintf (stderr, "%s\n", "-----------------------------------------------------------------------------");
    
    for (i = 0; i != ax->dim; ++i)
    {
        fprintf (stderr, "%5s%12ld%12ld%12ld%12ld%12ld%12lu\n",
                ax->sm[i].name, ax->P, ax->sm[i].p, ax->sm[i].p_err, ax->V, ax->sm[i].v, ax->A);
    }
    

    return OK;
}


/*==============================================================================
**  report_short
**
*/
int report_short (Axis *ax)
{
    int i;

    axis_status (ax);

    for (i = 0; i != ax->dim; ++i)
    {
        if (0 == i) fprintf (stderr, "%10s %6s", ax->name, ax->fault ? "[FAIL]" : "  [OK]");
        else (fprintf (stderr, "%17s", "                 ")); 

        fprintf (stderr, "%12s", ax->sm[i].name);

        if (ax->sm[i].com_fail)
        {
            fprintf (stderr, "%s\n", "          ERROR: not responding");
            continue;
        }

        fprintf (stderr, "%12s",   ax->sm[i].moving ? "yes" : "no");
        fprintf (stderr, "%12s",   ax->sm[i].servo ? "on" : "off");
        fprintf (stderr, "%12s",   ax->sm[i].index_available ? "ready" : "no");
        fprintf (stderr, "%12s\n", ax->sm[i].fault ? "yes" : "no");
    }

    return OK;
}


/*==============================================================================
**  send_axis_cmd
**
*/
void send_axis_cmd (Axis *ax, char *cmd)
{
    int i;
    for (i = 0; i != ax->dim; ++i) send_sm_cmd (&ax->sm[i], cmd);
}


/*==============================================================================
**  send_axis_val
**
*/
void send_axis_val (Axis *ax, char *cmd, long val)
{
    int i;
    for (i = 0; i != ax->dim; ++i) send_sm_val (&ax->sm[i], cmd, val);
}


/*==============================================================================
**  send_sm_cmd
**
*/
char* send_sm_cmd (Smart_Motor *sm, char *cmd)
{
    char *reply = 0;
    strcpy (sm->cmd, cmd);
    singlecast (sm->CMD, &reply);
    return reply;
}


/*==============================================================================
**  send_sm_val
**
*/
char* send_sm_val (Smart_Motor *sm, char *var, long val)
{
    char cmd [256];
    sprintf (cmd, "%s=%ld", var, val);
    return send_sm_cmd (sm, cmd);
}


/*==============================================================================
**  wakeup_toggle
**  returns true if asleep
*/
int wakeup_toggle (Axis *ax)
{
    if (ax->asleep)
    {
        send_axis_cmd (ax, ax->CMD_WAKEUP);
    }
    else
    {
        send_axis_cmd (ax, ax->CMD_SLEEP);
    }

    return (ax->asleep = ( ! ax->asleep ));
}
