
/****************************************************

		archive.c
   
   These routines allow the current configuration of
   the system to be saved to disk so that termination
   of the program will not cause loss of the locations
   of the buttons.

   Functions in this file include:

   *  FILE *openArchive()
   *  closeArchive()
   *  archivePosition()
   *  restorePosition()
   *  restoreAll()
   *  dateStamp()
   *  purgeArchive()
   *  listArchive()

   ( * indicates that function is declared as an extern )

****************************************************/
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <sys/time.h>
#include <setjmp.h>
#include <signal.h>

#include "errors.h"
#include "death.h"
#include "miscMath.h"
#include "motor.h"
#include "kaboom.h"
#include "occult.h"
#include "translate.h"
#include "detrot.h"
#include "flex.h"
#include "buttons.h"
#include "archive.h"
#include "fiborg.h"
#include "log.h"

/******* globals ******/

long EndPtr;

/**********************/

static struct flock LockAll = {
    F_WRLCK,                    /* locks the entire file */
    SEEK_SET,
    0,
    0,
    0
};

static struct flock UnLockAll = {
    F_UNLCK,                    /* unlocks the entire file */
    SEEK_SET,
    0,
    0,
    0
};

/*local prototypes */
FILE *openArchive (char *fileName, char *mode);
FILE *reOpenArchive (char *fileName, char *type, FILE * stream);
int closeArchive (FILE * theArchive);
int archivePosition (struct fiber_dbase *fiber, FILE * theArchive);
int archiveDataSet (int id, struct aPoint *coord1, struct aPoint *coord2,
                    char *occultations, int errorStatusCode, int moveId,
                    FILE * theArchive);
int restoreFromArchive (FILE * theArchive, struct fiber_dbase theDbase[],
                        int which);
int getArchiveLine (FILE * theArchive, char *theLine);
int parseArchiveLine (char *theLine, struct archiveLineData *result);
int applyArchiveLine (struct archiveLineData *theStuff);

/*
      openArchive

   open an archive file in the current directory
   with the name pointed to by "name".  Return the
   file pointer or NULL on failure.
*/

FILE *openArchive (char *fileName, char *mode)
{
    char *archivePath;
    char archiveFile[128];

    FILE *temp;

    if ((archivePath = getenv (FIBSTATUS)) != NULL)
        (void) strcpy (archiveFile, archivePath);

    (void) strcat (archiveFile, "/");
    (void) strcat (archiveFile, fileName);

    if ((temp = fopen (archiveFile, mode)) != NULL)
        setbuf (temp, (char *) NULL);

    return (temp);
}

/*
      reOpenArchive

   Re-open the position archive to allow the reading
   of information that has been appended to it.
*/

FILE *reOpenArchive (char *fileName, char *type, FILE * stream)
{
    char *archivePath;
    char archiveFile[128];

    FILE *temp;

    if ((archivePath = getenv (FIBSTATUS)) != NULL)
        (void) strcpy (archiveFile, archivePath);

    (void) strcat (archiveFile, "/");
    (void) strcat (archiveFile, fileName);

    if ((temp = freopen (archiveFile, type, stream)) != NULL)
        setbuf (temp, (char *) NULL);

    return (temp);
}

/*
      closeArchive

   close an archive file.
*/

closeArchive (FILE * theArchive)
{
    int status;

    status = fclose (theArchive);

    if (status != 0)
        fputs ("Can't close position archive!\n", stderr);
}

/*
      archivePosition

   Write the current location of the fiber in the first arg. 
   Assume no error.
*/

archivePosition (struct fiber_dbase *fiber, FILE * theArchive)
{
    struct aPoint p1;

    p1.x = fiber->CurPos.x;
    p1.y = fiber->CurPos.y;

    /* use of TrailGeometry is a trick to get to a record of where
       the button was before the move... */

    archiveDataSet (fiber->fiber_id, &p1, &fiber->TrialGeometry->button, occultList (fiber), STATIC,    /* problem type -- none if this ftn called. */
                    0,          /* move id code */
                    theArchive);
}

/*
      archiveDataSet

   Write the arguments to the locations archive 
   Date stamp it.
   Flush the new entry to disk.
*/

archiveDataSet (int id, struct aPoint *coord1, struct aPoint *coord2,
                char *occultations, int errorStatusCode, int moveId,
                FILE * theArchive)
                        /* id number of the fiber */
                        /* where we think the button is now */
                        /* where we think the button was before */
                        /* fibers blocking us */
                        /* status of last attempt to move this fiber */
                        /* a numerical id for which move this was. */
                        /* which archive to use */
{
    struct timeval theTime;
    char *timeString;

    char archiveLine[256];

    /* get the current time and process it into a string */

    if (gettimeofday (&theTime, (struct timezone *) NULL) != 0)
        labtyd (SYSTEM_CALL, "Unable to obtain system time");

    timeString = ctime (&theTime.tv_sec);

    /* create the line for the file */

    (void) sprintf (archiveLine,
                    "%3d  %6d %6d  %6d %6d  %-16s  %2d %6d  %.24s.  %3d\n",
                    id, (int) coord1->x, (int) coord1->y, (int) coord2->x,
                    (int) coord2->y, occultations, errorStatusCode,
                    NextMove++, timeString, id);

#ifdef IGNORE

    /* lock the file, write the line, flush, and unlock  */

    if (fcntl (fileno (theArchive), F_SETLKW, &LockAll) == -1)
        fputs ("WARNING:  Could not lock position archive file. \n", stderr);
#endif

    if (!isatty (fileno (theArchive)))
        if (fseek (theArchive, 0L, 2))
            labtyd (SYSTEM_CALL, "Unable to find end of archive file");

    fputs (archiveLine, theArchive);

    fflush (theArchive);

    if (!isatty (fileno (theArchive)))
        if (fsync (fileno (theArchive)))
            labtyd (SYSTEM_CALL, "Failure to flush archive info to disk.");

#ifdef IGNORE
    if (fcntl (fileno (theArchive), F_SETLK, &UnLockAll) == -1)
        labtyd (1,
                "FATAL ERROR:  Could not unlock position archive file. \n");
#endif

}

/*
      restoreFromArchive
   
   Restore the internal database entries for ether all fibers,
   or recently changed ones

   The most recent entry for any fiber takes presidence.

*/

restoreFromArchive (FILE * theArchive, struct fiber_dbase *theDbase,
                    int which)
                                /* either ALL or RECENT */
{
    char theLine[128];
    struct archiveLineData result;
    int error = FALSE;

    /* lock the file */

#ifdef IGNORE
    if (fcntl (fileno (theArchive), F_SETLKW, &LockAll) != -1)
        fputs ("WARNING:  Could not lock position archive file. \n", stderr);
#endif

    /* get to the beginning */

    rewind (theArchive);

    /* if only reading recent changes, find the previous end */

    if (which == RECENT)
        fseek (theArchive, EndPtr, 0);

    /* read lines and process them */

    while (getArchiveLine (theArchive, theLine))
    {

        if (parseArchiveLine (theLine, &result) != 0)
        {

            fprintf (stderr, "ERROR:  Bad archive line:  %s\n", theLine);
            error = TRUE;
            break;
        }

        if (applyArchiveLine (&result) != 0)
        {

            fprintf (stderr, "ERROR:  Couldn't apply archive line:  %s\n",
                     theLine);
            error = TRUE;
            break;
        }
    }

    /* remember where we are */

    EndPtr = ftell (theArchive);

    /* unlock the file */

#ifdef IGNORE
    if (fcntl (fileno (theArchive), F_SETLK, &UnLockAll) == -1)
        labtyd (1,
                "FATAL ERROR:  Could not unlock position archive file. \n");
#endif

    /* deal with errors */

    if (error)
    {
        fputs ("WARNING:  Error while restoring fiber positions from disk.\n",
               stderr);
        fputs
            ("          Indicated positions for some fibers may be wrong.\n",
             stderr);
        fputs ("          Do NOT continue unless you are trained to do so.\n",
               stderr);
    }

}

getArchiveLine (FILE * theArchive, char *theLine)
{
    return (fgets (theLine, 128, theArchive) != NULL);
}

parseArchiveLine (char *theLine, struct archiveLineData * result)
{
    char ourLine[256];

    if (theLine[0] == '#')
        return (OK);

    strcpy (ourLine, theLine);

    /* fputs( ourLine, stderr ); */

    /* scan the fiber id number */

    result->id = atoi (strtok (ourLine, " "));

    /* scan the current coord entry */

    result->p1.x = (double) atol (strtok ((char *) NULL, " "));

    result->p1.y = (double) atol (strtok (NULL, " "));

    /* scan the "previous coord" entry */

    result->p2.x = (double) atol (strtok (NULL, " "));
    result->p2.y = (double) atol (strtok (NULL, " "));

    /* create an occultation structure and fill it */

    result->occultations = occListToStruct (strtok (NULL, " "));

    /* doesn't read back time at the moment, that would go here... */

    return (OK);
}

applyArchiveLine (struct archiveLineData * theStuff)
{
    struct fiber_dbase *pFib;
    struct plate_coords tempCoords;

    /* be sure we don't access a fiber that doesn't exist! */

    if ((theStuff->id >= MAXFIBER) || (theStuff->id < 0))
    {
        return (ERROR);
    }

    tempCoords.x = theStuff->p1.x;
    tempCoords.y = theStuff->p1.y;

    pFib = &FiBase[theStuff->id];

    /* link to the database */

    applyOccultationList (pFib, theStuff->occultations);

    pFib->outOfDate = TRUE;

    if ((tempCoords.x != pFib->CurPos.x) || (tempCoords.y != pFib->CurPos.y))
    {

        setLocation (pFib, &tempCoords, DONT_FILE);

        pFib->parkedStatus = EXTENDED;  /* might get overwritten below */

        if ((abs (pFib->CurPos.x - pFib->ParkPos.x) < DEFAULT_DEAD_ZONE) &&
            (abs (pFib->CurPos.y - pFib->ParkPos.y) < DEFAULT_DEAD_ZONE))
        {

            pFib->parkedStatus = PARKED;
        }

        if ((abs (pFib->CurPos.x - pFib->StowPos.x) < DEFAULT_DEAD_ZONE) &&
            (abs (pFib->CurPos.y - pFib->StowPos.y) < DEFAULT_DEAD_ZONE))
        {

            pFib->parkedStatus = STOWED;
        }

        /* graphics use this for selective re-draw */

    }
    return (OK);
}
