#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "miscMath.h"
#include "nesslib.h"
#include "motor.h"
#include "kaboom.h"
#include "xref.h"
#include "nessIO.h"
#include "log.h"
#include "findIt.h"
#include "sla.h"

double tempmonths[] = {
    33.5,
    33.4,
    35.6,
    41.4,
    49.7,
    57.3,
    61.6,
    60.0,
    57.1,
    49.9,
    41.0,
    28.3
};

char *FieldEnvItems[] = {       /* environment items with paths in them */
    FIBFIELDS,
    STDFIELDS,
    NULL
};

double FieldRotationalOffset;

/* This is the list of the keywords that we expect in a Hydra file */
struct keyRouterChecker keyList[] = {
    /*    keyname          type              where it goes      req?  found? */

    {KEY_FIELD_NAME, CHAR, Exposure.expName, TRUE, FALSE},
    {KEY_INPUT_EPOCH, DOUBLE, (char *) &Exposure.epoch, TRUE, FALSE},
    {KEY_CURRENT_EPOCH, DOUBLE, (char *) &Exposure.current, TRUE, FALSE},
    {KEY_SIDERIAL_TIME, DOUBLE, (char *) &Exposure.st, TRUE, FALSE},
    {KEY_EXPOSURE_LENGTH, DOUBLE, (char *) &Exposure.exptime, TRUE, FALSE},
    {KEY_WAVELENGTH, DOUBLE, (char *) &Exposure.wave, TRUE, FALSE},
    {KEY_CABLE, CHAR, Exposure.cable, TRUE, FALSE},
    {KEY_GUIDEWAVE, DOUBLE, (char *) &Exposure.guidewave, FALSE, FALSE},
    {KEY_WEIGHTING, CHAR, (char *) Exposure.weighting, FALSE, FALSE}
};

/* this one is used to generate the output for nessie format
   to hydra format conversion.  This gets copied into the
   above structure and tells the hydra format output routine
   that certain keywords were found and therefore should
   be written. */
struct keyRouterChecker keyListNess[] = {
    /*    keyname          type              where it goes      req?  found? */
    {KEY_FIELD_NAME, CHAR, Exposure.expName, TRUE, TRUE},
    {KEY_INPUT_EPOCH, DOUBLE, (char *) &Exposure.epoch, TRUE, TRUE},
    {KEY_CURRENT_EPOCH, DOUBLE, (char *) &Exposure.current, TRUE, TRUE},
    {KEY_SIDERIAL_TIME, DOUBLE, (char *) &Exposure.st, TRUE, TRUE},
    {KEY_EXPOSURE_LENGTH, DOUBLE, (char *) &Exposure.exptime, TRUE, TRUE},
    {KEY_WAVELENGTH, DOUBLE, (char *) &Exposure.wave, TRUE, TRUE},
    {KEY_CABLE, CHAR, Exposure.cable, TRUE, TRUE},
    {KEY_GUIDEWAVE, DOUBLE, (char *) &Exposure.guidewave, FALSE, FALSE},
    {KEY_WEIGHTING, CHAR, (char *) Exposure.weighting, FALSE, FALSE}
};

/* local prototypes */
int LoadHydraFile (FILE * dataFile, struct starinfo *theStars[],
                   struct exposinfo *exposureInfo);
int LoadHydraHeader (FILE * dataFile, struct starinfo *theStars[],
                     struct exposinfo *exposureInfo);
int getNextLine (FILE * dataFile, char *theLine);
void appendCommentList (char *theLine);
void parseHeaderItem (char *theLine, struct exposinfo *exposureInfo);
void parseObject (char *theLine, struct starinfo *theStars[]);
struct starinfo *createObject (struct starinfo *theStars[]);
int buttonStringToToken (char *theString);
int requiredKeyMissing (void);
void clearKeysFound (void);
int searchForKey (char *theLine);
int keyCmp (char *s1, char *s2);
void prepExposureInfo (struct exposinfo *exposureInfo);
char *stringstring (char *s1, char *s2);
void hydraFileDump (struct starinfo *theStars[], struct exposinfo *pExposure,
                    FILE * infoFile, int flag);
void printHydraHeader (struct exposinfo *pExposure, FILE * infoFile);
void sPrintKey (char *stringOut, char *key);
void printHydraObject (struct starinfo *theStar, FILE * theFile,
                       int printCoords);
void sprintObjectCoord (struct starinfo *theStar, char *theString, int flag,
                        int precision);
int LoadStarFile (FILE * dataFile, struct starinfo *theStars[],
                  struct exposinfo *pExposure);
int ReadExpInfo (FILE * dataFile, struct exposinfo *Exposure);
int readRecord (FILE * dataFile, struct starinfo **ppTheStar);
void nessToHydra (struct starinfo *stars[], struct exposinfo *exposure);
FILE *createStarFile (char *fileName);
FILE *openStarFile (char *fileName, char *mode);
FILE *openCoordFile (char *fileName, char *mode);
double stor (int ira, int imn, double sec);
double astor (int idec, int imn, double arcsec);

/*
		LoadHydraFile

   Read and process a "hydra" type file.  Return a
   code to indicate the status of the file's contents.
*/
LoadHydraFile (FILE * dataFile, struct starinfo **theStars,
               struct exposinfo *exposureInfo)
{
    char theLine[256];
    int lineType;
    double dummy;

    xRefClear (FiBase);

    /* clear the key found list */

    clearKeysFound ();

    prepExposureInfo (exposureInfo);

    /* read and process all the lines in the file */

    while ((lineType = getNextLine (dataFile, theLine)) != EOF)
    {

        switch (lineType)
        {

            case EMPTY:
                break;

            case COMMENT:
                break;

            case HEADER:

                parseHeaderItem (theLine, exposureInfo);
                break;

            case OBJECT:

                parseObject (theLine, theStars);
                break;

            default:

                fprintf (stderr, "unreadable line in hydra file:\n%s",
                         theLine);
                break;
        }
    }

    /* calculate the month and the expected temperature */

    exposureInfo->month = (modf (exposureInfo->current, &dummy) * 12.0) + 0.5;

    if (exposureInfo->month > 12.0)
        exposureInfo->month -= 12.0;

    if (exposureInfo->month < 1.0)
        exposureInfo->month = 1.0;

    if (exposureInfo->month > 12.0)
        exposureInfo->month = 12.0;

    exposureInfo->tempadopt = tempmonths[(int) (exposureInfo->month - 1.0)];

    /* be sure all the required keywords are present and 
       then return the file status */

    if (requiredKeyMissing ())
        return (FILE_BAD);
    else
        return (FILE_OK);

}

/*
		LoadHydraHeader

   Read and process a "hydra" type file.  Return a
   code to indicate the status of the file's contents.
*/

LoadHydraHeader (FILE * dataFile, struct starinfo ** theStars,
                 struct exposinfo * exposureInfo)
{
    char theLine[256];
    int lineType;
    double dummy;
    int firstObjectFound = FALSE;

    /* clear out the field center entry, if one... */

    if (theStars[0] != NULL)
    {
        free ((char *) theStars[0]);
        theStars[0] = NULL;
    }

    /* clear the key found list */

    clearKeysFound ();

    prepExposureInfo (exposureInfo);

    /* read and process all the lines in the file */

    while ((lineType = getNextLine (dataFile, theLine)) != EOF)
    {

        switch (lineType)
        {

            case HEADER:

                parseHeaderItem (theLine, exposureInfo);
                break;

            case OBJECT:

                /* the first object is the field center and the
                   only object of interest when reading the header */

                if (!firstObjectFound)
                {
                    parseObject (theLine, theStars);
                    firstObjectFound = TRUE;
                }

                break;

            case EMPTY:
                break;

            case COMMENT:
                appendCommentList (theLine);
                break;

            default:
                fprintf (stderr, "unreadable line in hydra file:\n%s",
                         theLine);
                break;
        }
    }

    /* calculate the month and the expected temperature */

    exposureInfo->month = (modf (exposureInfo->current, &dummy) * 12.0) + 0.5;

    if (exposureInfo->month > 12.0)
        exposureInfo->month -= 12.0;

    exposureInfo->tempadopt = tempmonths[(int) (exposureInfo->month - 1.0)];

    /* be sure all the required keywords are present and 
       then return the file status */

    if (requiredKeyMissing ())
        return (FILE_BAD);
    else
        return (FILE_OK);

}

/*
		getNextLine

   Get the next line from the specified file and place it
   in the string pointed to by the second argument. 
   Determine the "type" of line, and return a description code.
   if no line is found or an error occurs, return EOF.
*/

getNextLine (FILE * dataFile, char *theLine)
{
    *theLine = '\0';

    if ((fgets (theLine, 128, dataFile) == NULL))
        return (EOF);

    /* get past any white space at the beginning */

    while ((*theLine == ' ') || (*theLine == '\t'))
        theLine++;

    if (*theLine == '\n')
        return (EMPTY);

    if (*theLine == '#')
        return (COMMENT);

    if (isalpha (*theLine))
        return (HEADER);

    if (isdigit (*theLine))
        return (OBJECT);

    return (OTHER);

}

/* 
		appendCommentList

   Take the specified line and append it to the
   list of comments in the exposure information.
*/

void appendCommentList (char *theLine)
{
    fputs (theLine, stderr);
}

/*
		parseHeaderItem

   Break down the specified line into a keyword
   and a value.  Save the value in the appropriate location
   in the exposure information structure.  If an unknown
   keyword or a bad line is encountered, print a message
   and return having taken no action.  This function uses
   the global "keyList" which is the list of valid header
   items.

*/

void parseHeaderItem (char *theLine, struct exposinfo *exposureInfo)
{
    if (searchForKey (theLine) == FAILURE)
        fprintf (stderr, "BAD HEADER ENTRY: \n%s", theLine);
}

/* 
		parseObject

   Break down an object entry line into the appropriate
   fields.  Check the values for reasonable range.  Create
   and fill a starinfo structure and link it into the star
   list.  If a bad entry is found, print a warning message
   and discard the line.  
*/

void parseObject (char *theLine, struct starinfo **theStars)
{
    struct starinfo *pTheStar;

    char restOfLine[128];

    char *rawNamePtr;
    char *nameStringPtr;
    char *statusStringPtr;

    int i;
    int error = 0;

    /*  input line iterms */

    int userID;
    char objectName[32];
    int ihr;                    /* RA *//*  input line iterms for ra */
    int imn;                    /* RA */
    double sec;                 /* RA */
    char sign;                  /* DEC */
    int ideg;                   /* DEC */
    int idmn;                   /* DEC */
    double dsec;
    char objectType;
    int button;
    long plateX;
    long plateY;
    long camX;
    long camY;

    char buttonString[16];
    char tail[64];

    /* start scanning for the fields.  set error and exit on
       the first one that is bad */

    /* try to get the id number */

    if (sscanf (theLine, "%d%128c", &userID, restOfLine) < 2)
        error = TRUE;

    else
    {

        /* now try to get the name of the object.  Guaranteed
           to be 20 characters long--Yucky fixed format... */

        /* first skip over the initial space */

        rawNamePtr = restOfLine;
        rawNamePtr++;

        nameStringPtr = objectName;

        /* be sure that what we are about to copy is long enough! */

        if (strlen (rawNamePtr) < NAME_FLD_WIDTH)
        {
            error = TRUE;
        }

        else
        {

            /* copy whatever is there */

            for (i = 0; i < NAME_FLD_WIDTH; i++)
                *nameStringPtr++ = *rawNamePtr++;

            /* terminate it */

            *nameStringPtr = '\0';

            /* now get the RA and DEC */

            if (sscanf (rawNamePtr,     /* poor choice of name--now pts to RA */
                        "%d%*[ :'\t']%d%*[ :'\t']%lf %63c", &ihr, &imn, &sec,
                        tail) < 4)
                error = TRUE;

            sscanf (tail, " %c", &sign);

            if (sscanf (tail,
                        "%d%*[ :'\t']%d%*[ :'\t']%lf %c",
                        &ideg, &idmn, &dsec, &objectType) < 4)

                error = TRUE;

            if (!error)
            {

                /* see if this is the special case of field center */

                if (*(statusStringPtr = stringstring (theLine, "Ag=")) !=
                    '\0')
                {

                    statusStringPtr += strlen ("Ag=");

                    if (sscanf
                        (statusStringPtr, "%lf", &FieldRotationalOffset) < 1)
                        FieldRotationalOffset = 0.0;
                    else
                    {
                        FieldRotationalOffset += ROTATION_CORRECTION;
                        FieldRotationalOffset *= DEG2RAD;
                    }

                    button = BLANK;
                }
                else
                {               /* this is a "normal" object */

                    /* get past the assignment "keyword" and read the assignment */

                    if ((statusStringPtr = stringstring (theLine, "STATUS=") +
                         strlen ("STATUS=")) != '\0')
                    {

                        if (sscanf (statusStringPtr, "%s", buttonString) < 1)
                            button = BLANK;

                        else
                        {       /* breakdown the string */
                            button = buttonStringToToken (buttonString);
                        }
                    }
                }

                /* get past the plate coordinate "keyword" and read the x,y coords */

                if ((statusStringPtr = stringstring (theLine, "PLATE=") +
                     strlen ("PLATE=")) != '\0')
                {

                    if (sscanf (statusStringPtr, "%ld %ld", &plateX, &plateY)
                        < 2)
                    {
                        plateX = 0;
                        plateY = 0;
                    }
                }

                /* get past the camera coordinate "keyword" and read the x,y coords */

                if ((statusStringPtr = stringstring (theLine, "CAMERA=") +
                     strlen ("CAMERA=")) != '\0')
                {

                    if (sscanf (statusStringPtr, "%ld %ld", &camX, &camY) < 2)
                    {
                        camX = 0;
                        camY = 0;
                    }
                }
            }
        }
    }

    /* now decide what to do with the line */

    if (!error)
    {                           /* all the pieces are there */

        /* create the space for the record */

        if ((pTheStar = (struct starinfo *) malloc ((unsigned)
                                                    sizeof (struct starinfo)))
            == NULL)
        {
            fputs ("malloc failed to allocate starinfo space\n", stderr);
            exit (1);
        }

        /* now fill in the record with the items from the input line */

/*  debug only
   fprintf( stderr, "% 5d  ",  userID );
   fprintf( stderr, "%20s  ",  objectName );
   fprintf( stderr, "%3d ",  ihr );
   fprintf( stderr, "%3d ",  imn );
   fprintf( stderr, "%6.3f   ",  sec );
   fprintf( stderr, "% 4d ",  ideg );
   fprintf( stderr, "%2d ",  idmn );
   fprintf( stderr, "%6.3f   ",  dsec );
   fprintf( stderr, "%c  ",  objectType );
   fprintf( stderr, "% 4d\n",  button );
*/

        pTheStar->IDnumber = userID;

        strcpy (pTheStar->name, objectName);

        /* convert the RA and DEC to radians */

        pTheStar->ra = stor (ihr, imn, sec);

        pTheStar->rawCoord.ra = stor (ihr, imn, sec);

        if (sign == '-')
        {
            idmn = -idmn;
            dsec = -dsec;
        }

        pTheStar->dc = astor (ideg, idmn, dsec);

        pTheStar->rawCoord.dec = astor (ideg, idmn, dsec);

        pTheStar->objectType = objectType;

        pTheStar->button = button;

        if (button >= 0)
            FiBase[button].assignedObject = pTheStar;

        pTheStar->adxp = plateX;
        pTheStar->adyp = plateY;

        pTheStar->specEncoderCoord.x = plateX;
        pTheStar->specEncoderCoord.y = plateY;

        pTheStar->camEncoderCoord.x = camX;
        pTheStar->camEncoderCoord.y = camY;

        /* put a link to this record in the first empty slot in
           the list */

        for (i = 0; i < MAXSTARS; i++)
            if (theStars[i] == NULL)
            {

                theStars[i] = pTheStar;
                theStars[i]->internalID = i;
                break;
            }
    }
}

/*
		createObject

   Create a generic object in the object database,
   and return a pointer to it.

*/

struct starinfo *createObject (struct starinfo **theStars)
{
    struct starinfo *pTheStar;

    int i;

    /* create the space for the record */

    if ((pTheStar = (struct starinfo *) malloc ((unsigned)
                                                sizeof (struct starinfo))) ==
        NULL)
    {
        fputs ("malloc failed to allocate starinfo space\n", stderr);
        exit (1);
    }

    /* now fill in the record with the items from the input line */

    for (i = 0; i < MAXSTARS; i++)
        if (theStars[i] == NULL)
        {

            theStars[i] = pTheStar;
            theStars[i]->internalID = i;
            break;
        }

    /* initialize the critical variables */

    pTheStar->specEncoderCoord.x = 0;
    pTheStar->specEncoderCoord.y = 0;

    pTheStar->camEncoderCoord.x = 0;
    pTheStar->camEncoderCoord.y = 0;

    pTheStar->rawCoord.ra = 0;
    pTheStar->rawCoord.dec = 0;

    return (pTheStar);
}

int buttonStringToToken (char *theString)
{
    int button;

    /* return as soon as a match is found */

    if (strcmp (theString, "OK") == 0)
        return COORD_OK;

    if (strcmp (theString, "EDGE") == 0)
        return EDGE;

    if (strcmp (theString, "STP1") == 0)
        return STP1;

    if (strcmp (theString, "STP2") == 0)
        return STP2;

    if (sscanf (theString, "%d", &button) < 1)
        return (BLANK);

    else
    {
        if (button < 0)         /* neg. ints are undefined */
            return (BLANK);
        else
            return (button);    /* positive ints are button IDs */

    }

}

int requiredKeyMissing (void)
{
    int keycount = sizeof (keyList) / sizeof (struct keyRouterChecker);
    int i;

    for (i = 0; i < keycount; i++)
        if (keyList[i].required && !keyList[i].found)
            return (TRUE);

    return (FALSE);
}

void clearKeysFound (void)
{
    int keycount = sizeof (keyList) / sizeof (struct keyRouterChecker);
    int i;

    for (i = 0; i < keycount; i++)
        keyList[i].found = FALSE;
}

int searchForKey (char *theLine)
{
    char key[128];
    char value[128];

    char *valPtr;
    char *tailPtr;

    int matchFound = FALSE;
    int keycount = sizeof (keyList) / sizeof (struct keyRouterChecker);

    int i;

    for (i = 0; i < strlen (value); i++)
        value[i] = '\0';

    /* be sure the line has the right format by splitting it into
       the keyword and the value */

    if (sscanf (theLine, "%[^:]:", key) < 1)
        return (FAILURE);

    valPtr = theLine + strlen (key) + 2;

    while (*valPtr == ' ')
        valPtr++;

    strcpy (value, valPtr);

    tailPtr = value + strlen (value) - 1;

    while (isspace (*tailPtr))
    {
        *tailPtr = '\0';
        tailPtr--;
    }

/*
   if( sscanf( theLine, "%[^:]: %62s", key, value ) < 2 )
      return ( FAILURE );
*/

    /* search for a key match */

    for (i = 0; ((i < keycount) && (!matchFound)); i++)
    {

        /* if( strcmp( key , keyList[i].keyName ) == 0 ){  old way */

        if (keyCmp (key, keyList[i].keyName) == 0)
        {

            switch (keyList[i].type)
            {

                case INT:

                    if (sscanf (value, "%d", (int *) keyList[i].whereItGoes)
                        >= 1)
                    {
                        keyList[i].found = TRUE;
                        matchFound = TRUE;
                    }
                    break;

                case DOUBLE:

                    if (sscanf
                        (value, "%lf",
                         (double *) keyList[i].whereItGoes) >= 1)
                    {
                        keyList[i].found = TRUE;
                        matchFound = TRUE;
                    }
                    break;

                case CHAR:

                    strcpy (keyList[i].whereItGoes, value);
                    keyList[i].found = TRUE;
                    matchFound = TRUE;
                    break;

                default:
                    fputs ("Unknown type for hydra header key\n", stderr);
                    break;
            }
        }
    }

    if (!matchFound)
        return (FAILURE);
    else
        return (0);
}

/* 
 		  keyCmp

   See if the string in s1 is a match for the key in s2.
   s2 may contain a wildcard characters that allow multiple
   matches for that key. 
*/

int keyCmp (char *s1, char *s2)
{
    while (*s1 != '\0')
    {

        switch (*s2)
        {

            case '\0':

                return (1);     /* "failure". Works like "strcmp" */
                break;

            case '?':

                s1++;           /* doesn't matter, it's a match */
                s2 += 2;        /* skip wildcard AND the default output char */

                break;

            default:

                if (*s1 == *s2)
                {
                    s1++;
                    s2++;
                }
                else
                    return (1); /* failure */

                break;
        }
    }

    if (*s2 != '\0')
        return (1);             /* failure, incomplete key... */
    else
        return (0);             /* Success. Like "strcmp" */
}

void prepExposureInfo (struct exposinfo *exposureInfo)
{
    exposureInfo->guidewave = WAVECAM;
}

char *stringstring (char *s1, char *s2)
{
    if ((*s1 != '\0') &&
        ((*s1 != *s2) || (strncmp (s1, s2, strlen (s2)) != 0)))

        return stringstring (++s1, s2);

    else
        return (s1);
}

void hydraFileDump (struct starinfo **theStars, struct exposinfo *pExposure,
                    FILE * infoFile, int flag)
{
    int i;

    printHydraHeader (pExposure, infoFile);

    for (i = 0; i < MAXSTARS; i++)
    {

        if (theStars[i] != NULL)
        {
            printHydraObject (theStars[i], infoFile, flag);
        }
    }
}

void printHydraHeader (struct exposinfo *pExposure, FILE * infoFile)
{
    int keycount = sizeof (keyList) / sizeof (struct keyRouterChecker);

    int i;

    char nameString[64];

    for (i = 0; i < keycount; i++)
    {

        /* don't output a key that was never read in or generated! */

        if (keyList[i].found == TRUE)
        {

            sPrintKey (nameString, keyList[i].keyName);

            switch (keyList[i].type)
            {

                case INT:
                    fprintf (infoFile, "%s: %d\n", nameString,
                             *(int *) keyList[i].whereItGoes);
                    break;

                case DOUBLE:
                    fprintf (infoFile, "%s: %f\n", nameString,
                             *(double *) keyList[i].whereItGoes);
                    break;

                case CHAR:
                    fprintf (infoFile, "%s: %s\n", nameString,
                             keyList[i].whereItGoes);
                    break;

                default:
                    fputs ("Unknown type for hydra header key\n", stderr);
                    break;
            }
        }
    }
}

void sPrintKey (char *stringOut, char *key)
{
    while (*key != '\0')
    {

        switch (*key)
        {

            case '?':

                key++;          /* skip the wildcard character */
                break;

            default:

                *stringOut++ = *key++;
                break;
        }
    }
    *stringOut = '\0';
}

/* 
		printHydraObject

   This function outputs the astrometry and assignment information
   for an object in the standard Hydra file format.  If the 
   printCoords flag is set, it appends plate the coordinates of the
   object.
*/

void printHydraObject (struct starinfo *theStar, FILE * theFile,
                       int printCoords)
                        /* flag to print last field with coordinates */
{
    char buttonName[16];
    char fieldCoords[64];

    /* print the id and name */

    fprintf (theFile, "%4d %-20s ", theStar->IDnumber, theStar->name);

    /* print the ra and dec */

    sprintObjectCoord (theStar, fieldCoords, RA, 3);
    fprintf (theFile, "%s ", fieldCoords);

    sprintObjectCoord (theStar, fieldCoords, DEC, 2);
    fprintf (theFile, "%s ", fieldCoords);

    /* print the object type */

    fprintf (theFile, "%c  ", theStar->objectType);

    /* fprintf( stderr, "%d code %d \n", theStar->IDnumber, theStar->button ); */

    /* print the assignment info */

    switch (theStar->button)
    {

        case BLANK:
            sprintf (buttonName, "       ");    /* 4 blanks */
            break;

        case COORD_OK:
            sprintf (buttonName, "OK     ");    /* 2 blanks */
            break;

        case EDGE:
            sprintf (buttonName, "EDGE   ");
            break;

        case STP1:
            sprintf (buttonName, "STP1   ");
            break;

        case STP2:
            sprintf (buttonName, "STP2   ");
            break;

        default:
            sprintf (buttonName, "%7d", theStar->button);
            break;
    }

    /* skip the "status=" for the center only on .hydra files */

    if ((theStar->objectType == 'c') || (theStar->objectType == 'C'))
    {

        /* this is the center, print the rotator angle */

        fprintf (theFile, "   Ag=%7.3f ", (FieldRotationalOffset / DEG2RAD));
    }
    else
    {
        /* this is an object, print the assignment */

        fprintf (theFile, "STATUS=%7s ", buttonName);
    }

    if (printCoords == TRUE)
    {

        fprintf (theFile, "PLATE= % 7.0f % 7.0f  ",
                 theStar->specEncoderCoord.x, theStar->specEncoderCoord.y);

        fprintf (theFile, "CAMERA= % 7.0f % 7.0f\n",
                 theStar->camEncoderCoord.x, theStar->camEncoderCoord.y);

    }
    else
        fprintf (theFile, "\n");
}

/* 
		sprintObjectCoord

   This function Take the ra and dec of the object and 
   creats a string representing the values.
*/

void sprintObjectCoord (struct starinfo *theStar, char *theString, int flag,
                        int precision)
{
    int ra[4];
    int dec[4];
    char raSign;
    char decSign;

    /* convert ra or dec into x:x:x type fields */

    if (flag == 0)
    {                           /* do the RA */

        sla_dr2tf (precision, theStar->rawCoord.ra, &raSign, ra);

        sprintf (theString, "%2d %02d %02d.%0*d",
                 ra[0], ra[1], ra[2], precision, ra[3]);
    }
    else
    {                           /* do the DEC */

        sla_dr2af (precision, theStar->rawCoord.dec, &decSign, dec);

        if (decSign == '+')
            decSign = ' ';

        sprintf (theString, "%c%02d %02d %02d.%0*d",
                 decSign, dec[0], dec[1], dec[2], precision, dec[3]);
    }
}

LoadStarFile (FILE * dataFile, struct starinfo **theStars,
              struct exposinfo *pExposure)
{
    int i = 0;

    ReadExpInfo (dataFile, pExposure);

    for (i = 0; (i < MAXSTARS) && (readRecord (dataFile, &theStars[i]) != -1);
         i++)
    {
        theStars[i]->internalID = i;
    }

    theStars[i] = NULL;
}

/*  Read in exposure information and save in the Exposure structure */

ReadExpInfo (FILE * dataFile, struct exposinfo *Exposure)
{

    char buff[128];
    int count = 0;
    int empty = TRUE;
    double dummy;

    while (empty)
    {

        *buff = '\0';

        if ((fgets (buff, sizeof (buff), dataFile) == NULL))
            return (FAILURE);

        if (strlen (buff) > 1)
        {

            /* trash the newline */

            buff[strlen (buff) - 1] = '\0';

            empty = FALSE;
        }
    }

    count = sscanf (buff,
                    "%s %lf%*[\040\t,]%lf%*[\040\t,]%lf%*[\040\t,]%lf%*[\040\t,]%lf",
                    Exposure->expName,
                    &Exposure->epoch,
                    &Exposure->current,
                    &Exposure->st, &Exposure->exptime, &Exposure->wave);

    if (count < 6)
        return (FAILURE);

    if (Exposure->epoch <= 0.0)
        Exposure->epoch = 1950.0;

    if (Exposure->current <= 1987.5)
        Exposure->current = 1988.5;

    Exposure->month = (modf (Exposure->current, &dummy) * 12.0) + 0.5;

    if (Exposure->month > 12.0)
        Exposure->month -= 12.0;

    Exposure->tempadopt = tempmonths[(int) (Exposure->month - 1.0)];

}

/*
		readRecord

	Scans one "astro"format line and returns the id, ra, and dec
	ra & dec in radians.
*/

readRecord (FILE * dataFile, struct starinfo **ppTheStar)
{
    char buff[128];

    int button;

    char *nextField;

    int ihr;                    /*  input line iterms for ra */
    int imn;
    double sec;

    int ideg;
    int idmn;
    double dsec;

    int i;
    int ok = 0;

    struct starinfo *pTheStar;

    if ((pTheStar = (struct starinfo *) malloc ((unsigned)
                                                sizeof (struct starinfo))) ==
        NULL)
    {
        fputs ("malloc failed to allocate starinfo space\n", stderr);
        exit (1);
    }

    *ppTheStar = pTheStar;

    if (fgets (buff, sizeof (buff), dataFile) == NULL)
        return -1;

    /* trash the newline */

    buff[strlen (buff) - 1] = '\0';

    if (strlen (buff) == 0)
        return (-1);

    nextField = strtok (buff, " ");

    ok = sscanf (nextField, "%d", &pTheStar->IDnumber);

    if (ok > 0)
    {

        nextField = strtok (NULL, " ");

        for (i = 0; i <= NAME_FLD_WIDTH; i++)
        {
            pTheStar->name[i] = *nextField++;
            /* mega kludge, but should work */
            if (pTheStar->name[i] == '\0')
                pTheStar->name[i] = ' ';
        }

        pTheStar->name[i] = '\0';

        /* strtok is useless at this point
           but it is the only way to get
           past the title field */

        sscanf (nextField, "%d%d%lf %d%d%lf\n",
                &ihr, &imn, &sec, &ideg, &idmn, &dsec);

        nextField = stringstring (nextField, "Status=") + strlen ("Status=");

        if (sscanf (nextField, "%d", &button) >= 1)
        {
            pTheStar->button = button;
        }
        else
        {
            pTheStar->button = VOIDBUTTON;
        }

        pTheStar->ra = stor (ihr, imn, sec);

        if (ideg < 0)
        {
            idmn = -idmn;
            dsec = -dsec;
        }

        pTheStar->dc = astor (ideg, idmn, dsec);
    }

    else
        ok = 0;

    return 1;
}

/*
		nessToHydra
   
   Modify the object database to look like it was read in as
   a Hydra file instead of as a nessie file.
*/

void nessToHydra (struct starinfo **stars, struct exposinfo *exposure)
{
    int i;

    int numEntries = sizeof (keyListNess) / sizeof (keyListNess[0]);

    /* fix keylist */

    for (i = 0; i < numEntries; i++)
    {
        keyList[i] = keyListNess[i];
    }

    /* set the cable type */

    strcat (exposure->cable, "RED");

    /* set object types */

    stars[0]->objectType = CENTERCHAR;

    for (i = 0; i < MAXSTARS; i++)
    {

        if (stars[i] != NULL)
        {
            stars[i]->objectType = OBJECTCHAR;
        }
    }
}

FILE *createStarFile (char *fileName)
{
    char path[128];

    sprintf (path, "%s/%s", getenv (FieldEnvItems[0]), fileName);

    return (fopen (path, "w+"));
}

FILE *openStarFile (char *fileName, char *mode)
{
    char *path;

    path = findFile (fileName, FieldEnvItems);

    if (path != NULL)
        return (fopen (path, mode));
    else
        return (NULL);
}

FILE *openCoordFile (char *fileName, char *mode)
{

    char coordFileName[256];
    char *coordPath;

    FILE *tmpFile;

    coordFileName[0] = '\n';

    if ((coordPath = getenv (FIBCOORDS)) != NULL)
    {

        strcpy (coordFileName, coordPath);
        strcat (coordFileName, "/");
    }

    strcat (coordFileName, fileName);
    strcat (coordFileName, OUTPUT_SUFFIX);

    tmpFile = fopen (coordFileName, mode);

    if (tmpFile != NULL)
        fchmod (fileno (tmpFile), 511);

    return (tmpFile);
}

double stor (int ira, int imn, double sec)
{
    return (S2R * ((60.0 * ((60.0 * ira) + imn)) + sec));
}

double astor (int idec, int imn, double arcsec)
{
    return (AS2R * ((60.0 * ((60.0 * idec) + imn)) + arcsec));
}
