#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <stdlib.h>

#include "miscMath.h"
#include "nesslib.h"
#include "motor.h"
#include "kaboom.h"
#include "concen.h"
#include "xref.h"
#include "nessIO.h"
#include "irafIO.h"
#include "log.h"
#include "sla.h"

extern int faith;

void sprintObjectCoord (struct starinfo *theStar, char *theString, int flag,
                        int precision);

/* local prototypes */
void adump (void);
char *createUniqueName (char *fieldName);
void printIRAFfile (FILE * theFile);
void printSCSline (int slitPosition, FILE * theFile, char *theLine);
void printIRAFline (int slitPosition, FILE * theFile, char *theLine);
void printCoordPair (int slitPosition, char *raString, char *decString);
void printFieldCenter (char *raString, char *decString, char *epochString);
char *packFieldName (void);
char *packCable (void);
int packCableSize (void);
char *packFieldTime (void);
char *packFieldLST (void);
void generateStats (int *objCount, int *skyCount, int *fopCount);
void printIRAFheader (FILE * file);

void adump (void)
{
    FILE *irafFile;
    char buf[256];

    char *path;

    path = createUniqueName (Exposure.fileName);

    sprintf (buf, "Writing Aperture Identification file into: %s\n", path);
    log_fputs (buf, stderr);

    if ((irafFile = fopen (path, "w+")) != NULL)
    {

        printIRAFfile (irafFile);
        fclose (irafFile);
    }
    else
        labtyd (0, "Can't open file for IRAF cross reference list.");
}

char *createUniqueName (char *fieldName)
{
    static char path[255];

    char basicPath[255];
    char *directory;

    struct stat sbuf;

    int nonUnique = TRUE;
    int suffix = 1;

    basicPath[0] = '\0';

    if ((directory = (char *) getenv (FIBFIELDS)) != NULL)
    {
        strcpy (basicPath, directory);
        strcat (basicPath, "/");
    }

    strcat (basicPath, fieldName);

    if (faith)
    {
        strcat (basicPath, ".sim");
    }

    strcat (basicPath, ".iraf");

    /* keep trying suffix's until a usable filename is found */

    while (nonUnique)
    {

        /* create the entire path/filename/suffix */

        sprintf (path, "%s.%d", basicPath, suffix);

        /* test name for uniqueness */

        if (stat (path, &sbuf) == 0)
        {

            suffix++;
        }
        else
        {
            nonUnique = FALSE;
        }
    }

    /* return the unique path/filename */

    return (path);

}

/* 
		printIRAFfile

   This function outputs the object/fiber cross list 
   for use by IRAF.
*/

void printIRAFfile (FILE * theFile)
{
    int i;

    int lastFib = getMaxSlitInCable (CurrentCable);

    printIRAFheader (theFile);

    printSCSline (1, theFile, NULL);

    for (i = 2; i <= lastFib; i++)
    {

        printIRAFline (i, theFile, NULL);

    }
    printSCSline ((lastFib + 1), theFile, NULL);

}

/* 
		printSCSline

   Create an entry of a simultaneous comapison source.
*/

void printSCSline (int slitPosition, FILE * theFile, char *theLine)
{
    char objectName[64];
    int typeCode;

    strcpy (objectName, "SCS");
    typeCode = SCS_FIBER;

    if (theLine != NULL)
    {

        sprintf (theLine, "%3d % 1d %s$", slitPosition, typeCode, objectName);
    }

    if (theFile != NULL)
    {

        fprintf (theFile, "%3d % 1d %s\n",
                 slitPosition, typeCode, objectName);
    }
}

/* 
		printIRAFline

   This function outputs an object/fiber cross reference entry
   for use by IRAF. The argument "theLine" is used by the WIYN mods.
*/

void printIRAFline (int slitPosition, FILE * theFile, char *theLine)
{
    xItem theList;
    int onTarget;

    struct starinfo *obj;
    char objectName[64];
    char tmpName[64];

    char *tail;

    int typeCode;

    onTarget = xRefSlit (slitPosition, &theList, Stars, FiBase);

    if (fiberBroken (theList.fiber))
    {

        strcpy (objectName, "Broken Fiber");
        typeCode = UNASSIGNED_FIBER;

    }
    else
    {

        if ((!onTarget) || (theList.object == VOIDENTRY))
        {

            if (theList.fiber == VOIDENTRY)
            {
                strcpy (objectName, "Gap");
                typeCode = FIDUCIAL_FIBER;
            }
            else
            {

                if (!fiberParked (theList.fiber))
                {
                    strcpy (objectName, "Random position");
                    typeCode = RANDOM_FIBER;
                }
                else
                {               /* fiber is parked */
                    strcpy (objectName, "not assigned");
                    typeCode = UNASSIGNED_FIBER;
                }
            }
        }
        else
        {

            obj = Stars[theList.object];

            /* strip trailing spaces off the object name */

            strcpy (tmpName, obj->name);

            tail = tmpName + strlen (tmpName) - 1;

            while (*tail == ' ')
                tail--;

            *(++tail) = '\0';

            sprintf (objectName, "%s (%d)", tmpName, obj->IDnumber);

            if ((obj->objectType == 'S') || (obj->objectType == 's'))
            {
                typeCode = SKY_FIBER;
            }
            else
            {
                typeCode = OBJ_FIBER;
            }

            if (FiBase[theList.fiber].updated == TRUE)
            {
                fprintf (stderr, "Fiber %d was updated.\n", theList.fiber);
            }
        }
    }

    if (theLine != NULL)
    {

        sprintf (theLine, "%3d % 1d %s$", slitPosition, typeCode, objectName);
    }

    if (theFile != NULL)
    {

        fprintf (theFile, "%3d % 1d %s\n",
                 slitPosition, typeCode, objectName);
    }
}

/* 
		printCoordPair

   This function generates a rad/dec coordinate pair for the specified
   slit position.  This is used by the wiyn header stuff which
   replaces the .iraf file.
*/

void printCoordPair (int slitPosition, char *raString, char *decString)
{
    xItem theList;
    int onTarget;

    struct starinfo *obj;

    *raString = '\0';
    *decString = '\0';

    onTarget = xRefSlit (slitPosition, &theList, Stars, FiBase);

    /* if there is no assigned object, exit */

    if ((!onTarget) || (theList.object == VOIDENTRY))
        return;                 /* empty strings are failure indicator */

    obj = Stars[theList.object];

    sprintObjectCoord (obj, raString, RA, 2);
    sprintObjectCoord (obj, decString, DEC, 1);

}

/* 
		printFieldCenter

   This function generates a rad/dec coordinate pair for the 
   field center position.  This is used by the wiyn header stuff which
   replaces the .iraf file.
*/

void printFieldCenter (char *raString, char *decString, char *epochString)
{
    struct starinfo *obj = Stars[0];    /* always contains field center */

    *raString = '\0';
    *decString = '\0';
    *epochString = '\0';

    if (obj != NULL)
    {
        sprintObjectCoord (obj, raString, RA, 2);
        sprintObjectCoord (obj, decString, DEC, 1);
        sprintf (epochString, "%5.1f", Exposure.epoch);
    }
}

/*
   
   The following three functions pack up pieces of header
   information as strings for use by gwc.
*/

char *packFieldName (void)
{
    static char theString[50];

    sprintf (theString, "%s\n", Exposure.fileName);

    return theString;
}

char *packCable (void)
{
    static char theString[50];

    strcpy (theString, getNameForCable (CurrentCable));

    return theString;
}

int packCableSize (void)
{
    return (getMaxSlitInCable (CurrentCable) + 1);
}

char *packFieldTime (void)
{
    struct timeval theTime;
    static char timeString[128];

    strcpy (timeString, "No Time");

    if (gettimeofday (&theTime, (struct timezone *) NULL) == 0)
    {
        strcpy (timeString, ctime (&theTime.tv_sec));
    }
    else
    {
        fputs ("WARNING: Unable to obtain system time for header\n", stderr);
    }

    return (timeString);
}

char *packFieldLST (void)
{
    static char theString[50];

    int angle[4];
    char angleSign;

    sla_dr2tf (2, ((Exposure.st / 12) * M_PI), &angleSign, angle);

    if (angleSign == '+')
        angleSign = ' ';

    sprintf (theString, "%c%2d:%02d:%02d.%-2d\n",
             angleSign, angle[0], angle[1], angle[2], angle[3]);

    return theString;
}

/* 
		generateStats

   This function computes how many stars/skys/fobs are assigned.
*/

void generateStats (int *objCount, int *skyCount, int *fopCount)
{
    xItem theList;
    int onTarget;

    int i;

    /* clear the counts */
    *objCount = 0;
    *skyCount = 0;
    *fopCount = 0;

    for (i = 0; i < MAXFIBER; i++)
    {

        if (fiberParked (i))
            continue;

        onTarget = xRefFiber (i, &theList, Stars, FiBase);

        if (onTarget)
        {

            if (fiberIsFop (i))
            {
                (*fopCount)++;
            }
            else
            {
                if ((Stars[theList.object]->objectType == 'S') ||
                    (Stars[theList.object]->objectType == 's'))
                {
                    (*skyCount)++;
                }
                else
                {
                    (*objCount)++;
                }
            }
        }
        else
        {                       /* object < 0, hence unassigned extended fiber */
            (*skyCount)++;
        }
    }
}

void printIRAFheader (FILE * file)
{
    struct timeval theTime;
    char *timeString;

    int angle[4];
    char angleSign;

    /* get the current time and process it into a string */

    if (gettimeofday (&theTime, (struct timezone *) NULL) != 0)
    {
        fputs ("WARNING: Unable to obtain system time for .iraf file\n",
               stderr);
    }
    else
    {
        timeString = ctime (&theTime.tv_sec);
        fprintf (file, "# %s", timeString);
    }

    fprintf (file, "# Hydra Filename: %s\n", Exposure.fileName);

    sla_dr2tf (2, ((Exposure.st / 12) * M_PI), &angleSign, angle);

    if (angleSign == '+')
        angleSign = ' ';

    fprintf (file, "# LST Middle :  %c%2d:%02d:%02d.%-2d\n",
             angleSign, angle[0], angle[1], angle[2], angle[3]);

}
