#include <time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <string.h>

#include "log.h"

FILE *CurrentLog;

char Prefix[32] = { '\0' };     /* placed ahead of the filename if not empty */

static struct flock LockAll = {
    F_WRLCK,                    /* locks the entire file */
    SEEK_SET,
    0,
    0,
    0
};

static struct flock UnLockAll = {
    F_UNLCK,                    /* unlocks the entire file */
    SEEK_SET,
    0,
    0,
    0
};

/* local prototypes */
int logCreate (char *name);
void logOpenLocal (void);
FILE *logOpen (void);
int log_puts (char *s);
void log_putsQ (char *s);
int log_fputs (char *s, FILE * stream);
int log_printf (char *format, long a, long b, long c, long d, long e, long f,
                long g, long h, long i, long j, long l, long m, long n,
                long o, long p, long q);
void log_printfQ (char *format, long a, long b, long c, long d, long e,
                  long f, long g, long h, long i, long j, long l, long m,
                  long n, long o, long p, long q);
int log_fprintf (FILE * stream, char *format, long a, long b, long c, long d,
                 long e, long f, long g, long h, long i, long j, long l,
                 long m, long n, long o, long p, long q);
int logString (char *s, FILE * theLog);
int logTimeStamp (FILE * theLog);

/*********************************************************************
*
* logCreate - create and link a log file
*
* If the environment variable LOGPATH is not present, use the current
* directory.  If the argument is a null pointer, create the log
* using the current date as the filename.  If the argument points
* to a string, use that string as the  final component of the path.
* 
* If the file already exists, insure that it is linked, and exit.
* If the file does not exist, create it, and link it to the 
* working file name.
* 
* RETURNS
* Returns -1 on any error.
*/

logCreate (char *name)
{
    FILE *logFile;              /* temporary for creating the log */
    char *logPath;              /* path to log file directory */
    char logName[128];          /* complete path including log filename */
    char defaultName[64];       /* software generated log name */
    char linkedLogName[128];    /* the name that all programs use--
                                   linked to the file that we create here */
    struct timeval theTime;
    struct tm *tm;
    struct stat sbuf;

    char *temp;                 /* remove when we get 4.1 */

    /* clear the strings */

    logName[0] = '\0';          /* complete path including log filename */
    defaultName[0] = '\0';      /* software generated log name */
    linkedLogName[0] = '\0';    /* the name that all programs use */

    /* create the current log name */

    if ((logPath = getenv (LOGENV)) != NULL)
    {
        (void) strcpy (logName, logPath);
        (void) strcat (logName, "/");
    }

    if (*Prefix != '\0')
        (void) strcat (logName, Prefix);

    if (name != NULL)
        (void) strcat (logName, name);

    else
    {                           /* create a name using the current date */

        if (gettimeofday (&theTime, (struct timezone *) NULL) != 0)
            (void) fputs ("Unable to obtain system time", stderr);

        else
        {
            /* convert the long int system time to its components */

            tm = localtime (&theTime.tv_sec);

            /* convert date into a string of the right format. */

#ifdef SUN41

            /* when we get sun OS 4.1, we'll use the easy way... */

            strftime (defaultName, (sizeof (defaultName) - 1), "%h%d", tm);
#else
            /* the following is a kludge until we get 4.1 */

            temp = asctime (tm);

            (void) strcpy (defaultName, temp);

            defaultName[10] = '\0';     /* chop after the date */
            defaultName[3] = '_';       /* replace the first space */
            defaultName[7] = '_';       /* replace the second space */

            /* sometimes there are two spaces before the date. */

            if (defaultName[8] == ' ')
            {
                defaultName[8] = defaultName[9];        /* shuffle things forward */
                defaultName[9] = defaultName[10];       /* shuffle things forward */
            }

#endif

            /* append this date to the path as the filename */

            (void) strcat (logName, defaultName);
        }
    }

    /* check it for existence */

    if (stat (logName, &sbuf) < 0)
    {

        /* not found, so create it */

        logFile = fopen (logName, "w");

        if (logFile == (FILE *) NULL)
        {
            fprintf (stderr,
                     "Non-fatal System Error: cannot open logfile \"%s\"\n",
                     logName);
        }

        else
        {

            if (fchmod (fileno (logFile), 0666) != 0)
                fputs ("WARNING: Couldn't set permissions on log file",
                       stderr);

            fclose (logFile);   /* don't need it anymore... */

            /* build the path to the shared/linked log file */

            if (logPath != NULL)
            {
                (void) strcpy (linkedLogName, logPath);
                (void) strcat (linkedLogName, "/");
            }

            if (*Prefix != '\0')
                (void) strcat (linkedLogName, Prefix);

            (void) strcat (linkedLogName, LOGFILE);

            /* if there is an old log, unlink it */

            if (stat (linkedLogName, &sbuf) == 0)
                unlink (linkedLogName);

            /* now link to the new file */

            if (link (logName, linkedLogName) != 0)
            {
                fprintf (stderr,
                         "Non-fatal System Error: cannot link logfile \"%s\"\n",
                         logName);
            }
        }
    }

}

/*********************************************************************
*
* logOpenLocal - open the current log for logging routines only.
*
* Call logOpen and save the file pointer in the global "CurrentLog".
*
*/

void logOpenLocal (void)
{
    CurrentLog = logOpen ();
}

/*********************************************************************
*
* logOpen - open the current log file
* Open the current log file and return a file pointer to it.
* If the environment variable LOGPATH is not present, use the current
* directory.
* 
* RETURNS
* Returns null if the file can not be found.
*/

FILE *logOpen (void)
{
    FILE *logFile;
    char *logPath;
    char linkedLogName[128];

    /* set up the path */

    if ((logPath = getenv (LOGENV)) != NULL)
    {
        (void) strcpy (linkedLogName, logPath);
        (void) strcat (linkedLogName, "/");
    }

    if (*Prefix != '\0')
        (void) strcat (linkedLogName, Prefix);

    (void) strcat (linkedLogName, LOGFILE);

    /* open the file */

    logFile = fopen (linkedLogName, "a+");

    if (logFile == (FILE *) NULL)
    {
        fprintf (stderr,
                 "Non-fatal System Error: cannot open logfile \"%s\"\n",
                 linkedLogName);

        return ((FILE *) NULL);
    }
    else
    {
        setbuf (logFile, (char *) NULL);
        return (logFile);
    }
}

/*********************************************************************
*
* log_puts - log a string then send it to stdout via "puts"
*
* RETURNS
* Returns the result of the "puts".
*/

log_puts (char *s)
{
    /* send to the log file */

    logString (s, CurrentLog);
    putc ('\n', CurrentLog);

    /* send to stdout */

    return (puts (s));
}

/*********************************************************************
*
* log_putsQ - log a string 
*
* RETURNS
* Returns the result of the "puts".
*/

void log_putsQ (char *s)
{
    /* send to the log file */

    logString (s, CurrentLog);
    putc ('\n', CurrentLog);
}

/*********************************************************************
*
* log_fputs - log a string then send it to a stream via "fputs"
*
* RETURNS
* Returns the result of the "fputs".
*/

log_fputs (char *s, FILE * stream)
{
    /* send to the log file */

    logString (s, CurrentLog);

    /* send to the specified stream */

    return (fputs (s, stream));

}

/*********************************************************************
* log_printf - log and format an output line then send it to stdout via puts
*
* NOTE
* This routine can handle a limited number of arguments.
*
* RETURNS
* Returns the result of the "puts".
*/

 /*VARARGS*/
log_printf (char *format, long int a, long int b, long int c, long int d,
            long int e, long int f, long int g, long int h, long int i,
            long int j, long int l, long int m, long int n, long int o,
            long int p, long int q)
{
    char tempString[255];

    /* create the formatted string */

    sprintf (tempString, format, a, b, c, d, e, f, g,
             h, i, j, l, m, n, o, p, q);

    /* log it */

    logString (tempString, CurrentLog);

    /* send to stdout */

    return (fputs (tempString, stdout));

}

/*********************************************************************
* log_printfQ - log and format an output line 
*
* NOTE
* This routine can handle a limited number of arguments.
*
*/

 /*VARARGS*/
    void log_printfQ (char *format, long int a, long int b, long int c,
                      long int d, long int e, long int f, long int g,
                      long int h, long int i, long int j, long int l,
                      long int m, long int n, long int o, long int p,
                      long int q)
{
    char tempString[255];

    /* create the formatted string */

    sprintf (tempString, format, a, b, c, d, e, f, g,
             h, i, j, l, m, n, o, p, q);

    /* log it */

    logString (tempString, CurrentLog);

}

/*********************************************************************
*
* log_fprintf - log and format an output line then send it to a stream via puts
*
* NOTE
* This routine can handle a limited number of arguments.
*
* RETURNS
* Returns the result of the "puts".
*/

 /*VARARGS*/
log_fprintf (FILE * stream, char *format, long int a, long int b, long int c,
             long int d, long int e, long int f, long int g, long int h,
             long int i, long int j, long int l, long int m, long int n,
             long int o, long int p, long int q)
{
    char tempString[255];

    /* create the formatted string */

    sprintf (tempString, format, a, b, c, d, e, f, g,
             h, i, j, l, m, n, o, p, q);

    /* log it */

    logString (tempString, CurrentLog);

    /* send to stdout */

    return (fputs (tempString, stream));

}

/*********************************************************************
*
* logString - Send a string to the log file.
*
* Before writing to the log file, put a lock on it so that no
* other processes can disturb the file until we are done.
* 
* Release the lock on exit unless the string does not end with 
* a newline.  The file will then remain locked until the process
* that locked it returns and releases it.
*/

logString (char *s, FILE * theLog)
{
    if (fcntl (fileno (theLog), F_SETLKW, &LockAll) != -1)
    {

        if (fseek (theLog, (long) 0, 2) == 0)
        {
            logTimeStamp (theLog);
            fputs (s, theLog);
        }
        else
            fputs ("Can't find end of log file.\n", stderr);

        if (s[strlen (s) - 1] == '\n')
        {
            if (fsync (fileno (theLog)))
                fputs ("WARNING:  Could not sync log file. \n", stderr);

            if (fcntl (fileno (theLog), F_SETLK, &UnLockAll) == -1)
                fputs ("WARNING:  Could not unlock log file. \n", stderr);

        }
    }
    else
        fputs ("WARNING:  Could not lock log file. \n", stderr);
}

logTimeStamp (FILE * theLog)
{

    struct timeval theTime;
    char *timeString;
    char modString[128];

    gettimeofday (&theTime, (struct timezone *) NULL);

    timeString = ctime (&theTime.tv_sec);

    (void) sprintf (modString, "%.24s -- ", timeString);

    fputs (modString, theLog);

}

#ifdef TEST_ONLY

FILE *CurrentLog;

main ()
{
    FILE *test = logOpen ();

    logString ("Ho Ho Ho", test);
    sleep (60);
    logString ("Ha Ha Ha\n", test);
}

#endif
