#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "statusIO.h"

/* structures for fcntl */

static struct flock LockAll = {
    F_WRLCK,                    /* locks the entire file */
    SEEK_SET,
    0,
    0,
    0
};

static struct flock UnLockAll = {
    F_UNLCK,                    /* unlocks the entire file */
    SEEK_SET,
    0,
    0,
    0
};

/* next 3 are for testing */
int dummy1;
double dummy2;
char dummy3[64];

/* This is the list of the keywords that we expect in a status file */
#ifdef IGNORE
struct templateItem statusList[] = {

    /*    keyname          type         where it goes          req?  found? */

    {KEY_DUMMY_INT, INT, (char *) &dummy1, TRUE, FALSE},
    {KEY_DUMMY_DOUBLE, DOUBLE, (char *) &dummy2, TRUE, FALSE},
    {KEY_DUMMY_STRING, CHAR, dummy3, TRUE, FALSE},
    /* the next line is required and used to mark the end of the structure */
    {"", END, NULL, FALSE, FALSE}
};

#endif

/*********************************************************************/

/* local prototypes */
int LoadStatusFile (char *envName, struct templateItem template[]);
int getStatusLine (FILE * dataFile, char *theLine);
void parseItem (char *theLine, struct templateItem template[]);
int reqKeyMissing (struct templateItem template[]);
void clearFlagsFound (struct templateItem template[]);
int keySearch (char *theLine, struct templateItem template[]);
void compareToDefaults (struct templateItem template[]);
void setToDefaults (struct templateItem template[]);
char *strngstrng (char *s1, char *s2);
int saveStatus (char *envName, struct templateItem template[]);

#ifdef IGNORE
main ()
{

    char envName[64];

    (void) strcpy (envName, STATUSFILE);

    (void) LoadStatusFile (envName, statusList);

    (void) strcpy (envName, OUTFILE);

    (void) printStatus (envName, statusList);

    int getStatusLine (FILE * dataFile, char *theLine);
    int reqKeyMissing (struct templateItem *template);
    int keySearch (char *theLine, struct templateItem *template);

    int getStatusLine (FILE * dataFile, char *theLine);
    int reqKeyMissing (struct templateItem *template);
    int keySearch (char *theLine, struct templateItem *template);

}

#endif

/*
		LoadStatusFile

   Read and process a "status" file.  Return a
   code to indicate the status of the file's contents.
*/

LoadStatusFile (char *envName, struct templateItem *template)
{
    FILE *dataFile;

    char theLine[256];
    int lineType;

    char *statusPath;
    char *statusFile;
    char statusFileName[64];

    /* find out what directory to search for the data file */

    if ((statusPath = getenv (FIBSTATUS)) != NULL)
    {
        (void) strcpy (statusFileName, statusPath);
        (void) strcat (statusFileName, "/");
    }
    else
    {
        (void) fprintf (stderr, "\"%s\" environment variable not found\n",
                        FIBSTATUS);
        (void) fputs ("Searching current directory\n", stderr);
    }

    /* find out what file to use */

    if ((statusFile = getenv (envName)) != NULL)
    {
        (void) strcat (statusFileName, statusFile);
    }
    else
    {
        (void) fprintf (stderr, "\"%s\" environment variable not found\n",
                        envName);
        return (FAILURE);
    }

    /* see if the file exists and open it */

    if ((dataFile = fopen (statusFileName, "r+")) == NULL)
    {
        (void) fprintf (stderr, "Couldn't open status archive:\" %s\"\n",
                        statusFileName);
        return (FAILURE);
    }
    else
    {
        if (fcntl (fileno (dataFile), F_SETLKW, &LockAll) == -1)
        {
            perror ("ERROR:  Couldn't lock status file for reading");
            return (FAILURE);
        }
    }

    /* clear the key found list */

    clearFlagsFound (template);

    /* read and process all the lines in the file */

    while ((lineType = getStatusLine (dataFile, theLine)) != EOF)
    {

        switch (lineType)
        {

            case EMPTY:

                break;

            case COMMENT:

                (void) fputs (theLine, stderr);
                break;

            case HEADER:

                parseItem (theLine, template);
                break;

            default:

                (void) fprintf (stderr, "unreadable line in status file:\n%s",
                                theLine);
                break;
        }
    }

    if (fcntl (fileno (dataFile), F_SETLKW, &LockAll) == -1)
    {
        perror ("ERROR:  Couldn't unlock status file.");
    }

    fclose (dataFile);

    /* warn user if restoring to non standard values */

    compareToDefaults (template);
    setToDefaults (template);

    /* be sure all the required keywords are present and 
       then return the file status */

    if (reqKeyMissing (template))
        return (FILE_BAD);
    else
    {
        return (FILE_OK);
    }

}

/*
		getStatusLine

   Get the next line from the specified file and place it
   in the string pointed to by the second argument. 
   Determine the "type" of line, and return a description code.
   if no line is found or an error occurs, return EOF.
*/

getStatusLine (FILE * dataFile, char *theLine)
{
    *theLine = '\0';

    if ((fgets (theLine, 128, dataFile) == NULL))
        return (EOF);

    /* get past any white space at the beginning */

    while ((*theLine == ' ') || (*theLine == '\t'))
        theLine++;

    if (*theLine == '\n')
        return (EMPTY);

    if (*theLine == '#')
        return (COMMENT);

    if (isalpha (*theLine))
        return (HEADER);

    return (OTHER);

}

/*
		parseItem

   Break down the specified line into a keyword
   and a value. If an unknown
   keyword or a bad line is encountered, print a message
   and return having taken no action.  This function uses
   the global "statusFlagList" which is the list of valid header
   items.

*/

void parseItem (char *theLine, struct templateItem *template)
{
    if (keySearch (theLine, template) == FAILURE)
        (void) fprintf (stderr, "BAD HEADER ENTRY: \n%s", theLine);
}

int reqKeyMissing (struct templateItem *template)
{
    int i;

    for (i = 0; (template[i].type != END); i++)
        if (template[i].required && !template[i].found)
            return (TRUE);

    return (FALSE);
}

void clearFlagsFound (struct templateItem *template)
{
    int i;

    for (i = 0; (template[i].type != END); i++)
        template[i].found = FALSE;
}

int keySearch (char *theLine, struct templateItem *template)
{
    char key[64];
    char value[64];

    int matchFound = FALSE;
    int i;

    /* be sure the line has the right format by splitting it into
       the keyword and the value */

    if (sscanf (theLine, "%[^:]: %s", key, value) < 2)
        return (FAILURE);

    /* search for a key match */

    for (i = 0; ((template[i].type != END) && (!matchFound)); i++)
    {

        if (strcmp (key, template[i].keyName) == 0)
        {

            switch (template[i].type)
            {

                case INT:

                    if (sscanf (value, "%d", (int *) template[i].whereItGoes)
                        >= 1)
                    {
                        template[i].found = TRUE;
                        matchFound = TRUE;
                    }
                    break;

                case DOUBLE:

                    if (sscanf
                        (value, "%lf",
                         (double *) template[i].whereItGoes) >= 1)
                    {
                        template[i].found = TRUE;
                        matchFound = TRUE;
                    }
                    break;

                case CHAR:

                    (void) strcpy (template[i].whereItGoes, value);
                    template[i].found = TRUE;
                    matchFound = TRUE;
                    break;

                default:
                    (void) fputs ("Unknown type for hydra header key\n",
                                  stderr);
                    break;
            }
        }
    }

    if (!matchFound)
        return (FAILURE);
    else
        return (0);
}

/*
		compareToDefaults

   Test the value loaded from the file against the
   default value.  Print a message if they differ.
*/

void compareToDefaults (struct templateItem *template)
{
    int i;

    for (i = 0; (template[i].type != END); i++)
    {

        switch (template[i].type)
        {

            case INT:
                if (template[i].defaultValue != NULL)
                    if ((int) *(int *) template[i].whereItGoes !=
                        (int) *(int *) template[i].defaultValue)
                    {

                        fprintf (stderr,
                                 "\n%s has non default value of: %d\n",
                                 template[i].keyName,
                                 (int) *(int *) template[i].whereItGoes);

                        if (template[i].bootToDefault)
                            fprintf (stderr,
                                     "   --will be forced to default value: %d\n\n",
                                     (int) *(int *) template[i].defaultValue);

                    }
                break;

            case DOUBLE:

                if (template[i].defaultValue != NULL)
                    if ((double) *(double *) template[i].whereItGoes !=
                        (double) *(double *) template[i].defaultValue)
                    {

                        fprintf (stderr,
                                 "\n%s has non default value of: %f\n",
                                 template[i].keyName,
                                 (double) *(double *) template[i].
                                 whereItGoes);

                        if (template[i].bootToDefault)
                            fprintf (stderr,
                                     "   --will be forced to default value: %f\n\n",
                                     (double) *(double *) template[i].
                                     defaultValue);

                    }
                break;

            case CHAR:
                if (template[i].defaultValue != NULL)
                    if (strcmp
                        (template[i].whereItGoes,
                         template[i].defaultValue) != 0)
                    {
                        fprintf (stderr,
                                 "\n%s has non default value of: %s\n",
                                 template[i].keyName,
                                 template[i].whereItGoes);

                        if (template[i].bootToDefault)
                            fprintf (stderr,
                                     "   --will be forced to default value:\"%s\"\n\n",
                                     template[i].defaultValue);
                    }
                break;

            default:
                (void) fputs ("Unknown type for statusIO key\n", stderr);
                break;
        }
    }
}

/*
		setToDefaults

   Test the value loaded from the file against the
   default value.  Print a message if they differ.
*/

void setToDefaults (struct templateItem *template)
{
    int i;

    for (i = 0; (template[i].type != END); i++)
    {

        if ((template[i].bootToDefault) && (template[i].defaultValue != NULL))
        {

            switch (template[i].type)
            {

                case INT:
                    *(int *) template[i].whereItGoes =
                        *(int *) template[i].defaultValue;
                    break;

                case DOUBLE:
                    *(double *) template[i].whereItGoes =
                        *(double *) template[i].defaultValue;
                    break;

                case CHAR:
                    strcpy (template[i].whereItGoes,
                            template[i].defaultValue);
                    break;

                default:
                    (void) fputs ("Unknown type for statusIO key\n", stderr);
                    break;
            }
        }

    }
}

char *strngstrng (char *s1, char *s2)
{
    if ((*s1 != '\0') &&
        ((*s1 != *s2) || (strncmp (s1, s2, strlen (s2)) != 0)))

        return strngstrng (++s1, s2);

    else
        return (s1);
}

saveStatus (char *envName, struct templateItem * template)
{
    char *statusPath;
    char *statusFile;
    char statusFileName[64];

    FILE *dataFile;

    int i;

    /* find out what directory to search for the data file */

    if ((statusPath = getenv (FIBSTATUS)) != NULL)
    {
        (void) strcpy (statusFileName, statusPath);
        (void) strcat (statusFileName, "/");
    }
    else
    {
        (void) fprintf (stderr, "\"%s\" environment variable not found\n",
                        FIBSTATUS);
        (void) fputs ("Searching current directory\n", stderr);
    }

    /* find out what file to use */

    if ((statusFile = getenv (envName)) != NULL)
    {
        (void) strcat (statusFileName, statusFile);
    }
    else
    {
        (void) fprintf (stderr, "\"%s\" environment variable not found\n",
                        envName);
        return (FAILURE);
    }

    /* see if the file exists and open it */

    if ((dataFile = fopen (statusFileName, "w")) == NULL)
    {
        (void) fprintf (stderr, "Couldn't open status archive:\" %s\"\n",
                        statusFileName);
        return (FAILURE);
    }
    else
    {
        if (fcntl (fileno (dataFile), F_SETLKW, &LockAll) == -1)
        {
            perror ("ERROR:  Couldn't lock status file for writing");
            return (FAILURE);
        }
    }

    for (i = 0; (template[i].type != END); i++)
    {

        /* don't output a key that was never read in or generated! */

        if (template[i].found == TRUE)
        {

            switch (template[i].type)
            {

                case INT:
                    (void) fprintf (dataFile, "%s: %d\n",
                                    template[i].keyName,
                                    *(int *) template[i].whereItGoes);
                    break;

                case DOUBLE:
                    (void) fprintf (dataFile, "%s: %25.12f\n",
                                    template[i].keyName,
                                    *(double *) template[i].whereItGoes);
                    break;

                case CHAR:
                    (void) fprintf (dataFile, "%s: %s\n",
                                    template[i].keyName,
                                    template[i].whereItGoes);
                    break;

                default:
                    (void) fputs ("Unknown type for hydra header key\n",
                                  stderr);
                    break;
            }
        }
    }

    if (fcntl (fileno (dataFile), F_SETLK, &UnLockAll) == -1)
        perror ("ERROR:  Couldn't unlock status file.");

    fclose (dataFile);

    return (0);
}

/*
if( fcntl( fileno( theLog ), F_SETLKW, &LockAll ) != -1 )
if( fcntl( fileno( theLog ), F_SETLK, &UnLockAll ) == -1 )
*/
