
/* Generic WIYN Client: gwcTkInit.c

 *  Module name:
        gwcTkInit.c

 *  Function:
        This file is the Tk specific code for GWC.

 *  Description:
        This file interfaces GWC to the TK event handling mechanism.
        Connection/reconnect code specific to Tk is here too.

 *  Language:
      C

 *  Support: Kim Gillies, NOAO

 *

 *  History:
      12 Dec 94: Version 2 - goes into CVS

 *  Rcs Id:  $Id: gwcTkInit.c,v 1.1.1.1 2004/08/13 20:46:04 behzad Exp $

 */
#include <stdio.h>
#include <unistd.h>
#include <assert.h>
#include <tk.h>
#include "gwc/gwc.h"

/* In file data */
static int gDelay = 20000;
static int gRestartCount = 0;
static int gRestartTotal = 10000;

/* Tk Specific */
static Tk_TimerToken gRestartToken;

/* External reference from gwcConnect */
int gwcSetupHandlers (gwcConnectInfo info);
int gwcGetProcessEDM (gwcConnectInfo);
int gwcMakeFinalConnection (gwcConnectInfo info, connectType yesorno);

/* External reference from hydra code */

extern int noGWC;

/* In file prototypes */
int gwcLibSetup (void);
int gwcLibSetupHandlers (gwcConnectInfo info);
int gwcLibDestroyHandlers (gwcConnectInfo info);
void gwcLibDefaultConnectCallback (ClientData clientData);
void gwcLibPassRestartParameters (int delay, int restartcount);
static void tkReceiver (ClientData clientData, int mask);
static void gwcTkConnectCallback (ClientData clientData);

/* System-specific message system connection */
int gwcLibSetup (void)
{
    /* Nothing is needed here */
    return TCL_OK;
}

/* Library specific connection setup */
int gwcLibSetupHandlers (gwcConnectInfo info)
{
    int sock;

    assert (info != (gwcConnectInfo) NULL);

    /*fprintf(stderr, "Add Handler for socket: %d\n", sock); */
    sock = gwcGetClientSocket (info);

    /* Set the message receiver code */
    Tk_CreateFileHandler (sock, TK_READABLE, tkReceiver, (ClientData) info);
    return TCL_OK;
}

/* System specific break connection code */
int gwcLibDestroyHandlers (gwcConnectInfo info)
{
    int sock;

    assert (info != (gwcConnectInfo) NULL);

    /*fprintf(stderr, "Delete Handler for socket: %d\n", sock); */
    sock = gwcGetClientSocket (info);

    /* Set the message receiver code */
    Tk_DeleteFileHandler (sock);
    /* Free up the socket */
    (void) close (sock);
    /* Set to a safe value (0 is stdout) */
    gwcSetClientSocket (info, 0);
    return TCL_OK;
}

/* Start a reconnect sequence */
void gwcLibDefaultConnectCallback (ClientData clientData)
{
    gwcConnectInfo info = (gwcConnectInfo) clientData;

    gRestartCount = 0;
    gwcTkConnectCallback (info);
}

/* Accept restart parameters from upper layer */
void gwcLibPassRestartParameters (int delay, int restartcount)
{
    /* Convert seconds to milliseconds for tk */
    gDelay = delay * 1000;
    gRestartTotal = restartcount;
}

/* --------------------- Internal Private Functions --------------------- */

/* This function is called by the socket interface to process incoming
 * messages.
 */
 /*ARGSUSED*/ static void tkReceiver (ClientData clientData, int mask)
{
    gwcConnectInfo info = (gwcConnectInfo) clientData;

    assert (info != (gwcConnectInfo) NULL);

    /* Process incoming EDM messages */
    (void) gwcGetProcessEDM (info);
}

/* The tk reconnect callback */
static void gwcTkConnectCallback (ClientData clientData)
{
    gwcConnectInfo info = (gwcConnectInfo) clientData;

    /* Check for the first time - delay if it's zero */
    if (gRestartCount == 0)
    {
        /* Up the count and try again later */
        ++gRestartCount;
        gRestartToken = Tk_CreateTimerHandler (gDelay, gwcTkConnectCallback,
                                               (ClientData) info);
        return;
    }

    /* Print a message on the users screen */
    /* Add one so that numbers go from 1 to limit rather than 0 */
    gwcPrintStatus (info, "Trying to connect to router on: %s - (%d)",
                    gwcGetRouterHost (info), gRestartCount);
    if (gRestartCount < gRestartTotal)
    {
        if (gwcRouterConnect (info) == TCL_OK)
        {
            gwcPrintStatus (info, "Connected!");
            /* Now run the restart func if there is one */
            if (gwcMakeFinalConnection (info, CONNECT_YES) != TCL_OK)
            {
                return;
            }
        }
        else
        {
            /* Up the count and try again later */
            gwcPrintStatus (info, "Failed...");
            ++gRestartCount;
            gRestartToken =
                Tk_CreateTimerHandler (gDelay, gwcTkConnectCallback,
                                       (ClientData) info);
        }
    }
    else
    {
        gwcPrintStatus (info,
                        "Reconnection failed. Giving up and disconnecting from hydra");
        noGWC = 1;
    }
}
