
/*

		rotatorCmd.c

   This file contains the functions that issue commands to
   the wiyn tcs for operating the MNIR port.  This includes
   setting the offset angle, the absolute position, and 
   turning tracking on and off.  Note that all status
   requests are in "rotatorVar.c"

*/

#include <tcl.h>
#include <tk.h>
#include <assert.h>

#include "gwc/gwc.h"
#include "miscMath.h"
#include "motor.h"
#include "plate.h"
#include "kaboom.h"
#include "nesslib.h"
#include "nessIO.h"

/****   Some defines just to save looking through the code ****/

#define CMD_ROT_TRACK	"mnir tracker"
#define CMD_ROT_OFFSET	"mnir offset set"
#define CMD_ROT_POS	"mnir position set"
#define ARG_ENABLE	"enable"
#define ARG_DISABLE	"disable"

#define ROTATOR_OFF	0
#define ROTATOR_ON	1

/* declerations of globals */

extern gwcConnectInfo hostInfo;

extern double CurrentTargetPos;

static double LastSavedOffset;

/*
   A command to send the rotator to the field and start tracking.
*/

/* local prototypes */
int rotatorPrepObserve (double offsetAngle);
int rotatorTrackFromZero (double offsetAngle);
int resumeTrackFromZero (void);
int rotatorStopAndZero (void);
int rotatorSetTracking (int mode);
int rotatorSetOffset (double offsetAngle);
int rotatorSetAbsAngle (double absoluteAngle);

int rotatorPrepObserve (double offsetAngle)
{

    int error = TCL_OK;

    /* set the offset */

    if ((error = rotatorSetOffset (offsetAngle)) != TCL_OK)
        return (error);

    /* start the tracking */

    if ((error = rotatorSetTracking (ROTATOR_ON)) != TCL_OK)
        return (error);

    return (error);
}

/*
   A command to start the rotator tracking from the zero position.
*/

int rotatorTrackFromZero (double offsetAngle)
{

    int error = TCL_OK;

    double tmpAngle = 0.0;

    /* set the offset */

    if ((error = rotatorSetOffset (0.0)) != TCL_OK)
        return (error);

    if ((error = rotatorSetTracking (ROTATOR_ON)) != TCL_OK)
        return (error);

    while (tmpAngle == 0.0)
    {

        while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
        {
            /* next line is just for testing */
            /* fputs( "FooTick...\n", stderr ); */
        }

        tmpAngle = CurrentTargetPos;
        /*fprintf( stderr, "\n 'tmpAngle in loop' = %f\n", tmpAngle ); */
    }

/*
   while( tmpAngle > M_PI )
      tmpAngle -= (M_PI * 2);

   while( tmpAngle < -M_PI )
      tmpAngle += (M_PI * 2);
*/

    /* fprintf( stderr, "\n 'foooo' = %f\n", tmpAngle ); */

    tmpAngle *= (180 / M_PI);

    LastSavedOffset = -tmpAngle;

    if ((error = rotatorSetOffset (-tmpAngle)) != TCL_OK)
        return (error);

    return (error);
}

/*
   A command to resume the rotator tracking from the offset
   last found from the track from zero.
*/

int resumeTrackFromZero (void)
{

    int error = TCL_OK;

    if ((error = rotatorSetTracking (ROTATOR_ON)) != TCL_OK)
        return (error);

    if ((error = rotatorSetOffset (LastSavedOffset)) != TCL_OK)
        return (error);

    return (error);
}

int rotatorStopAndZero (void)
{

    int error = TCL_OK;

    /* stop the tracking */

    if ((error = rotatorSetTracking (ROTATOR_OFF)) != TCL_OK)
        return (error);

    /* zero the offset */

    if ((error = rotatorSetOffset (0.0)) != TCL_OK)
        return (error);

    /* send to zero angle */

    if ((error = rotatorSetAbsAngle (0.0)) != TCL_OK)
        return (error);

    return (error);
}

int rotatorSetTracking (int mode)
{
    char cliString[128];

    if (mode == ROTATOR_ON)
        sprintf (cliString, "%s %s\n", CMD_ROT_TRACK, ARG_ENABLE);
    else
        sprintf (cliString, "%s %s\n", CMD_ROT_TRACK, ARG_DISABLE);

    /* fputs( cliString, stderr ); */

    return gwcSendTcsCommand (hostInfo, cliString);     /* hostInfo is a global */
}

int rotatorSetOffset (double offsetAngle)
{
    char cliString[128];

    /* offsetAngle += ROTATION_CORRECTION; */
    sprintf (cliString, "%s %10.5fd", CMD_ROT_OFFSET, offsetAngle);

    /* fputs( cliString, stderr ); */

    return gwcSendTcsCommand (hostInfo, cliString);     /* hostInfo is a global */
}

int rotatorSetAbsAngle (double absoluteAngle)
{
    char cliString[128];

    sprintf (cliString, "%s %10.5fd", CMD_ROT_POS, absoluteAngle);

    /* fputs( cliString, stderr ); */

    return gwcSendTcsCommand (hostInfo, cliString);     /* hostInfo is a global */
}
