
/*
		
		rotatorVar.c

   This file has the functions to create variables for the gwc
   client that relate to the instrument rotator .  It also
   has the callback for the rotator moving.

*/

#include <tk.h>
#include <assert.h>
#include <string.h>

#include "gwc/gwc.h"
#include "miscMath.h"
#include "motor.h"
#include "plate.h"
#include "kaboom.h"

#define ROTATOR_LIMIT  0.0349   /* one degree */
#define MAX_ROTATION_TIME  90000        /* 90 seconds in milliseconds */

/* hydra fiber data */
static double CurrentPlantPos = 0.0;
double CurrentRotatorOffset = 0.0;
double CurrentTargetPos = 0.0;
char CurrentRotatorEncoderstate[32];
int CurrentRotatorAmpStatus;

/* loacal prototypes */
void doRotatorSetup (gwcConnectInfo info);
static int rotatorUpdate (gwcEventArgs args);
static int targetUpdate (gwcEventArgs args);
double getCurrentRotatorPosition (void);
double getCurrentRotatorTarget (void);
int rotatorNotZero (void);
double getCurrentRotatorOffset (void);
int encoderStateUpdate (gwcEventArgs args);
int encoderAmpUpdate (gwcEventArgs args);
int rotatorNotInitialized (void);
int rotatorWaitForZero (void);
int doRotatorInitialValue (gwcConnectInfo info);
int doRotatorInitialOffset (gwcConnectInfo info);
int doRotatorInitialEncoderstate (gwcConnectInfo info);
int doRotatorInitialAmpStatus (gwcConnectInfo info);
static int offsetUpdate (gwcEventArgs args);
static void rotatorTimeoutFunction (void);

/* Setup the Hydra send database */

void doRotatorSetup (gwcConnectInfo info)
{

    /* subscribe to the tcs data stream */

    if (gwcStreamSubscribe (info, "tcs.mnir") != TCL_OK)
    {
        fputs ("WARNING: Subscribe to tcs info failed\n", stderr);
        return;
    }

    /* subscribe to the dcs data stream */

    if (gwcStreamSubscribe (info, "dcs.mnir") != TCL_OK)
    {
        fputs ("WARNING: Subscribe to dcs info failed\n", stderr);
        return;
    }

    /* callback for the plant position */

    if (gwcAddEvent (info, "tcs.mnir.plantpos",
                     EV_CHANGED, rotatorUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for rotator angle failed\n",
               stderr);
        return;
    }

    /* callback for the target position */

    if (gwcAddEvent (info, "tcs.mnir.position",
                     EV_CHANGED, targetUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for rotator position failed\n",
               stderr);
        return;
    }

    /* subscribe to the rotator offset */

    if (gwcAddEvent (info, "tcs.mnir.offset",
                     EV_CHANGED, offsetUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for rotator offset failed\n",
               stderr);
        return;
    }

    /* callback for the rotator encoder state */

    if (gwcAddEvent (info, "tcs.mnir.encoderstate",
                     EV_CHANGED, encoderStateUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for rotator position failed\n",
               stderr);
        return;
    }

    /* callback for the rotator amp */

    if (gwcAddEvent (info, "dcs.mnir.ampcmd",
                     EV_CHANGED, encoderAmpUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for rotator position failed\n",
               stderr);
        return;
    }
}

/* callback handler for rotator */

static int rotatorUpdate (gwcEventArgs args)
{

    double *value;

    value = (double *) args.value;

    CurrentPlantPos = value[0];

    /* fprintf ( stderr, "New rotator angle of %f\n", CurrentPlantPos ); */

    return TCL_OK;
}

static int targetUpdate (gwcEventArgs args)
{

    double *value;

    value = (double *) args.value;

    CurrentTargetPos = value[0];

    /* fprintf( stderr, "New rotator target of %f\n", CurrentTargetPos ); */

    return TCL_OK;
}

double getCurrentRotatorPosition (void)
{
    return (CurrentPlantPos);
}

double getCurrentRotatorTarget (void)
{
    return (CurrentTargetPos);
}

int rotatorNotZero (void)
{
    return (abs (CurrentPlantPos) > ROTATOR_LIMIT);
}

double getCurrentRotatorOffset (void)
{
    return (CurrentRotatorOffset);
}

int encoderStateUpdate (gwcEventArgs args)
{
    char **value;

    value = (char **) args.value;

    /* fprintf ( stderr, "New rotator encoder status of %s\n", value[0] ); */

    strcpy (CurrentRotatorEncoderstate, value[0]);

    return TCL_OK;
}

int encoderAmpUpdate (gwcEventArgs args)
{
    int *value;

    value = (int *) args.value;

    /* fprintf ( stderr, "New rotator amp state of %d\n", value[0] ); */

    CurrentRotatorAmpStatus = value[0];

    return TCL_OK;
}

int rotatorNotInitialized (void)
{
    if ((strcmp (CurrentRotatorEncoderstate, "absolute") != 0) ||
        (CurrentRotatorAmpStatus != 1))
    {

        return (TRUE);
    }
    else
    {
        return (FALSE);
    }
}

static int TimeOut;

int rotatorWaitForZero (void)
{
    Tk_TimerToken theTimer;

    TimeOut = 0;

    theTimer = Tk_CreateTimerHandler (MAX_ROTATION_TIME,
                                      (void (*)()) rotatorTimeoutFunction,
                                      (ClientData *) NULL);

    while (abs (CurrentPlantPos) > ROTATOR_LIMIT)
    {

        Tk_DoOneEvent (0);

        if (TimeOut)
            break;
    }

    Tk_DeleteTimerHandler (theTimer);

    return (TimeOut);
}

int doRotatorInitialValue (gwcConnectInfo info)
{
    valueType type;
    void *value;
    int count = 0;

    Tk_TimerToken theTimer;

    TimeOut = 0;

    theTimer = Tk_CreateTimerHandler (MAX_ROTATION_TIME,
                                      (void (*)()) rotatorTimeoutFunction,
                                      (ClientData *) NULL);

    while (count == 0)
    {

        if (gwcGetAttrValue (info,
                             "tcs.mnir.plantpos", &type, &count,
                             &value) != TCL_OK)
        {

            fputs ("WARNING: Couldn't get rotator initial value!\n", stderr);
        }
        else
        {
            if (count != 0)
            {
                CurrentPlantPos = *(double *) value;
                fprintf (stderr, "Initial rotator angle: %f\n",
                         CurrentPlantPos);
            }
        }

        Tk_DoOneEvent (0);

        if (TimeOut)
        {

            fputs
                ("ERROR: Timed out trying to get initial rotator position!\n",
                 stderr);
            break;
        }
    }

    Tk_DeleteTimerHandler (theTimer);

    return (TimeOut);
}

int doRotatorInitialOffset (gwcConnectInfo info)
{
    valueType type;
    void *value;
    int count = 0;

    Tk_TimerToken theTimer;

    TimeOut = 0;

    theTimer = Tk_CreateTimerHandler (MAX_ROTATION_TIME,
                                      (void (*)()) rotatorTimeoutFunction,
                                      (ClientData *) NULL);

    while (count == 0)
    {

        if (gwcGetAttrValue (info,
                             "tcs.mnir.offset", &type, &count,
                             &value) != TCL_OK)
        {

            fputs ("WARNING: Couldn't get rotator initial value!\n", stderr);
        }
        else
        {
            if (count != 0)
            {
                CurrentRotatorOffset = *(double *) value;
                fprintf (stderr, "Initial rotator offset: %f\n",
                         CurrentRotatorOffset);
            }
        }

        Tk_DoOneEvent (0);

        if (TimeOut)
        {

            fputs ("ERROR: Timed out trying to get initial rotator offset!\n",
                   stderr);
            break;
        }
    }

    Tk_DeleteTimerHandler (theTimer);

    return (TimeOut);
}

int doRotatorInitialEncoderstate (gwcConnectInfo info)
{
    valueType type;
    void *value;
    int count = 0;

    Tk_TimerToken theTimer;

    TimeOut = 0;

    theTimer = Tk_CreateTimerHandler (5000,
                                      (void (*)()) rotatorTimeoutFunction,
                                      (ClientData *) NULL);

    while (count == 0)
    {

        if (gwcGetAttrValue (info,
                             "tcs.mnir.encoderstate", &type, &count,
                             &value) != TCL_OK)
        {

            fputs ("WARNING: Couldn't get rotator initial encoder state!\n",
                   stderr);
        }
        else
        {
            if (count != 0)
            {
                strcpy (CurrentRotatorEncoderstate, *(char **) value);
                fprintf (stderr, "Initial rotator encoder state: %s\n",
                         CurrentRotatorEncoderstate);
            }
        }

        Tk_DoOneEvent (0);

        if (TimeOut)
        {

            fputs
                ("ERROR: Timed out trying to get initial rotator encoder state!\n",
                 stderr);
            break;
        }
    }

    Tk_DeleteTimerHandler (theTimer);

    return (TimeOut);
}

int doRotatorInitialAmpStatus (gwcConnectInfo info)
{
    valueType type;
    void *value;
    int count = 0;

    Tk_TimerToken theTimer;

    TimeOut = 0;

    theTimer = Tk_CreateTimerHandler (5000,
                                      (void (*)()) rotatorTimeoutFunction,
                                      (ClientData *) NULL);

    while (count == 0)
    {

        if (gwcGetAttrValue (info,
                             "dcs.mnir.ampcmd", &type, &count,
                             &value) != TCL_OK)
        {

            fputs ("WARNING: Couldn't get rotator initial Amp status!\n",
                   stderr);
        }
        else
        {
            if (count != 0)
            {
                CurrentRotatorAmpStatus = *(int *) value;
                fprintf (stderr, "Initial rotator encoder state: %d\n",
                         CurrentRotatorAmpStatus);
            }
        }

        Tk_DoOneEvent (0);

        if (TimeOut)
        {

            fputs
                ("ERROR: Timed out trying to get initial rotator Amp status!\n",
                 stderr);
            break;
        }
    }

    Tk_DeleteTimerHandler (theTimer);

    return (TimeOut);
}

static int offsetUpdate (gwcEventArgs args)
{
    double *value;

    value = (double *) args.value;

    /* fputs( "offset Update \n", stderr ); */

    /* sprintf ( temp, "%8.3f", ( value[0] / DEG2RAD) ); */

    CurrentRotatorOffset = (double) value[0];

    return TCL_OK;
}

static void rotatorTimeoutFunction (void)
{
    TimeOut = 1;
}
