
/****************************************************

		concenIO.c
   
   These functions Open and load the concentricity file. 
   eccentricity structure.

   Functions in this file include:

   *  FILE *eccenOpen()
   *  eccenLoad()
   *  eccenCalcVec()

   ( * indicates that function is declared as an extern )

****************************************************/

#include <setjmp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "errors.h"
#include "death.h"
#include "miscMath.h"
#include "motor.h"
#include "kaboom.h"
#include "plate.h"
#include "translate.h"
#include "flex.h"
#include "buttons.h"
#include "fiborg.h"
#include "nesslib.h"
#include "findIt.h"
#include "xref.h"
#include "concen.h"
#include "log.h"

/*****   globals *****/
char buf[256];
char CurrentCable;

/*****   end globals *****/

/*****   opaque structures *****/

struct cableInfo
{

    char cableID;
    char cableLabelType;
    char cableName[16];
    char selectable;
    int maxSlitValue;
};

static struct cableInfo *CableList[32]; /* should be plenty */

char *ConcenEnvItems[] = {      /* environment items with paths in them */
    "FIBINITS",
    "PWD",                      /* this one is for concentricities */
    "HOME",
    "PATH",
    NULL
};

/*****   opaque functions *****/

static void parseCableEntry (char *theLine);
static void parseFiberEntry (char *theLine, struct fiber_dbase theDbase[]);
static void registerSlitWithCable (char cable, int slitPosition);

/*****   end of opaque *****/

/*
      eccenOpen

   open an eccentricity file
   with the name pointed to by "name".  Return the
   file pointer or NULL on failure.
*/

FILE *eccenOpen (char *fileName)
{
    char *archivePath;

    archivePath = findFile (fileName, ConcenEnvItems);

    if (archivePath == NULL)
    {
        fprintf (stdout, "ERROR: couldn't find file: \"%s\"\n", fileName);
        return ((FILE *) NULL);
    }

    return (fopen (archivePath, "r"));
}

/*
      eccenLoad
   
   Restore the internal database entries for all fiber
   eccentricities from the contents of the archive. 
*/

void eccenLoad (FILE * theArchive, struct fiber_dbase theDbase[])
{
    char theLine[128];

    if (theArchive == NULL)
    {
#ifdef LOG
        log_fputs
            ("ERROR:  Attempt to load from null concentricities file.\n",
             stderr);
#else
        fputs ("ERROR:  Attempt to load from null concentricities file.\n",
               stderr);
#endif
        return;
    }

    /* clear the cable list --doesn't bother to dealloc previous one... */

    CableList[0] = NULL;

    rewind (theArchive);

    while (fgets (theLine, 128, theArchive) != NULL)
    {

        /* handle empty lines */

        if (theLine[0] == '\n')
            continue;

        /* handle comments */

        if (theLine[0] == '#')
        {
            if (theLine[1] == '#')
            {
#ifdef LOG
                log_fputs (theLine, stderr);
#endif
            }
            continue;
        }

        /* handle real data */

        if (isalpha (theLine[0]))
        {
            parseCableEntry (theLine);
        }
        else
        {
            parseFiberEntry (theLine, theDbase);
        }
    }

    fclose (theArchive);
}

static void parseCableEntry (char *theLine)
{
    int i = 0;

    /* get to the first empty entry */

    while (CableList[i] != NULL)
        i++;

    CableList[i] = (struct cableInfo *) malloc (sizeof (struct cableInfo));

    CableList[i]->maxSlitValue = 0;

    CableList[i + 1] = NULL;

    if (sscanf (theLine, "%1c %1c %1c %s",
                &(CableList[i]->cableID),
                &(CableList[i]->cableLabelType),
                &(CableList[i]->selectable), CableList[i]->cableName) < 4)
    {
#ifdef LOG
        sprintf (buf,
                 "WARNING: Bad entry in concentricities file! (Cable descr#)--\n	\"%s\"",
                 theLine);
        log_fputs (buf, stderr);
#else
        fprintf (stderr,
                 "WARNING: Bad entry in concentricities file! (Cable descr#)--\n	\"%s\"",
                 theLine);
#endif
        /* clear this entry to avoid trouble */
        CableList[i] = NULL;
        return;
    }
}

static void parseFiberEntry (char *theLine, struct fiber_dbase theDbase[])
{
    int scanCount;

    int fiberID;
    int slitID;
    char cableID[2];
    char statusChar[2];
    struct eccentric_pair eccen;

    struct fiber_dbase *fib;

    scanCount = sscanf (theLine,
                        "%d%d%1s%1s%lf%lf",
                        &fiberID,
                        &slitID,
                        cableID, statusChar, &eccen.mag, &eccen.angle);

    /* check for correct number of items */

    if (scanCount != 6)
    {
#ifdef LOG
        sprintf (buf,
                 "WARNING: Bad line (field count) in concentricities file! --\n	\"%s\"",
                 theLine);
        log_fputs (buf, stderr);
#else
        fprintf (stderr,
                 "WARNING: Bad line (field count) in concentricities file! --\n	\"%s\"",
                 theLine);
#endif
        return;
    }
    else
    {
        fib = &theDbase[fiberID];
    }

    /* be sure fiber entry is in range before we start using pointer to it
       to store variables ! */

    if ((fiberID < 0) || (fiberID >= MAXFIBER))
    {
#ifdef LOG
        sprintf (buf,
                 "WARNING: Bad entry in concentricities file! (fiber#)--\n	\"%s\"",
                 theLine);
        log_fputs (buf, stderr);
#else
        fprintf (stderr,
                 "WARNING: Bad entry in concentricities file! (fiber#)--\n	\"%s\"",
                 theLine);
#endif
        return;
    }
    else
    {
        fib = &theDbase[fiberID];
    }

    /* link up the concentricity information */

    fib->Eccentricity = eccen;  /* structure copy */

    /* now verify and load  slit position */

    if (slitID < 0)
    {
#ifdef LOG
        sprintf (buf,
                 "WARNING: Bad entry in concentricities file! (slit#)--\n	\"%s\"",
                 theLine);
        log_fputs (buf, stderr);
#else
        fprintf (stderr,
                 "WARNING: Bad entry in concentricities file! (slit#)--\n	\"%s\"",
                 theLine);
#endif
        return;
    }
    else
    {
        fib->slitPosition = slitID;
    }

    /* now make use of the various flags in the line */

    switch (statusChar[0])
    {

        case 'A':              /* fiber is Active */

            /* setting these is redundant, but safe */

            fib->virtual = FALSE;
            fib->locked = FALSE;
            fib->broken = FALSE;
            break;

        case 'B':              /* fiber is broken */

            fib->broken = TRUE;
            break;

        case 'L':              /* fiber is locked */

            fib->locked = TRUE;
            break;

        case 'Z':              /* fiber is nonexistent */

            fib->virtual = TRUE;
            break;

        default:
#ifdef LOG
            sprintf (buf,
                     "WARNING: unknown status character in concentricities file!--\n	\"%s\"",
                     theLine);
            log_fputs (buf, stderr);
#else
            fprintf (stderr,
                     "WARNING: unknown status character in concentricities file!--\n	\"%s\"",
                     theLine);
#endif
            break;
    }

    /* now make use of the various flags in the line */

    if (cableIsValid (cableID[0]))
    {

        fib->cable = cableID[0];

        registerSlitWithCable (fib->cable, fib->slitPosition);
    }
    else
    {
#ifdef LOG
        sprintf (buf,
                 "WARNING: unknown cable character in concentricities file!--\n	\"%s\"",
                 theLine);
        log_fputs (buf, stderr);
#else
        fprintf (stderr,
                 "WARNING: unknown cable character in concentricities file!--\n	\"%s\"",
                 theLine);
#endif
    }

}

char getCableIdentifier (char *targetCable)
{
    int i = 0;

    while (CableList[i] != NULL)
    {

        if (strncasecmp
            (targetCable, CableList[i]->cableName,
             sizeof (CableList[i]->cableName) - 1) == 0)
            return (CableList[i]->cableID);

        i++;
    }

    return -99;
}

char *getNameForCable (char targetCable)
{
    int i = 0;

    while (CableList[i] != NULL)
    {

        if (CableList[i]->cableID == toupper (targetCable))
        {
            return (CableList[i]->cableName);
        }

        i++;
    }

    strcpy (buf, "UNKNOWN");
    return buf;
}

int cableIsValid (char targetCable)
{
    int i = 0;

    while (CableList[i] != NULL)
    {

        if (CableList[i]->cableID == targetCable)
        {
            return (TRUE);
        }

        i++;
    }

    return (FALSE);
}

char *getCableList (void)
{
    static char theList[255];
    int i = 0;

    /* clear the list first */

    theList[0] = '\0';

    while (CableList[i] != NULL)
    {

        if (CableList[i]->selectable == 'Y')
        {
            strcat (theList, CableList[i]->cableName);
            strcat (theList, ", ");
        }

        i++;
    }

    /* clean up the end... */

    theList[strlen (theList) - 2] = '\0';

    return (theList);
}

int cableIsAlphaLabeled (char targetCable)
{
    int i = 0;

    while (CableList[i] != NULL)
    {

        if (CableList[i]->cableID == targetCable)
        {
            return (CableList[i]->cableLabelType == CABLE_ALPHA);
        }

        i++;
    }
    return (FALSE);
}

void registerSlitWithCable (char cable, int slitPosition)
{

    int i = 0;

    /* clear the list first */

    while (CableList[i] != NULL)
    {

        if (CableList[i]->cableID == cable)
        {

            if (slitPosition > CableList[i]->maxSlitValue)
            {
                CableList[i]->maxSlitValue = slitPosition;
            }

            break;
        }
        i++;
    }
}

int getMaxSlitInCable (char cable)
{

    int i = 0;

    /* clear the list first */

    while (CableList[i] != NULL)
    {

        if (CableList[i]->cableID == cable)
        {

            return (CableList[i]->maxSlitValue);
        }

        i++;
    }

    return (0);
}
