
/********************************************************************

		f77toC

   These functions provide an interface between the C based
   collision routines and a specified set of fortran calls.

********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "miscMath.h"
#include "motor.h"
#include "flex.h"
#include "buttons.h"
#include "kaboom.h"
#include "nesslib.h"
#include "coordConvert.h"
#include "concen.h"
#include "xref.h"

#define CONCENTRICITIES_FILE "concentricities"

/*
		libinit

   Make a set of calls to set up the database for future use.
*/

void libinit_ (void)
{
    /* load the button concentricities */
    eccenLoad (eccenOpen (CONCENTRICITIES_FILE), FiBase);

    initDbase ();

    initRadiusList ();
}

void rotmode_ (int *flag)
{
    Rotflag = *flag;
}

void concenmode_ (int *flag)
{
    EccenFlag = *flag;
}

void rotangle_ (double *angle)
{
    RotationAngle = *angle;
}

/*
		setfib

   if a location is in range for a fiber, set the fiber's entry
   in the database to that location.  If the location is bad,
   return a code that gives the error
*/

void setfib_ (int *button, int *x, int *y, int *istat)
{
    struct fiber_dbase *fib = &FiBase[*button];
    struct aPoint target;
    struct flex_geometry *tmpGeo;

    int status;

    target.x = *x;
    target.y = *y;

    *istat = 0;

    skyToButton (fib, &target, &target);

    prepNewLocation (fib, &target);

    /* save a link to the existing geometry */

    tmpGeo = fib->CurGeometry;

    /* set the current Geometry to the target */

    fib->CurGeometry = fib->TrialGeometry;

    /* see if the target is allowable for this fiber */

    status = coordStatus (fib, NO_RESTRICT);

    /* put back the current geometry */

    fib->CurGeometry = tmpGeo;

    if (status == 0)
    {
        recordNewLocation (fib, 0 /* don't try to archive it */ );
    }

    *istat = (int) status;
}

/*
		setfibphys

   if a location is in range for a fiber, set the fiber's entry
   in the database to that location.  If the location is bad,
   return a code that gives the error
*/

void setfibphys_ (int *button, int *x, int *y, int *istat)
{
    struct fiber_dbase *fib = &FiBase[*button];
    struct aPoint target;
    struct flex_geometry *tmpGeo;

    int status;

    target.x = *x;
    target.y = *y;

    *istat = 0;

    prepNewLocation (fib, &target);

    /* save a link to the existing geometry */

    tmpGeo = fib->CurGeometry;

    /* set the current Geometry to the target */

    fib->CurGeometry = fib->TrialGeometry;

    /* see if the target is allowable for this fiber */

    status = coordStatus (fib, NO_RESTRICT);

    /* put back the current geometry */

    fib->CurGeometry = tmpGeo;

    if (status == 0)
    {
        recordNewLocation (fib, 0 /* don't try to archive it */ );
    }

    *istat = (int) status;
}

/*
		checkfib

   See if a location is in range for a fiber.
   If the location is bad, return a code that gives the error
*/

void checkfib_ (int *button, int *x, int *y, int *istat)
{
    struct fiber_dbase *fib = &FiBase[*button];
    struct aPoint target;
    struct flex_geometry *tmpGeo;

    target.x = *x;
    target.y = *y;

    *istat = 0;

    /* fprintf( stderr, "%f %f", target.x, target.y ); */

    skyToButton (fib, &target, &target);

    /* fprintf( stderr, "   %f %f\n", target.x, target.y ); */

    prepNewLocation (fib, &target);

    /* save a link to the existing geometry */

    tmpGeo = fib->CurGeometry;

    /* set the current Geometry to the target */

    fib->CurGeometry = fib->TrialGeometry;

    /* see if the target is allowable for this fiber */

    *istat = (int) coordStatus (fib, RESTRICT_TO_PLATE);

    fib->CurGeometry = tmpGeo;

}

/*
		checkfibphys

   See if a location is in range for a fiber.
   If the location is bad, return a code that gives the error
*/

void checkfibphys_ (int *button, int *x, int *y, int *istat)
{
    struct fiber_dbase *fib = &FiBase[*button];
    struct aPoint target;
    struct flex_geometry *tmpGeo;

    target.x = *x;
    target.y = *y;

    *istat = 0;

    prepNewLocation (fib, &target);

    /* save a link to the existing geometry */

    tmpGeo = fib->CurGeometry;

    /* set the current Geometry to the target */

    fib->CurGeometry = fib->TrialGeometry;

    /* see if the target is allowable for this fiber */

    *istat = (int) coordStatus (fib, RESTRICT_TO_PLATE);

    fib->CurGeometry = tmpGeo;

}

/*
		park

   Set the location of the specified fiber to either the park
   or stowed locations, depending on the flag.
*/

void park_ (int *button, int *flag)
{
    struct fiber_dbase *fib = &FiBase[*button];
    struct plate_coords *target;

    if (*flag != 0)
    {
        target = &fib->StowPos;
        setLocation (fib, target, 0 /* don't archive */ );
        fib->parkedStatus = STOWED;
    }
    else
    {
        target = &fib->ParkPos;
        setLocation (fib, target, 0 /* don't archive */ );
        fib->parkedStatus = PARKED;
    }

}

/*
		interact

   See if two fibers touch.
*/

int interract_ (int *buttona, int *buttonb)
{
    int status;

    struct fiber_dbase *fib1 = &FiBase[*buttona];
    struct fiber_dbase *fib2 = &FiBase[*buttonb];

    status = pairHits (fib1, fib2);

/*
   fprintf( stderr, "status = %d\n", status );
*/

    return (status != 0);

}

int parkstatus_ (int *fib)
{
    return (FiBase[*fib].parkedStatus);
}

void whereis_ (int *button, int *x1, int *y1, int *x2, int *y2, int *x3,
               int *y3)
{
    struct fiber_dbase *buttonPtr = &FiBase[*button];

    *x1 = (int) buttonPtr->CurPos.x;
    *y1 = (int) buttonPtr->CurPos.y;

    *x2 = (int) buttonPtr->CurGeometry->button.x;
    *y2 = (int) buttonPtr->CurGeometry->button.y;

    *x3 = (int) buttonPtr->TrialGeometry->button.x;
    *y3 = (int) buttonPtr->TrialGeometry->button.y;
}

void whereskyphys (int *button, int *x1, int *y1, int *x2, int *y2)
{
    struct aPoint skyPos;

    struct fiber_dbase *buttonPtr = &FiBase[*button];

    *x1 = (int) buttonPtr->CurPos.x;
    *y1 = (int) buttonPtr->CurPos.y;

    buttonToSky (buttonPtr, &skyPos);

    *x2 = (int) skyPos.x;
    *y2 = (int) skyPos.y;

}

/*
	ibuttonhitsanother

   See if the specified button interacts with
   any of the others.
*/

int ibuttonhitsanother_ (int *buttonid)
{
    struct fiber_dbase *button;
    struct fiber_dbase *targetButton;
    int statusCode = 0;         /* non-zero means there is a problem. */
    int i;

    button = &FiBase[*buttonid];

    /* loop through all other fibers and check for collisions */

    for (i = 0; ((i < MAXFIBER) && (statusCode == 0)); i++)
    {

        if (i != button->fiber_id)
        {                       /* don't do current fiber! */

            targetButton = &FiBase[i];

            statusCode = pairHits (button, targetButton);
        }
    }

#ifdef DEBUG
    if (statusCode != 0)
        fprintf (stderr, "%d hits %d", button->fiber_id,
                 targetButton->fiber_id);
#endif
    return (statusCode);
}

void labtyd (int error, char *msg)
{
    fprintf (stderr, "KABOOMLIB FATAL ERROR: %s\n", msg);
    exit (error);
}

/* Added [d][a]{sin cos}d functions
 * These were initially added to make whydra.f work with g77,
 * which does not have the degree version of the trig functins.
 * However, the functions below, though seemingly producing
 * correct results in tests, made whydra.f produce funny results
 * for certain .ast files. Therefore, all references to these
 * functions within whydra.f were replaced with their arc
 * counterpart, and the necessary unit conversions were taken
 * care of within whydra.f. This fixed the problem with whydra.f
 * being compiled with g77. Hence why the functions below 
 * were deactivated.
 * Behzad 20 Aug 2003.
 */

/*
float sind_ (float d)
{
	return (float) (sin (M_PI * d / 180.0));
}

float cosd_ (float d)
{
	return (float) (cos (M_PI * d / 180.0));
}

float asind_ (float x)
{
	if (x < -1.0 || x > 1.0)
	{
		fprintf (stderr, "ERROR (kaboom): asind\n");
		exit (-1);
	}

	return (float) (asin (x) * 180.0 / M_PI);
}

float acosd_ (float x)
{
	if (x < -1.0 || x > 1.0)
	{
		fprintf (stderr, "ERROR (kaboom): acosd\n");
		exit (-1);
	}

	return (float) (acos (x) * 180.0 / M_PI);
}
*/

/*
double sind_ (double d)
{
	return dsind_ (d);
}

double cosd_ (double d)
{
	return dcosd_ (d);
}

double asind_ (double x)
{
	return dasind_ (x);
}

double acosd_ (double x)
{
	return dacosd_ (x);
}

double dsind_ (double d)
{
	return sin (M_PI * d / 180.0);
}

double dcosd_ (double d)
{
	return cos (M_PI * d / 180.0);
}

double dasind_ (double x)
{
	if (x < -1.0 || x > 1.0)
	{
		fprintf (stderr, "ERROR (kaboom): dasind\n");
		exit (-1);
	}

	return asin (x) * 180.0 / M_PI;
}

double dacosd_ (double x)
{
	if (x < -1.0 || x > 1.0)
	{
		fprintf (stderr, "ERROR (kaboom): dacosd\n");
		exit (-1);
	}

	return acos (x) * 180.0 / M_PI;
}
*/
