
/**
 *
 *     - - - - - -
 *      a d d e t
 *     - - - - - -
 *
 *
 *  Add the E-terms (elliptic component of annual aberration)
 *  to a pre IAU 1976 mean place to conform to the old
 *  catalogue convention
 *
 *  (double precision)
 *
 *
 *  given:
 *     rm,dm     dp     ra,dec (radians) without e-terms
 *     eq        dp     besselian epoch of mean equator and equinox
 *
 *  returned:
 *     rc,dc     dp     ra,dec (radians) with e-terms included
 *
 *  called:
 *     sla_etrms, sla_dcs2c, sla_dcc2s, sla_dranrm, sla_drange
 *
 *
 *  explanation:
 *     Most star positions from pre-1984 optical catalogues (or
 *     derived from astrometry using such stars) embody the
 *     E-terms.  If it is necessary to convert a formal mean
 *     place (for example a pulsar timing position) to one
 *     consistent with such a star catalogue, then the ra,dec
 *     should be adjusted using this routine.
 *
 *
 *  reference:
 *     Explanatory Supplement to the Astronomical Ephemeris,
 *     section 2D, page 48.
 *
 *
 *  p.t.wallace   starlink   july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_etrms (double ep, double *ev);
extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_addet (double rm, double dm, double eq, double *rc, double *dc)
{

    double sla_dranrm (double angle);
    double a[3], v[3];
    int i;

    /* E-terms vector */
    sla_etrms (eq, a);

    /* spherical to cartesian */
    sla_dcs2c (rm, dm, v);

    /* include the E-terms */
    for (i = 0; i <= 2; i++)
    {
        v[i] += a[i];
    }

    /* cartesian to spherical */
    sla_dcc2s (v, rc, dc);

    /* bring ra into conventional range */
    *rc = sla_dranrm (*rc);
}
