#include <math.h>
double sla_dranrm (double angle);

/**
 *
 *     - - - -
 *      a m p
 *     - - - -
 *
 *
 *  convert star ra,dec from geocentric apparent to mean place
 *
 *  (double precision)
 *
 *
 *  The mean coordinate system is the IAU 1976 system
 *  (loosely FK5).
 *
 *
 *  references:
 *     1984 astronomical almanac, pp b39-b41.
 *     (also Lederle & Schwan, Astron. Astrophys. 134,
 *      1-6, 1984)
 *
 *
 *  given:
 *
 *     ra,da    dp     apparent ra,dec (rad)
 *     date     dp     tdb for apparent place (jd-2400000.5)
 *     eq       dp     equinox:  julian epoch of mean place
 *
 *
 *  returned:
 *
 *     rm,dm    dp     mean ra,dec (rad)
 *
 *
 *  called:
 *
 *     sla_evp         earth position & velocity
 *     sla_dcs2c       spherical to cartesian
 *     sla_prenut      precession/nutation matrix
 *     sla_dimxv       inverse of matrix x vector
 *     sla_dvn         normalise vector
 *     sla_dvdv        dot product
 *     sla_dcc2s       cartesian to spherical
 *     sla_dranrm      normalise angle 0-2pi
 *
 *
 *  notes:
 *
 *     1)  The distinction between the required TDB and TDT is
 *         always negligible.  Moreover, for all but the most
 *         critical applications UTC is adequate.
 *
 *     2)  The accuracy is limited by the routine sla_evp (qv),
 *         which computes the earth positions and velocities using
 *         the methods of Stumpff.  The maximum error is about
 *         0.3 milliarcsecond.
 *
 *     3)  Iterative techniques are used for the aberration and light
 *         deflection corrections so that the routines sla_amp and
 *         sla_map are exact inverses (to vax double precision
 *         accuracy and up to 15 arcminutes from the centre of
 *         the sun).
 *
 *
 *  p.t.wallace   starlink   august 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_evp (double date, double deqx, double *dvb, double *dpb,
                    double *dvh, double *dph);
extern int sla_dcs2c (double a, double b, double *v);
extern int sla_prenut (double epoch, double date, double (*rmatpn)[3]);
extern int sla_dimxv (double (*dm)[3], double *va, double *vb);
extern int sla_dvn (double *v, double *uv, double *vm);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_amp (double ra, double da, double date, double eq, double *rm, double *dm)
{

    /* gravitational radius of the sun x 2 (2*mu/c**2, au) */
#define gr2  (double)(2e0*9.87063e-9)

    /* light time for 1 au (sec) */
#define cr  (double)(499.004782e0)

    int i, j;
    double sla_dvdv (double *va, double *vb), sla_dranrm (double angle);
    double ebd[3], eb[3], ehd[3], eh[3];
    double p3[3], pnmat[3][3], p2[3];
    double ehn[3], e;
    double v[3], vn[3], vm, br, brp1;
    double p1dv, p1dvp1, p1[3], w;
    double gr2e, pde, pdep1, p[3];

    /* get earth barycentric and heliocentric position and velocity */
    sla_evp (date, eq, ebd, eb, ehd, eh);

    /* geocentric apparent ra,dec */
    sla_dcs2c (ra, da, p3);

    /* precession and nutation */
    sla_prenut (eq, date, pnmat);
    sla_dimxv (pnmat, p3, p2);

    /* heliocentric direction of earth (normalised) and modulus */
    sla_dvn (eh, ehn, &e);

    /* aberration */
    for (i = 0; i <= 2; i++)
    {
        v[i] = ebd[i] * cr;
    }
    sla_dvn (v, vn, &vm);
    br = sqrt (1e0 - vm * vm);
    brp1 = br + 1e0;
    for (i = 0; i <= 2; i++)
    {
        p1[i] = p2[i];
    }
    for (j = 0; j <= 1; j++)
    {
        p1dv = sla_dvdv (p1, v);
        p1dvp1 = p1dv + 1e0;
        for (i = 0; i <= 2; i++)
        {
            p1[i] = (br * p2[i] - v[i] - p1dv * v[i] / brp1) / p1dvp1;
        }
        sla_dvn (p1, p3, &w);
        for (i = 0; i <= 2; i++)
        {
            p1[i] = p3[i];
        }
    }

    /* light deflection */
    gr2e = gr2 / e;
    for (i = 0; i <= 2; i++)
    {
        p[i] = p1[i];
    }
    for (j = 0; j <= 5; j++)
    {
        pde = sla_dvdv (p, ehn);
        pdep1 = 1e0 + pde;
        for (i = 0; i <= 2; i++)
        {
            p[i] = p1[i] - gr2e * (ehn[i] - pde * p[i]) /
                (pdep1 < 1e-6 ? 1e-6 : pdep1);
        }
        sla_dvn (p, p2, &w);
        for (i = 0; i <= 2; i++)
        {
            p[i] = p2[i];
        }
    }

    /* mean ra,dec */
    sla_dcc2s (p, rm, dm);
    *rm = sla_dranrm (*rm);
}
