#include <math.h>

/**
 *
 *     - - - - - -
 *      d j c a l
 *     - - - - - -
 *
 *  modified julian date to gregorian calendar, expressed
 *  in a form convenient for formatting messages
 *
 *  given:
 *     ndp      int    number of decimal places of days in fraction
 *     djm      dp     modified julian date (JD-2400000.5)
 *
 *  returned:
 *     iymdf    int    year, month, day, fraction in gregorian
 *                     calendar
 *     j        int    status:  nonzero = out of range
 *
 *  Any date after 4701bc march 1 is accepted.
 *
 *  The algorithm is derived from that of Hatcher 1984
 *  (qjras 25, 53-55).
 *
 *  p.t.wallace   starlink   28 october 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_djcal (int ndp, double djm, int *iymdf, int *j)
{

    int nfd;
    double fd, df, f, d;
    int jd, n4, nd10;

    /* validate */
    if (djm <= (-2395520e0) || djm >= 1e9)
    {
        *j = 1;
    }
    else
    {

        /* denominator of fraction */
        /* nfd = 10 **  max(ndp, 0) */
        nfd = 1;
        while (ndp-- > 0)
            nfd *= 10;
        fd = (double) nfd;

        /* round date and express in units of fraction */
        df = rint ((double) djm * fd);

        /* separate day and fraction */
        /* f = mod(df, fd) */
        f = modf ((df / fd), &d) * fd;
        if (f < 0e0)
            f = f + fd;
        d = (df - f) / fd;

        /* express day in gregorian calendar */
        jd = (int) rint (d) + 2400001;
        n4 = 4 *
            (jd + ((2 * ((4 * jd - 17918) / 146097) * 3) / 4 + 1) / 2 - 37);
        nd10 = 10 * (((n4 - 237) % 1461) / 4) + 5;
        iymdf[0] = n4 / 1461 - 4712;
        iymdf[1] = ((nd10 / 306 + 2) % 12) + 1;
        iymdf[2] = (nd10 % 306) / 10 + 1;
        iymdf[3] = (int) rint (f);
        *j = 0;
    }
}
