#include <math.h>

/**
 *
 *     - - - - - -
 *      e a r t h
 *     - - - - - -
 *
 *
 *  approximate heliocentric position and velocity of the earth
 *
 *
 *  given:
 *     iy       i       year ad
 *     id       i       day in year (1 = jan 1st)
 *     fd       r       fraction of day
 *
 *  returned:
 *     posvel   r(6)    earth position & velocity vector
 *
 *
 *  The date and time is TDB (loosely ET) in the Gregorian
 *  calendar, and is interpreted in a manner which is valid
 *  between 1900 March 1 and 2100 February 28.
 *
 *  The earth heliocentric 6-vector is mean equator and equinox
 *  of date.  Position part, posvel(0-2), is in AU;  velocity part,
 *  posvel(3-5), is in AU/sec.
 *
 *  max/rms errors 1950-2050:
 *     13/5 e-5 AU = 19200/7600 km in position
 *     47/26 e-10 AU/s = 0.0070/0.0039 km/s in speed
 *
 *  More precise results are obtainable with the routine sla_evp.
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_earth (int iy, int id, double fd, double *posvel)
                                        /* variables were REAL */
{

    /* variables were REAL */
    double yi, yf, t, elm, gamma;
    double em, elt, eps0, e, esq, v, r, elmm;
    double coselt, sineps, coseps, w1, w2, selmm, celmm;
    double junk;

#define twopi  (double)6.28318530718

    /* mean orbital speed of earth, au/s */
#define speed  (double)1.9913e-7

    /* mean earth:emb distance and speed, au and au/s */
#define remb  (double)3.12e-5
#define semb  (double)8.31e-11

    /* whole years & fraction of year, and years since j1900.0 */
    yi = (double) (iy - 1900);
    yf = ((double) (4 * (id - 1 / ((iy % 4) + 1)) - (iy % 4) - 2)
          + 4.0 * fd) / 1461.0;
    t = yi + yf;

    /* geometric mean longitude of sun */
    /* (cf 4.881627938+6.283319509911*t mod 2pi) */
    elm = modf ((4.881628 + twopi * yf + 0.00013420 * t) / twopi, &junk)
        * twopi;

    /* mean longitude of perihelion */
    gamma = 4.908230 + 3.0005e-4 * t;

    /* mean anomaly */
    em = elm - gamma;

    /* mean obliquity */
    eps0 = 0.40931975 - 2.27e-6 * t;

    /* eccentricity */
    e = 0.016751 - 4.2e-7 * t;
    esq = e * e;

    /* true anomaly */
    v = em + 2.0 * e * sin (em) + 1.25 * esq * sin (2.0 * em);

    /* true ecliptic longitude */
    elt = v + gamma;

    /* true distance */
    r = (1.0 - esq) / (1.0 + e * cos (v));

    /* moon's mean longitude */
    elmm = modf ((4.72 + 83.9971 * t) / twopi, &junk) * twopi;

    /* useful functions */
    coselt = cos (elt);
    sineps = sin (eps0);
    coseps = cos (eps0);
    w1 = -r * sin (elt);
    w2 = -speed * (coselt + e * cos (gamma));
    selmm = sin (elmm);
    celmm = cos (elmm);

    /* earth position and velocity */
    posvel[0] = -r * coselt - remb * celmm;
    posvel[1] = (w1 - remb * selmm) * coseps;
    posvel[2] = w1 * sineps;
    posvel[3] = speed * (sin (elt) + e * sin (gamma)) + semb * selmm;
    posvel[4] = (w2 - semb * celmm) * coseps;
    posvel[5] = w2 * sineps;
}
