#include <math.h>

/**
 *
 *     - - - - - -
 *      e c m a t
 *     - - - - - -
 *
 *
 *  form the equatorial to ecliptic rotation matrix (IAU 1980 theory)
 *
 *  (double precision)
 *
 *
 *  given:
 *     date     dp         TDB (loosely ET) as modified julian date
 *                                            (JD-2400000.5)
 *  returned:
 *     rmat     dp(3,3)    matrix
 *
 *  references:
 *     Murray,C.a., Vectorial Astrometry, section 4.3.
 *
 *  note:
 *    The matrix is in the sense   v(ecl)  =  rmat * v(equ);  the
 *    equator, equinox and ecliptic are mean of date.
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_ecmat (double date, double (*rmat)[3])
{

    /* arc seconds to radians */
#define as2r  (double)0.4848136811095359949e-05

    double t, eps0, s, c;

    /* interval between basic epoch j2000.0 and current epoch (jc) */
    t = (date - 51544.5e0) / 36525e0;

    /* mean obliquity */
    eps0 =
        as2r * (84381.448 + (-46.8150 + (-0.00059 + 0.001813 * t) * t) * t);

    /* matrix */
    s = sin (eps0);
    c = cos (eps0);
    rmat[0][0] = 1e0;
    rmat[1][0] = 0e0;
    rmat[2][0] = 0e0;
    rmat[0][1] = 0e0;
    rmat[1][1] = c;
    rmat[2][1] = -s;
    rmat[0][2] = 0e0;
    rmat[1][2] = s;
    rmat[2][2] = c;
}
