
/**
 *
 *     - - - - -
 *      e c o r
 *     - - - - -
 *
 *
 *  component of earth orbit velocity and heliocentric
 *  light time in a given direction
 *
 *  (single precision)
 *
 *
 *  given:
 *     rm,dm    real    mean ra, dec of date (radians)
 *     iy       int     year ad
 *     id       int     day in year (1 = Jan 1st)
 *     fd       real    fraction of day
 *
 *  returned:
 *     rv       real    component of earth orbital velocity (km/sec)
 *     tl       real    component of heliocentric light time (sec)
 *
 *  The date and time is TDB (loosely ET) in the gregorian
 *  calendar, and is interpreted in a manner which is valid
 *  between 1900 March 1 and 2100 February 28.
 *
 *  sign convention:
 *    The velocity component is +ve when the earth is receding from
 *    the given point on the sky.  The light time component is +ve
 *    when the earth lies between the sun and the given point on
 *    the sky.
 *
 *  accuracy:
 *    The velocity component is usually within 0.004 km/s of the
 *    correct value and is never in error by more than 0.007 km/s.
 *    The error in light time correction is about 0.03s at worst,
 *    but is usually better than 0.01s. For applications requiring
 *    higher accuracy, see the sla_evp routine.
 *
 *  called:  sla_earth, sla_cs2c, sla_vdv
 *
 *
 *  p t wallace   starlink   april 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_earth (int iy, int id, double fd, double *posvel);
extern int sla_cs2c (double a, double b, double *v);

sla_ecor (double rm, double dm, int iy, int id, double fd, double *rv,
          double *tl)
                                        /* variables were REAL */
                                        /* variables were REAL */
{

    double sla_vdv (double *va, double *vb);    /* function was REAL */
    double pv[6], v[3];         /* variables were REAL */

    /* au to km and light sec (1985 almanac) */
#define aukm  (double)1.4959787066e8
#define ausec  (double)499.0047837

    /* sun:earth position & velocity vector */
    sla_earth (iy, id, fd, pv);

    /* star position vector */
    sla_cs2c (rm, dm, v);

    /* velocity component */
    *rv = -aukm * sla_vdv (&pv[3], v);

    /* light time component */
    *tl = ausec * sla_vdv (pv, v);
}
