
/**
 *
 *     - - - - -
 *      e g 5 0
 *     - - - - -
 *
 *
 *  transformation from b1950.0 'FK4' equatorial coordinates to
 *  IAU 1958 galactic coordinates
 *
 *  (double precision)
 *
 *
 *  given:
 *     dr,dd       dp       b1950.0 'FK4' ra,dec
 *
 *  returned:
 *     dl,db       dp       galactic longitude and latitude l2,b2
 *
 *  (all arguments are radians)
 *
 *
 *  called:
 *     sla_dcs2c, sla_dmxv, sla_dcc2s, sla_subet, sla_dranrm, sla_drange
 *
 *
 *  note:
 *     The equatorial coordinates are b1950.0 'FK4'.  Use the
 *     routine sla_eqgal if conversion from j2000.0 coordinates
 *     is required.
 *
 *
 *  reference:
 *     Blaauw et al, Mon.Not.R.Astron.Soc.,121,123 (1960)
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_subet (double rc, double dc, double eq, double *rm,
                      double *dm);
extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dmxv (double (*dm)[3], double *va, double *vb);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_eg50 (double dr, double dd, double *dl, double *db)
{

    double sla_dranrm (double angle), sla_drange (double angle);
    double v1[3], v2[3], r, d;

    /*
     * l2,b2 system of galactic coordinates
     *
     * p = 192.25       ra of galactic north pole (mean b1950.0)
     * q =  62.6        inclination of galactic to mean b1950.0 equator
     * r =  33          longitude of ascending node
     *
     * p,q,r are degrees
     *
     *
     * equatorial to galactic rotation matrix
     *
     * the eulerian angles are p, q, 90-r
     *
     * +cp.cq.sr-sp.cr     +sp.cq.sr+cp.cr     -sq.sr
     *
     * -cp.cq.cr-sp.sr     -sp.cq.cr+cp.sr     +sq.cr
     *
     * +cp.sq              +sp.sq              +cq
     */

    static double rmat[3][3] = {
        -0.066988739415e0, -0.872755765852e0, -0.483538914632,
        0.492728466075e0, -0.450346958020e0, 0.744584633283,
        -0.867600811151e0, -0.188374601723e0, 0.460199784784
    };

    /* remove e-terms */
    sla_subet (dr, dd, 1950e0, &r, &d);

    /* spherical to cartesian */
    sla_dcs2c (r, d, v1);

    /* rotate to galactic */
    sla_dmxv (rmat, v1, v2);

    /* cartesian to spherical */
    sla_dcc2s (v2, dl, db);

    /* express angles in conventional ranges */
    *dl = sla_dranrm (*dl);
    *db = sla_drange (*db);
}
