
/**
 *
 *     - - - - - -
 *      e q g a l
 *     - - - - - -
 *
 *
 *  transformation from j2000.0 equatorial coordinates to
 *  iau 1958 galactic coordinates
 *
 *
 *  (double precision)
 *
 *
 *  given:
 *     dr,dd       dp       j2000.0 ra,dec
 *
 *  returned:
 *     dl,db       dp       galactic longitude and latitude l2,b2
 *
 *  (all arguments are radians)
 *
 *
 *  called:
 *     sla_dcs2c, sla_dmxv, sla_dcc2s, sla_dranrm, sla_drange
 *
 *
 *  note:
 *     The equatorial coordinates are j2000.0.  Use the routine
 *     sla_eg50 if conversion from b1950.0 'FK4' coordinates is
 *     required.
 *
 *
 *  reference:
 *     Blaauw et al, Mon.Not.R.Astron.Soc.,121,123 (1960)
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dmxv (double (*dm)[3], double *va, double *vb);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_eqgal (double dr, double dd, double *dl, double *db)
{

    double sla_dranrm (double angle), sla_drange (double angle);
    double v1[3], v2[3];

    /*
     * l2,b2 system of galactic coordinates
     *
     * p = 192.25       ra of galactic north pole (mean b1950.0)
     * q =  62.6        inclination of galactic to mean b1950.0 equator
     * r =  33          longitude of ascending node
     *
     * p,q,r are degrees
     *
     * equatorial to galactic rotation matrix (j2000.0):
     */
    static double rmat[3][3] = {
        -0.054875539726e0, -0.873437108010e0, -0.483834985808e0,
        0.494109453312e0, -0.444829589425e0, 0.746982251810e0,
        -0.867666135858e0, -0.198076386122e0, 0.455983795705e0
    };

    /* spherical to cartesian */
    sla_dcs2c (dr, dd, v1);

    /* equatorial to galactic */
    sla_dmxv (rmat, v1, v2);

    /* cartesian to spherical */
    sla_dcc2s (v2, dl, db);

    /* express in conventional ranges */
    *dl = sla_dranrm (*dl);
    *db = sla_drange (*db);
}
