#include <math.h>

/**
 *
 *     - - - - - -
 *      e t r m s
 *     - - - - - -
 *
 *  compute the e-terms (elliptic component of annual aberration)
 *  vector
 *
 *  (double precision)
 *
 *  given:
 *     ep      dp      besselian epoch
 *
 *  returned:
 *     ev      dp(3)   E-terms as (dx,dy,dz)
 *
 *
 *  references:
 *
 *     1  The transformation of astrometric catalog systems to the
 *        equinox j2000.0.  Smith, C.A. et al, paper to be submitted
 *        to the Astronomical Journal.  Draft dated 1987 august 26.
 *
 *     2  Transformation of mean star places from FK4 b1950.0 to
 *        FK5 j2000.0 using matrices in 6-space.  Yallop, B.D. et al,
 *        paper to be submitted to the astronomical journal.  Draft
 *        dated 1987 october 8.
 *
 *  Note the use of the j2000 aberration constant (20.49552 arcsec).
 *  This is a reflection of the fact that the E-terms embodied in
 *  existing star catalogues were computed from a variety of
 *  aberration constants.  Rather than adopting one of the old
 *  constants the latest value is used here.
 *
 *
 *  p.t.wallace   starlink   28 october 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_etrms (double ep, double *ev)
{

    /* arcseconds to radians */
#define as2r  (double)0.4848136811095359949e-5
    double t, e, e0, p, ek, cp;

    /* julian centuries since b1950 */
    t = (ep - 1950e0) * 1.00002135903e-2;

    /* eccentricity */
    e = 0.01673011e0 - (0.00004193e0 + 0.000000126e0 * t) * t;

    /* mean obliquity */
    e0 = (84404.836e0 - (46.8495e0 + (0.00319e0 + 0.00181e0 * t) * t) * t)
        * as2r;

    /* mean longitude of perihelion */
    p = (1015489.951e0 + (6190.67e0 + (1.65e0 + 0.012e0 * t) * t) * t) * as2r;

    /* e-terms */
    ek = e * 20.49552e0 * as2r;
    cp = cos (p);
    ev[0] = ek * sin (p);
    ev[1] = -ek * cp * cos (e0);
    ev[2] = -ek * cp * sin (e0);
}
