#include <math.h>

/**
 *
 *     - - - -
 *      e v p
 *     - - - -
 *
 *
 *  barycentric and heliocentric velocity and position of the earth
 *
 *
 *  all arguments are double precision
 *
 *
 *  given:
 *
 *     date          TDB (loosely ET) as a modified julian date
 *                                         (JD-2400000.5)
 *
 *     deqx          Julian epoch (e.g. 2000.0d0) of mean equator and
 *                   equinox of the vectors returned.  If deqx .le. 0d0,
 *                   all vectors are referred to the mean equator and
 *                   equinox (FK5) of date date.
 *
 *
 *  returned (all 3d cartesian vectors):
 *
 *     dvb,dpb       barycentric velocity, position
 *
 *     dvh,dph       heliocentric velocity, position
 *
 *  (units are au/s for velocity and au for position)
 *
 *
 *  called:  sla_epj, sla_prec, sla_dmxv
 *
 *
 *  accuracy:
 *
 *     The maximum deviations from the JPL DE96 ephemeris are as
 *     follows:
 *
 *     barycentric velocity                  42  cm/s
 *     barycentric position           0.000 046  au
 *
 *     heliocentric velocity                 42  cm/s
 *     heliocentric position          0.000 011  au
 *
 *  This routine is adapted from the barvel and barcor
 *  subroutines of P.Stumpff, which are described in
 *  Astron. Astrophys. Suppl. ser. 41, 1-8 (1980).  Most of the
 *  changes are merely cosmetic and do NOT affect the results at
 *  all.  However, some adjustments have been made so as to give
 *  results that refer to the new (IAU 1976 'FK5') equinox
 *  and precession, although the differences these changes make
 *  relative to the results from stumpff's original 'FK4' version
 *  are smaller than the inherent accuracy of the algorithm.  One
 *  minor shortcoming in the original routines that has not been
 *  corrected is that better numerical accuracy could be achieved
 *  if the various polynomial evaluations were nested.  Note also
 *  that one of Stumpff's precession constants differs by 0.001 arcsec
 *  from the value given in the explanatory supplement to the A.E.
 *
 *
 *  p t wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_prec (double ep0, double ep1, double (*rmatp)[3]);
extern int sla_dmxv (double (*dm)[3], double *va, double *vb);

sla_evp (double date, double deqx, double *dvb, double *dpb, double *dvh,
         double *dph)
{

    int ideq, k;

    /* not used 
     * int i, j; 
     * double g, e, param;
     */
    /* float t, tsq; */
    double junk;

    /* following variables were REAL */
    double a, pertl, pertld, pertr, pertrd, cosa, sina, esq;
    double twoe, twog, phi, f;
    double sinf, cosf, phid, psid, pertp, pertpd, tl, sinlm, coslm;
    double sigma, b, plon, pomg, pecc, flatm, flat;

    /* following variables were (and are) double */
    double dt, dtsq, dlocal, dml, deps;
    double dparam, dpsi, d1pdro, drd, drld, dtl, dsinls, dcosls, dxhd;
    double dyhd, dzhd, dxbd, dybd, dzbd, dcosep, dsinep, dyahd, dzahd;
    double dyabd, dzabd, dr, dxh, dyh, dzh, dxb, dyb, dzb, dyah;
    double dzah, dyab, dzab, depj, deqcor;

    /*
     * to simplify initialization in a C program and to avoid
     * having to rearrange the given fortran data arrays, the
     * two dimensional arrays ccsel, ccamps, ccsec, ccampm,
     * dcfel, dcargm, and dcargs have had their dimensions
     * exchanged.  This is OK since none are used except in
     * explicit computations (as opposed to arguments to
     * other routines).  One just has to be sure to change
     * the indexing in the computations too!
     */

    /* following 11 variables were REAL */
    /* double ccsel[3][17], ccamps[5][15], ccsec[3][4], ccampm[4][3]; */
    /* double ccpamv[4], ccpam[4] */
    double forbel[7], sorbel[17];
    double sn[4], sinlp[4], coslp[4];

    /* equivalence[sorbel[1], e],[forbel[1], g]; */
#define e sorbel[0]
#define g forbel[0]

    /* following 5 variables were (and are) double */
    /* double dcfel[3][8], dcargm[2][3], dcargs[2][15], dceps[3]; */
    double dprema[3][3];

    double sla_epj (double date);

#define dc2pi  (double)6.2831853071796e0
#define ds2r  (double)0.7272205216643e-04
    /* cc2pi was real */
#define cc2pi  (double)6.283185

    /*
     * constants dcfel(k,i) of fast changing elements
     *      i=1                i=2              i=3
     */
    static double dcfel[8][3] = {
        1.7400353e+00, 6.2833195099091e+02, 5.2796e-06,
        6.2565836e+00, 6.2830194572674e+02, -2.6180e-06,
        4.7199666e+00, 8.3997091449254e+03, -1.9780e-05,
        1.9636505e-01, 8.4334662911720e+03, -5.6044e-05,
        4.1547339e+00, 5.2993466764997e+01, 5.8845e-06,
        4.6524223e+00, 2.1354275911213e+01, 5.6797e-06,
        4.2620486e+00, 7.5025342197656e+00, 5.5317e-06,
        1.4740694e+00, 3.8377331909193e+00, 5.6093e-06
    };

    /*
     * constants dceps and ccsel(k,i) of slowly changing elements
     *          i=1           i=2           i=3
     */
    static double dceps[3] = {
        4.093198e-01, -2.271110e-04, -2.860401e-08
    };

    static double ccsel[17][3] = {
        1.675104e-02, -4.179579e-05, -1.260516e-07,
        2.220221e-01, 2.809917e-02, 1.852532e-05,
        1.589963e+00, 3.418075e-02, 1.430200e-05,
        2.994089e+00, 2.590824e-02, 4.155840e-06,
        8.155457e-01, 2.486352e-02, 6.836840e-06,
        1.735614e+00, 1.763719e-02, 6.370440e-06,
        1.968564e+00, 1.524020e-02, -2.517152e-06,
        1.282417e+00, 8.703393e-03, 2.289292e-05,
        2.280820e+00, 1.918010e-02, 4.484520e-06,
        4.833473e-02, 1.641773e-04, -4.654200e-07,
        5.589232e-02, -3.455092e-04, -7.388560e-07,
        4.634443e-02, -2.658234e-05, 7.757000e-08,
        8.997041e-03, 6.329728e-06, -1.939256e-09,
        2.284178e-02, -9.941590e-05, 6.787400e-08,
        4.350267e-02, -6.839749e-05, -2.714956e-07,
        1.348204e-02, 1.091504e-05, 6.903760e-07,
        3.106570e-02, -1.665665e-04, -1.590188e-07
    };

    /*
     * constants of the arguments of the short-period perturbations
     * by the planets:   dcargs(k,i)
     *         i=1               i=2
     */
    static double dcargs[15][2] = {
        5.0974222e+00, -7.8604195454652e+02,
        3.9584962e+00, -5.7533848094674e+02,
        1.6338070e+00, -1.1506769618935e+03,
        2.5487111e+00, -3.9302097727326e+02,
        4.9255514e+00, -5.8849265665348e+02,
        1.3363463e+00, -5.5076098609303e+02,
        1.6072053e+00, -5.2237501616674e+02,
        1.3629480e+00, -1.1790629318198e+03,
        5.5657014e+00, -1.0977134971135e+03,
        5.0708205e+00, -1.5774000881978e+02,
        3.9318944e+00, 5.2963464780000e+01,
        4.8989497e+00, 3.9809289073258e+01,
        1.3097446e+00, 7.7540959633708e+01,
        3.5147141e+00, 7.9618578146517e+01,
        3.5413158e+00, -5.4868336758022e+02
    };

    /*
     * amplitudes ccamps(k,n) of the short-period perturbations
     *   n=1           n=2           n=3           n=4           n=5
     */
    static double ccamps[15][5] = {
        -2.279594e-5, 1.407414e-5, 8.273188e-6, 1.340565e-5, -2.490817e-7,
        -3.494537e-5, 2.860401e-7, 1.289448e-7, 1.627237e-5, -1.823138e-7,
        6.593466e-7, 1.322572e-5, 9.258695e-6, -4.674248e-7, -3.646275e-7,
        1.140767e-5, -2.049792e-5, -4.747930e-6, -2.638763e-6, -1.245408e-7,
        9.516893e-6, -2.748894e-6, -1.319381e-6, -4.549908e-6, -1.864821e-7,
        7.310990e-6, -1.924710e-6, -8.772849e-7, -3.334143e-6, -1.745256e-7,
        -2.603449e-6, 7.359472e-6, 3.168357e-6, 1.119056e-6, -1.655307e-7,
        -3.228859e-6, 1.308997e-7, 1.013137e-7, 2.403899e-6, -3.736225e-7,
        3.442177e-7, 2.671323e-6, 1.832858e-6, -2.394688e-7, -3.478444e-7,
        8.702406e-6, -8.421214e-6, -1.372341e-6, -1.455234e-6, -4.998479e-8,
        -1.488378e-6, -1.251789e-5, 5.226868e-7, -2.049301e-7, 0.0e0,
        -8.043059e-6, -2.991300e-6, 1.473654e-7, -3.154542e-7, 0.0e0,
        3.699128e-6, -3.316126e-6, 2.901257e-7, 3.407826e-7, 0.0e0,
        2.550120e-6, -1.241123e-6, 9.901116e-8, 2.210482e-7, 0.0e0,
        -6.351059e-7, 2.341650e-6, 1.061492e-6, 2.878231e-7, 0.0e0
    };

    /*
     * constants of the secular perturbations in longitude
     * ccsec3 and ccsec(k,n)
     *          n=1           n=2           n=3
     */
    static double ccsec[4][3] = {
        1.289600e-06, 5.550147e-01, 2.076942e+00,
        3.102810e-05, 4.035027e+00, 3.525565e-01,
        9.124190e-06, 9.990265e-01, 2.622706e+00,
        9.793240e-07, 5.508259e+00, 1.559103e+01
    };

#define ccsec3  (double)-7.757020e-08

    /* sidereal rate dcsld in longitude, rate ccsgd in mean anomaly */
#define dcsld  (double)1.990987e-07
#define ccsgd  (double)1.990969e-07

    /* some constants used in the calculation of the lunar contribution */
#define cckm   (double)3.122140e-05
#define ccmld  (double)2.661699e-06
#define ccfdi  (double)2.399485e-07

    /*
     * constants dcargm(k,i) of the arguments of the perturbations
     * of the motion of the moon
     *           i=1               i=2
     */
    static double dcargm[3][2] = {
        5.1679830e+00, 8.3286911095275e+03,
        5.4913150e+00, -7.2140632838100e+03,
        5.9598530e+00, 1.5542754389685e+04
    };

    /*
     * amplitudes ccampm(k,n) of the perturbations of the moon
     *           n=1          n=2           n=3            n=4
     */
    static double ccampm[3][4] = {
        1.097594e-01, 2.896773e-07, 5.450474e-02, 1.438491e-07,
        -2.223581e-02, 5.083103e-08, 1.002548e-02, -2.291823e-08,
        1.148966e-02, 5.658888e-08, 8.249439e-03, 4.063015e-08
    };

    /*
     * ccpamv(k)=a*m*dl/dt (planets), dc1mme=1-mass(earth+moon)
     */
    static double ccpamv[4] = {
        8.326827e-11, 1.843484e-11, 1.988712e-12, 1.881276e-12
    };

#define dc1mme  (double)0.99999696e0

    /* ccpam(k)=a*m(planets), ccim=inclination(moon) */
    static double ccpam[4] = {
        4.960906e-3, 2.727436e-3, 8.392311e-4, 1.556861e-3
    };

#define ccim  (double)8.978749e-2

    /*
     * execution
     * ---------
     * control parameter ideq, and time arguments
     */
    ideq = 0;
    if (deqx > 0e0)
        ideq = 1;
    dt = (date - 15019.5e0) / 36525e0;
    /* t = (float)(dt); */
    dtsq = dt * dt;
    /* tsq = (float)(dtsq); */

    /* values of all elements for the instant date */
    for (k = 0; k < 8; k++)
    {
        dlocal = modf ((dcfel[k][0] + dt * dcfel[k][1] + dtsq * dcfel[k][2]) /
                       dc2pi, &junk) * dc2pi;
        if (k == 0)
        {
            dml = dlocal;
        }
        else
        {
            forbel[k - 1] = (float) (dlocal);
        }
    }
    deps = modf ((dceps[0] + dt * dceps[1] + dtsq * dceps[2]) / dc2pi, &junk)
        * dc2pi;
    /* use of t, tsq changed to dt, dtsq in next three eqn */
    for (k = 0; k < 17; k++)
    {
        sorbel[k] = modf ((ccsel[k][0] + dt * ccsel[k][1] +
                           dtsq * ccsel[k][2]) / cc2pi, &junk) * cc2pi;
    }

    /* secular perturbations in longitude */
    for (k = 0; k < 4; k++)
    {
        a = modf ((ccsec[k][1] + dt * ccsec[k][2]) / cc2pi, &junk) * cc2pi;
        sn[k] = sin (a);
    }

    /* periodic perturbations of the emb (earth-moon barycentre) */
    pertl = ccsec[0][0] * sn[0] + ccsec[1][0] * sn[1] +
        (ccsec[2][0] + dt * ccsec3) * sn[2] + ccsec[3][0] * sn[3];
    pertld = 0.0;
    pertr = 0.0;
    pertrd = 0.0;
    for (k = 0; k < 15; k++)
    {
        a = modf ((dcargs[k][0] + dt * dcargs[k][1]) / dc2pi, &junk) * dc2pi;
        cosa = cos (a);
        sina = sin (a);
        pertl += ccamps[k][0] * cosa + ccamps[k][1] * sina;
        pertr += ccamps[k][2] * cosa + ccamps[k][3] * sina;
        if (k < 10)
        {
            pertld += (ccamps[k][1] * cosa - ccamps[k][0] * sina)
                * ccamps[k][4];
            pertrd += (ccamps[k][3] * cosa - ccamps[k][2] * sina)
                * ccamps[k][4];
        }
    }

    /* elliptic part of the motion of the emb */
    esq = e * e;
    dparam = 1e0 - (double) (esq);
    /* param = (float)dparam; */
    twoe = e + e;
    twog = g + g;
    phi = twoe * ((1.0 - esq * 0.125) * sin (g) + e * 0.625 * sin (twog)
                  + esq * 0.5416667 * sin (g + twog));
    f = g + phi;
    sinf = sin (f);
    cosf = cos (f);
    dpsi = dparam / (1e0 + (double) (e * cosf));
    phid = twoe * ccsgd * ((1.0 + esq * 1.5) * cosf +
                           e * (1.25 - sinf * sinf * 0.5));
    /* was param instead of dparam */
    psid = ccsgd * e * sinf / sqrt (dparam);

    /* perturbed heliocentric motion of the emb */
    d1pdro = 1e0 + (double) pertr;
    drd = d1pdro * ((double) psid + dpsi * (double) pertrd);
    drld = d1pdro * dpsi * (dcsld + (double) phid + (double) pertld);
    dtl = modf ((dml + (double) phi + (double) pertl) / dc2pi, &junk) * dc2pi;
    dsinls = sin (dtl);
    dcosls = cos (dtl);
    dxhd = drd * dcosls - drld * dsinls;
    dyhd = drd * dsinls + drld * dcosls;

    /* influence of eccentricity, evection and variation on the */
    /* geocentric motion of the moon */
    pertl = 0.0;
    pertld = 0.0;
    pertp = 0.0;
    pertpd = 0.0;
    for (k = 0; k < 3; k++)
    {
        a = modf ((dcargm[k][0] + dt * dcargm[k][1]) / dc2pi, &junk) * dc2pi;
        sina = sin (a);
        cosa = cos (a);
        pertl += ccampm[k][0] * sina;
        pertld += ccampm[k][1] * cosa;
        pertp += ccampm[k][2] * cosa;
        pertpd -= ccampm[k][3] * sina;
    }

    /* heliocentric motion of the earth */
    tl = forbel[1] + pertl;
    sinlm = sin (tl);
    coslm = cos (tl);
    sigma = cckm / (1.0 + pertp);
    a = sigma * (ccmld + pertld);
    b = sigma * pertpd;
    dxhd += (double) (a * sinlm) + (double) (b * coslm);
    dyhd -= (double) (a * coslm) + (double) (b * sinlm);
    dzhd = -(double) (sigma * ccfdi * cos (forbel[2]));

    /* barycentric motion of the earth */
    dxbd = dxhd * dc1mme;
    dybd = dyhd * dc1mme;
    dzbd = dzhd * dc1mme;
    for (k = 0; k < 4; k++)
    {
        plon = forbel[k + 3];
        pomg = sorbel[k + 1];
        pecc = sorbel[k + 9];
        tl = modf ((plon + 2.0 * pecc * sin (plon - pomg)) / cc2pi, &junk)
            * cc2pi;
        sinlp[k] = sin (tl);
        coslp[k] = cos (tl);
        dxbd += (double) (ccpamv[k] * (sinlp[k] + pecc * sin (pomg)));
        dybd -= (double) (ccpamv[k] * (coslp[k] + pecc * cos (pomg)));
        dzbd -= (double) (ccpamv[k] * sorbel[k + 13] *
                          cos (plon - sorbel[k + 5]));
    }

    /* transition to mean equator of date */
    dcosep = cos (deps);
    dsinep = sin (deps);
    dyahd = dcosep * dyhd - dsinep * dzhd;
    dzahd = dsinep * dyhd + dcosep * dzhd;
    dyabd = dcosep * dybd - dsinep * dzbd;
    dzabd = dsinep * dybd + dcosep * dzbd;

    /* heliocentric coordinates of the earth */
    dr = dpsi * d1pdro;
    flatm = ccim * sin (forbel[2]);
    a = sigma * cos (flatm);
    dxh = dr * dcosls - (double) (a * coslm);
    dyh = dr * dsinls - (double) (a * sinlm);
    dzh = -(double) (sigma * sin (flatm));

    /* barycentric coordinates of the earth */
    dxb = dxh * dc1mme;
    dyb = dyh * dc1mme;
    dzb = dzh * dc1mme;
    for (k = 0; k < 4; k++)
    {
        flat = sorbel[k + 13] * sin (forbel[k + 3] - sorbel[k + 5]);
        a = ccpam[k] * (1.0 - sorbel[k + 9] * cos (forbel[k + 3] -
                                                   sorbel[k + 1]));
        b = a * cos (flat);
        dxb -= (double) (b * coslp[k]);
        dyb -= (double) (b * sinlp[k]);
        dzb -= (double) (a * sin (flat));
    }

    /* transition to mean equator of date */
    dyah = dcosep * dyh - dsinep * dzh;
    dzah = dsinep * dyh + dcosep * dzh;
    dyab = dcosep * dyb - dsinep * dzb;
    dzab = dsinep * dyb + dcosep * dzb;

    /* copy result components into vectors, correcting for FK4 equinox */
    depj = sla_epj (date);
    deqcor = ds2r * (0.035 + 0.00085 * (depj - 1950e0));
    dvh[0] = dxhd - deqcor * dyahd;
    dvh[1] = dyahd + deqcor * dxhd;
    dvh[2] = dzahd;
    dvb[0] = dxbd - deqcor * dyabd;
    dvb[1] = dyabd + deqcor * dxbd;
    dvb[2] = dzabd;
    dph[0] = dxh - deqcor * dyah;
    dph[1] = dyah + deqcor * dxh;
    dph[2] = dzah;
    dpb[0] = dxb - deqcor * dyab;
    dpb[1] = dyab + deqcor * dxb;
    dpb[2] = dzab;

    /* was precession to another equinox requested? */
    if (ideq != 0)
    {

        /*
         * yes: compute precession matrix from mjd date to julian
         * epoch deqx 
         */
        sla_prec (depj, deqx, dprema);

        /* rotate dvh */
        sla_dmxv (dprema, dvh, dvh);
        /* code was:
           * for (j = 0; j < 3; j++) {
           *    w = 0e0;
           *    for (i = 0; i < 3; i++) {
           *            w += dprema[j][i] * dvh[i];
           *    }
           *    vw[j] = w;
           * }
           * for (j = 0; j < 3; j++) {
           *    dvh[j] = vw[j];
           * }
           * end of "code was"
         */

        /* rotate dvb */
        sla_dmxv (dprema, dvb, dvb);

        /* rotate dph */
        sla_dmxv (dprema, dph, dph);

        /* rotate dpb */
        sla_dmxv (dprema, dpb, dpb);
    }
}
