#include <math.h>

/**
 *
 *     - - - - - -
 *      f k 4 2 5
 *     - - - - - -
 *
 *  convert b1950.0 FK4 star data to j2000.0 FK5
 *
 *  (double precision)
 *
 *  This routine converts stars from the old, Bessel-Newcomb, FK4
 *  system to the new, IAU 1976, FK5, Fricke system, using Yallop's
 *  implementation (see ref 2) of a matrix method due to Standish.
 *  The numerical values of ref 2 are used canonically.
 *
 *  given:  (all b1950.0,FK4)
 *
 *     r1950,d1950     dp    b1950.0 ra,dec (rad)
 *     dr1950,dd1950   dp    b1950.0 proper motions (rad/trop.yr)
 *     p1950           dp    parallax (arcsec)
 *     v1950           dp    radial velocity (km/s, +ve = moving away)
 *
 *  returned:  (all j2000.0,FK5)
 *
 *     r2000,d2000     dp    j2000.0 ra,dec (rad)
 *     dr2000,dd2000   dp    j2000.0 proper motions (rad/jul.yr)
 *     p2000           dp    parallax (arcsec)
 *     v2000           dp    radial velocity (km/s, +ve = moving away)
 *
 *  notes:
 *
 *     1)  The proper motions in ra are dra/dt rather than
 *         cos(dec)*dra/dt, and are per year rather than per century.
 *
 *     2)  Conversion from Besselian epoch 1950.0 to Julian epoch
 *         2000.0 only is provided for.  Conversions involving other
 *         epochs will require use of the appropriate precession,
 *         proper motion, and E-terms routines before and/or
 *         after fk425 is called.
 *
 *     3)  In the FK4 catalogue the proper motions of stars within
 *         10 degrees of the poles do not embody the differential
 *         E-term effect and should, strictly speaking, be handled
 *         in a different manner from stars outside these regions.
 *         However, given the general lack of homogeneity of the star
 *         data available for routine astrometry, the difficulties of
 *         handling positions that may have been determined from
 *         astrometric fields spanning the polar and non-polar regions,
 *         the likelihood that the differential E-terms effect was not
 *         taken into account when allowing for proper motion in past
 *         astrometry, and the undesirability of a discontinuity in
 *         the algorithm, the decision has been made in this routine to
 *         include the effect of differential E-terms on the proper
 *         motions for all stars, whether polar or not.  At epoch 2000,
 *         and measuring on the sky rather than in terms of dra, the
 *         errors resulting from this simplification are less than
 *         1 milliarcsecond in position and 1 milliarcsecond per
 *         century in proper motion.
 *
 *  references:
 *
 *     1  The transformation of astrometric catalog systems to the
 *        equinox j2000.0.  Smith, C.A. et al, paper to be submitted
 *        to the Astronomical Journal.  draft dated 1987 august 26.
 *
 *     2  Transformation of mean star places from FK4 b1950.0 to
 *        FK5 j2000.0 using matrices in 6-space.  Yallop, B.D. et al,
 *        paper to be submitted to the Astronomical Journal.  draft
 *        dated 1987 october 8.
 *
 *
 *  p.t.wallace   starlink   27 october 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_fk425 (double r1950, double d1950, double dr1950, double dd1950,
           double p1950, double v1950, double *r2000, double *d2000,
           double *dr2000, double *dd2000, double *p2000, double *v2000)
{

    /* miscellaneous */
    double r, d, ur, ud, px, rv, sr, cr, sd, cd, w, wd;
    double x, y, z, xd, yd, zd;
    double rxysq, rxyzsq, rxy, rxyz, spxy, spxyz;
    int i, j;

    /* star position and velocity vectors */
    double r0[3], rd0[3];

    /* combined position and velocity vectors */
    double v1[6], v2[6];

    /* 2pi */
#define d2pi  (double)6.283185307179586476925287e0

    /* radians per year to arcsec per century */
#define pmf  ((double)100e0*60e0*60e0*360e0/d2pi)

    /* small number to avoid arithmetic problems */
#define tiny  (double)1e-30

    /*
     * canonical constants  (see references)
     *
     * km per sec to au per tropical century
     * = 86400 * 36524.2198782 / 1.49597870e8
     */
#define vf  (double)21.0945e0

    /* constant vector and matrix (by columns) */
    static double a[3] = { -1.62557e-6, -0.31919e-6, -0.13843e-6 };
    static double ad[3] = { 1.245e-3, -1.580e-3, -0.659e-3 };

    /*
     * changing from Fortran to C should involve transposing
     * the em array.  Too error prone.  So, we swap coordinates
     * where em is used, and do row rather than column init.
     */
    static double em[6][6] = {
        0.999925678186902e0, 0.011182059571766e0, 0.004857946721186e0,
        -0.000541652366951e0, 0.237917612131583e0, -0.436111276039270e0,

        -0.011182059642247e0, 0.999937478448132e0, -0.000027147426498e0,
        -0.237968129744288e0, -0.002660763319071e0, 0.012259092261564e0,

        -0.004857946558960e0, -0.000027176441185e0, 0.999988199738770e0,
        0.436227555856097e0, -0.008537771074048e0, 0.002119110818172e0,

        0.000002423950176e0, 0.000000027106627e0, 0.000000011776559e0,
        0.999947035154614e0, 0.011182506007242e0, 0.004857669948650e0,

        -0.000000027106627e0, 0.000002423978783e0, -0.000000000065816e0,
        -0.011182506121805e0, 0.999958833818833e0, -0.000027137309539e0,

        -0.000000011776558e0, -0.000000000065874e0, 0.000002424101735e0,
        -0.004857669684959e0, -0.000027184471371e0, 1.000009560363559e0
    };

    /* pick up b1950 data (units radians and arcsec/TC) */
    r = r1950;
    d = d1950;
    ur = dr1950 * pmf;
    ud = dd1950 * pmf;
    px = p1950;
    rv = v1950;

    /* spherical to cartesian */
    sr = sin (r);
    cr = cos (r);
    sd = sin (d);
    cd = cos (d);
    r0[0] = cr * cd;
    r0[1] = sr * cd;
    r0[2] = sd;
    w = vf * rv * px;
    rd0[0] = -sr * cd * ur - cr * sd * ud + w * r0[0];
    rd0[1] = cr * cd * ur - sr * sd * ud + w * r0[1];
    rd0[2] = cd * ud + w * r0[2];

    /* allow for E-terms and express as position+velocity 6-vector */
    w = r0[0] * a[0] + r0[1] * a[1] + r0[2] * a[2];
    wd = r0[0] * ad[0] + r0[1] * ad[1] + r0[2] * ad[2];
    for (i = 0; i < 3; i++)
    {
        v1[i] = r0[i] - a[i] + w * r0[i];
        v1[i + 3] = rd0[i] - ad[i] + wd * r0[i];
    }

    /* convert position+velocity vector to Fricke system */
    for (i = 0; i < 6; i++)
    {
        w = 0e0;
        for (j = 0; j < 6; j++)
        {
            /* see above; reversed subscript order in C */
            w += em[j][i] * v1[j];
        }
        v2[i] = w;
    }

    /* revert to spherical coordinates */
    x = v2[0];
    y = v2[1];
    z = v2[2];
    xd = v2[3];
    yd = v2[4];
    zd = v2[5];
    rxysq = x * x + y * y;
    rxyzsq = rxysq + z * z;
    rxy = sqrt (rxysq);
    rxyz = sqrt (rxyzsq);
    spxy = x * xd + y * yd;
    spxyz = spxy + z * zd;
    if (x == 0e0 && y == 0e0)
    {
        r = 0e0;
    }
    else
    {
        r = atan2 (y, x);
        if (r < 0.0e0)
            r += d2pi;
    }
    d = atan2 (z, rxy);
    if (rxy > tiny)
    {
        ur = (x * yd - y * xd) / rxysq;
        ud = (zd * rxysq - z * spxy) / (rxyzsq * rxy);
    }
    if (px > tiny)
    {
        rv = spxyz / (px * rxyz * vf);
        px = px / rxyz;
    }
    /* return results */
    *r2000 = r;
    *d2000 = d;
    *dr2000 = ur / pmf;
    *dd2000 = ud / pmf;
    *v2000 = rv;
    *p2000 = px;
}
