
/**
 *
 *     - - - - - -
 *      f k 4 5 z
 *     - - - - - -
 *
 *  convert b1950.0 FK4 star data to j2000.0 FK5 assuming zero
 *  proper motion in an inertial frame.
 *
 *  (double precision)
 *
 *  This routine converts stars from the old, Bessel-Newcomb, FK4
 *  system to the new, IAU 1976, FK5, Fricke system, in such a
 *  way that the FK5 proper motion is zero.  Because such a star
 *  has, in general, a non-zero proper motion in the FK4 system,
 *  the routine requires the epoch at which the position in the
 *  FK4 system was determined.
 *
 *  The method is from appendix 2 of ref 1, but using the constants
 *  of ref 2.
 *
 *  given:
 *     r1950,d1950     dp    b1950.0 FK4 ra,dec at epoch (rad)
 *     bepoch          dp    Besselian epoch (e.g. 1979.3d0)
 *
 *  returned:
 *     r2000,d2000     dp    j2000.0 FK5 ra,dec (rad)
 *
 *  notes:
 *
 *     1)  The epoch bepoch is strictly speaking Besselian, but
 *         if a Julian epoch is supplied the result will be
 *         affected only to a negligible extent.
 *
 *     2)  Conversion from Besselian epoch 1950.0 to Julian epoch
 *         2000.0 only is provided for.  Conversions involving other
 *         epochs will require use of the appropriate precession,
 *         proper motion, and E-terms routines before and/or
 *         after FK425 is called.
 *
 *     3)  In the FK4 catalogue the proper motions of stars within
 *         10 degrees of the poles do not embody the differential
 *         E-term effect and should, strictly speaking, be handled
 *         in a different manner from stars outside these regions.
 *         However, given the general lack of homogeneity of the star
 *         data available for routine astrometry, the difficulties of
 *         handling positions that may have been determined from
 *         astrometric fields spanning the polar and non-polar regions,
 *         the likelihood that the differential E-terms effect was not
 *         taken into account when allowing for proper motion in past
 *         astrometry, and the undesirability of a discontinuity in
 *         the algorithm, the decision has been made in this routine to
 *         include the effect of differential E-terms on the proper
 *         motions for all stars, whether polar or not.  At epoch 2000,
 *         and measuring on the sky rather than in terms of dra, the
 *         errors resulting from this simplification are less than
 *         1 milliarcsecond in position and 1 milliarcsecond per
 *         century in proper motion.
 *
 *  references:
 *
 *     1  Aoki,S., et al, 1983.  Astron.Astrophys., 128, 263.
 *
 *     2  The transformation of astrometric catalog systems to the
 *        equinox j2000.0.  Smith, C.A. et al, paper to be submitted
 *        to the Astronomical Journal.  draft dated 1987 august 26.
 *
 *     3  Transformation of mean star places from FK4 b1950.0 to
 *        FK5 j2000.0 using matrices in 6-space.  Yallop, B.D. et al,
 *        paper to be submitted to the Astronomical Journal.  draft
 *        dated 1987 october 8.
 *
 *  called:  sla_dcs2c, sla_epj, sla_epb2d, sla_dcc2s, sla_dranrm
 *
 *  p.t.wallace   starlink   27 october 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_fk45z (double r1950, double d1950, double bepoch, double *r2000,
           double *d2000)
{

#define d2pi  (double)6.283185307179586476925287e0
    double w;
    int i, j;

    /* position and position+velocity vectors */
    double r0[3], a1[3], v1[3], v2[6];

    /* radians per year to arcsec per century */
#define pmf  ((double)100e0*60e0*60e0*360e0/d2pi)

    /* functions */
    double sla_epj (double date), sla_epb2d (double epb),
        sla_dranrm (double angle);

    /*
     * canonical constants  (see references)
     *
     *
     * vectors a and adot, and matrix m (only half of which is needed
     * here) 
     */
    static double a[3] = { -1.62557e-6, -0.31919e-6, -0.13843e-6 };
    static double ad[3] = { 1.245e-3, -1.580e-3, -0.659e-3 };

    /*
     * changing from Fortran to C should involve transposing
     * the em array.  Too error prone.  So, we swap coordinates
     * where em is used, and do row rather than column init.
     */
    static double em[3][6] = {
        0.999925678186902e0, 0.011182059571766e0, 0.004857946721186e0,
        -0.000541652366951e0, 0.237917612131583e0, -0.436111276039270e0,

        -0.011182059642247e0, 0.999937478448132e0, -0.000027147426498e0,
        -0.237968129744288e0, -0.002660763319071e0, 0.012259092261564e0,

        -0.004857946558960e0, -0.000027176441185e0, 0.999988199738770e0,
        0.436227555856097e0, -0.008537771074048e0, 0.002119110818172e0
    };

    /* spherical to cartesian */
    sla_dcs2c (r1950, d1950, r0);

    /* adjust vector a to give zero proper motion in FK5 */
    w = (bepoch - 1950e0) / pmf;
    for (i = 0; i < 3; i++)
    {
        a1[i] = a[i] + w * ad[i];
    }

    /* remove E-terms */
    w = r0[0] * a1[0] + r0[1] * a1[1] + r0[2] * a1[2];
    for (i = 0; i < 3; i++)
    {
        v1[i] = r0[i] - a1[i] + w * r0[i];
    }

    /* convert position vector to fricke system */
    for (i = 0; i < 6; i++)
    {
        w = 0e0;
        for (j = 0; j < 3; j++)
        {
            /* em indices swapped; see above */
            w += em[j][i] * v1[j];
        }
        v2[i] = w;
    }

    /* allow for fictitious proper motion in fk4 */
    w = (sla_epj (sla_epb2d (bepoch)) - 2000e0) / pmf;
    for (i = 0; i < 3; i++)
    {
        v2[i] += w * v2[i + 3];
    }

    /* revert to spherical coordinates */
    sla_dcc2s (v2, &w, d2000);
    *r2000 = sla_dranrm (w);
}
