#include <math.h>

/**
 *
 *     - - - - - -
 *      f k 5 2 4
 *     - - - - - -
 *
 *  convert j2000.0 FK5 star data to b1950.0 FK4
 *
 *  (double precision)
 *
 *  This routine converts stars from the new, IAU 1976, FK5, Fricke
 *  system, to the old, Bessel-Newcomb, FK4 system, using Yallop's
 *  implementation (see ref 2) of a matrix method due to Standish.
 *  The numerical values of ref 2 are used canonically.
 *
 *  given:  (all j2000.0,FK5)
 *
 *     r2000,d2000     dp    j2000.0 ra,dec (rad)
 *     dr2000,dd2000   dp    j2000.0 proper motions (rad/jul.yr)
 *     p2000           dp    parallax (arcsec)
 *     v2000           dp    radial velocity (km/s, +ve = moving away)
 *
 *  returned:  (all b1950.0,FK4)
 *
 *     r1950,d1950     dp    b1950.0 ra,dec (rad)
 *     dr1950,dd1950   dp    b1950.0 proper motions (rad/trop.yr)
 *     p1950           dp    parallax (arcsec)
 *     v1950           dp    radial velocity (km/s, +ve = moving away)
 *
 *  notes:
 *
 *     1)  The proper motions in ra are dra/dt rather than
 *         cos(dec)*dra/dt, and are per year rather than per century.
 *
 *     2)  Note that conversion from Julian epoch 2000.0 to Besselian
 *         epoch 1950.0 only is provided for.  Conversions involving
 *         other epochs will require use of the appropriate precession,
 *         proper motion, and E-terms routines before and/or after
 *         fk524 is called.
 *
 *     3)  In the FK4 catalogue the proper motions of stars within
 *         10 degrees of the poles do not embody the differential
 *         E-term effect and should, strictly speaking, be handled
 *         in a different manner from stars outside these regions.
 *         However, given the general lack of homogeneity of the star
 *         data available for routine astrometry, the difficulties of
 *         handling positions that may have been determined from
 *         astrometric fields spanning the polar and non-polar regions,
 *         the likelihood that the differential E-terms effect was not
 *         taken into account when allowing for proper motion in past
 *         astrometry, and the undesirability of a discontinuity in
 *         the algorithm, the decision has been made in this routine to
 *         include the effect of differential e-terms on the proper
 *         motions for all stars, whether polar or not.  At epoch 2000,
 *         and measuring on the sky rather than in terms of dra, the
 *         errors resulting from this simplification are less than
 *         1 milliarcsecond in position and 1 milliarcsecond per
 *         century in proper motion.
 *
 *  references:
 *
 *     1  The transformation of astrometric catalog systems to the
 *        equinox j2000.0.  Smith, C.A. et al, paper to be submitted
 *        to the Astronomical Journal.  draft dated 1987 august 26.
 *
 *     2  Transformation of mean star places from FK4 b1950.0 to
 *        FK5 j2000.0 using matrices in 6-space.  Yallop, B.D. et al,
 *        paper to be submitted to the Astronomical Journal.  draft
 *        dated 1987 october 8.
 *
 *  p.t.wallace   starlink   27 october 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_fk524 (double r2000, double d2000, double dr2000, double dd2000,
           double p2000, double v2000, double *r1950, double *d1950,
           double *dr1950, double *dd1950, double *p1950, double *v1950)
{

    /* miscellaneous */
    double r, d, ur, ud, px, rv;
    double sr, cr, sd, cd, x, y, z, w;
    double v1[6], v2[6];
    double xd, yd, zd;
    double rxyz, rxysq, rxy;
    int i, j;

    /* 2pi */
#define d2pi  (double)6.283185307179586476925287e0

    /* radians per year to arcsec per century */
#define pmf  ((double)100e0*60e0*60e0*360e0/d2pi)

    /* small number to avoid arithmetic problems */
#define tiny  (double)1e-30

    /*
     * canonical constants  (see references)
     *
     * km per sec to au per tropical century
     * = 86400 * 36524.2198782 / 1.49597870e8
     */
#define vf  (double)21.0945e0

    /* constant vector and matrix (by columns) */
    static double a[6] = {
        -1.62557e-6, -0.31919e-6, -0.13843e-6,
        1.245e-3, -1.580e-3, -0.659e-3
    };

    /*
     * changing from Fortran to C should involve transposing
     * the emi array.  Too error prone.  So, we swap coordinates
     * where emi is used, and do row rather than column init.
     */
    static double emi[6][6] = {
        0.999925679499910e0, -0.011181482788805e0, -0.004859004008828e0,
        -0.000541640798032e0, -0.237963047085011e0, 0.436218238658637e0,

        0.011181482840782e0, 0.999937484898031e0, -0.000027155744957e0,
        0.237912530551179e0, -0.002660706488970e0, -0.008537588719453e0,

        0.004859003889183e0, -0.000027177143501e0, 0.999988194601879e0,
        -0.436101961325347e0, 0.012258830424865e0, 0.002119065556992e0,

        -0.000002423898405e0, 0.000000027105439e0, 0.000000011777422e0,
        0.999904322043106e0, -0.011181451601069e0, -0.004858519608686e0,

        -0.000000027105439e0, -0.000002423927017e0, 0.000000000065851e0,
        0.011181451608968e0, 0.999916125340107e0, -0.000027162614355e0,

        -0.000000011777422e0, 0.000000000065846e0, -0.000002424049954e0,
        0.004858519590501e0, -0.000027165866691e0, 0.999966838131419e0
    };

    /* the above values were obtained by inverting C.Hohenkerk's
     * forward matrix (private communication), which agrees with
     * the one given in reference 2 but which has one additional
     * decimal place.
     */

    /* pick up j2000 data (units radians and arcsec/JC) */
    r = r2000;
    d = d2000;
    ur = dr2000 * pmf;
    ud = dd2000 * pmf;
    px = p2000;
    rv = v2000;

    /* spherical to cartesian */
    sr = sin (r);
    cr = cos (r);
    sd = sin (d);
    cd = cos (d);
    x = cr * cd;
    y = sr * cd;
    z = sd;
    w = vf * rv * px;
    v1[0] = x;
    v1[1] = y;
    v1[2] = z;
    v1[3] = -ur * y - cr * sd * ud + w * x;
    v1[4] = ur * x - sr * sd * ud + w * y;
    v1[5] = cd * ud + w * z;

    /* convert position+velocity vector to bn system */
    for (i = 0; i < 6; i++)
    {
        w = 0e0;
        for (j = 0; j < 6; j++)
        {
            /* emi subscripts exchanged; see above */
            w += emi[j][i] * v1[j];
        }
        v2[i] = w;
    }

    /* vector components */
    x = v2[0];
    y = v2[1];
    z = v2[2];
    xd = v2[3];
    yd = v2[4];
    zd = v2[5];

    /* magnitude of position vector */
    rxyz = sqrt (x * x + y * y + z * z);

    /* radial velocity and parallax */
    if (px > tiny)
    {
        rv = (x * xd + y * yd + z * zd) / (px * vf * rxyz);
        px = px / rxyz;
    }
    /* include E-terms */
    x += a[0] * rxyz;
    y += a[1] * rxyz;
    z += a[2] * rxyz;
    xd += a[3] * rxyz;
    yd += a[4] * rxyz;
    zd += a[5] * rxyz;

    /* convert to spherical */
    rxysq = x * x + y * y;
    rxy = sqrt (rxysq);
    if (x == 0e0 && y == 0e0)
    {
        r = 0e0;
    }
    else
    {
        r = atan2 (y, x);
        if (r < 0.0e0)
            r += d2pi;
    }
    d = atan2 (z, rxy);
    if (rxy > tiny)
    {
        ur = (x * yd - y * xd) / rxysq;
        ud = (zd * rxysq - z * (x * xd + y * yd)) / ((rxysq + z * z) * rxy);
    }
    /* return results */
    *r1950 = r;
    *d1950 = d;
    *dr1950 = ur / pmf;
    *dd1950 = ud / pmf;
    *v1950 = rv;
    *p1950 = px;
}
