
/**
 *
 *     - - - - - -
 *      f k 5 4 z
 *     - - - - - -
 *
 *
 *  convert a j2000.0 FK5 star position to b1950.0 FK4 assuming
 *  zero proper motion and parallax.
 *
 *  (double precision)
 *
 *  This routine converts star positions from the new, IAU 1976,
 *  FK5, Fricke system to the old, Bessel-Newcomb, FK4 system.
 *
 *
 *  notes:
 *
 *     1) Conversion from Julian epoch 2000.0 to Besselian epoch 1950.0
 *        only is provided for.  Conversions involving other epochs will
 *        require use of the appropriate precession routines before and
 *        after this routine is called.
 *
 *     2) Unlike in the sla_fk524 routine, the FK5 proper motions, the
 *        parallax and the radial velocity are presumed zero.
 *
 *     3) It is the intention that FK5 should be a close approximation
 *        to an inertial frame, so that distant objects have zero proper
 *        motion;  such objects have (in general) non-zero proper motion
 *        in FK4, and this routine returns those fictitious proper
 *        motions.
 *
 *     4) The position returned by this routine is in the b1950
 *        reference frame but at Besselian epoch bepoch.  For
 *        comparison with catalogues the bepoch argument will
 *        frequently be 1950d0.
 *
 *  given:
 *     r2000,d2000     dp    j2000.0 FK5 ra,dec (rad)
 *     bepoch          dp    Besselian epoch (e.g. 1950d0)
 *
 *  returned:
 *     r1950,d1950     dp    b1950.0 FK4 ra,dec (rad) at epoch bepoch
 *     dr1950,dd1950   dp    b1950.0 FK4 proper motions (rad/trop.yr)
 *
 *  n.b.  the proper motion in ra is dra/dt rather than cos(dec)*dra/dt.
 *
 *  called:  sla_fk524, sla_pm
 *
 *  p.t.wallace   starlink   30 july 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_fk524 (double r2000, double d2000, double dr2000,
                      double dd2000, double p2000, double v2000,
                      double *r1950, double *d1950, double *dr1950,
                      double *dd1950, double *p1950, double *v1950);
extern int sla_pm (double r0, double d0, double pr, double pd, double px,
                   double rv, double ep0, double ep1, double *r1, double *d1);

sla_fk54z (double r2000, double d2000, double bepoch, double *r1950,
           double *d1950, double *dr1950, double *dd1950)
{

    double r, d, px, rv;

    /* fk5 equinox j2000 (any epoch) to fk4 equinox b1950 epoch b1950 */
    sla_fk524 (r2000, d2000, 0e0, 0e0, 0e0, 0e0,
               &r, &d, dr1950, dd1950, &px, &rv);

    /* fictitious proper motion to epoch bepoch */
    sla_pm (r, d, *dr1950, *dd1950, 0e0, 0e0, 1950e0, bepoch, r1950, d1950);
}
