
/**
 *
 *     - - - - - - -
 *      g a l s u p
 *     - - - - - - -
 *
 *
 *  transformation from IAU 1958 galactic coordinates to
 *  de Vaucouleurs supergalactic coordinates
 *
 *  (double precision)
 *
 *
 *  given:
 *     dl,db       dp       galactic longitude and latitude l2,b2
 *
 *  returned:
 *     dsl,dsb     dp       supergalactic longitude and latitude
 *
 *  (all arguments are radians)
 *
 *
 *  called:
 *     sla_dcs2c, sla_dimxv, sla_dcc2s, sla_dranrm, sla_drange
 *
 *
 *  references:
 *
 *     de Vaucouleurs, de Vaucouleurs, & Corwin, Second Reference
 *     Catalogue of Bright Galaxies, U. Texas, page 8.
 *
 *     Systems & Applied Sciences Corp., Documentation for the
 *     machine-readable version of the above catalogue,
 *     Contract NAS 5-26490.
 *
 *    (These two references give different values for the galactic
 *     longitude of the supergalactic origin.  Both are wrong;  the
 *     correct value is l2=137.37.)
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dmxv (double (*dm)[3], double *va, double *vb);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_galsup (double dl, double db, double *dsl, double *dsb)
{

    double sla_dranrm (double angle), sla_drange (double angle);
    double v1[3], v2[3];

    /*
     * system of supergalactic coordinates:
     *
     * sgl   sgb        l2     b2      (deg)
     *  -    +90      47.37  +6.32
     *  0     0         -      0
     *
     * galactic to supergalactic rotation matrix:
     *
     */
    static double rmat[3][3] = {
        -0.735742574804e0, 0.677261296414e0, 0.000000000000e0,
        -0.074553778365e0, -0.080991471307e0, 0.993922590400e0,
        0.673145302109e0, 0.731271165817e0, 0.110081262225e0
    };

    /* spherical to cartesian */
    sla_dcs2c (dl, db, v1);

    /* galactic to supergalactic */
    sla_dmxv (rmat, v1, v2);

    /* cartesian to spherical */
    sla_dcc2s (v2, dsl, dsb);

    /* express in conventional ranges */
    *dsl = sla_dranrm (*dsl);
    *dsb = sla_drange (*dsb);
}
