
/**
 *
 *     - - - - -
 *      g e 5 0
 *     - - - - -
 *
 *
 *  transformation from IAU 1958 galactic coordinates to
 *  b1950.0 'FK4' equatorial coordinates
 *
 *  (double precision)
 *
 *
 *  given:
 *     dl,db       dp       galactic longitude and latitude l2,b2
 *
 *  returned:
 *     dr,dd       dp       b1950.0 'FK4' ra,dec
 *
 *  (all arguments are radians)
 *
 *
 *  called:
 *     sla_dcs2c, sla_dimxv, sla_dcc2s, sla_addet, sla_dranrm, sla_drange
 *
 *
 *  note:
 *     The equatorial coordinates are b1950.0 'FK4'.  Use the
 *     routine sla_galeq if conversion to j2000.0 coordinates
 *     is required.
 *
 *
 *  reference:
 *     Blaauw et al, Mon.Not.R.Astron.Soc.,121,123 (1960)
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dimxv (double (*dm)[3], double *va, double *vb);
extern int sla_dcc2s (double *v, double *a, double *b);
extern int sla_addet (double rm, double dm, double eq, double *rc,
                      double *dc);

sla_ge50 (double dl, double db, double *dr, double *dd)
{

    double sla_dranrm (double angle), sla_drange (double angle);
    double v1[3], v2[3], r, d, re, de;

    /*
     * l2,b2 system of galactic coordinates
     *
     * p = 192.25       ra of galactic north pole (mean b1950.0)
     * q =  62.6        inclination of galactic to mean b1950.0 equator
     * r =  33          longitude of ascending node
     *
     * p,q,r are degrees
     *
     *
     * equatorial to galactic rotation matrix
     *
     * the eulerian angles are p, q, 90-r
     *
     * +cp.cq.sr-sp.cr     +sp.cq.sr+cp.cr     -sq.sr
     *
     * -cp.cq.cr-sp.sr     -sp.cq.cr+cp.sr     +sq.cr
     *
     * +cp.sq              +sp.sq              +cq
     *
     */
    static double rmat[3][3] = {
        -0.066988739415e0, -0.872755765852e0, -0.483538914632,
        0.492728466075e0, -0.450346958020e0, 0.744584633283,
        -0.867600811151e0, -0.188374601723e0, 0.460199784784
    };

    /* spherical to cartesian */
    sla_dcs2c (dl, db, v1);

    /* rotate to mean b1950.0 */
    sla_dimxv (rmat, v1, v2);

    /* cartesian to spherical */
    sla_dcc2s (v2, &r, &d);

    /* introduce E-terms */
    sla_addet (r, d, 1950e0, &re, &de);

    /* express in conventional ranges */
    *dr = sla_dranrm (re);
    *dd = sla_drange (de);
}
