#include <math.h>

/**
 *
 *     - - - - -
 *      g e o c
 *     - - - - -
 *
 *
 *  convert geodetic position to geocentric
 *
 *  (double precision)
 *
 *
 *  given:
 *     p     dp     latitude (geodetic, radians)
 *     h     dp     height above reference spheroid (geodetic, metres)
 *
 *  returned:
 *     r     dp     distance from earth axis (au)
 *     z     dp     distance from plane of earth equator (au)
 *
 *
 *  notes:
 *     1)  Geocentric latitude can be obtained by evaluating atan2(z,r).
 *     2)  IAU 1976 constants are used.
 *
 *
 *  reference:
 *     Explanatory Supplement to the A.E. (1961), page 57.
 *
 *
 *  p.t.wallace   starlink   october 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_geoc (double p, double h, double *r, double *z)
{

    double sp, cp, c2p, c, s, cc, ss, rho;
    double b;

    /* earth equatorial radius (metres) */
#define a0  (double)6378140e0

    /* reference spheroid flattening factor and useful function thereof */
#define f  (double)0.00335281e0
    b = (1e0 - f) * (1e0 - f);

    /* astronomical unit in metres */
#define au  (double)1.49597870e11

    /* geodetic to geocentric conversion */
    sp = sin (p);
    cp = cos (p);
    c2p = cos (2e0 * p);
    c = 1e0 / sqrt (cp * cp + b * sp * sp);
    s = b * c;
    cc = c * c;
    ss = s * s;
    rho = a0 * sqrt ((ss + cc + (cc - ss) * c2p) / 2e0);
    *r = (rho * c + h) * cp / au;
    *z = (rho * s + h) * sp / au;
}
