
/**
 *
 *     - - - -
 *      m a p
 *     - - - -
 *
 *
 *  transform star ra,dec from mean place to geocentric apparent
 *
 *
 *  The reference frames and timescales used are post IAU 1976.
 *
 *  references:
 *     1984 Astronomical Almanac, pp b39-b41.
 *     (also Lederle & Schwan, Astron. Astrophys. 134,
 *      1-6, 1984)
 *
 *  given:
 *     rm,dm    dp     mean ra,dec (rad)
 *     pr,pd    dp     proper motions:  ra,dec changes per julian year
 *     px       dp     parallax (arcsec)
 *     rv       dp     radial velocity (km/sec, +ve if receeding)
 *     eq       dp     epoch and equinox of star data (Julian)
 *     date     dp     TDB for apparent place (JD-2400000.5)
 *
 *  returned:
 *     ra,da    dp     apparent ra,dec (rad)
 *
 *  called:
 *     sla_mapp        star-independent parameters
 *     sla_mapq        quick mean to apparent
 *
 *
 *  notes:
 *
 *     1)  Eq is the Julian epoch specifying both the reference
 *         frame and the epoch of the position - usually 2000.
 *         For positions where the epoch and equinox are
 *         different, use the routine sla_pm to apply proper
 *         motion corrections before using this routine.
 *
 *     2)  The distinction between the required TDB and TDT is
 *         always negligible.  Moreover, for all but the most
 *         critical applications UTC is adequate.
 *
 *     3)  The proper motions in ra are dra/dt rather than
 *         cos(dec)*dra/dt.
 *
 *     4)  This routine may be wasteful for some applications
 *         because it recomputes the earth position/velocity and
 *         the precession/nutation matrix each time, and because
 *         it allows for parallax and proper motion.  Where
 *         multiple transformations are to be carried out for one
 *         epoch, a faster method is to call the sla_mapp routine
 *         once and then either the sla_mapq routine (which includes
 *         parallax and proper motion) or sla_mapqz (which assumes
 *         zero parallax and proper motion).
 *
 *
 *  p.t.wallace   starlink   july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_mapp (double eq, double date, double *pmt, double *eb,
                     double *ehn, double *gr2e, double *abv, double *ab1,
                     double *ab2, double (*pnm)[3]);
extern int sla_mapq (double rm, double dm, double pr, double pd, double px,
                     double rv, double pmt, double *eb, double *ehn,
                     double gr2e, double *abv, double ab1, double ab2,
                     double (*pnm)[3], double *ra, double *da);

sla_map (double rm, double dm, double pr, double pd, double px, double rv,
         double eq, double date, double *ra, double *da)
{

    double pmt, eb[3], ehn[3], gr2e, abv[3], ab1, ab2, pnm[3][3];

    /* star-independent parameters */
    sla_mapp (eq, date, &pmt, eb, ehn, &gr2e, abv, &ab1, &ab2, pnm);

    /* mean to apparent */
    sla_mapq (rm, dm, pr, pd, px, rv, pmt, eb, ehn, gr2e,
              abv, ab1, ab2, pnm, ra, da);
}
