#include <math.h>

/**
 *
 *     - - - - -
 *      m a p p
 *     - - - - -
 *
 *
 *  compute star-independent parameters in preparation for
 *  conversions between mean place and geocentric apparent place.
 *
 *  The parameters produced by this routine are required in the
 *  parallax, light deflection, aberration, and precession/nutation
 *  parts of the mean/apparent transformations.
 *
 *  The reference frames and timescales used are post IAU 1976.
 *
 *  references:
 *     1984 Astronomical Almanac, pp b39-b41.
 *     (also Lederle & Schwan, Astron. Astrophys. 134,
 *      1-6, 1984)
 *
 *  given:
 *     eq       dp     epoch of mean equinox to be used (Julian)
 *     date     dp     TDB (JD-2400000.5)
 *
 *  returned:
 *     pmt      dp     time interval for proper motion (Julian years)
 *     eb      dp(3)   barycentric position of the earth (au)
 *     ehn     dp(3)   heliocentric direction of the earth (unit vector)
 *     gr2e     dp     (grav rad sun)*2/(sun-earth distance)
 *     abv     dp(3)   barycentric earth velocity in units of c
 *     ab1      dp     sqrt(1-v**2) where v=modulus(abv)
 *     ab2      dp     ab1+1
 *     pnm    dp(3,3)  precession/nutation matrix
 *
 *  called:
 *     sla_epj         mdj to julian epoch
 *     sla_evp         earth position & velocity
 *     sla_dvn         normalise vector
 *     sla_prenut      precession/nutation matrix
 *
 *
 *  notes:
 *
 *    1) The distinction between the required TDB and TDT is
 *       always negligible.  Moreover, for all but the most
 *       critical applications UTC is adequate.
 *
 *    2) The accuracy is limited by the routine sla_evp (qv),
 *       which computes the earth positions and velocities using
 *       the methods of Stumpff.  The maximum error in resulting
 *       aberration corrections is about 0.3 milliarcsecond.
 *
 *    3) The vectors eb and ehn are referred to the mean equinox
 *       and equator of epoch eq.
 *
 *
 *  p.t.wallace   starlink   july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_evp (double date, double deqx, double *dvb, double *dpb,
                    double *dvh, double *dph);
extern int sla_dvn (double *v, double *uv, double *vm);
extern int sla_prenut (double epoch, double date, double (*rmatpn)[3]);

sla_mapp (double eq, double date, double *pmt, double *eb, double *ehn,
          double *gr2e, double *abv, double *ab1, double *ab2,
          double (*pnm)[3])
{

    /* light time for 1 au (sec) */
#define cr  (double)499.004782e0

    /* gravitational radius of the sun x 2 (2*mu/c**2, au) */
#define gr2  (double)2e0*9.87063e-9

    int i;
    double ebd[3], ehd[3], eh[3], e, vn[3], vm;
    double sla_epj (double date);

    /* time interval for proper motion correction */
    *pmt = sla_epj (date) - eq;

    /* get earth barycentric and heliocentric position and velocity */
    sla_evp (date, eq, ebd, eb, ehd, eh);

    /* heliocentric direction of earth (normalised) and modulus */
    sla_dvn (eh, ehn, &e);

    /* aberration parameters */
    for (i = 0; i < 3; i++)
    {
        abv[i] = ebd[i] * cr;
    }
    sla_dvn (abv, vn, &vm);
    *ab1 = sqrt (1e0 - vm * vm);
    *ab2 = *ab1 + 1e0;

    /* light deflection parameter */
    *gr2e = gr2 / e;

    /* precession/nutation matrix */
    sla_prenut (eq, date, pnm);
}
