#include <math.h>

/**
 *
 *     - - - - -
 *      m a p q
 *     - - - - -
 *
 *
 *  quick mean to apparent place:  transform a star ra,dec from
 *  mean place to geocentric apparent place, given the
 *  star-independent parameters.
 *
 *  Use of this routine is appropriate when efficiency is important
 *  and where many star positions, all referred to the same equator
 *  and equinox, are to be transformed for one epoch.  The
 *  star-independent parameters can be obtained by calling the
 *  sla_mapp routine.
 *
 *  If the parallax and proper motions are zero the sla_mapqz can
 *  be used instead.
 *
 *  The reference frames and timescales used are post IAU 1976.
 *
 *  references:
 *     1984 Astronomical Almanac, pp b39-b41.
 *     (also Lederle & Schwan, Astron. Astrophys. 134,
 *      1-6, 1984)
 *
 *  given:
 *     rm,dm    dp     mean ra,dec (rad)
 *     pr,pd    dp     proper motions:  ra,dec changes per julian year
 *     px       dp     parallax (arcsec)
 *     rv       dp     radial velocity (km/sec, +ve if receeding)
 *     pmt      dp     time interval for proper motion (julian years)
 *     eb      dp(3)   barycentric position of the earth (au)
 *     ehn     dp(3)   heliocentric direction of the earth (unit vector)
 *     gr2e     dp     (grav rad sun)*2/(sun-earth distance)
 *     abv     dp(3)   barycentric earth velocity in units of c
 *     ab1      dp     sqrt(1-v**2) where v=modulus(abv)
 *     ab2      dp     ab1+1
 *     pnm    dp(3,3)  precession/nutation matrix
 *
 *  returned:
 *     ra,da    dp     apparent ra,dec (rad)
 *
 *  called:
 *     sla_dcs2c       spherical to cartesian
 *     sla_dvdv        dot product
 *     sla_dmxv        matrix x vector
 *     sla_dcc2s       cartesian to spherical
 *     sla_dranrm      normalise angle 0-2pi
 *
 *  notes:
 *    1)  The vectors eb and ehn are referred to the mean equinox
 *        and equator of epoch eq.
 *    2)  Within about 300 arcsec of the centre of the sun the
 *        gravitational deflection term is set to zero to avoid
 *        overflow.  Otherwise no account is taken of the
 *        impossibility of observing stars which lie behind
 *        the sun.
 *
 *  p.t.wallace   starlink   february 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dvn (double *v, double *uv, double *vm);
extern int sla_dmxv (double (*dm)[3], double *va, double *vb);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_mapq (double rm, double dm, double pr, double pd, double px, double rv,
          double pmt, double *eb, double *ehn, double gr2e, double *abv,
          double ab1, double ab2, double (*pnm)[3], double *ra, double *da)
{

    /* arc seconds to radians */
#define as2r  (double)0.4848136811095359949e-5

    /* km/s to au/year */
#define vf  (double)0.21094502e0

    int i;
    double q[3], pxr, w, em[3], p[3], pn[3], pde, pdep1;
    double p1[3], p1dv, p1dvp1, p2[3], p3[3];
    double sla_dvdv (double *va, double *vb), sla_dranrm (double angle);

    /* spherical to x,y,z */
    sla_dcs2c (rm, dm, q);

    /* space motion (radians per year) */
    pxr = px * as2r;
    w = vf * rv * pxr;
    em[0] = -pr * q[1] - pd * cos (rm) * sin (dm) + w * q[0];
    em[1] = pr * q[0] - pd * sin (rm) * sin (dm) + w * q[1];
    em[2] = pd * cos (dm) + w * q[2];

    /* geocentric direction of star (normalised) */
    for (i = 0; i < 3; i++)
    {
        p[i] = q[i] + pmt * em[i] - pxr * eb[i];
    }
    sla_dvn (p, pn, &w);

    /* light deflection */
    pde = sla_dvdv (pn, ehn);
    pdep1 = 1e0 + pde;
    for (i = 0; i < 3; i++)
    {
        p1[i] = pn[i] + gr2e *
            (ehn[i] - pde * pn[i]) / (pdep1 > 1e-6 ? pdep1 : 1e-6);
    }

    /* aberration */
    p1dv = sla_dvdv (p1, abv);
    p1dvp1 = p1dv + 1e0;
    for (i = 0; i < 3; i++)
    {
        p2[i] = (ab1 * p1[i] + abv[i] + p1dv * abv[i] / ab2) / p1dvp1;
    }

    /* precession and nutation */
    sla_dmxv (pnm, p2, p3);

    /* geocentric apparent ra,dec */
    sla_dcc2s (p3, ra, da);
    *ra = sla_dranrm (*ra);
}
