
/**
 *
 *     - - - - - -
 *      m a p q z
 *     - - - - - -
 *
 *
 *  quick mean to apparent place:  transform a star ra,dec from
 *  mean place to geocentric apparent place, given the
 *  star-independent parameters, and assuming zero parallax
 *  and proper motion.
 *
 *
 *  Use of this routine is appropriate when efficiency is important
 *  and where many star positions, all with parallax and proper
 *  motion either zero or already allowed for, and all referred to
 *  the same equator and equinox, are to be transformed for one
 *  epoch.  The star-independent parameters can be obtained by
 *  calling the sla_mapp routine.
 *
 *  The corresponding routine for the case of non-zero parallax
 *  and proper motion is sla_mapq.
 *
 *  The reference frames and timescales used are post IAU 1976.
 *
 *  references:
 *     1984 Astronomical Almanac, pp b39-b41.
 *     (also Lederle & Schwan, Astron. Astrophys. 134,
 *      1-6, 1984)
 *
 *  given:
 *     rm,dm    dp     mean ra,dec (rad)
 *     ehn     dp(3)   heliocentric direction of the earth (unit vector)
 *     gr2e     dp     (grav rad sun)*2/(sun-earth distance)
 *     abv     dp(3)   barycentric earth velocity in units of c
 *     ab1      dp     sqrt(1-v**2) where v=modulus(abv)
 *     ab2      dp     ab1+1
 *     pnm    dp(3,3)  precession/nutation matrix
 *
 *  returned:
 *     ra,da    dp     apparent ra,dec (rad)
 *
 *  called:
 *     sla_dcs2c       spherical to cartesian
 *     sla_dvdv        dot product
 *     sla_dmxv        matrix x vector
 *     sla_dcc2s       cartesian to spherical
 *     sla_dranrm      normalise angle 0-2pi
 *
 *  note: The vectors eb and ehn are referred to the mean equinox
 *        and equator of epoch eq.
 *
 *  p.t.wallace   starlink   july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dmxv (double (*dm)[3], double *va, double *vb);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_mapqz (double rm, double dm, double *ehn, double gr2e, double *abv,
           double ab1, double ab2, double (*pnm)[3], double *ra, double *da)
{

    int i;
    double p[3], pde, pdep1, p1[3], p1dv, p1dvp1, p2[3], p3[3];
    double sla_dvdv (double *va, double *vb), sla_dranrm (double angle);

    /* spherical to x,y,z */
    sla_dcs2c (rm, dm, p);

    /* light deflection */
    pde = sla_dvdv (p, ehn);
    pdep1 = 1e0 + pde;
    for (i = 0; i < 3; i++)
    {
        p1[i] = p[i] + gr2e * (ehn[i] - pde * p[i]) / pdep1;
    }

    /* aberration */
    p1dv = sla_dvdv (p1, abv);
    p1dvp1 = p1dv + 1e0;
    for (i = 0; i < 3; i++)
    {
        p2[i] = (ab1 * p1[i] + abv[i] + p1dv * abv[i] / ab2) / p1dvp1;
    }

    /* precession and nutation */
    sla_dmxv (pnm, p2, p3);

    /* geocentric apparent ra,dec */
    sla_dcc2s (p3, ra, da);
    *ra = sla_dranrm (*ra);
}
