#include <math.h>

/**
 *
 *     - - - - -
 *      m o o n
 *     - - - - -
 *
 *
 *  approximate geocentric position and velocity of the moon
 *
 *
 *  given:
 *     iy       i       year AD
 *     id       i       day in year (1 = Jan 1st)
 *     fd       r       fraction of day
 *
 *  returned:
 *     posvel   r(6)    moon position & velocity vector
 *
 *  called:
 *     sla_cs2c6, sla_mxv
 *
 *
 *  The date and time is TDB (loosely ET) in the Gregorian
 *  calendar, and is interpreted in a manner which is valid
 *  between 1900 March 1 and 2100 February 28.
 *
 *  The moon 6-vector is moon centre relative to earth centre,
 *  mean equator and equinox of date.  Position part, posvel(0-2),
 *  is in au;  velocity part, posvel(3-5), is in au/sec.
 *
 *  The position is accurate to better than 0.5 arcminute
 *  in direction and 1e-5 au in distance.  The velocity
 *  is accurate to better than 0.5"/hour in direction and
 *  4 m/s in distance.  Note that the distance accuracy is
 *  comparatively poor because this routine is merely
 *  intended for use in computing topocentric direction.
 *
 *  reference:
 *     Meeus, L'Astronomie, June 1984, p348.
 *
 *  This routine is only a partial implementation of the original
 *  algorithm, which offers somewhat higher accuracy in direction
 *  and much higher accuracy in distance (or rather in horizontal
 *  parallax) when fully implemented.
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_cs2c6 (double a, double b, double r, double ad, double bd,
                      double rd, double *v);
extern int sla_mxv (double (*rm)[3], double *va, double *vb);

sla_moon (int iy, int id, double fd, double *posvel)
                                        /* variables were REAL */
{

    int i, m, n;

    /* all following variables were REAL */
    double v1[6], v2[6], rmat[3][3];

    /* declared below 
     * double  elp0, elp1, elp1i, elp1f;
     * double  em0, em1, em1i, em1f;
     * double  emp0, emp1, emp1i, emp1f;
     * double  d0, d1, d1i, d1f;
     * double  f0, f1, f1i, f1f;
     * double  tl[39];
     * double  tb[29];
     * double  tp[4];
     */
    double junk;
    double yi, yf, t, elp, em, emp, d, f, el, eld, coeff, cem, cemp;
    double cd, cf, theta, thetad, b, bd, p, pd, r, rd;
    double eps, seps, ceps;

    /* degrees to radians */
#define d2r  1.745329252e-2

    /* rate conversion factor: d2r**2 / (86400 * 365.25) */
#define ratcon  9.652743551e-12

    /* earth radius in au */
#define eradau  4.26352e-5

    /*
     * coefficients for fundamental arguments
     *
     * fixed term (deg), term in t (deg & whole revs + fraction per year)
     *
     * moon's mean longitude
     */
    static double elp0 = 270.4342;
    static double elp1 = 4812.678831;
    static double elp1i = 4680.;
    static double elp1f = 132.678831;

    /*
     * sun's mean anomaly
     */
    static double em0 = 358.4758;
    static double em1 = 359.990498;
    static double em1i = 0.;
    static double em1f = 359.990498;

    /*
     * moon's mean anomaly
     */
    static double emp0 = 296.1046;
    static double emp1 = 4771.988491;
    static double emp1i = 4680.;
    static double emp1f = 91.988491;

    /*
     * moon's mean elongation
     */
    static double d0 = 350.7375;
    static double d1 = 4452.671142;
    static double d1i = 4320.;
    static double d1f = 132.671142;

    /*
     * mean distance of the moon from its ascending node
     */
    static double f0 = 11.2509;
    static double f1 = 4832.020251;
    static double f1i = 4680.;
    static double f1f = 152.020251;

    /*
     * Converting between FORTRAN and C should mean that
     * the itl, itb, itp matrices have their columns and
     * rows swapped, so initialization can be done and the
     * code run as is.  Too error prone.  So, leave init
     * "as is" and swap indices where arrays are used.
     */

    /*
     * coefficients for moon position
     *
     * t(n)       = coefficient of term (deg)
     * it(n,1-4)  = coefficients of m, m', d, f in argument
     *
     * longitude
     */
    static double tl[39] = {
        6.288750, 1.274018, 0.658309, 0.213616, -0.185596, -0.114336,
        0.058793, 0.057212, 0.053320, 0.045874, 0.041024, -0.034718,
        -0.030465, 0.015326, -0.012528, -0.010980, 0.010674, 0.010034,
        0.008548, -0.007910, -0.006783, 0.005162, 0.005000, 0.004049,
        0.003996, 0.003862, 0.003665, 0.002695, 0.002602, 0.002396,
        -0.002349, 0.002249, -0.002125, -0.002079, 0.002059, -0.001773,
        -0.001595, 0.001220, -0.001110
    };
    static short itl[39][4] = {
        /* m   m'  d   f */
        0, 1, 0, 0,
        0, -1, 2, 0,
        0, 0, 2, 0,
        0, 2, 0, 0,
        1, 0, 0, 0,
        0, 0, 0, 2,
        0, -2, 2, 0,
        -1, -1, 2, 0,
        0, 1, 2, 0,
        -1, 0, 2, 0,
        -1, 1, 0, 0,
        0, 0, 1, 0,
        1, 1, 0, 0,
        0, 0, 2, -2,
        0, 1, 0, 2,
        0, -1, 0, 2,
        0, -1, 4, 0,
        0, 3, 0, 0,
        0, -2, 4, 0,
        1, -1, 2, 0,
        1, 0, 2, 0,
        0, 1, -1, 0,
        1, 0, 1, 0,
        -1, 1, 2, 0,
        0, 2, 2, 0,
        0, 0, 4, 0,
        0, -3, 2, 0,
        -1, 2, 0, 0,
        0, 1, -2, -2,
        -1, -2, 2, 0,
        0, 1, 1, 0,
        -2, 0, 2, 0,
        1, 2, 0, 0,
        2, 0, 0, 0,
        -2, -1, 2, 0,
        0, 1, 2, -2,
        0, 0, 2, 2,
        -1, -1, 4, 0,
        0, 2, 0, 2
    };

    /*
     * latitude
     */
    static double tb[29] = {
        5.128189, 0.280606, 0.277693, 0.173238, 0.055413, 0.046272,
        0.032573, 0.017198, 0.009267, 0.008823, 0.008247, 0.004323,
        0.004200, 0.003372, 0.002472, 0.002222, 0.002072, 0.001877,
        0.001828, -0.001803, -0.001750, 0.001570, -0.001487, -0.001481,
        0.001417, 0.001350, 0.001330, 0.001106, 0.001020
    };

    static short itb[29][4] = {
        /* m   m'  d   f */
        0, 0, 0, 1,
        0, 1, 0, 1,
        0, 1, 0, -1,
        0, 0, 2, -1,
        0, -1, 2, 1,
        0, -1, 2, -1,
        0, 0, 2, 1,
        0, 2, 0, 1,
        0, 1, 2, -1,
        0, 2, 0, -1,
        -1, 0, 2, -1,
        0, -2, 2, -1,
        0, 1, 2, 1,
        -1, 0, -2, 1,
        -1, -1, 2, 1,
        -1, 0, 2, 1,
        -1, -1, 2, -1,
        -1, 1, 0, 1,
        0, -1, 4, -1,
        1, 0, 0, 1,
        0, 0, 0, 3,
        -1, 1, 0, -1,
        0, 0, 1, 1,
        1, 1, 0, 1,
        -1, -1, 0, 1,
        -1, 0, 0, 1,
        0, 0, -1, 1,
        0, 3, 0, 1,
        0, 0, 4, -1
    };

    /*
     * parallax
     */
    static double tp[4] = {
        0.051818, 0.009531, 0.007843, 0.002824
    };

    static short itp[4][4] = {
        /* m   m'  d   f */
        0, 1, 0, 0,
        0, -1, 2, 0,
        0, 0, 2, 0,
        0, 2, 0, 0
    };

    /* whole years & fraction of year, and years since j1900.0 */
    yi = (double) (iy - 1900);
    yf = ((double) (4 * (id - 1 / ((iy % 4) + 1)) - (iy % 4) - 2) + 4.0 * fd)
        / 1461.0;
    t = yi + yf;

    /* moon's mean longitude */
    elp = d2r * modf ((elp0 + elp1i * yf + elp1f * t) / 360.0, &junk) * 360.0;

    /* sun's mean anomaly */
    em = d2r * modf ((em0 + em1i + em1f * t) / 360.0, &junk) * 360.0;

    /* moon's mean anomaly */
    emp = d2r * modf ((emp0 + emp1i * yf + emp1f * t) / 360.0, &junk) * 360.0;

    /* moon's mean elongation */
    d = d2r * modf ((d0 + d1i * yf + d1f * t) / 360.0, &junk) * 360.0;

    /* mean distance of the moon from its ascending node */
    f = d2r * modf ((f0 + f1i * yf + f1f * t) / 360.0, &junk) * 360.0;

    /*
     * itl, itb, itp use swapped indices relative to original code.
     * See discussion above.
     */

    /* longitude */
    el = 0.0;
    eld = 0.0;
    for (m = 0; m < 39; m++)
    {
        n = 38 - m;
        coeff = tl[n];
        cem = (double) itl[n][0];
        cemp = (double) itl[n][1];
        cd = (double) itl[n][2];
        cf = (double) itl[n][3];
        theta = cem * em + cemp * emp + cd * d + cf * f;
        thetad = cem * em1 + cemp * emp1 + cd * d1 + cf * f1;
        el += coeff * sin (theta);
        eld += coeff * cos (theta) * thetad;
    }
    el = el * d2r + elp;
    eld = ratcon * (eld + elp1 / d2r);

    /* latitude */
    b = 0.0;
    bd = 0.0;
    for (m = 0; m < 29; m++)
    {
        n = 28 - m;
        coeff = tb[n];
        cem = (double) itb[n][0];
        cemp = (double) itb[n][1];
        cd = (double) itb[n][2];
        cf = (double) itb[n][3];
        theta = cem * em + cemp * emp + cd * d + cf * f;
        thetad = cem * em1 + cemp * emp1 + cd * d1 + cf * f1;
        b += coeff * sin (theta);
        bd += coeff * cos (theta) * thetad;
    }
    b = b * d2r;
    bd = ratcon * bd;

    /* parallax */
    p = 0.0;
    pd = 0.0;
    for (m = 0; m < 4; m++)
    {
        n = 3 - m;
        coeff = tp[n];
        cem = (double) itp[n][0];
        cemp = (double) itp[n][1];
        cd = (double) itp[n][2];
        cf = (double) itp[n][3];
        theta = cem * em + cemp * emp + cd * d + cf * f;
        thetad = cem * em1 + cemp * emp1 + cd * d1 + cf * f1;
        p += coeff * cos (theta);
        pd -= coeff * sin (theta) * thetad;
    }
    p = (p + 0.950724) * d2r;
    pd = ratcon * pd;

    /* transform parallax to distance (au, au/sec) */
    r = eradau / p;
    rd = -r * pd / p;

    /* longitude, latitude to x,y,z (au) */
    sla_cs2c6 (el, b, r, eld, bd, rd, v1);

    /* mean obliquity */
    eps = d2r * (23.45229 - 0.00013 * t);

    /* rotation matrix - ecliptic to equatorial */
    seps = sin (eps);
    ceps = cos (eps);
    rmat[0][0] = 1.0;
    rmat[1][0] = 0.0;
    rmat[2][0] = 0.0;
    rmat[0][1] = 0.0;
    rmat[1][1] = ceps;
    rmat[2][1] = seps;
    rmat[0][2] = 0.0;
    rmat[1][2] = -seps;
    rmat[2][2] = ceps;

    /* rotate into equatorial system */
    sla_mxv (rmat, v1, v2);
    sla_mxv (rmat, &v1[3], &v2[3]);

    /* return results */
    for (i = 0; i < 6; i++)
    {
        posvel[i] = v2[i];
    }
}
