#include <math.h>

/**
 *
 *     - - - - -
 *      n u t c
 *     - - - - -
 *
 *
 *  nutation:  longitude & obliquity components and
 *             mean obliquity (IAU 1980 theory)
 *
 *  (double precision)
 *
 *
 *  references:
 *     Final report of the IAU working group on nutation,
 *      chairman P.K.Seidelmann, 1980.
 *     Kaplan,G.H., 1981, USNO circular no. 163, pa3-6.
 *
 *
 *  given:
 *
 *     date        dp    TDB (loosely ET) as modified Julian date
 *                                            (JD-2400000.5)
 *
 *  returned:
 *
 *     dpsi,deps   dp    nutation in longitude,obliquity
 *     eps0        dp    mean obliquity
 *
 *
 *  p.t.wallace   starlink   september 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_nutc (double date, double *dpsi, double *deps, double *eps0)
{

    double t, el, el2, el3;
    double elp, elp2;
    double f, f2, f4;
    double d, d2, d4;
    double om, om2;
    double dp, de;
    double a;

    /* turns to arc seconds */
#define t2as  1296000e0

    /* arc seconds to radians */
#define as2r  0.4848136811095359949e-05

    /* units of 0.0001 arcsec to radians */
#define u2r  (as2r/1e4)

    /* interval between basic epoch j2000.0 and current epoch (JC) */
    t = (date - 51544.5e0) / 36525e0;

    /*
     * fundamental arguments in the FK5 reference system
     *
     *
     * mean longitude of the moon minus mean longitude of the moon's
     * perigee 
     */
    el = as2r * (485866.733e0 +
                 (1325e0 * t2as + 715922.633e0 +
                  (31.310e0 + 0.064e0 * t) * t) * t);

    /*
     * mean longitude of the sun minus mean longitude of the sun's
     * perigee 
     */
    elp = as2r * (1287099.804e0 +
                  (99e0 * t2as + 1292581.224e0 +
                   (-0.577e0 - 0.012e0 * t) * t) * t);

    /* mean longitude of the moon minus mean longitude of the moon's node */
    f = as2r * (335778.877e0 +
                (1342e0 * t2as + 295263.137e0 +
                 (-13.257e0 + 0.011e0 * t) * t) * t);

    /* mean elongation of the moon from the sun */
    d = as2r * (1072261.307e0 +
                (1236e0 * t2as + 1105601.328e0 +
                 (-6.891e0 + 0.019e0 * t) * t) * t);

    /* longitude of the mean ascending node of the lunar orbit on the
     * ecliptic, measured from the mean equinox of date
     */
    om = as2r * (450160.280e0 +
                 (-5e0 * t2as - 482890.539e0 +
                  (7.455e0 + 0.008e0 * t) * t) * t);

    /* multiples of arguments */
    el2 = el + el;
    el3 = el2 + el;
    elp2 = elp + elp;
    f2 = f + f;
    f4 = f2 + f2;
    d2 = d + d;
    d4 = d2 + d2;
    om2 = om + om;

    /*
     * series for the nutation
     */
    dp = 0e0;
    de = 0e0;

    /* 106 */
    dp += sin (elp + d);

    /* 105 */
    dp -= sin (f2 + d4 + om2);

    /* 104 */
    dp += sin (el2 + d2);

    /* 103 */
    dp -= sin (el - f2 + d2);

    /* 102 */
    dp -= sin (el + elp - d2 + om);

    /* 101 */
    dp -= sin (-elp + f2 + om);

    /* 100 */
    dp -= sin (el - f2 - d2);

    /* 99 */
    dp -= sin (elp + d2);

    /* 98 */
    dp -= sin (f2 - d + om2);

    /* 97 */
    dp -= sin (-f2 + om);

    /* 96 */
    dp += sin (-el - elp + d2 + om);

    /* 95 */
    dp += sin (elp + f2 + om);

    /* 94 */
    dp -= sin (el + f2 - d2);

    /* 93 */
    dp += sin (el3 + f2 - d2 + om2);

    /* 92 */
    dp += sin (f4 - d2 + om2);

    /* 91 */
    dp -= sin (el + d2 + om);

    /* 90 */
    dp -= sin (el2 + f2 + d2 + om2);

    /* 89 */
    a = el2 + f2 - d2 + om;
    dp += sin (a);
    de -= cos (a);

    /* 88 */
    dp += sin (el - elp - d2);

    /* 87 */
    dp += sin (-el + f4 + om2);

    /* 86 */
    a = -el2 + f2 + d4 + om2;
    dp -= sin (a);
    de += cos (a);

    /* 85 */
    a = el + f2 + d2 + om;
    dp -= sin (a);
    de += cos (a);

    /* 84 */
    a = el + elp + f2 - d2 + om2;
    dp += sin (a);
    de -= cos (a);

    /* 83 */
    dp -= sin (el2 - d4);

    /* 82 */
    a = -el + f2 + d4 + om2;
    dp -= 2e0 * sin (a);
    de += cos (a);

    /* 81 */
    a = -el2 + f2 + d2 + om2;
    dp += sin (a);
    de -= cos (a);

    /* 80 */
    dp -= sin (el - d4);

    /* 79 */
    a = -el + om2;
    dp += sin (a);
    de -= cos (a);

    /* 78 */
    a = f2 + d + om2;
    dp += 2e0 * sin (a);
    de -= cos (a);

    /* 77 */
    dp += 2e0 * sin (el3);

    /* 76 */
    a = el + om2;
    dp -= 2e0 * sin (a);
    de += cos (a);

    /* 75 */
    a = el2 + om;
    dp += 2e0 * sin (a);
    de -= cos (a);

    /* 74 */
    a = -el + f2 - d2 + om;
    dp -= 2e0 * sin (a);
    de += cos (a);

    /* 73 */
    a = el + elp + f2 + om2;
    dp += 2e0 * sin (a);
    de -= cos (a);

    /* 72 */
    a = -elp + f2 + d2 + om2;
    dp -= 3e0 * sin (a);
    de += cos (a);

    /* 71 */
    a = el3 + f2 + om2;
    dp -= 3e0 * sin (a);
    de += cos (a);

    /* 70 */
    a = -el2 + om;
    dp -= 2e0 * sin (a);
    de += cos (a);

    /* 69 */
    a = -el - elp + f2 + d2 + om2;
    dp -= 3e0 * sin (a);
    de += cos (a);

    /* 68 */
    a = el - elp + f2 + om2;
    dp -= 3e0 * sin (a);
    de += cos (a);

    /* 67 */
    dp += 3e0 * sin (el + f2);

    /* 66 */
    dp -= 3e0 * sin (el + elp);

    /* 65 */
    dp -= 4e0 * sin (d);

    /* 64 */
    dp += 4e0 * sin (el - f2);

    /* 63 */
    dp -= 4e0 * sin (elp - d2);

    /* 62 */
    a = el2 + f2 + om;
    dp -= 5e0 * sin (a);
    de += 3e0 * cos (a);

    /* 61 */
    dp += 5e0 * sin (el - elp);

    /* 60 */
    a = -d2 + om;
    dp -= 5e0 * sin (a);
    de += 3e0 * cos (a);

    /* 59 */
    a = el + f2 - d2 + om;
    dp += 6e0 * sin (a);
    de -= 3e0 * cos (a);

    /* 58 */
    a = f2 + d2 + om;
    dp -= 7e0 * sin (a);
    de += 3e0 * cos (a);

    /* 57 */
    a = d2 + om;
    dp -= 6e0 * sin (a);
    de += 3e0 * cos (a);

    /* 56 */
    a = el2 + f2 - d2 + om2;
    dp += 6e0 * sin (a);
    de -= 3e0 * cos (a);

    /* 55 */
    dp += 6e0 * sin (el + d2);

    /* 54 */
    a = el + f2 + d2 + om2;
    dp -= 8e0 * sin (a);
    de += 3e0 * cos (a);

    /* 53 */
    a = -elp + f2 + om2;
    dp -= 7e0 * sin (a);
    de += 3e0 * cos (a);

    /* 52 */
    a = elp + f2 + om2;
    dp += 7e0 * sin (a);
    de -= 3e0 * cos (a);

    /* 51 */
    dp -= 7e0 * sin (el + elp - d2);

    /* 50 */
    a = -el + f2 + d2 + om;
    dp -= 10e0 * sin (a);
    de += 5e0 * cos (a);

    /* 49 */
    a = el - d2 + om;
    dp -= 13e0 * sin (a);
    de += 7e0 * cos (a);

    /* 48 */
    a = -el + d2 + om;
    dp += 16e0 * sin (a);
    de -= 8e0 * cos (a);

    /* 47 */
    a = -el + f2 + om;
    dp += 21e0 * sin (a);
    de -= 10e0 * cos (a);

    /* 46 */
    dp += 26e0 * sin (f2);
    de -= cos (f2);

    /* 45 */
    a = el2 + f2 + om2;
    dp -= 31e0 * sin (a);
    de += 13e0 * cos (a);

    /* 44 */
    a = el + f2 - d2 + om2;
    dp += 29e0 * sin (a);
    de -= 12e0 * cos (a);

    /* 43 */
    dp += 29e0 * sin (el2);
    de -= cos (el2);

    /* 42 */
    a = f2 + d2 + om2;
    dp -= 38e0 * sin (a);
    de += 16e0 * cos (a);

    /* 41 */
    a = el + f2 + om;
    dp -= 51e0 * sin (a);
    de += 27e0 * cos (a);

    /* 40 */
    a = -el + f2 + d2 + om2;
    dp -= 59e0 * sin (a);
    de += 26e0 * cos (a);

    /* 39 */
    a = -el + om;
    dp += (-58e0 - 0.1e0 * t) * sin (a);
    de += 32e0 * cos (a);

    /* 38 */
    a = el + om;
    dp += (63e0 + 0.1e0 * t) * sin (a);
    de -= 33e0 * cos (a);

    /* 37 */
    dp += 63e0 * sin (d2);
    de -= 2e0 * cos (d2);

    /* 36 */
    a = -el + f2 + om2;
    dp += 123e0 * sin (a);
    de -= 53e0 * cos (a);

    /* 35 */
    a = el - d2;
    dp -= 158e0 * sin (a);
    de -= cos (a);

    /* 34 */
    a = el + f2 + om2;
    dp -= 301e0 * sin (a);
    de += (129e0 - 0.1e0 * t) * cos (a);

    /* 33 */
    a = f2 + om;
    dp += (-386e0 - 0.4e0 * t) * sin (a);
    de += 200e0 * cos (a);

    /* 32 */
    dp += (712e0 + 0.1e0 * t) * sin (el);
    de -= 7e0 * cos (el);

    /* 31 */
    a = f2 + om2;
    dp += (-2274e0 - 0.2e0 * t) * sin (a);
    de += (977e0 - 0.5e0 * t) * cos (a);

    /* 30 */
    dp -= sin (elp + f2 - d2);

    /* 29 */
    dp += sin (-el + d + om);

    /* 28 */
    dp += sin (elp + om2);

    /* 27 */
    dp -= sin (elp - f2 + d2);

    /* 26 */
    dp += sin (-f2 + d2 + om);

    /* 25 */
    dp += sin (el2 + elp - d2);

    /* 24 */
    dp -= 4e0 * sin (el - d);

    /* 23 */
    a = elp + f2 - d2 + om;
    dp += 4e0 * sin (a);
    de -= 2e0 * cos (a);

    /* 22 */
    a = el2 - d2 + om;
    dp += 4e0 * sin (a);
    de -= 2e0 * cos (a);

    /* 21 */
    a = -elp + f2 - d2 + om;
    dp -= 5e0 * sin (a);
    de += 3e0 * cos (a);

    /* 20 */
    a = -el2 + d2 + om;
    dp -= 6e0 * sin (a);
    de += 3e0 * cos (a);

    /* 19 */
    a = -elp + om;
    dp -= 12e0 * sin (a);
    de += 6e0 * cos (a);

    /* 18 */
    a = elp2 + f2 - d2 + om2;
    dp += (-16e0 + 0.1e0 * t) * sin (a);
    de += 7e0 * cos (a);

    /* 17 */
    a = elp + om;
    dp -= 15e0 * sin (a);
    de += 9e0 * cos (a);

    /* 16 */
    dp += (17e0 - 0.1e0 * t) * sin (elp2);

    /* 15 */
    dp -= 22e0 * sin (f2 - d2);

    /* 14 */
    a = el2 - d2;
    dp += 48e0 * sin (a);
    de += cos (a);

    /* 13 */
    a = f2 - d2 + om;
    dp += (129e0 + 0.1e0 * t) * sin (a);
    de -= 70e0 * cos (a);

    /* 12 */
    a = -elp + f2 - d2 + om2;
    dp += (217e0 - 0.5e0 * t) * sin (a);
    de += (-95e0 + 0.3e0 * t) * cos (a);

    /* 11 */
    a = elp + f2 - d2 + om2;
    dp += (-517e0 + 1.2e0 * t) * sin (a);
    de += (224e0 - 0.6e0 * t) * cos (a);

    /* 10 */
    dp += (1426e0 - 3.4e0 * t) * sin (elp);
    de += (54e0 - 0.1e0 * t) * cos (elp);

    /* 9 */
    a = f2 - d2 + om2;
    dp += (-13187e0 - 1.6e0 * t) * sin (a);
    de += (5736e0 - 3.1e0 * t) * cos (a);

    /* 8 */
    dp += sin (el2 - f2 + om);

    /* 7 */
    a = -elp2 + f2 - d2 + om;
    dp -= 2e0 * sin (a);
    de += 1e0 * cos (a);

    /* 6 */
    dp -= 3e0 * sin (el - elp - d);

    /* 5 */
    a = -el2 + f2 + om2;
    dp -= 3e0 * sin (a);
    de += 1e0 * cos (a);

    /* 4 */
    dp += 11e0 * sin (el2 - f2);

    /* 3 */
    a = -el2 + f2 + om;
    dp += 46e0 * sin (a);
    de -= 24e0 * cos (a);

    /* 2 */
    dp += (2062e0 + 0.2e0 * t) * sin (om2);
    de += (-895e0 + 0.5e0 * t) * cos (om2);

    /* 1 */
    dp += (-171996e0 - 174.2e0 * t) * sin (om);
    de += (92025e0 + 8.9e0 * t) * cos (om);

    /* convert results to radians */
    *dpsi = dp * u2r;
    *deps = de * u2r;

    /* mean obliquity */
    *eps0 = as2r * (84381.448e0 +
                    (-46.8150e0 + (-0.00059e0 + 0.001813e0 * t) * t) * t);
}
