#include <string.h>

/**
 *
 *     - - - -
 *      o b s
 *     - - - -
 *
 *
 *  parameters of selected groundbased observing stations
 *
 *
 *  given:
 *     n       int     number specifying observing station
 *
 *  either given or returned
 *     c       *char   identifier specifying observing station
 *
 *  returned:
 *     name    *char   name of specified observing station
 *     w       dp      longitude (radians, west +ve)
 *     p       dp      geodetic latitude (radians, north +ve)
 *     h       dp      height above sea level (metres)
 *
 *
 *  notes:
 *
 *     Station identifiers c may be up to 10 characters long,
 *     and station names name may be up to 40 characters long.
 *
 *     c and n are alternative ways of specifying the observing
 *     station.  The c option, which is the most generally useful,
 *     may be selected by specifying an n value of zero or less.
 *     If n is 1 or more, the parameters of the nth station
 *     in the currently supported list are interrogated, and
 *     the station identifier c is returned as well as name, w,
 *     p and h.
 *
 *     If the station parameters are not available, either because
 *     the station identifier c is not recognised, or because an
 *     n value greater than the number of stations supported is
 *     given, a name of '?' is returned and w, p and h are left in
 *     their current states.
 *
 *     Programs can obtain a list of all currently supported
 *     stations by calling the routine repeatedly, with n=1,2,3...
 *     when name='?' is seen, the list of stations has been
 *     exhausted.
 *
 *     Programs must not assume that the n value corresponding to
 *     a given station will remain the same between successive
 *     releases of this routine.  The station identifiers c will,
 *     however, remain substantially unchanged from one release
 *     to the next.
 *
 *     All station identifiers c are lower case only.
 *     The station names returned may contain both upper
 *     and lower case.
 *
 *     Users are urged to inform the author of any improvements
 *     they would like to see made.  For example:
 *
 *         typographical corrections
 *         more accurate parameters
 *         better station identifiers or names
 *         additional stations
 *
 *
 *  p t wallace   starlink   june 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */
struct dms
{
    int id;
    int iam;
    float as;
};

#define north 0
#define south 1
#define east 1
#define west 0

struct teli
{
    char *name;
    int e_w_flag;               /* true (!= 0) if east */
    struct dms e_w;
    int n_s_flag;               /* true (!= 0) if south */
    struct dms n_s;
    double altitude;
}
telinfo[] =
{

    {
    "anglo-australian 3.9m telescope",
            east, 149, 03, 57.91, south, 31, 16, 37.34, 1164e0}
    ,
    {
    "william herschel 4.2m telescope",
            west, 17, 52, 53.9, north, 28, 45, 38.1, 2332e0}
    ,
    {
    "isaac newton 2.5m telescope",
            west, 17, 52, 39.5, north, 28, 45, 43.2, 2336e0}
    ,
    {
    "jacobus kapteyn 1m telescope",
            west, 17, 52, 41.2, north, 28, 45, 39.9, 2364e0}
    ,
    {
    "lick 120 inch", west, 121, 38, 09.9, north, 37, 20, 35.2, 1290e0}
    ,
    {
    "mmt, mt hopkins", west, 110, 53, 04.3, north, 31, 41, 19.6, 2608e0}
    ,
    {
    "victoria b.c. 1.85 metre",
            west, 123, 25, 01.18, north, 48, 31, 11.9, 238e0}
    ,
    {
    "du pont 2.5m telescope, las campanas",
            west, 70, 42, 9., south, 29, 00, 11., 2280e0}
    ,
    {
    "mt hopkins 1.5 metre",
            west, 110, 52, 39.00, north, 31, 40, 51.4, 2344e0}
    ,
    {
    "mount stromlo 74 inch",
            east, 149, 00, 27.59, south, 35, 19, 14.3, 767e0}
    ,
    {
    "siding spring 2.3 metre",
            east, 149, 03, 40.3, south, 31, 16, 24.1, 1149e0}
    ,
    {
    "greenbank 140 foot", west, 79, 50, 09.61, north, 38, 26, 15.4, 881e0}
    ,
    {
    "cerro tololo 4 metre", west, 70, 48, 53.6, south, 30, 9, 57.8, 2235e0}
    ,
    {
    "cerro tololo 1.5 metre",
            west, 70, 48, 54.5, south, 30, 9, 56.3, 2225e0}
    ,
    {
    "tidbinbilla 64 metre",
            east, 148, 58, 48.20, south, 35, 24, 14.3, 670e0}
    ,
    {
    "bloemfontein 1.52 metre",
            east, 26, 24, 18., south, 29, 02, 18., 1387e0}
    ,
    {
    "bosque alegre 1.54 metre",
            west, 64, 32, 48.0, south, 31, 35, 53., 1250e0}
    ,
    {
    "usno 61 inch astrograph, flagstaff",
            west, 111, 44, 23.6, north, 35, 11, 02.5, 2316e0}
    ,
    {
    "perkins 72 inch, lowell",
            west, 111, 32, 09.3, south, 35, 05, 48.6, 2198e0}
    ,
    {
    "harvard college observatory 1.55m",
            west, 71, 33, 29.32, north, 42, 30, 19.0, 185e0}
    ,
    {
    "okayama 1.88 metre", east, 133, 35, 47.29, north, 34, 34, 26.1, 372e0}
    ,
    {
    "kitt peak 158 inch",
            west, 111, 35, 57.61, north, 31, 57, 50.3, 2120e0}
    ,
    {
    "kitt peak 90 inch", west, 111, 35, 58.24, north, 31, 57, 46.9, 2071e0}
    ,
    {
    "kitt peak 84 inch", west, 111, 35, 51.56, north, 31, 57, 29.2, 2096e0}
    ,
    {
    "kitt peak 36 foot", west, 111, 36, 51.12, north, 31, 57, 12.1, 1939e0}
    ,
    {
    "kottamia 74 inch", east, 31, 49, 30., north, 29, 55, 54., 476e0}
    ,
    {
    "eso 3.6 metre", west, 70, 43, 36., south, 29, 15, 36., 2428e0}
    ,
    {
    "mauna kea 88 inch", west, 155, 28, 20., north, 19, 49, 34., 4215e0}
    ,
    {
    "uk infra red telescope",
            west, 155, 28, 18., north, 19, 49, 35., 4200e0}
    ,
    {
    "quebec 1.6 metre", west, 71, 9, 09.7, north, 45, 27, 20.6, 1114e0}
    ,
    {
    "mt ekar 1.82 metre", east, 11, 34, 15., north, 45, 50, 48., 1365e0}
    ,
    {
    "mt lemmon 60 inch", west, 110, 42, 16.9, north, 32, 26, 33.9, 2790e0}
    ,
    {
    "mcdonald 2.7 metre",
            west, 104, 01, 17.60, north, 30, 40, 17.7, 2075e0}
    ,
    {
    "mcdonald 2.1 metre",
            west, 104, 01, 20.10, north, 30, 40, 17.7, 2075e0}
    ,
    {
    "palomar 200 inch", west, 116, 51, 50., north, 33, 21, 22., 1706e0}
    ,
    {
    "palomar 60 inch", west, 116, 51, 31., north, 33, 20, 56., 1706e0}
    ,
    {
    "david dunlap 74 inch", west, 79, 25, 20., north, 43, 51, 46., 244e0}
    ,
    {
    "haute provence 1.93 metre",
            east, 5, 42, 46.75, north, 43, 55, 53.3, 665e0}
    ,
    {
    "haute provence 1.52 metre",
            east, 5, 42, 43.82, north, 43, 56, 00.2, 667e0}
    ,
    {
    "san pedro martir 83 inch",
            west, 115, 27, 47., north, 31, 02, 38., 2830e0}
    ,
    {
    "sutherland 74 inch", east, 20, 48, 44.3, south, 32, 22, 43.4, 1771e0}
    ,
    {
    "tautenburg 2 metre", east, 11, 42, 45., north, 50, 58, 51., 331e0}
    ,
    {
    "catalina 61 inch", west, 110, 43, 55.1, north, 32, 25, 00.7, 2510e0}
    ,
    {
    "steward 90 inch", west, 111, 35, 58.24, north, 31, 57, 46.9, 2071e0}
    ,
    {
    "ussr 6 metre", east, 41, 26, 30.0, north, 43, 39, 12., 2100e0}
    ,
    {
    "arecibo 1000 foot", west, 66, 45, 11.1, north, 18, 20, 36.6, 496e0}
    ,
    {
    "cambridge 5km", east, 0, 02, 37.23, north, 52, 10, 12.2, 17e0}
    ,
    {
    "cambridge 1 mile", east, 0, 02, 21.64, north, 52, 9, 47.3, 17e0}
    ,
    {
    "effelsberg 100 metre", east, 6, 53, 01.5, north, 50, 31, 28.6, 366e0}
    ,
    {
    "greenbank 300 foot", west, 79, 50, 56.36, north, 38, 25, 46.3, 894e0}
    ,
    {
    "jodrell bank 250 foot", west, 2, 18, 25., north, 53, 14, 10.5, 78e0}
    ,
    {
    "parkes 64 metre", east, 148, 15, 44.15, south, 33, 00, 00.0, 392e0}
    ,
    {
    "very large array", west, 107, 37, 03.82, north, 34, 04, 43.5, 2124e0}
    ,
    {
    "sugar grove 150 foot", west, 79, 16, 23., north, 38, 31, 14., 705e0}
    ,
    {
    "ussr 600 foot", east, 41, 35, 25.5, north, 43, 49, 32., 973e0}
    ,
    {
    "nobeyama 45 metre", east, 138, 29, 12., north, 35, 56, 19., 1350e0}
    ,
    {
    "jcmt 15 metre", west, 155, 28, 47., north, 19, 49, 33., 4111e0}
    ,
    {
    "?", west, 0, 0, 0., north, 0, 0, 0., 0.}
};

sla_obs (int n, char *c, char *name, double *w, double *p, double *h)
{

    struct teli *telp;
    int i;
    double torad (struct dms *coord, int flag);

    /* table of station identifiers */
    static char *ctab[] = {
        "aat",
        "lpo4.2",
        "lpo2.5",
        "lpo1",
        "lick120",
        "mmt",
        "vicbc",
        "dupont",
        "mthop1.5",
        "stromlo74",
        "anu2.3",
        "gbva140",
        "tololo4m",
        "tololo1.5m",
        "tidbinbla",
        "bloemf",
        "bosqalegre",
        "flagstf61",
        "lowell72",
        "harvard",
        "okayama",
        "kpno158",
        "kpno90",
        "kpno84",
        "kpno36ft",
        "kottamia",
        "eso3.6",
        "maunak88",
        "ukirt",
        "quebec1.6",
        "mtekar",
        "mtlemmon60",
        "mcdonld2.7",
        "mcdonld2.1",
        "palomar200",
        "palomar60",
        "dunlap74",
        "hprov1.93",
        "hprov1.52",
        "sanpm83",
        "saao74",
        "tautnbg",
        "catalina61",
        "steward90",
        "ussr6",
        "arecibo",
        "camb5km",
        "camb1mile",
        "effelsberg",
        "gbva300",
        "jodrell1",
        "parkes",
        "vla",
        "sugargrove",
        "ussr600",
        "nobeyama",
        "jcmt",
        "?"
    };

/*
#define NMAX  ((sizeof (ctab)/sizeof (*char)) - 1)
 */
#define NMAX 57

    /* station specified by number or identifier? */
    if (n > 0)
    {

        /* station specified by number */
        i = n - 1;
        if (i >= NMAX)
            i = NMAX;           /* out of range, use '?' entry */
        strcpy (c, ctab[i]);
    }
    else
    {

        /*
         * station specified by identifier:  determine corresponding
         * number 
         */
        for (i = 0; i < NMAX; i++)
        {
            if (strcmp (c, ctab[i]) == 0)
                break;
        }
    }
    telp = &telinfo[i];
    strcpy (name, telp->name);
    if (*name != '?')
    {
        *w = torad (&(telp->e_w), telp->e_w_flag);
        *p = torad (&(telp->n_s), telp->n_s_flag);
        *h = telp->altitude;
    }
}

/* degrees, arcminutes, arcseconds to radians */

/* flag is true if east or south */

double torad (struct dms *coord, int flag)
{

    /* variables were REAL */
#define as2r  (double)0.4848136811095359949e-05

    register struct dms *p;
    double radians;

    p = coord;
    radians = as2r * ((double) (60 * (60 * p->id + p->iam))
                      + (double) (p->as));
    if (flag)
        radians = -radians;
    return (radians);
}
