#include <math.h>

/**
 *
 *     - - -
 *      p m
 *     - - -
 *
 *
 *  apply corrections for proper motion to a star ra,dec
 *
 *  (double precision)
 *
 *
 *  references:
 *     1984 Astronomical Almanac, pp b39-b41.
 *     (also Lederle & Schwan, Astron. Astrophys. 134,
 *      1-6, 1984)
 *
 *
 *  given:
 *
 *     r0,d0    dp     ra,dec at epoch ep0 (rad)
 *     pr,pd    dp     proper motions:  ra,dec changes per year of epoch
 *     px       dp     parallax (arcsec)
 *     rv       dp     radial velocity (km/sec, +ve if receeding)
 *     ep0      dp     start epoch in years (e.g Julian epoch)
 *     ep1      dp     end epoch in years (same system as ep0)
 *
 *
 *  returned:
 *
 *     r1,d1    dp     ra,dec at epoch ep1 (rad)
 *
 *
 *  called:
 *
 *     sla_dcs2c       spherical to cartesian
 *     sla_dcc2s       cartesian to spherical
 *     sla_dranrm      normalise angle 0-2pi
 *
 *
 *  notes:
 *
 *     1)  The proper motions in ra are dra/dt rather than
 *         cos(dec)*dra/dt, and are in the same coordinate
 *         system as r0,d0.
 *
 *
 *  p.t.wallace   starlink   june 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_pm (double r0, double d0, double pr, double pd, double px, double rv,
        double ep0, double ep1, double *r1, double *d1)
{

    /* km/s to au/year multiplied by arc seconds to radians */
#define vfr  (double)0.21094502e0*0.4848136811095359949e-05

    int i;
    double sla_dranrm (double angle);
    double w, em[3], t, p[3];

    /* spherical to cartesian */
    sla_dcs2c (r0, d0, p);

    /* space motion (radians per year) */
    w = vfr * rv * px;
    em[0] = -pr * p[1] - pd * cos (r0) * sin (d0) + w * p[0];
    em[1] = pr * p[0] - pd * sin (r0) * sin (d0) + w * p[1];
    em[2] = pd * cos (d0) + w * p[2];

    /* apply the motion */
    t = ep1 - ep0;
    for (i = 0; i < 3; i++)
    {
        p[i] += t * em[i];
    }

    /* cartesian to spherical */
    sla_dcc2s (p, r1, d1);
    *r1 = sla_dranrm (*r1);
}
