#include <math.h>

/**
 *
 *     - - - - - -
 *      p r e b n
 *     - - - - - -
 *
 *
 *  generate the matrix of precession between two epochs,
 *  using the old, pre-IAU1976, Bessel-Newcomb model
 *
 *  (double precision)
 *
 *
 *  given:
 *     bep0    dp         beginning Besselian epoch
 *     bep1    dp         ending Besselian epoch
 *
 *  returned:
 *     rmatp  dp(3,3)    precession matrix
 *
 *
 *  reference:
 *     Explanatory Supplement to the A.E., 1960, section
 *      2b, p30.
 *
 *
 *  The matrix is in the sense   v(bep1)  =  rmatp * v(bep0) .
 *
 *
 *  p.t.wallace   starlink   july 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_prebn (double bep0, double bep1, double (*rmatp)[3])
{

    double t0, t;
    double tas2r;
    double zeta0, z, theta;
    double sze, cze, sz, cz, sth, cth, cthsz, cthcz;

    /* arc seconds to radians */
#define as2r  0.4848136811095359949e-05

    /* interval between basic epoch b1900.0 and beginning epoch, */
    /* in tropical centuries */
    t0 = (bep0 - 1900e0) / 100e0;

    /* interval over which precession required, in tropical centuries */
    t = (bep1 - bep0) / 100e0;

    /* rotations (euler angles) */
    tas2r = t * as2r;
    zeta0 = (2304.250e0 + 1.396 * t0 + (0.302e0 + 0.018e0 * t) * t) * tas2r;
    z = zeta0 + 0.791e0 * t * tas2r;
    theta = (2004.682e0 - 0.853e0 * t0 +
             (-0.42665 - 0.042e0 * t) * t) * tas2r;

    /* elements of rotation matrix */
    sze = sin (zeta0);
    cze = cos (zeta0);
    sz = sin (z);
    cz = cos (z);
    sth = sin (theta);
    cth = cos (theta);
    cthsz = cth * sz;
    cthcz = cth * cz;
    rmatp[0][0] = cze * cthcz - sze * sz;
    rmatp[1][0] = cze * cthsz + sze * cz;
    rmatp[2][0] = cze * sth;
    rmatp[0][1] = -sze * cthcz - cze * sz;
    rmatp[1][1] = -sze * cthsz + cze * cz;
    rmatp[2][1] = -sze * sth;
    rmatp[0][2] = -sth * cz;
    rmatp[1][2] = -sth * sz;
    rmatp[2][2] = cth;
}
