#include <math.h>

/**
 *
 *     - - - - -
 *      p r e c
 *     - - - - -
 *
 *
 *  form the matrix of precession between two epochs (IAU1976/FK5)
 *
 *
 *  (double precision)
 *
 *
 *  references:
 *     Lieske,J.H., 1979. Astron.Astrophys.,73,282.
 *      equations (6) & (7), p283.
 *     Kaplan,G.H., 1981. USNO circular no. 163, pa2.
 *
 *
 *  given:
 *
 *     ep0    dp         beginning epoch
 *     ep1    dp         ending epoch
 *
 *
 *  returned:
 *
 *     rmatp  dp(3,3)    precession matrix
 *
 *
 *  notes:
 *
 *  1)  The epochs are TDB (loosely ET) Julian epochs.
 *
 *  2)  The matrix is in the sense   v(ep1)  =  rmatp * v(ep0) .
 *
 *
 *  p.t.wallace   starlink   february 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_prec (double ep0, double ep1, double (*rmatp)[3])
{

    double t0, t;
    double tas2r;
    double w;
    double zeta, z, theta;
    double sze, cze, sz, cz, sth, cth, cthsz, cthcz;

    /* arc seconds to radians */
#define as2r  0.4848136811095359949e-05

    /* interval between basic epoch j2000.0 and beginning epoch (jc) */
    t0 = (ep0 - 2000e0) / 100e0;

    /* interval over which precession required (jc) */
    t = (ep1 - ep0) / 100e0;

    /* rotations (euler angles) */
    tas2r = t * as2r;
    w = 2306.2181e0 + (1.39656e0 - 0.000139e0 * t0) * t0;
    zeta = (w + ((0.30188e0 - 0.000344e0 * t0) + 0.017998e0 * t) * t) * tas2r;
    z = (w + ((1.09468e0 + 0.000066e0 * t0) + 0.018203e0 * t) * t) * tas2r;
    theta = ((2004.3109e0 + (-0.85330e0 - 0.000217e0 * t0) * t0)
             + ((-0.42665e0 - 0.000217e0 * t0) - 0.041833e0 * t) * t) * tas2r;

    /* elements of rotation matrix */
    sze = sin (zeta);
    cze = cos (zeta);
    sz = sin (z);
    cz = cos (z);
    sth = sin (theta);
    cth = cos (theta);
    cthsz = cth * sz;
    cthcz = cth * cz;
    rmatp[0][0] = cze * cthcz - sze * sz;
    rmatp[1][0] = cze * cthsz + sze * cz;
    rmatp[2][0] = cze * sth;
    rmatp[0][1] = -sze * cthcz - cze * sz;
    rmatp[1][1] = -sze * cthsz + cze * cz;
    rmatp[2][1] = -sze * sth;
    rmatp[0][2] = -sth * cz;
    rmatp[1][2] = -sth * sz;
    rmatp[2][2] = cth;
}
