#include <string.h>
#include <ctype.h>

/**
 *
 *     - - - - - - -
 *      p r e c e s
 *     - - - - - - -
 *
 *
 *  precession - either FK4 (Bessel-Newcomb, pre-IAU1976) or
 *  FK5 (Fricke, post-IAU1976) as required.
 *
 *
 *  given:
 *     system     char   precession to be applied: 'fk4' or 'fk5'
 *     ep0,ep1    dp     starting and ending epoch
 *     ra,dc      dp     ra,dec, mean equator & equinox of epoch ep0
 *
 *  returned:
 *     ra,dc      dp     ra,dec, mean equator & equinox of epoch ep1
 *
 *  called:    sla_dranrm, sla_prebn, sla_prec, sla_dcs2c,
 *             sla_dmxv, sla_dcc2s
 *
 *  notes:
 *
 *  1)  The epochs are Besselian if system='fk4' and Julian if 'fk5'.
 *      For example, to precess coordinates in the old system from
 *      equinox 1900.0 to 1950.0 the call would be:
 *          call sla_preces ('fk4', 1900e0, 1950e0, &ra, &dc)
 *
 *  2)  This routine will not correctly convert between the old and
 *      the new systems - for example conversion from b1950 to j2000.
 *      For these purposes see sla_fk425, sla_fk45z, sla_fk54z.
 *
 *  3)  If an invalid system is supplied, values of -99d0,-99d0 will
 *      be returned for both ra and dc.
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_prebn (double bep0, double bep1, double (*rmatp)[3]);
extern int sla_prec (double ep0, double ep1, double (*rmatp)[3]);
extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dmxv (double (*dm)[3], double *va, double *vb);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_preces (char *system, double ep0, double ep1, double *ra, double *dc)
{

    register char *p;
    double pm[3][3], v1[3], v2[3];
    double sla_dranrm (double angle);

    /* make sure lower case */
    for (p = system; *p != 0; p++)
        if (isupper (*p))
            *p = tolower (*p);
    /* if not fk[45], mark invalid */
    if (strcmp (system, "fk4") != 0 && strcmp (system, "fk5") != 0)
    {
        *ra = -99e0;
        *dc = -99e0;
    }
    else
    {

        /* generate appropriate precession matrix */
        if (system[2] == '4')
        {
            sla_prebn (ep0, ep1, pm);
        }
        else
        {
            sla_prec (ep0, ep1, pm);
        }

        /* convert ra,dec to x,y,z */
        sla_dcs2c (*ra, *dc, v1);

        /* precess */
        sla_dmxv (pm, v1, v2);

        /* back to ra,dec */
        sla_dcc2s (v2, ra, dc);
        *ra = sla_dranrm (*ra);
    }
}
