#include <math.h>

/**
 *
 *     - - - -
 *      r c c
 *     - - - -
 *
 *
 *  relativistic clock correction:  the difference between proper time
 *  on earth and coordinate time in the solar system barycentric
 *  space-time frame of reference.
 *
 *  The proper time is TDT;  the coordinate time is an
 *  implementation of TDB.
 *
 *
 *  given:
 *    tdb   dp   coordinate time - MJD
 *    ut1   dp   universal time - fraction of one day
 *    wl    dp   clock longitude - radians West
 *    u     dp   clock distance from earth spin axis - km
 *    v     dp   clock distance north of earth equatorial plane - km
 *
 *
 *  returned:
 *    The clock correction:  TDB - TDT, where TDB may be considered to
 *    be the coordinate time in the solar system barycentre frame of
 *    reference, and TDT is the proper time given by clocks at mean sea
 *    level on the earth, in seconds.
 *
 *    The result has a main (annual) sinusoidal term of amplitude
 *    approximately 0.00166 seconds, plus planetary terms up to 20
 *    microseconds, and lunar and diurnal terms up to 2 microseconds.
 *
 *  The argument TDB is in the form of a modified Julian date:
 *  JD - 2400000.5.  Though coordinate time (TDB) is strictly required,
 *  the terrestrial proper time (TDT) can in practice be used.
 *
 *
 *  references:
 *    van Flandern T C & Pulkkinen K F, 1979, Astrophys. J. Suppl., 41,
 *          391-411
 *    Meeus J, 1984, L'Astronomie, 348-354.
 *    Moyer T D, 1981, Cel. Mech., 23, 33.
 *    Murray C A, 1983, Vectorial Astrometry, Adam Hilger.
 *    Newcomb S, 1898, Astron. Pap. Washington, vi, part 1.
 *    Stumpff P, 1981, Astron. Astrophys., 101, 52-71.
 *
 *  The algorithm is based on Moyer 1981 (see also Murray 1983), but with
 *  all terms of 0.01 microsecond or more included.  The coefficients
 *  have all been recomputed, using IAU 1976 (or where available
 *  JPL DE200/LE200) constants.
 *
 *  Note that Moyer's accuracy objectives were for spacecraft tracking
 *  applications, where the doppler effect is of particular importance.
 *  A result of this is that some of his short period terms are smaller
 *  than the absolute precision of the algorithm;  all Moyer's terms
 *  have, nevertheless, been included.  During the interval 1950-2050,
 *  the absolute accuracy is about 0.09/0.27 microsecond rms/max
 *  relative to direct numerical integrations of the JPL DE200/LE200
 *  solar system ephemeris.  Over a 10-year interval the errors are
 *  about half these values.
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

double sla_rcc (double tdb, double ut1, double wl, double u, double v)
{

    double t, e, w, tsol;
    double rlv, rtv, qgv;
    double rle, rge, dg;
    double qgm;
    double rlm, rgm, rtm;
    double rlj, rgj, rtj, qgj;
    double rls, rgs, rts, qgs;
    double rlu, rgu, rtu;
    double rln, rgn, rtn;
    double pj, ps;
    double rgmoon, rdmoon;
    double rlmoon, rpmoon;
    double xmoon, ymoon, xde, yde;
    double value, junk;

#define d2pi  (double)6.283185307179586476925287e0

    /* initialise accumulator */
    w = 0e0;

    /* convert TDB to Julian centuries past 2000 Jan 1.5 */
    t = (tdb - 51544.5e0) / 36525e0;

    /* convert UT1 to local solar time in radians */
    /* tsol = mod(ut1,1d0) * d2pi - wl; */
    tsol = modf (ut1, &junk) * d2pi - wl;

    /*
     * earth orbit mean anomaly and eccentricity
     *
     * derived from Newcomb 1898 with modifications by Stumpff 1981
     */
    value = 6.24005225e0 + (628.301973182e0 -
                            (0.0000002909e0 + 0.00000005818e0 * t) * t) * t;
    rge = modf (value / d2pi, &junk) * d2pi;
    e = 0.01670847e0 - (0.000041979e0 + 0.0000000503e0 * t) * t;

    /*
     * long period perturbations in mean anomaly and longitude
     */
    /* 4mars-7earth+3venus */
    dg = 1.2896e-6 * sin (2.631957e0 + 2.076942e0 * t)
        /* 3jupiter-8mars+4earth */
        + 31.0281e-6 * sin (4.387583e0 + 0.352557e0 * t);
    rge += dg;

    /*
     * planetary arguments
     *
     * derived from van Flandern & Pulkkinen 1979
     *
     * rlp = mean longitude, mean equinox of date
     * rgp = mean anomaly
     *
     * venus
     */
    rlv = modf ((3.176138e0 + 1021.352935e0 * t) / d2pi, &junk) * d2pi;

    /* earth */
    rle = modf ((1.753461e0 + 628.331961e0 * t) / d2pi, &junk) * d2pi;

    /* mars */
    rlm = modf ((6.203722e0 + 334.085687e0 * t) / d2pi, &junk) * d2pi;
    rgm = modf ((0.338387e0 + 334.053556e0 * t) / d2pi, &junk) * d2pi;

    /* jupiter */
    rlj = modf ((0.599550e0 + 52.993465e0 * t) / d2pi, &junk) * d2pi;
    rgj = modf ((0.355195e0 + 52.969113e0 * t) / d2pi, &junk) * d2pi;

    /* saturn */
    rls = modf ((0.873980e0 + 21.354271e0 * t) / d2pi, &junk) * d2pi;
    rgs = modf ((5.547971e0 + 21.329963e0 * t) / d2pi, &junk) * d2pi;

    /* uranus */
    rlu = modf ((5.467588e0 + 7.503143e0 * t) / d2pi, &junk) * d2pi;
    rgu = modf ((2.516975e0 + 7.503143e0 * t) / d2pi, &junk) * d2pi;

    /* neptune */
    rln = modf ((5.321305e0 + 3.837340e0 * t) / d2pi, &junk) * d2pi;
    rgn = modf ((4.557622e0 + 3.837340e0 * t) / d2pi, &junk) * d2pi;

    /* jupiter and saturn perturbations (van Flandern & Pulkkinen) */
    pj = 0.005789e0 * sin (2e0 * rgj - 5e0 * rgs + 2.023208e0) +
        0.000970e0 * sin (2e0 * rgj - 2e0 * rgs - 2.751801e0) +
        0.000722e0 * sin (3e0 * rgj - 5e0 * rgs + 0.436627e0) +
        0.000635e0 * sin (rgj - 2e0 * rgs + 3.141593e0);
    ps = 0.014172e0 * sin (2e0 * rgj - 5e0 * rgs - 1.118317e0) +
        0.003990e0 * sin (2e0 * rgj - 4e0 * rgs - 1.558646e0);
    rlj += pj;
    rls += ps;

    /* differences in mean longitude, planet-EMB */
    rtv = rlv - rle;
    rtm = rlm - rle;
    rtj = rlj - rle;
    rts = rls - rle;
    rtu = rlu - rle;
    rtn = rln - rle;

    /*
     * lunar contribution (arguments derived from Meeus 1984;  they are
     * substantially the same as those given in the 1984 Astronomical
     * Almanac, page d46).
     */
    rgmoon = modf ((2.355547e0 + 8328.691425e0 * t) / d2pi, &junk) * d2pi;
    rdmoon = modf ((5.198468e0 + 7771.377144e0 * t) / d2pi, &junk) * d2pi;

    rlmoon = 3.810336e0 + 8399.709105e0 * t +
        0.109759e0 * sin (rgmoon) +
        0.022236e0 * sin (2e0 * rdmoon - rgmoon) +
        0.011490e0 * sin (2e0 * rdmoon) +
        0.003728e0 * sin (2e0 * rgmoon) - 0.003239e0 * sin (rge);

    rpmoon = 0.01659326e0 + 0.00090439e0 * cos (rgmoon) +
        0.00016635e0 * cos (2e0 * rdmoon - rgmoon) +
        0.00013689e0 * cos (2e0 * rdmoon);
    xmoon = cos (rlmoon);
    ymoon = sin (rlmoon);
    xde = -sin (rle) - e * sin (rle + rge);
    yde = cos (rle) + e * cos (rle + rge);

    /* transverse doppler:  mean amplitude approximately 1.55 microsec */
    w -= 0.0256829e-6 * (xmoon * xde + ymoon * yde) / rpmoon;

    /* gravitational redshift */
    w += 0.00292e-6 * sin (rgmoon);

    /* topocentric terms (Moyer 1981 and Murray 1983) */
    w -= 0.00029e-10 * u * sin (tsol - rts);
    w += 0.00100e-10 * u * sin (tsol - 2e0 * rge);
    w += 0.00133e-10 * u * sin (tsol - rdmoon);
    w -= 0.00133e-10 * u * sin (tsol - rtj);
    w -= 0.00229e-10 * u * sin (tsol + 2e0 * rle + rge);
    w += 0.0220e-10 * v * sin (rle + rge);
    w += 0.05312e-10 * u * sin (tsol - rge);
    w -= 0.13677e-10 * u * sin (tsol + 2e0 * rle);
    w += 1.3184e-10 * v * cos (rle);
    w += 3.17679e-10 * u * sin (tsol);

    /*
     * effects due to planetary perturbations on the motion of the earth
     *
     * The model is derived from Newcomb's tables A-E, but using Stumpff's 
     * values for the coefficients (though the difference between the
     * original Newcomb values and those of Stumpff are negligible in
     * this application).
     *
     * quasi mean anomalies
     */
    qgv = modf ((0.871590e0 + 1021.322923e0 * t) / d2pi, &junk) * d2pi;
    qgm = modf ((0.341377e0 + 334.055617e0 * t) / d2pi, &junk) * d2pi;
    qgj = modf ((0.346691e0 + 52.963465e0 * t + pj) / d2pi, &junk) * d2pi;
    qgs = modf ((5.539505e0 + 21.324328e0 * t + ps) / d2pi, &junk) * d2pi;

    /* effects depending on radius vector perturbations */
    w -= 0.0260e-6 * sin (-2e0 * qgv + 3e0 * rge - 1.01755e0);
    w += 0.0116e-6 * sin (-3e0 * qgv + 5e0 * rge + 1.82778e0);
    w -= 0.0110e-6 * sin (-8e0 * qgv + 13e0 * rge - 3.01464e0);
    w += 0.0310e-6 * sin (2e0 * qgm - rge + 1.85352e0);
    w += 0.0800e-6 * sin (qgj - 1.56367e0);
    w += 0.0120e-6 * sin (qgs - 0.19464e0);

    /* effects depending on longitude perturbations */
    w += 0.0182e-6 * sin (-qgv + 2.68189e0);
    w += 1.1630e-6 * sin (-qgv + rge + 2.63361e0);
    w += 0.0182e-6 * sin (-qgv + 2e0 * rge - 2.04146e0);
    w += 0.0279e-6 * sin (-2e0 * qgv + rge - 1.02977e0);
    w += 1.3283e-6 * sin (-2e0 * qgv + 2e0 * rge - 1.01777e0);
    w += 0.5968e-6 * sin (-2e0 * qgv + 3e0 * rge + 2.33692e0);
    w += 0.0112e-6 * sin (-2e0 * qgv + 4e0 * rge + 2.39994e0);
    w += 0.1574e-6 * sin (-3e0 * qgv + 3e0 * rge - 1.53424e0);
    w += 0.3750e-6 * sin (-3e0 * qgv + 4e0 * rge + 1.82826e0);
    w += 0.2459e-6 * sin (-3e0 * qgv + 5e0 * rge + 2.30794e0);
    w += 0.0506e-6 * sin (-4e0 * qgv + 4e0 * rge - 2.02928e0);
    w += 0.0346e-6 * sin (-4e0 * qgv + 5e0 * rge - 1.84025e0);
    w += 0.0369e-6 * sin (-4e0 * qgv + 6e0 * rge + 1.79696e0);
    w += 0.0203e-6 * sin (-5e0 * qgv + 5e0 * rge - 2.53740e0);
    w += 0.0305e-6 * sin (-5e0 * qgv + 7e0 * rge - 1.88632e0);
    w += 0.0375e-6 * sin (-5e0 * qgv + 8e0 * rge + 1.57958e0);
    w += 0.0657e-6 * sin (qgm - rge - 2.22577e0);
    w += 0.0117e-6 * sin (qgm - 2.98295e0);
    w += 0.4917e-6 * sin (2e0 * qgm - 2e0 * rge + 1.85306e0);
    w += 0.4255e-6 * sin (2e0 * qgm - rge - 1.92854e0);
    w += 0.0310e-6 * sin (3e0 * qgm - 3e0 * rge + 2.72439e0);
    w += 0.1024e-6 * sin (3e0 * qgm - 2e0 * rge + 1.93986e0);
    w += 0.1203e-6 * sin (4e0 * qgm - 3e0 * rge - 0.26836e0);
    w += 0.1405e-6 * sin (4e0 * qgm - 2e0 * rge + 2.01909e0);
    w += 0.0204e-6 * sin (5e0 * qgm - 4e0 * rge + 0.61009e0);
    w += 0.0492e-6 * sin (5e0 * qgm - 3e0 * rge - 0.18510e0);
    w += 0.0370e-6 * sin (6e0 * qgm - 4e0 * rge - 2.39793e0);
    w += 0.0242e-6 * sin (6e0 * qgm - 3e0 * rge - 0.10746e0);
    w += 0.0117e-6 * sin (7e0 * qgm - 5e0 * rge - 1.51224e0);
    w += 0.0256e-6 * sin (7e0 * qgm - 4e0 * rge - 2.30920e0);
    w += 0.0124e-6 * sin (8e0 * qgm - 5e0 * rge + 1.78535e0);
    w += 0.0148e-6 * sin (9e0 * qgm - 5e0 * rge + 1.83586e0);
    w += 0.0101e-6 * sin (11e0 * qgm - 6e0 * rge - 0.24509e0);
    w += 0.0109e-6 * sin (13e0 * qgm - 7e0 * rge - 2.42043e0);
    w += 0.0470e-6 * sin (15e0 * qgm - 8e0 * rge + 1.70527e0);
    w += 0.0392e-6 * sin (qgj - 2e0 * rge - 1.89772e0);
    w += 1.7332e-6 * sin (qgj - rge - 1.56329e0);
    w += 0.6307e-6 * sin (qgj - 3.02734e0);
    w += 0.0177e-6 * sin (qgj + rge + 3.03866e0);
    w += 0.0165e-6 * sin (2e0 * qgj - 3e0 * rge + 0.15296e0);
    w += 0.6565e-6 * sin (2e0 * qgj - 2e0 * rge + 0.04852e0);
    w += 0.3865e-6 * sin (2e0 * qgj - rge - 0.34458e0);
    w += 0.0179e-6 * sin (2e0 * qgj - 2.84069e0);
    w += 0.0396e-6 * sin (3e0 * qgj - 3e0 * rge - 1.40633e0);
    w += 0.1342e-6 * sin (3e0 * qgj - 2e0 * rge + 0.12294e0);
    w += 0.0505e-6 * sin (3e0 * qgj - rge - 0.15280e0);
    w += 0.0106e-6 * sin (4e0 * qgj - 3e0 * rge - 1.37709e0);
    w += 0.0194e-6 * sin (4e0 * qgj - 2e0 * rge + 0.20095e0);
    w += 0.1005e-6 * sin (qgs - rge - 0.19410e0);
    w += 0.0766e-6 * sin (qgs + 3.12985e0);
    w += 0.0258e-6 * sin (2e0 * qgs - 2e0 * rge + 2.76845e0);
    w += 0.0257e-6 * sin (2e0 * qgs - rge + 2.72025e0);

    /*
     * long period terms
     * 13earth-8venus term in mean longitude
     */
    w += (0.4485e-6 - 0.00385e-6 * t) * sin (3.621733e0 + 2.622706e0 * t);

    /* 4mars-7earth+3venus and 3jupiter-8mars+4earth terms in both
     * mean longitude and mean anomaly:  amplitudes approximately
     * 0.0639 and 1.5381 microsec respectively
     */
    w += dg * 0.04957e0;

    /*
     * planetary terms:  transverse doppler from the motion of the ssb
     * plus gravitational redshift.
     */
    /* venus */
    w += 0.0155e-6 * sin (3e0 * rtv);
    w += 0.0431e-6 * sin (2e0 * rtv);
    w -= 0.0611e-6 * sin (rtv);

    /* mars */
    w += 0.0115e-6 * sin (rtm + rgm);

    /* jupiter */
    w += 0.0142e-6 * sin (2e0 * rtj + rge);
    w -= 0.0144e-6 * sin (3e0 * rtj);
    w -= 0.0227e-6 * sin (2e0 * rtj + rgj);
    w += 0.0559e-6 * sin (rtj + 2e0 * rgj);
    w -= 0.1238e-6 * sin (2e0 * rtj);
    w += 0.1271e-6 * sin (2e0 * rgj);
    w += 0.1729e-6 * sin (rtj - rge);
    w -= 0.5287e-6 * sin (rtj + rge);
    w += 0.9969e-6 * sin (rtj + rgj);
    w += 5.3448e-6 * sin (rgj);
    w += 20.6750e-6 * sin (rtj);

    /* saturn */
    w += 0.0164e-6 * sin (rts + 2e0 * rgs);
    w += 0.0382e-6 * sin (rts - rge);
    w += 0.0690e-6 * sin (2e0 * rgs);
    w -= 0.1148e-6 * sin (rts + rge);
    w += 0.2565e-6 * sin (rts + rgs);
    w += 2.4664e-6 * sin (rgs);
    w += 4.5706e-6 * sin (rts);

    /* uranus */
    w += 0.0101e-6 * sin (2e0 * rgu);
    w -= 0.0123e-6 * sin (rtu + rge);
    w += 0.0228e-6 * sin (rtu + rgu);
    w += 0.4364e-6 * sin (rgu);
    w += 0.4933e-6 * sin (rtu);

    /* neptune */
    w -= 0.0117e-6 * sin (rtn + rge);
    w += 0.1446e-6 * sin (rgn);
    w += 0.4671e-6 * sin (rtn);

    /*
     * main terms due to earth orbital eccentricity:  amplitudes
     * are 1656.64, 13.84, 0.17 microsec approximately at j2000
     */
    w += e * (10.381e-6 * sin (3e0 * rge) +
              828.380e-6 * sin (2e0 * rge) + 99149.657e-6 * sin (rge));

    /* result */
    return (w);
}
