
/**
 *
 *     - - - - - -
 *      r e f c o
 *     - - - - - -
 *
 *
 *  determine constants a and b in atmospheric refraction model
 *  dz = a tan z + b tan**3 z (for optical wavelengths).
 *
 *
 *  given:
 *    h0    dp    height of the observer above sea level (metre)
 *    t0    dp    ambient temperature at the observer (deg K)
 *    p0    dp    pressure at the observer (millibar)
 *    ups   dp    relative humidity at the observer (range 0-1)
 *    wl    dp    effective wavelength of the source (micrometre)
 *    ph    dp    latitude of the observer (radian, astronomical)
 *    as    dp    temperature lapse rate in the troposphere (degK/metre)
 *    eps   dp    precision required to terminate iteration (radian)
 *
 *  returned:
 *    refa  dp    tan z coefficient (radian)
 *    refb  dp    tan**3 coefficient (radian)
 *
 *  called:  sla_refro
 *
 *  Typical values for the as and eps arguments would be 0.0065d0 and
 *  1d-10 respectively.
 *
 *  Relative to the comprehensive refraction model used by this routine,
 *  the simple a tan z + b tan**3 z formula achieves 0.5 arcsec accuracy
 *  for zd < 80 deg, 0.01 arcsec accuracy for zd < 60 deg, and
 *  0.001 arcsec accuracy for zd < 45 deg.
 *
 *  p.t.wallace   starlink   july 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_refro (double z0, double h0, double t0, double p0, double ups,
                      double wl, double ph, double as, double eps,
                      double *ref);

sla_refco (double h0, double t0, double p0, double ups, double wl, double ph,
           double as, double eps, double *refa, double *refb)
{

    double r1, r2;

    /* sample zenith distances: arctan(1) and arctan(4) */
#define atn1  (double)0.7853981634e0
#define atn4  (double)1.325817664e0

    /* determine refraction for the two sample zenith distances */
    sla_refro (atn1, h0, t0, p0, ups, wl, ph, as, eps, &r1);
    sla_refro (atn4, h0, t0, p0, ups, wl, ph, as, eps, &r2);

    /* solve for refraction constants */
    *refa = (64e0 * r1 - r2) / 60e0;
    *refb = (r2 - 4e0 * r1) / 60e0;
}
