#include <math.h>

/**
 *
 *     - - - - - -
 *      r e f r o
 *     - - - - - -
 *
 *
 *  atmospheric refraction for optical wavelengths
 *
 *
 *  given:
 *    z0    dp    observed zenith distance of the source (radian)
 *    h0    dp    height of the observer above sea level (metre)
 *    t0    dp    ambient temperature at the observer (deg K)
 *    p0    dp    pressure at the observer (millibar)
 *    ups   dp    relative humidity at the observer (range 0-1)
 *    wl    dp    effective wavelength of the source (micrometre)
 *    ph    dp    latitude of the observer (radian, astronomical)
 *    as    dp    temperature lapse rate in the troposphere (degK/metre)
 *    eps   dp    precision required to terminate iteration (radian)
 *
 *  returned:
 *    ref   dp    refraction: in vacuo zd minus observed zd (radian)
 *
 *  called:  sla_atmt, sla_atms
 *
 *  Typical values for the as and eps arguments would be 0.0065d0 and
 *  1d-10 respectively.
 *
 *  This routine computes the refraction for zenith distances up to
 *  and beyond 90 deg using the method of Hohenkerk and Sinclair (NAO
 *  technical notes 59 and 63).  The code is a trivially modified form
 *  of the aref subroutine of C.Hohenkerk (HMNAO, September 1984).
 *  Most of the modifications are cosmetic;  in addition the angle
 *  arguments have been changed to radians and the z0 = 0 case is
 *  handled.  For z0 > 90 deg it is up to the caller to supply an
 *  h0 consistent with the depression of the horizon thus implied.
 *
 *  Fixed values of the water vapour exponent, height of tropopause, and
 *  height at which refraction is negligible are used.
 *
 *  Original version by C.Hohenkerk, HMNAO, September 1986.
 *  starlink version by p.t.wallace, july 1986.
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_atmt (double r0, double t0, double *a, double r, double *t,
                     double *dn, double *dndr);
extern int sla_atms (double rt, double tt, double dnt, double gmdr, double r,
                     double *dn, double *dndr);

sla_refro (double z0, double h0, double t0, double p0, double ups, double wl,
           double ph, double as, double eps, double *ref)
{

    /*
     * fixed parameters
     *
     */

    /* universal gas constant */
#define gcr  (double)8314.36e0

    /* molecular weight of dry air */
#define dmd  (double)28.966e0

    /* molecular weight of water vapour */
#define dmw  (double)18.016e0

    /* mean earth radius (metre) */
#define s  (double)6378120e0

    /* exponent of temperature dependence of water vapour pressure */
#define gamma  (double)18.36e0

    /* height of tropopause (metre) */
#define ht  (double)11000e0

    /* upper limit for refractive effects (metre) */
#define hs  80000e0

    int in, is, k, istart, i, j;
    double wlsq, gb, z1, a[10], pw0, r0, t0o, dn0, dndr0, sk0;
    double f0, rt, tt, dnt, dndrt, zt, ft, dnts, dndrts, sine;
    double zts, fts, rs, dns, dndrs, zs, fs, refo, fe, fo, h, fb;
    double ff, step, z, r, rg, tg, t, dn, dndr, f, refp, reft;
    double temp;

    /* the refraction integrand */
#define	refi(r, dn, dndr)  (r * dndr / (dn + r * dndr))

    /* set up model atmosphere parameters defined at the observer */
    wlsq = wl * wl;
    gb = 9.784e0 * (1e0 - 0.0026e0 * cos (2e0 * ph) - 0.00000028e0 * h0);
    z1 = ((287.604e0 + 1.6288e0 / wlsq + 0.0136e0 / (wlsq * wlsq))
          * 273.15e0 / 1013.25e0) * 1e-6;
    a[0] = fabs (as);
    a[1] = (gb * dmd) / gcr;
    a[2] = a[1] / a[0];
    a[3] = gamma;
    pw0 = ups * pow ((t0 / 247.1e0), a[3]);
    a[4] = pw0 * (1e0 - dmw / dmd) * a[2] / (a[3] - a[2]);
    a[5] = p0 + a[4];
    a[6] = z1 * a[5] / t0;
    a[7] = (z1 * a[4] + 11.2684e-6 * pw0) / t0;
    a[8] = (a[2] - 1e0) * a[0] * a[6] / t0;
    a[9] = (a[3] - 1e0) * a[0] * a[7] / t0;
    a[2] -= 2e0;
    a[3] -= 2e0;

    /* at the observer */
    r0 = s + h0;
    sla_atmt (r0, t0, a, r0, &t0o, &dn0, &dndr0);
    sk0 = dn0 * r0 * sin (z0);
    f0 = refi (r0, dn0, dndr0);

    /* at the tropopause in the troposphere */
    rt = s + ht;
    sla_atmt (r0, t0, a, rt, &tt, &dnt, &dndrt);
    sine = sk0 / (rt * dnt);
    temp = 1e0 - sine * sine;
    if (temp < 0.0)
        temp = 0.0;
    zt = atan2 (sine, sqrt (temp));
    ft = refi (rt, dnt, dndrt);

    /* at the tropopause in the stratosphere */
    sla_atms (rt, tt, dnt, a[1], rt, &dnts, &dndrts);
    sine = sk0 / (rt * dnts);
    temp = 1e0 - sine * sine;
    if (temp < 0.0)
        temp = 0.0;
    zts = atan2 (sine, sqrt (temp));
    fts = refi (rt, dnts, dndrts);

    /* at the stratosphere limit */
    rs = s + hs;
    sla_atms (rt, tt, dnt, a[1], rs, &dns, &dndrs);
    sine = sk0 / (rs * dns);
    temp = 1e0 - sine * sine;
    if (temp < 0.0)
        temp = 0.0;
    zs = atan2 (sine, sqrt (temp));
    fs = refi (rs, dns, dndrs);

    /*
     * integrate the refraction integral in two parts;  first in the
     * troposphere (k=1), then in the stratosphere (k=2)
     */
    refo = -999.999e0;
    is = 16;
    for (k = 1; k <= 2; k++)
    {
        istart = 0;
        fe = 0e0;
        fo = 0e0;
        if (k == 1)
        {
            h = (zt - z0) / (double) is;
            fb = f0;
            ff = ft;
        }
        else
        {
            h = (zs - zts) / (double) is;
            fb = fts;
            ff = fs;
        }

        in = is - 1;
        is = is / 2;
        step = h;
        for (;;)
        {
            for (i = 1; i <= in; i++)
            {
                if (i == 1 && k == 1)
                {
                    z = z0 + h;
                    r = r0;
                }
                else if (i == 1 && k == 2)
                {
                    z = zts + h;
                    r = rt;
                }
                else
                {
                    z = z + step;
                }

                /*
                 * given the zenith distance
                 * (z) find r 
                 */
                rg = r;
                for (j = 1; j <= 4; j++)
                {
                    if (k == 1)
                    {
                        sla_atmt (r0, t0, a, rg, &tg, &dn, &dndr);
                    }
                    else
                    {
                        sla_atms (rt, tt, dnt, a[1], rg, &dn, &dndr);
                    }
                    if (z > 1e-20)
                        rg -= ((rg * dn - sk0 / sin (z)) / (dn + rg * dndr));
                }
                r = rg;

                /*
                 * find * refractive index and integrand
                 * at r 
                 */
                if (k == 1)
                {
                    sla_atmt (r0, t0, a, r, &t, &dn, &dndr);
                }
                else
                {
                    sla_atms (rt, tt, dnt, a[1], r, &dn, &dndr);
                }
                f = refi (r, dn, dndr);
                if (istart == 0 && (i % 2) == 0)
                {
                    fe += f;
                }
                else
                {
                    fo += f;
                }
            }

            /*
             * evaluate the integrand using Simpson's rule
             */
            refp = h * (fb + 4e0 * fo + 2e0 * fe + ff) / 3e0;

            /*
             * skip if required precision reached
             */
            if (fabs (refp - refo) <= 0.5e0 * eps)
                break;

            /*
             * prepare for next interation
             */
            is *= 2;
            in = is;
            step = h;
            h /= 2e0;
            fe += fo;
            fo = 0e0;
            refo = refp;
            if (istart == 0)
                istart = 1;
        }
        if (k == 1)
            reft = refp;
    }

    /*
     * result
     */
    *ref = reft + refp;
}
