#include <math.h>

/**
 *
 *     - - - - -
 *      r e f v
 *     - - - - -
 *
 *
 *  adjust an unrefracted cartesian vector to include the effect of
 *  atmospheric refraction, using the simple a tan z + b tan**3 z
 *  model.
 *
 *
 *  given:
 *    vu    dp    unrefracted position of the source (az/el 3-vector)
 *    refa  dp    tan z coefficient (radian)
 *    refb  dp    tan**3 z coefficient (radian)
 *
 *  returned:
 *    vr    dp    refracted position of the source (az/el 3-vector)
 *
 *  Note that this routine applies the adjustment for refraction in
 *  the opposite sense to the usual one - it takes an unrefracted
 *  (in vacuo) position and produces an observed (refracted)
 *  position, whereas the basic a tan z + b tan**3 z model strictly
 *  applies to the case where a refracted position is available and
 *  must be corrected for refraction.  This requires an inverted form of
 *  the refraction expression;  the formula which is the basis of the
 *  algorithm used here is taken from RGO/LaPalma technical note
 *  no. 42, "A Control System for the William Herschel Telescope",
 *  by R.Laing.
 *
 *  See also the routine sla_refz, which performs the adjustment to
 *  the zenith distance rather than in cartesian az/el coordinates.
 *
 *  To avoid arithmetic overflow at the horizon, the z-component is
 *  artificially constrained to the positive non-zero range.
 *
 *  p.t.wallace   starlink   september 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_refv (double *vu, double refa, double refb, double *vr)
{

    double x, y, z, zsq, rsq, r, wb, wt, d, cd, f;

    /* unrefracted vector */
    x = vu[0];
    y = vu[1];
    z = fabs (vu[2]);
    if (z < 1e-10)
        z = 1e-10;

    /* functions */
    zsq = z * z;
    rsq = x * x + y * y;
    r = sqrt (rsq);
    wb = refb * rsq / zsq;
    wt = (refa + wb) / (1e0 + (refa + 3e0 * wb) / zsq);
    d = wt * r / z;
    cd = 1e0 - d * d / 2e0;
    f = cd * (1e0 - wt);

    /* refracted vector */
    vr[0] = x * f;
    vr[1] = y * f;
    vr[2] = cd * (z + d * r);
}
