#include <math.h>

/**
 *
 *     - - - - -
 *      r e f z
 *     - - - - -
 *
 *
 *  adjust an unrefracted zenith distance to include the effect of
 *  atmospheric refraction, using the simple a tan z + b tan**3 z
 *  model.
 *
 *
 *  given:
 *    zu    dp    unrefracted zenith distance of the source (radian)
 *    refa  dp    tan z coefficient (radian)
 *    refb  dp    tan**3 z coefficient (radian)
 *
 *  returned:
 *    zr    dp    refracted zenith distance (radian)
 *
 *  Note that this routine applies the adjustment for refraction in
 *  the opposite sense to the usual one - it takes an unrefracted
 *  (in vacuo) position and produces an observed (refracted)
 *  position, whereas the basic a tan z + b tan**3 z model strictly
 *  applies to the case where a refracted position is available and
 *  must be corrected for refraction.  This requires an inverted form of
 *  the refraction expression; the formula used here is taken from
 *  RGO/LaPalma technical note no. 42, "A Control System for
 *  the William Herschel Telescope", by R.Laing.
 *
 *  For zd > 85 deg, a fixed value zd = 85 deg is used.
 *
 *  See also the routine sla_refv, which performs the adjustment in
 *  cartesian az/el coordinates.
 *
 *  p.t.wallace   starlink   july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */
sla_refz (double zu, double refa, double refb, double *zr)
{

    double zl, s, c, t, tsq, tcu;

    /* keep zd within usable range */
    zl = fabs (zu);
    if (zl > 1.483529864e0)
        zl = 1.483529864e0;

    /* functions of zd */
    s = sin (zl);
    c = cos (zl);
    t = s / c;
    tsq = t * t;
    tcu = t * tsq;

    /* refracted zd */
    *zr = zu - (refa * t + refb * tcu) /
        (1e0 + (refa + 3e0 * refb * tsq) / (c * c));
}
