#include <math.h>

/**
 *
 *     - - - - - - -
 *      s m a t r x
 *     - - - - - - -
 *
 *
 *  matrix inversion & solution of simultaneous equations
 *
 *
 *  (was single precision)
 *
 *
 *  for the set of n simultaneous equations in n unknowns:
 *     a.y = x
 *
 *  where:
 *     a is a non-singular n x n matrix
 *     y is the vector of n unknowns
 *     x is the known vector
 *
 *  smatrx computes:
 *     the inverse of matrix a
 *     the determinant of matrix a
 *     the vector of n unknowns
 *
 *
 *  arguments:
 *
 *     symbol  type dimension           before              after
 *
 *       n      int                 no. of unknowns       unchanged
 *       ap	pointer to a(n,n)       matrix             inverse
 *       y      real   (n)              vector            solution
 *       d      real                       -             determinant
 *     * jf     int                        -           singularity flag
 *       wp	pointer to w(n,n)          -              workspace
 *       iw     int    (n)                 -              workspace
 *
 *
 *  *  jf is the singularity flag.  If the matrix is non-singular,
 *    jf=0 is returned.  If the matrix is singular, jf=-1 & d=0.0 are
 *    returned.  In the latter case, the contents of array a on return
 *    are undefined.
 *
 *  algorithm:
 *     Gaussian elimination with partial pivoting.
 *
 *  speed:
 *     very fast.
 *
 *  accuracy:
 *     fairly accurate - errors 1 to 4 times those of routines optimised
 *     for accuracy.
 *
 *
 *  p.t.wallace   starlink   october 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_smatrx (int n, double *ap, double *y, double *d, int *jf, double *wp,
            int *iw)
    /* variables were REAL */
    /* variables were REAL */
{

    int k, imx, i, j, nm1mk, ki;
    register double *p1, *p2, *pn;
    double amx, t, akk, yk, aik;        /* variables were REAL */
    double td;                  /* local temp "d" */

#define sfa  (double)1e-20
#define a(i,j) (*(&ap[i*n+j]))

    *jf = 0;
    td = 1.0;
    for (k = 0; k < n; k++)
    {
        pn = &ap[k * n + k];
        amx = fabs (*pn);
        imx = k;
        if (k != n - 1)
        {
            for (i = k + 1; i < n; i++)
            {
                t = fabs (a (i, k));
                if (t > amx)
                {
                    amx = t;
                    imx = i;
                }
            }
        }
        if (amx < sfa)
        {
            *jf = -1;
        }
        else
        {
            if (imx != k)
            {
                for (j = 0; j < n; j++)
                {
                    p1 = &a (k, j);
                    p2 = &a (imx, j);
                    t = *p1;
                    *p1 = *p2;
                    *p2 = t;
                }
                t = y[k];
                y[k] = y[imx];
                y[imx] = t;
                td = -td;
            }
            iw[k] = imx;
            akk = *pn;
            td *= akk;
            if (fabs (td) < sfa)
            {
                *jf = -1;
            }
            else
            {
                akk = 1e0 / akk;
                *pn = akk;
                for (j = 0; j < n; j++)
                {
                    if (j != k)
                        a (k, j) *= akk;
                }
                yk = y[k] * akk;
                y[k] = yk;
                for (i = 0; i < n; i++)
                {
                    aik = a (i, k);
                    if (i != k)
                    {
                        for (j = 0; j < n; j++)
                        {
                            if (j != k)
                                a (i, j) -= aik * a (k, j);
                        }
                        y[i] -= aik * yk;
                    }
                }
                for (i = 0; i < n; i++)
                {
                    if (i != k)
                        a (i, k) *= -akk;
                }
            }
        }
    }
    if (*jf != 0)
    {
        *d = 0e0;
    }
    else
    {
        *d = td;
        for (k = 0; k < n; k++)
        {
            nm1mk = n - 1 - k;
            ki = iw[nm1mk];
            if (nm1mk != ki)
            {
                for (i = 0; i < n; i++)
                {
                    p1 = &a (i, nm1mk);
                    p2 = &a (i, ki);
                    t = *p1;
                    *p1 = *p2;
                    *p2 = t;
                }
            }
        }
    }
}
