
/**
 *
 *     - - - - - -
 *      s u b e t
 *     - - - - - -
 *
 *
 *  remove the E-terms (elliptic component of annual aberration)
 *  from a pre IAU 1976 catalogue ra,dec to give a mean place
 *
 *  (double precision)
 *
 *
 *  given:
 *     rc,dc     dp     ra,dec (radians) with E-terms included
 *     eq        dp     Besselian epoch of mean equator and equinox
 *
 *  returned:
 *     rm,dm     dp     ra,dec (radians) without E-terms
 *
 *  called:
 *     sla_etrms, sla_dcs2c, sla_,dvdv, sla_dcc2s, sla_dranrm
 *
 *
 *  explanation:
 *     Most star positions from pre-1984 optical catalogues (or
 *     derived from astrometry using such stars) embody the
 *     E-terms.  This routine converts such a position to a
 *     formal mean place (allowing, for example, comparison with a
 *     pulsar timing position).
 *
 *
 *  reference:
 *     Explanatory Supplement to the Astronomical Ephemeris,
 *     section 2D, page 48.
 *
 *
 *  p.t.wallace   starlink   july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_etrms (double ep, double *ev);
extern int sla_dcs2c (double a, double b, double *v);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_subet (double rc, double dc, double eq, double *rm, double *dm)
{

    double sla_dranrm (double angle), sla_dvdv (double *va, double *vb),
        sla_drange (double angle);
    double a[3], v[3], f;
    int i;

    /* e-terms */
    sla_etrms (eq, a);

    /* spherical to cartesian */
    sla_dcs2c (rc, dc, v);

    /* include the e-terms */
    f = 1e0 + sla_dvdv (v, a);
    for (i = 0; i < 3; i++)
    {
        v[i] = f * v[i] - a[i];
    }

    /* cartesian to spherical */
    sla_dcc2s (v, rm, dm);

    /* bring ra into conventional range */
    *rm = sla_dranrm (*rm);
}
