
/***********************************************************************

		     hydAstro.c

This program takes a "nessie" format file of 
Ra's and Dec's and converts it to a hydraformat file.

The program expects one argument, a filename.

The output is left in a file with the same name as the input,
and the original file is over-written.

***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "findIt.h"
#include "miscMath.h"
#include "nesslib.h"
#include "nessIO.h"
#include "proto.h"

#define FIBBIN    	"FIBBIN"

#define FIBSTATUS       "FIBSTATUS"     /* env. variable for status files */
#define FIBINITS        "FIBINITS"      /* env. variable for status files */
#define FIBSCRIPTS      "FIBSCRIPTS"    /* env. variable for scripts */

extern int Verbose;

void overrideHeader (void);

struct starinfo *stars[MAXSTARS];
struct exposinfo Exposure;

int expTimeFromArgs = 0;
double userExpTime;

int siderialTimeFromArgs = 0;
double userSiderialTime;

int doReverseAstrometry = 0;

extern int calcPlateCoords (struct starinfo **theStars,
                            struct exposinfo *exposure);
extern int calcAstrometricCoords (struct starinfo **theStars,
                                  struct exposinfo *exposure);

main (int argc, char **argv, char **envp)
{
    FILE *outFile;
    FILE *starFile;

    int i;

    if (argc < 2)
    {
        fputs ("missing argument(s) to astrometry routine\n", stderr);
        exit (1);
    }

    for (i = 2; i < argc; i++)
    {

        if (argv[i][0] != '-')
        {
            fprintf (stderr,
                     "unknown argument \"%s\" to astrometry routine\n",
                     argv[i]);
        }
        else
        {
            switch (argv[i][1])
            {

                case 'T':
                    if (sscanf (argv[i], "-T%lf", &userExpTime) >= 1)
                    {
                        expTimeFromArgs = TRUE;
                    }
                    else
                    {
                        expTimeFromArgs = FALSE;
                        fputs
                            ("Astrometry could not parse exposure time override\n",
                             stderr);
                    }
                    break;

                case 'S':

                    if (sscanf (argv[i], "-S%lf", &userSiderialTime) >= 1)
                        siderialTimeFromArgs = TRUE;
                    else
                    {
                        siderialTimeFromArgs = FALSE;
                        fputs
                            ("Astrometry could not parse sidereal time override\n",
                             stderr);
                    }
                    break;

                case 'R':
                    doReverseAstrometry = TRUE;
                    break;

                default:
                    break;
            }
        }
    }

    if (doReverseAstrometry != TRUE)
    {                           /* do normal astrometry */

        starFile = openStarFile (argv[1], "r");

        if (starFile == NULL)
        {
            fprintf (stderr, "Astrometry Couldn't find file \"%s\"\n",
                     argv[1]);
            exit (1);
        }

        /* load the coordinates and process */

        else
        {
            if (LoadHydraFile (starFile, stars, &Exposure) == FILE_OK)
            {

                /* kludge alert!!!
                   This un-does the instrument mis-alignment correction
                   so that the original value in the field center entry
                   gets written back out un-touched. Otherwise it would
                   be applied here *and* by the programs reading our 
                   output file. */

                FieldRotationalOffset -=
                    (ROTATION_CORRECTION * (M_PI / 180.0));

                /* end kludge */

                overrideHeader ();

                /* do the actual astrometry */

                calcPlateCoords (stars, &Exposure);

            }
            else
            {

                fprintf (stderr,
                         "ERROR: File \"%s\"\nNot readable as Hydra file\n",
                         argv[1]);
            }
        }

        /* create the output file and open it */

        outFile = openCoordFile (argv[1], "w+");

        if (outFile != NULL)
        {

            /* write out the new file */

            hydraFileDump (stars, &Exposure, outFile, 1);

        }
        else
        {
            fputs ("ERROR: Couldn't create output file for plate coords\n",
                   stderr);
            exit (1);
        }
    }

    else
    {                           /* do reverse astrometry */

        starFile = openCoordFile (argv[1], "r");

        if (starFile == NULL)
        {
            fprintf (stderr,
                     " Reverse astrometry Couldn't find file \"%s\"\n",
                     argv[1]);
            exit (1);
        }

        /* load the coordinates and process */

        else
        {
            if (LoadHydraFile (starFile, stars, &Exposure) == FILE_OK)
            {

                /* kludge alert!!!
                   This un-does the instrument mis-alignment correction
                   so that the original value in the field center entry
                   gets written back out un-touched. Otherwise it would
                   be applied here *and* by the programs reading our 
                   output file. */

                FieldRotationalOffset -=
                    (ROTATION_CORRECTION * (M_PI / 180.0));

                if (Verbose)
                    fprintf (stderr, "rotational offset(load) = %f\n",
                             (FieldRotationalOffset * (180 / M_PI)));

                /* end kludge */

                overrideHeader ();

                /* do the actual astrometry in reverse */

                calcAstrometricCoords (stars, &Exposure);

            }
            else
            {

                fprintf (stderr,
                         "ERROR: File \"%s\"\nNot readable as Hydra file\n",
                         argv[1]);
            }
        }

        /* create the output file and open it */

        outFile = createStarFile (argv[1]);

        if (outFile != NULL)
        {

            /* write out the new file without coords */

            hydraFileDump (stars, &Exposure, outFile, 0);
        }
        else
        {

            fputs ("ERROR: Couldn't create output file for plate coords\n",
                   stderr);
            exit (1);
        }
    }

}

void overrideHeader (void)
{

    if (expTimeFromArgs)
    {
        Exposure.exptime = userExpTime;
        /*
           fprintf( stderr, "replacing exposure time with: %f\n", userExpTime );
         */
    }

    if (siderialTimeFromArgs)
    {
        Exposure.st = userSiderialTime;
        /*
           fprintf( stderr, "replacing sidereal time with: %f\n", userSiderialTime );
         */
    }

}
