
/* Generic WIYN Client: gwcXviewInit.c

 *  Module name:
        gwcXviewInit.c

 *  Function:
        This file is the Xview specific code for GWC integration.

 *  Description:
        This file interfaces GWC to the Xview event handling mechanism.
        Connection/reconnect code specific to Xview is here too.

 *  Language:
      C

 *  Support: Kim Gillies, NOAO

 *

 *  History:
      12 Dec 94: Version 2 - goes into CVS

 *  Rcs Id:  $Id: gwcXviewInit.c,v 1.1.1.1 2004/08/13 20:46:05 behzad Exp $

 */
#include <stdio.h>
#include <assert.h>
#include <xview/xview.h>
#include <gwc/gwc.h>

/* In file data */
static int gDelay = 10;         /* 10 seconds */
static int gRestartCount = 0;
static int gRestartTotal = 10000;

/* Connection hash table */
static Tcl_HashTable fdTable;
static int tableinited = 0;

/* Structure stored as part of the connection data */
typedef struct _xviewClientData
{
    Notify_client outclient;
    Notify_client inclient;
    Notify_client exclient;
}
xviewClientData;

/* External reference from gwcConnect */
extern int gwcSetupHandlers (gwcConnectInfo info);
extern int gwcRouterConnect (gwcConnectInfo info);
extern int gwcMakeFinalConnection (gwcConnectInfo info, connectType yesorno);

/* In file prototypes */
static Notify_value xviewSender (Notify_client client, int fd);
static Notify_value xviewReceiver (Notify_client client, int fd);
static Notify_value xviewException (Notify_client client, int fd);
static void gwcXviewConnectCallback (ClientData clientData);
static void xviewSetDelay (Notify_client cl,
                           int seconds, int useconds, Notify_func f);
static Notify_value xviewRestartFunc (Notify_client cl, int fd);

/* System-specific message system connection */
int gwcLibSetup (void)
{
    /* Initialize the file descriptor table  first time */
    if (!tableinited)
    {
        Tcl_InitHashTable (&fdTable, TCL_ONE_WORD_KEYS);
        tableinited = 1;
    }
    return TCL_OK;
}

int gwcLibSetupHandlers (gwcConnectInfo info)
{
    int sock;
    int new;
    Tcl_HashEntry *entryPtr;
    xviewClientData *xdata;

    assert (info != (gwcConnectInfo) NULL);

    sock = gwcGetClientSocket (info);
    xdata = (xviewClientData *) malloc (sizeof (xviewClientData));
    assert (xdata != (xviewClientData *) NULL);

    xdata->inclient = (Notify_client) xv_unique_key ();
    xdata->outclient = (Notify_client) xv_unique_key ();
    xdata->exclient = (Notify_client) xv_unique_key ();
    gwcSetClientData (info, (ClientData) xdata);

    entryPtr = Tcl_CreateHashEntry (&fdTable, (char *) sock, &new);
    /* Check that it is new */
    assert (new == 1);
    Tcl_SetHashValue (entryPtr, info);

    /*fprintf(stderr, "Add Handler for socket: %d\n", sock); */
    notify_set_input_func (xdata->inclient, xviewReceiver, sock);
    /*notify_set_output_func(xdata->outclient, xviewOutput, sock); */
    notify_set_exception_func (xdata->exclient, xviewException, sock);
    return TCL_OK;
}

/* System specific break connection code */
int gwcLibDestroyHandlers (gwcConnectInfo info)
{
    int sock;
    xviewClientData *xdata;
    Tcl_HashEntry *entryPtr;

    assert (info != (gwcConnectInfo) NULL);

    /* Get the data directly but then look up the entry for deletion */
    xdata = (xviewClientData *) gwcGetClientData (info);
    if (xdata == (xviewClientData *) NULL)
    {
        fprintf (stderr, "xviewCientData NULL\n");
        return TCL_ERROR;
    }
    sock = gwcGetClientSocket (info);
    /*fprintf(stderr, "Delete Handler for socket: %d\n", sock); */

    /* Check for a handler for deletion */
    entryPtr = Tcl_FindHashEntry (&fdTable, (char *) sock);
    if (entryPtr == (Tcl_HashEntry *) NULL)
    {
        fprintf (stderr, "Found no fd table entry for: %d\n", sock);
        return NOTIFY_DONE;
    }

    /* Set the message receiver code */
    notify_set_input_func (xdata->inclient, NOTIFY_FUNC_NULL, sock);
    /*notify_set_output_func(xdata->outclient, NOTIFY_FUNC_NULL, sock); */
    notify_set_exception_func (xdata->exclient, NOTIFY_FUNC_NULL, sock);

    /* Free up the socket */
    (void) close (sock);

    /* Free up the hash table entry but not the info structure */
    Tcl_DeleteHashEntry (entryPtr);

    /* Set to a safe value (0 is stdout) */
    gwcSetClientSocket (info, 0);

    return TCL_OK;
}

/* Start a reconnect sequence */
void gwcLibDefaultConnectCallback (ClientData clientData)
{
    gwcConnectInfo info = (gwcConnectInfo) clientData;

    gRestartCount = 0;
    gwcXviewConnectCallback (info);
}

/* Accept restart parameters from upper layer */
void gwcLibPassRestartParameters (int delay, int restartcount)
{
    gDelay = delay;
    gRestartTotal = restartcount;
}

/* --------------------- Internal Private Functions --------------------- */
 /*ARGSUSED*/ static Notify_value xviewReceiver (Notify_client client, int fd)
{
    gwcConnectInfo info;
    Tcl_HashEntry *entryPtr;

    entryPtr = Tcl_FindHashEntry (&fdTable, (char *) fd);
    if (entryPtr == (Tcl_HashEntry *) NULL)
    {
        fprintf (stderr, "Found no fd table entry for: %d\n", fd);
        return NOTIFY_DONE;
    }

    info = (gwcConnectInfo) Tcl_GetHashValue (entryPtr);
    assert (info != (gwcConnectInfo) NULL);
    /*fprintf(stderr, "Sock: %d\n", CONNECT_SOCK(info)); */

    /* Process the incoming data */
    (void) gwcGetProcessEDM (info);
    return NOTIFY_DONE;
}

 /*ARGSUSED*/ static Notify_value xviewSender (Notify_client client, int fd)
{
    gwcConnectInfo info;
    Tcl_HashEntry *entryPtr;

    entryPtr = Tcl_FindHashEntry (&fdTable, (char *) fd);
    if (entryPtr == (Tcl_HashEntry *) NULL)
    {
        fprintf (stderr, "Found no fd table entry for: %d\n", fd);
        return NOTIFY_DONE;
    }
    info = (gwcConnectInfo) Tcl_GetHashValue (entryPtr);
    assert (info != (gwcConnectInfo) NULL);

    /*fprintf(stderr, "Send data on: %d\n",  gwcGetClientSocket(info)); */
    return NOTIFY_DONE;
}

 /*ARGSUSED*/
    static Notify_value xviewException (Notify_client client, int fd)
{
    fprintf (stderr, "Exception on fd: %d\n", fd);
    return NOTIFY_DONE;
}

/* The xview reconnect callback */
static void gwcXviewConnectCallback (ClientData clientData)
{
    gwcConnectInfo info = (gwcConnectInfo) clientData;

    /* Check for the first time - delay if it's zero */
    if (gRestartCount == 0)
    {
        /* Up the count and try again later */
        ++gRestartCount;
        xviewSetDelay ((Notify_client) info, gDelay, 0, xviewRestartFunc);
        return;
    }

    /* Print a message on the users screen */
    /* Add one so that numbers go from 1 to limit rather than 0 */
    gwcPrintStatus (info,
                    "Trying to connect to router on: %s - (attempt:%d) ",
                    gwcGetRouterHost (info), gRestartCount);
    if (gRestartCount < gRestartTotal)
    {
        /* Make a connection attempt here */
        if (gwcRouterConnect (info) == TCL_OK)
        {
            gwcPrintStatus (info, "Connected!\n");
            /* Now run the restart func if there is one */
            if (gwcMakeFinalConnection (info, CONNECT_YES) != TCL_OK)
            {
                return;
            }
        }
        else
        {
            /* Up the count and try again later */
            gwcPrintStatus (info, "Failed...\n");
            ++gRestartCount;
            xviewSetDelay ((Notify_client) info, gDelay, 0, xviewRestartFunc);
        }
    }
    else
    {
        gwcPrintStatus (info, "Reconnection failed.\n");
        /* Give the app a chance to notice the failure */
        gwcMakeFinalConnection (info, CONNECT_NO);
    }
}

static Notify_value xviewRestartFunc (Notify_client cl, int fd)
{
    gwcConnectInfo info;

    info = (gwcConnectInfo) cl;
    assert (info != (gwcConnectInfo) NULL);

    gwcXviewConnectCallback ((ClientData) info);

    return NOTIFY_DONE;
}

/* Function: xviewSetDelay
 * Description: This function is called to set a single alarm.   The
 *              Notify_func f will be called after seconds and useconds
 *              delay.
 * Arguments: Notify_client cl: the xview Notify_client - some value
 *            int seconds: number of seconds to delay +
 *            int useconds: number of microseconds to delay
 *            Notify_fun f: function called once delay is up.
 * Returns: nothing
 */
static void xviewSetDelay (Notify_client cl,
                           int seconds, int useconds, Notify_func f)
{
    static struct itimerval atime;      /* for the wake up timer */

    /* Set the next timeout */
    atime.it_value.tv_sec = seconds;
    atime.it_value.tv_usec = useconds;
    atime.it_interval.tv_sec = 0;
    atime.it_interval.tv_usec = 0;
    notify_set_itimer_func (cl, f, ITIMER_REAL, &atime, NULL);
}
