
/***********************************************************

		fibSim.c

   This file contains the routines for drawing a picture
   of the current system configuration.  This version
   is for xview/xlib.

   Included functions are:

      * refresh()	 clears the screen and re-draws everything 
      * initScreen()	 clears the screen at sets graphics mode 
      * drawButton()	 draws a button without the tube attached
      * drawTube()	 draws the tube, but not the button 
      * drawTarget()	 draws a "crosshair" at the requested locations 

   ( * indicated functions that is declared as an extern )

***********************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

#include "gc.h"

#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "buttons.h"
#include "kaboom.h"
#include "nesslib.h"
#include "coordConvert.h"
#include "concen.h"
#include "xref.h"
#include "xFibSim.h"
#include "fibwindow.h"
#include "inits.h"
#include "idIt.h"

void drawScreenAtRadius ();
void animateScreen ();

void drawSteps ()
{
    drawStepBoundry (PLATE, HydraWinInfo.plateGCzones);
}

/*
		drawStepBoundry

     draw circles to mark the important locations on the focal plane.
*/

drawStepBoundry (radius, gc)
long radius;
GC gc;

{
    double tweakedDia;

    double x2;
    double y2;

    double dx = HydraWinInfo.canvasSizeX / 2;
    double dy = HydraWinInfo.canvasSizeY / 2;

    double dia = ((2 * radius) * HydraWinInfo.scaleFactor);

    /* x2 and y2 are the coordinates of the corner of the box that
       bounds the circle */

    if (((int) (dia + 0.5) % 2) == 0)
        dia = dia -= 1.0;

    x2 = dx - ((dia / 2) + 0.5);
    y2 = dy - ((dia / 2) + 0.5);

    XDrawArc (HydraWinInfo.display,
              HydraWinInfo.canvasXid,
              gc,
              (int) x2, (int) y2,
              (int) (dia + 0.5), (int) (dia + 0.5),
              0, 23040 /* full circle */ );

}

/*
		drawButton

   Draw the button specified by the argument.
   This function does not draw the tube.
*/

void drawButton (fib, geo, gc)
struct fiber_dbase *fib;
struct flex_geometry *geo;
GC gc;

{
    double tweakedDia;

    double x2;
    double y2;

    double dx = HydraWinInfo.canvasSizeX / 2;
    double dy = HydraWinInfo.canvasSizeY / 2;

    double dia = (fib->ButtonDia * HydraWinInfo.scaleFactor);

    /* x1 and y1 are the coordinates of the center of the button */

    double x1 = (geo->button.x * HydraWinInfo.scaleFactor) + dx;
    double y1 = -(geo->button.y * HydraWinInfo.scaleFactor) + dy;

    /* x2 and y2 are the coordinates of the corner of the box that
       bounds the he button */

    if (((int) (dia + 0.5) % 2) == 0)
        dia = dia -= 1.0;

    x2 = x1 - (dia / 2) + 0.5;
    y2 = y1 - (dia / 2) + 0.5;

    XDrawArc (HydraWinInfo.display,
              HydraWinInfo.canvasXid,
              gc,
              (int) x2, (int) y2,
              (int) (dia + 0.5), (int) (dia + 0.5),
              0, 23040 /* full circle */ );

}

/*
		drawTube

   Draw the tube as a series of straight segments.
   This function does not draw the button.
*/

void drawTube (fib, geo, gc)
struct fiber_dbase *fib;
struct flex_geometry *geo;
GC gc;

{
    int i;

    double dx = HydraWinInfo.canvasSizeX / 2;
    double dy = HydraWinInfo.canvasSizeY / 2;

    XPoint points[NUM_TUBE_SEGS + 1];

    struct aPoint pointN;       /* the point on the tube */

    /* set the starting point */

    points[0].x = (int) ((geo->segments[0]->p1->x *
                          HydraWinInfo.scaleFactor) + dx + 0.5);

    points[0].y = (int) (-(geo->segments[0]->p1->y *
                           HydraWinInfo.scaleFactor) + dy + 0.5);

    for (i = 0; i < (NUM_TUBE_SEGS); i++)
    {

        /* draw the segment */

        points[i + 1].x = (int) ((geo->segments[i]->p2->x *
                                  HydraWinInfo.scaleFactor) + dx + 0.5);

        points[i + 1].y = (int) (-(geo->segments[i]->p2->y *
                                   HydraWinInfo.scaleFactor) + dy + 0.5);

    }

    XDrawLines (HydraWinInfo.display,
                HydraWinInfo.canvasXid,
                gc, points, (NUM_TUBE_SEGS + 1), CoordModeOrigin);

}

/*
		drawBackground

   Draw the objects in the field
*/

void drawBackground ()
{
    int i;

    for (i = 1; i < MAXSTARS; i++)
    {
        drawStar (i, OBJ_DEFAULT_MODE);
    }
}

/*
		drawStar

   Draw a specific sky object in the requested mode
*/

drawStar (id, flag)
int id;
int flag;

{
    int targetType;
    int mode;

    struct aPoint target;

    if (Stars[id] != NULL)
    {

        target = Stars[id]->specEncoderCoord;

        switch (Stars[id]->objectType)
        {

            case 'C':
                targetType = T_TARG;
                mode = STAR_NORMAL;
                break;

            case 'A':
                targetType = STAR;
                mode = ACQ_NORMAL;
                break;

            case 'F':
                targetType = X_TARG;
                mode = ACQ_NORMAL;
                break;

            case 'S':
                targetType = T_BOX;
                mode = SKY_NORMAL;
                break;

            case 'O':
                targetType = T_TARG;
                mode = STAR_NORMAL;
                break;

            default:
                break;
        }

        if (Rotflag)
            rotate (&target, RotationAngle);

        /* "flag" can override the above decision on GC mode */

        if (flag == OBJ_SELECTED_MODE)
            mode = STAR_SELECTED;

        drawTarget (&target, targetType, mode);
    }
}

/*
		drawTarget

   draw a target of the type specified.
*/

void drawTarget (coords, type, mode)
struct aPoint *coords;
int type;
int mode;

{
    double dx = HydraWinInfo.canvasSizeX / 2;
    double dy = HydraWinInfo.canvasSizeY / 2;

    GC gc;

    /* set up the end points for each segment */

    /* the 0.5 factor insures the values round during the casts below */

    double xCntr = (coords->x * HydraWinInfo.scaleFactor) + dx + 0.5;
    double yCntr = -(coords->y * HydraWinInfo.scaleFactor) + dy + 0.5;

    double xStart =
        (coords->x * HydraWinInfo.scaleFactor) + dx - XV_TARGETSIZE + 0.5;
    double xEnd =
        (coords->x * HydraWinInfo.scaleFactor) + dx + XV_TARGETSIZE + 0.5;

    double yStart =
        -(coords->y * HydraWinInfo.scaleFactor) + dy - XV_TARGETSIZE + 0.5;
    double yEnd =
        -(coords->y * HydraWinInfo.scaleFactor) + dy + XV_TARGETSIZE + 0.5;

    switch (mode)
    {

        case STAR_NORMAL:
            gc = HydraWinInfo.starGCnormal;
            break;

        case STAR_SELECTED:
            gc = HydraWinInfo.starGCselect;
            break;

        case SKY_NORMAL:
            gc = HydraWinInfo.skyGCnormal;
            break;

        case ACQ_NORMAL:
            gc = HydraWinInfo.acqGCnormal;
            break;

        default:
            gc = HydraWinInfo.fiberGCnormal;
            fputs ("Using default GC for target\n", stderr);
            break;
    }

    switch (type)
    {

        case T_TARG:

            /* draw the horizontal segment */

            XDrawLine (HydraWinInfo.display,
                       HydraWinInfo.canvasXid,
                       gc,
                       (int) xStart, (int) yCntr, (int) xEnd, (int) yCntr);

            /* draw the vertical segment */

            XDrawLine (HydraWinInfo.display,
                       HydraWinInfo.canvasXid,
                       gc,
                       (int) xCntr, (int) yStart, (int) xCntr, (int) yEnd);

            break;

        case X_TARG:

            /* draw the first segment */

            XDrawLine (HydraWinInfo.display,
                       HydraWinInfo.canvasXid,
                       gc,
                       (int) xStart, (int) yStart, (int) xEnd, (int) yEnd);

            /* draw the second segment */

            XDrawLine (HydraWinInfo.display,
                       HydraWinInfo.canvasXid,
                       gc,
                       (int) xStart, (int) yEnd, (int) xEnd, (int) yStart);
            break;

        case BOX:

            drawBox (coords, XV_SMALLBOX, mode);
            break;

        case STAR:

            /* recurse and do both types */

            drawTarget (coords, T_TARG, mode);
            drawTarget (coords, X_TARG, mode);
            break;

        case T_BOX:

            /* recurse and do two types */

            drawTarget (coords, T_TARG, mode);
            drawTarget (coords, BOX, mode);
            break;

        default:
            break;

    }
}

/*
		drawBox

   Draw a hexagon with the "radius" specified by the argument.
*/

void drawBox (coords, size, mode)
struct aPoint *coords;
int size;
int mode;

{
    double dx = HydraWinInfo.canvasSizeX / 2;
    double dy = HydraWinInfo.canvasSizeY / 2;

    XPoint points[5];

    GC gc;

    switch (mode)
    {

        case STAR_NORMAL:
            gc = HydraWinInfo.starGCnormal;
            break;

        case STAR_SELECTED:
            gc = HydraWinInfo.starGCselect;
            break;

        case SKY_NORMAL:
            gc = HydraWinInfo.skyGCnormal;
            break;

        case ACQ_NORMAL:
            gc = HydraWinInfo.acqGCnormal;
            break;

        default:
            gc = HydraWinInfo.fiberGCnormal;
            fputs ("Using default GC for target\n", stderr);
            break;
    }

    /* upper left corner */
    points[0].x =
        (int) ((coords->x * HydraWinInfo.scaleFactor) + dx - (size / 2) +
               0.5);
    points[0].y =
        (int) (-(coords->y * HydraWinInfo.scaleFactor) + dy - (size / 2) +
               0.5);

    /* lower right corner */
    points[2].x =
        (int) ((coords->x * HydraWinInfo.scaleFactor) + dx + (size / 2) +
               0.5);
    points[2].y =
        (int) (-(coords->y * HydraWinInfo.scaleFactor) + dy + (size / 2) +
               0.5);

    /* upper right corner */
    points[1].x = points[2].x;
    points[1].y = points[0].y;

    /* lower left corner */
    points[3].x = points[0].x;
    points[3].y = points[2].y;

    /* upper left again... */
    points[4].x = points[0].x;
    points[4].y = points[0].y;

    XDrawLines (HydraWinInfo.display,
                HydraWinInfo.canvasXid, gc, points, 5, CoordModeOrigin);

}

/*
		drawCrosshair

   Draw a crosshair in a circle  with the "radius" specified by the argument.
*/

void drawCrosshair (coords, size, mode)
struct aPoint *coords;
int size;
int mode;

{
    int dx = HydraWinInfo.canvasSizeX / 2;
    int dy = HydraWinInfo.canvasSizeY / 2;

    int x = (int) (coords->x * HydraWinInfo.scaleFactor + 0.5) + dx;
    int y = (int) -(coords->y * HydraWinInfo.scaleFactor + 0.5) + dy;

    int xa;
    int xb;
    int xc;
    int xd;

    int ya;
    int yb;
    int yc;
    int yd;

    int xCntr;
    int yCntr;

    int xCir;
    int yCir;

    int outerDist;
    int innerDist;

    GC gc;

    switch (mode)
    {

            /* currently only one kind available */

        default:
            gc = HydraWinInfo.crosshairGC;
            break;
    }

    outerDist = (5 * size) / 4;
    innerDist = size / 4;

    xCntr = x;
    yCntr = y;

    xa = x - outerDist;
    xb = x - innerDist;

    xc = x + innerDist;
    xd = x + outerDist;

    ya = y - outerDist;
    yb = y - innerDist;

    yc = y + innerDist;
    yd = y + outerDist;

    xCir = x - size;
    yCir = y - size;

    XDrawLine (HydraWinInfo.display, HydraWinInfo.canvasXid, gc,
               xa, yCntr, xb, yCntr);

    XDrawLine (HydraWinInfo.display, HydraWinInfo.canvasXid, gc,
               xc, yCntr, xd, yCntr);

    XDrawLine (HydraWinInfo.display, HydraWinInfo.canvasXid, gc,
               xCntr, ya, xCntr, yb);

    XDrawLine (HydraWinInfo.display, HydraWinInfo.canvasXid, gc,
               xCntr, yc, xCntr, yd);

    XDrawArc (HydraWinInfo.display, HydraWinInfo.canvasXid, gc,
              (int) xCir, (int) yCir,
              (2 * size + 1), (2 * size + 1), 0, 23040 /* full circle */ );

}

/*
		drawHex

   Draw a hexagon with the "radius" specified by the argument.
*/

void drawHex (x, y, radius)
long x, y;
long radius;

{

}

/*
		drawFib

   Draw the specified fiber in the specified mode.

*/

void drawFiber (fib, geo, mode)
struct fiber_dbase *fib;
struct flex_geometry *geo;
int mode;

{
    GC gc;                      /* the appropriate graphics context */

    switch (mode)
    {

        case FIBER_ERASE:
            gc = HydraWinInfo.fiberGCerase;
            break;

        case FIBER_NORMAL:
            gc = HydraWinInfo.fiberGCnormal;
            break;

        case FIBER_LOCKED:
            gc = HydraWinInfo.fiberGClocked;
            break;

        case FIBER_OCCULTED:
            gc = HydraWinInfo.fiberGCoccult;
            break;

        case FOPS_NORMAL:
            gc = HydraWinInfo.fopsGCnormal;
            break;

        case FIBER_SELECT:
            gc = HydraWinInfo.fiberGCselect;
            break;

        case FIBER_DRAG:
            gc = HydraWinInfo.fiberGCdrag;
            break;

        default:
            break;
    }

    drawButton (fib, geo, gc);
    drawTube (fib, geo, gc);

}

/*
		labelAt

   draw a label at the specified coordinate.
*/

void labelAt (string, coords)
char *string;
struct plate_coords *coords;

{
}

/*
		refresh

   Clear the screen then draw the system in its current configuration. 
   This re-draws all buttons and tubes. 
*/

void refresh ()
{
    int i;
    int drawMode;

    drawSteps ();

    if (BackgroundFlag)
        drawBackground ();

    for (i = 0; i < MAXFIBER; i++)
    {

        drawMode = getFiberType (i);

        drawFiber (&FiBase[i], FiBase[i].CurGeometry, getFiberType (i));
    }
}

refreshSelections ()
{
    if (CurrentFiber.status == SELECTED)
    {

        drawFiber (&FiBase[CurrentFiber.id],
                   FiBase[CurrentFiber.id].CurGeometry, FIBER_SELECT);
    }

    if (CurrentObject.status == SELECTED)
        drawStar (CurrentObject.id, OBJ_SELECTED_MODE);

    if (CurrentCrosshair.status == SELECTED)
        drawCrosshair (&CurrentCrosshair.location, 15, 0);
}

/*
		screenUpdate

   Redraw any fibers that are out of date, and clear
   the outOfDate flag.
*/

void screenUpdate ()
{
    struct flex_geometry *tmpGeo;
    int i;

    for (i = 0; i < MAXFIBER; i++)
    {

        if (FiBase[i].outOfDate)
        {

            /*
               The TrialGeometry has the *previous* location of the fiber.
               (At least it does at the time this routine is called!)
               Get back to that location temporarily so that we can
               erase the fiber.
             */

            drawFiber (&FiBase[i], FiBase[i].TrialGeometry, FIBER_ERASE);

            /* now put the geometry back and draw the fiber at the
               new location */

            drawFiber (&FiBase[i], FiBase[i].CurGeometry, getFiberType (i));

            /* clear the flag */

            FiBase[i].outOfDate = FALSE;

        }
    }
}

getFiberType (id)
int id;

{

    if (isOcculted (&FiBase[id]))
        return (FIBER_OCCULTED);

    if (fiberInCable (id, getCableIdentifier ("FOPS")))
        return (FOPS_NORMAL);

    if (fiberInCable (id, CurrentCable))
        return (FIBER_NORMAL);

    if (fiberInCable (id, getCableIdentifier ("NULL")))
        return (FIBER_VOID);

    /* finally, the default */
    return (FIBER_LOCKED);
}
