
/***********************************************************************

		autorot.c

   This file contains the functions that implement interactive
   user interface.  These are very device specific commands,
   and have be deliberately isolated from more general commands
   for ease of reading.  They are called from the main parsing
   loop in "parser.c"

***********************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>

#include "miscMath.h"
#include "nesslib.h"
#include "nessIO.h"
#include "flex.h"
#include "detrot.h"
#include "prompter.h"
#include "fiborg.h"
#include "log.h"

#define ADD_POINT_CHAR 'a'
#define ADD_POINT_STRING "a"

/**************  Local Structures ******************/

char *detrotMenuItems[] = {

    /* the prompt */
    "What next?",
    /* the choices (automatically labeled in order of appearance) */
    "Measure additional objects.",      /* choice a */
    "List data points.",        /* choice b */
    "Start over.",              /* choice c */
    "Exit without updating rotation value.",    /* choice d */
    "Exit with new rotation value in use.",     /* choice d */
    NULL,
    ADD_POINT_STRING
};

char *listrotMenuItems[] = {

    /* the prompt */
    "",                         /*  no prompt required */
    /* the choices (automatically labeled in order of appearance) */
    "Delete a data point.",     /* choice a */
    "Re-list points.",          /* choice b */
    "Continue.",                /* choice c */
    NULL,
    "c"
};

struct sigvec OldVec;

void listAndEdit (void);
void RotDeath (int sig, int code, struct sigcontext *scp);

struct sigvec DeathVec = {
    RotDeath,
    0,
    SV_INTERRUPT
};

/****************** externs **********************/

extern int GripperParked;
extern int setBlock (int flag);
extern void gripin (char *args);
extern int cload (char *args);
extern void aload (char *args, int flag1);
extern void clearRot (char *args);
extern char getMenuSelection (char **menuItems, FILE * devIn, FILE * devOut);
extern int userToInternalID (aStar **, int, int *);
extern int detrot (char *args);
extern void listrot (char *args);
extern void applyRot (char *args);

/*
		autoRotation

   Automated rotation determination routine.  Walks user through
   the steps.
*/
void autoRotation (char *args)
{
    char fileName[128];
    char *tmpName;
    char *starFile = args;

    double siderialTime;
    double siderialTimeDefault;

    int precision;

    double doubleMin;
    double doubleMax;

    char theCommand[128];

    int pointCounter;
    int exit;
    char nextAction;

    char *object;
    char objectString[32];

    int tmpId;
    int internalId;

    if (sigvec (SIGINT, &DeathVec, &OldVec))
        fputs ("Warning: interrupt handler for \"setup\" failed to install\n",
               stderr);

    /* turn off requests from the graphics */

    setBlock (TRUE);

    /* be sure the gripper is in. */

    if (GripperParked)
    {

        fputs ("ATTENTION:  Gripper was disabled--re-enabling...\n\n",
               stderr);
        gripin (0);
    }

   /******  get the file name if not on command line ******/

    /* skip over whitespace */

    while (*starFile == ' ')
        starFile++;

    if (*starFile == '\0')
    {

      namePrompt:

        tmpName = getUserInput (stderr, stdin, CHAR, NO_DEFAULT,
                                "\nEnter name of file.", NULL, NULL, NULL,
                                NULL);

        if (tmpName == NULL)
        {
            fputs ("You must specify a star field!\n", stderr);
            goto namePrompt;
        }

        /* starFile points to a reusable static, so get a copy quick! */

        strcpy (fileName, tmpName);

        /* get rid of the newline at the end... */

        fileName[strlen (fileName) - 1] = '\0';

        /* now link it to the pointer we'll use for the rest of this */

        starFile = fileName;
    }

   /*****  get the header of the file  *****/

    if (cload (starFile) != 0)
    {
        setBlock (FALSE);
        return;
    }

    /* prompt sidereal time. Default has to be in radians... */

    siderialTimeDefault = (Exposure.st / 12) * M_PI;

    doubleMin = 0.0;
    doubleMax = (2 * M_PI);
    precision = 0;

    siderialTime =
        *(double *) getUserInput (stderr, stdin, TIME, SHOW_DEFAULT,
                                  "\nEnter the anticipated sidereal time at midpoint.",
                                  &siderialTimeDefault, &doubleMin,
                                  &doubleMax, &precision);

    /* finish loading the file */
    /* set up the command using the recently obtained values */

    sprintf (theCommand, "%s -T%f -S%f",
             starFile, Exposure.exptime, ((siderialTime / M_PI) * 12));

   /***** computer astrometry and load the coordinates *****/

    aload (starFile, NO_CABLE_CHECK);

  Beginning:

    pointCounter = 0;
    exit = 0;
    nextAction = ADD_POINT_CHAR;

    clearRot ("0");             /* clears the rotation points */

    while (!exit)
    {

        if (pointCounter >= 2)
            nextAction = getMenuSelection (detrotMenuItems, stdin, stderr);

        switch (nextAction)
        {

                /* see 'detrotMenuItems for explanation of items */

            case 'a':

                object = getUserInput (stderr, stdin, CHAR, NO_DEFAULT,
                                       "Enter object number.", NULL, NULL,
                                       NULL, NULL);

                if (object != NULL)
                {
                    strcpy (objectString, object);

                    if (sscanf (objectString, " %d", &tmpId) < 1)
                    {
                        fputs ("ERROR: non numeric id#, try again..\n",
                               stderr);
                        continue;
                    }

                    if (userToInternalID (Stars, tmpId, &internalId) == 0)
                    {
                        fputs
                            ("ERROR: Requested object does not exist, try again..\n",
                             stderr);
                        continue;
                    }

                    if (detrot (objectString))
                    {
                        pointCounter++;
                    }

                    if (pointCounter >= 2)
                        listrot ("w");
                }

                break;

            case 'b':

                listAndEdit ();

                break;

            case 'c':
                goto Beginning;
                break;

            case 'd':
                fputs ("\nExiting detrot. --No Change to rotation value.\n",
                       stderr);
                exit = TRUE;
                break;

            case 'e':
                applyRot ("");
                fputs ("Exited, with update\n", stderr);
                exit = TRUE;
                break;

            default:
                break;
        }

    }

    setBlock (FALSE);
}

void listAndEdit (void)
{

    char nextAction;
    char *dataPoint;
    char dataPointString[32];
    int exit = FALSE;

    /* print an initial copy of the list */

    listrot ("s");              /* list "sets" of rotation points */

    while (!exit)
    {

        nextAction = getMenuSelection (listrotMenuItems, stdin, stderr);

        switch (nextAction)
        {

                /* see 'listrotMenuItems for explanation of items */

            case 'a':

                dataPoint = getUserInput (stderr, stdin, CHAR, NO_DEFAULT,
                                          "delete which point?", NULL, NULL,
                                          NULL, NULL);

                if (dataPoint != NULL)
                {
                    strcpy (dataPointString, dataPoint);
                    clearRot (dataPointString);
                }

                break;

            case 'b':

                listrot ("s");  /* list "sets" of rotation points */

                break;

            case 'c':

                exit = TRUE;
                break;

            default:
                break;
        }
    }
}

/*
	Special handler for user ctrl-c.
*/

void RotDeath (int sig, int code, struct sigcontext *scp)
{

    if (sigvec (SIGINT, &OldVec, NULL))
        log_fputs ("Old interrupt handler failed to install\n", stderr);

    setBlock (FALSE);

    labtyd (0, "WARNING:  User interrupt. Abandoning \"detrot\"\n");

}
