
/*****************************************************

	 buttons.c

   Routines for picking up and moving buttons.
   Includes the following functions:

   *  RadialTest()
   *  ParkButt()
   *  Unstow()
   *  MoveButt()
   *  ViewButt()
      GrabButt()
      RemGrab()
      DropButt()
      RemDrop()
   *  buttonPresent()
   *  unresolved()

   (  * indicated functions are declared as externs )

   Routines for collision avoidance and fiber flexure
   geometry are in "kaboom.c"

*****************************************************/

#include <stdio.h>
#include <setjmp.h>
#include <sys/time.h>
#include <math.h>
#include <unistd.h>

#include "log.h"
#include "errors.h"
#include "miscMath.h"
#include "death.h"
#include "plate.h"
#include "motor.h"
#include "reports.h"
#include "translate.h"
#include "detrot.h"
#include "flex.h"
#include "archive.h"
#include "fiborg.h"
#include "kaboom.h"
#include "buttons.h"
#include "coordConvert.h"
#include "nesslib.h"
#include "nessIO.h"
#include "coordConvert.h"
#include "xref.h"

#ifdef STOPWATCH
#include "stopwatch.h"
#endif

#ifdef HYDRA_II
#include "hydra_system.h"
#include "hydra_mc.h"
#endif
// HYDRA_II

static char log_buf[256];

extern void eccenCalcVec (fibDB *, point *, point *);
int buttonAlreadyThere (struct fiber_dbase *fib, int deadZone);
extern int GoToXY (long int x, long int y, double vel, double accel,
                   int system);
int GrabButt (struct fiber_dbase *fib, struct aPoint *targetCoord);
int RemGrab (struct fiber_dbase *fib, struct aPoint *targetCoord);
int DropButt (struct fiber_dbase *fib, struct aPoint *targetCoord,
              int stepFlag);
int RemDrop (struct fiber_dbase *fib, struct aPoint *targetCoord,
             int stepFlag);
int FaithDrop (struct fiber_dbase *fib, struct aPoint *targetCoord,
               int stepFlag);
extern int OKtoMove (void);
extern int GoToZ (long int z, double vel, double accel, int sync);
extern void flushStatus (void);
extern int getYesNo (FILE * devOut, FILE * devIn, char *prompt,
                     int defaultValue);

/* 
		xyValidForButton

   Check that the location specified is a valid
   place to put the specified button. Return
   TRUE if it is acceptable, and FALSE if it is
   not. Leave the validated geometry for the
   new location in the TrialGeometry field

   This routine applys the rotation and 
   eccentricity corrections to the requested
   coordinate. 

   ATTENTION: 
   Once a move is validated by this routine,
   the only source of coordinates for any move
   or physical action should come from the
   geometry structure this routine has processed.
   This is because the requested coordinate is
   modified in place for rotation and eccentricity and 
   may not be the same as the requested coordinate.

   WARNING:
   If the new location is acceptable,
   This function will leave the FiberInTransit
   structure set to the specified fiber/button,
   and the state of this fiber as "move pending".

   When the move has been completed, recordNewLocation
   must be called to properly update the database.
   This will also clear the FiberInTransit.  Optionally,
   restoreOldLocation may be called to put things
   back the way they were before the call to
   this function.

*/
int xyValidForButton (struct fiber_dbase *button, struct aPoint *target)
{
    int NoKaboom;               /* flag for collision determination */

    /* TRUE means location is OK. */

    struct flex_geometry *originalGeometry;

    struct aPoint eccenVector;  /* correction for eccentricity */

    /* apply rotation correction if required */

    if (Rotflag)
        rotate (target, RotData.aveAngle);

    /* apply eccentricity correction if required */

    if (EccenFlag)
    {
        eccenCalcVec (button, target, &eccenVector);
        applyVec (target, &eccenVector, NORMAL);
    }

    /*
       Allocate the new geometry structure.
       Save a link back to the old one in the
       global FiberInTransit structure.
     */

    if (prepNewLocation (button, target))
        labtyd (SYSTEM_CALL, "Couldn't prep. new location in database");

    /* replace the link to the current geometry with that of the  target
       for the collision testing.  This is done because the collision detection 
       asks, "Do these fibers collide?" *not* "Will they collide?"
     */

    originalGeometry = button->CurGeometry;

    button->CurGeometry = button->TrialGeometry;

    NoKaboom = DestOK (FiBase, button);

    if (NoKaboom != TRUE)
    {                           /* this location is no good */

#ifdef LOG
        log_fputs ("WARNING:  Invalid target destination for button.\n",
                   stderr);
#else
        fputs ("WARNING:  Invalid target destination for button.\n", stderr);
#endif

        if (FiberInTransit.state != IN_TRANSIT)
        {
            restoreOldLocation ();
        }
    }

    /* put the current geometry back */

    button->CurGeometry = originalGeometry;

    return (NoKaboom);

}

/* 
		xyValidPlate

   Check that the location specified is a valid
   place to put the specified button. Return
   TRUE if it is acceptable, and FALSE if it is
   not. Leave the validated geometry for the
   new location in the TrialGeometry field

   Use this routine when the intent is to put
   a *button* at a specific plate coordinate.
   No rotation or concentricity corrections
   are applied to the target coordinate.

   ATTENTION: 
   Once a move is validated by this routine,
   the only source of coordinates for any move
   or physical action should come from the
   geometry structure this routine has processed.
   This is because the requested coordinate is
   modified in place for rotation and eccentricity and 
   may not be the same as the requested coordinate.

   WARNING:
   If the new location is acceptable,
   This function will leave the FiberInTransit
   structure set to the specified fiber/button,
   and the state of this fiber as "move pending".

   When the move has been completed, recordNewLocation
   must be called to properly update the database.
   This will also clear the FiberInTransit.  Optionally,
   restoreOldLocation may be called to put things
   back the way they were before the call to
   this function.

*/

int xyValidPlate (struct fiber_dbase *button, struct aPoint *target)
{
    int NoKaboom;               /* flag for collision determination */

    /* TRUE means location is OK. */

    struct flex_geometry *originalGeometry;

    /*
       Allocate the new geometry structure.
       Save a link back to the old one in the
       global FiberInTransit structure.
     */

    if (prepNewLocation (button, target))
        labtyd (SYSTEM_CALL, "Couldn't prep. new location in database");

    /* replace the link to the current geometry with that of the  target
       for the collision testing.  This is done because the collision detection 
       asks, "Do these fibers collide?" *not* "Will they collide?"
     */

    originalGeometry = button->CurGeometry;

    button->CurGeometry = button->TrialGeometry;

    NoKaboom = DestOK (FiBase, button);

    if (NoKaboom != TRUE)
    {                           /* this location is no good */

#ifdef LOG
        log_fputs ("WARNING:  Invalid target destination for button.\n",
                   stderr);
#else
        fputs ("WARNING:  Invalid target destination for button.\n", stderr);
#endif

        if (FiberInTransit.state != IN_TRANSIT)
        {
            restoreOldLocation ();
        }
    }

    /* put the current geometry back */

    button->CurGeometry = originalGeometry;

    return (NoKaboom);

}

/*
      ParkButt

   Move the button specified in the argument to its park position.
   Turn off rotation of coordinate system to insure it goes to
   a consistent location.
*/

void ParkButt (int id)
{
    struct fiber_dbase *fib = &FiBase[id];
    struct aPoint target;

    target.x = fib->ParkPos.x;
    target.y = fib->ParkPos.y;

    if (xyValidPlate (fib, &target))
    {

        if (fib->parkedStatus != PARKED)
        {
            sprintf (log_buf, "Parking fiber %d, which means:  ", id);
            log_fputs (log_buf, stderr);
        }

        if (FiberInTrouble.fiber == fib)
        {
            if (buttonPresent ())
                DisposeButt (fib, &target);
            else
                MoveButt (fib, DEFAULT_DEAD_ZONE);
        }
        else
            MoveButt (fib, DEFAULT_DEAD_ZONE);

        recordNewLocation (fib, UPDATE_FILE);

        fib->parkedStatus = PARKED;

        fib->tweaked = FALSE;
    }

}

/*
      Unstow
   
   Move the button specified in the argument from its stow position
   to its park position.  Assumes that the button *is* stowed!
   Uses explicit Z commands.
*/

void Unstow (int id)
{

    if (FiBase[id].parkedStatus == STOWED)
    {
        sprintf (log_buf, "Unstowing Button %d, which means:\n", id);
        log_fputs (log_buf, stderr);

        FiberInTrouble.fiber = NULL;
        FiberInTrouble.id = -1;

        ParkButt (id);

    }
    else
    {
        sprintf (log_buf, "Button %d isn't stowed--and can't be unstowed!\n",
                 id);
        log_fputs (log_buf, stderr);
    }
}

/*
		MoveButt

   Move a button to the location in its CurGeometry structure.
   Move must be pre-verified before calling this routine. 
   The deadZone is the distince in either axis by which
   the move must differ from the current location before
   a move will actually take place.
*/

void MoveButt (struct fiber_dbase *fib, int deadZone)
{

    struct aPoint target;

    /*
       Be sure that we aren't doing a null move and skip 
       it if we are.  The value deadZone is the number of
       steps that x or y must change before the button will
       be moved.  Note that if deadZone is less than zero,
       the move will always occur.
     */

#ifdef STOPWATCH
    clearEvents ();
#endif

    /* some checks to see if we should bother */

    if (fiberInCable (fib->fiber_id, 'Z'))
    {
        sprintf (log_buf, "SORRY, fiber #%d doesn't physically exist...\n",
                 fib->fiber_id);
        log_fputs (log_buf, stderr);
        return;
    }

#ifdef STOPWATCH
    event ("decision to move");
#endif

    if (!buttonAlreadyThere (fib, deadZone))
    {                           /* we must do the move... */
        FiberInTransit.fiber = fib;

/*
      FiberInTransit.id = fib->fiber_id;
*/
        (void) sprintf (log_buf, "moving button %3d to %6ld, %6ld\n",
                        fib->fiber_id,
                        (long) fib->TrialGeometry->button.x,
                        (long) fib->TrialGeometry->button.y);
        log_fputs (log_buf, stderr);

        /* Go to where button is now. */

        GoToXY (fib->CurPos.x, fib->CurPos.y, HIGH_VEL, HIGH_ACCEL, PHYSICAL);

#ifdef STOPWATCH
        event ("move to button");
#endif

        target.x = location_now.x;
        target.y = location_now.y;

        /*  Pick up the button. */

        if (!faith)
        {
            if (!remote)
            {
                GrabButt (fib, &target);
            }
            else
                RemGrab (fib, &target);
        }

        /* save a record of fiber in gripper in case of massive failure */

        FiberInTransit.id = FiberInTransit.fiber->fiber_id;
        flushStatus ();

#ifdef STOPWATCH
        event ("button grab");
#endif
        /*
           go to destination, carrying button
           Remember:  The TrialGeometry Structure
           has already been set to the target
           destination by a higher level function.
         */

        GoToXY ((long) fib->TrialGeometry->button.x,
                (long) fib->TrialGeometry->button.y,
                CARRY_VEL, CARRY_ACCEL, PHYSICAL);

#ifdef STOPWATCH
        event ("move to destination");
#endif

        /*  Put down the button. */

        target.x = location_now.x;      /* struct conversion... */
        target.y = location_now.y;

        if (!faith)
        {
            if (!remote)
            {
                DropButt (fib, &target, SOFT_BOUNDRIES);
            }
            else
            {
                RemDrop (fib, &target, SOFT_BOUNDRIES);
            }
        }
        else
        {
            FaithDrop (fib, &target, SOFT_BOUNDRIES);
        }

        FiberInTransit.id = -1;
        flushStatus ();

#ifdef STOPWATCH
        event ("button drop");
#endif

        FiberInTransit.fiber = (struct fiber_dbase *) NULL;
        FiberInTransit.state = STATIC;

        /* if we just moved the fiber in trouble, then it
           must no longer actually *be* in trouble... */

        if (FiberInTrouble.fiber == fib)
        {
            FiberInTrouble.fiber = NULL;
            FiberInTrouble.state = NO_TROUBLE;
        }

        if (graphic && !faith)
        {
            fputs (" update \n", graphCmds);
            fflush (graphCmds);
        }
    }

#ifdef STOPWATCH
    listEvents (stderr);
#endif
}

/*
		DisposeButt

   Move a button to the location in its CurGeometry structure,
   assuming that it is already in the gripper...
   Move must be pre-verified before calling this routine. 
*/

void DisposeButt (struct fiber_dbase *fib, struct aPoint *target)
{
    if (fib != FiberInTrouble.fiber)
    {

        log_fputs ("ERROR:  DisposeButt called with confusing system state",
                   stderr);
        return;
    }

    (void) sprintf (log_buf,
                    "moving button %d, in the gripper, to %ld, %ld\n",
                    fib->fiber_id, (long) target->x, (long) target->y);
    log_fputs (log_buf, stderr);

    /* be sure the gripper is in the correct state for moving a button */

    Relax ();

    /*
       go to destination, carrying button
       Remember:  The TrialGeometry Structure
       has already been set to the target
       destination by a higher level function.
     */

    /* this is a kludge to slow down moves involving
       the fops */

    GoToXY ((long) target->x, (long) target->y,
            (CARRY_VEL / 3), (CARRY_ACCEL / 3), PHYSICAL);

    /*  Put down the button. */

    if (!faith)
    {
        if (!remote)
        {
            DropButt (fib, &fib->TrialGeometry->button, SOFT_BOUNDRIES);
        }
        else
            RemDrop (fib, &fib->TrialGeometry->button, SOFT_BOUNDRIES);
    }
    else
    {
        FaithDrop (fib, &fib->TrialGeometry->button, SOFT_BOUNDRIES);
    }

    if (FiberInTransit.state == AT_NEW_LOC)
    {

        FiberInTransit.fiber = (struct fiber_dbase *) NULL;
        FiberInTransit.state = STATIC;
        FiberInTransit.id = -1;

        flushStatus ();
    }

    if (graphic && !faith)
    {
        fputs (" update \n", graphCmds);
        fflush (graphCmds);
    }
}

/*
		ViewButt

   Move gripper to the position of the specified button, but do not
   pick up.
*/

void ViewButt (struct fiber_dbase *fib)
{
    struct aPoint target;
    struct aPoint eccenVector;  /* correction for eccentricity */

    if (OKtoMove ())
    {

        target.x = (double) fib->CurPos.x;
        target.y = (double) fib->CurPos.y;

        if (EccenFlag)
        {

            eccenCalcVec (fib, &target, &eccenVector);
            applyVec (&target, &eccenVector, INVERT);
        }

        /* Go to where button is now. */

        GoToXY ((long) target.x, (long) target.y, HIGH_VEL, HIGH_ACCEL,
                PHYSICAL);

        (void) sprintf (log_buf, "viewing button #%d\n", fib->fiber_id);
        log_fputs (log_buf, stderr);
    }
    else
    {
        sprintf (log_buf, "ERROR:  Can't move gripper!\n");
        log_fputs (log_buf, stderr);
    }
}

/*
      GrabButt

   Pick up a button.
   Calculate proper distance from plate depth and button data.
   Return error if no button found.

*/

int GrabButt (struct fiber_dbase *fib, struct aPoint *targetCoord)
{
    int dummy;
    long Zvalue;
    long grip_dist;

    OpenWide ();

    /* z values extend down from zero! */

    if ((Zvalue = PlateZ (fib, targetCoord, &dummy, HARD_BOUNDRIES)) < 0)
    {

        grip_dist = Zvalue + fib->GrabHeight;

        /* put a test for gripper open here and open if required */

        GoToZ (grip_dist, Z_GRAB_VEL, Z_GRAB_ACCEL, SYNC);

        CloseWide ();

        GoToZ ((long) CRUISE_HEIGHT, Z_GRAB_VEL, Z_GRAB_ACCEL, SYNC);

#ifndef HYDRA_II_TEST_MODE
        if (!buttonPresent ())
        {
            (void) sprintf (log_buf, "ERROR:  Failure to grab button %d.\n",
                            fib->fiber_id);
            log_fputs (log_buf, stderr);

            /* set the fiber in trouble global for use by 
               recovery routines */

            if ((FiberInTrouble.fiber != NULL) &&
                (FiberInTrouble.state == CANT_GRAB))
            {
                (void) sprintf (log_buf,
                                "BIG TROUBLE:  Failure to grab button %d. after failure to grab\
		button %d\n", fib->fiber_id,
                                FiberInTrouble.fiber->fiber_id);
                log_fputs (log_buf, stderr);

                (void) sprintf (log_buf,
                                "button %d is now the \"fiber in trouble\".\n",
                                FiberInTrouble.fiber->fiber_id);
                log_fputs (log_buf, stderr);

                (void) sprintf (log_buf, "(will be default for \"thisis\")");
                log_fputs (log_buf, stderr);
            }

            FiberInTrouble.fiber = fib;
            FiberInTrouble.state = CANT_GRAB;

            labtyd (MISSED_BUTTON, "Gripper couldn't pick up button!");
        }
        else
#endif
// HYDRA_II_TEST_MODE
        {

            /* if appropriate, clear the fiber in trouble since
               we just succeeded in picking one up */

            if ((FiberInTrouble.fiber != NULL) &&
                (FiberInTrouble.state == CANT_GRAB) &&
                (FiberInTrouble.fiber == fib))
            {

                (void) sprintf (log_buf,
                                "GOOD NEWS:  button %d successfully picked up this time!\n",
                                fib->fiber_id);
                log_fputs (log_buf, stderr);

                FiberInTrouble.fiber = NULL;
                FiberInTrouble.state = NO_TROUBLE;
            }

            FiberInTransit.state = IN_TRANSIT;
        }

        Relax ();
    }

    else
        labtyd (SYS_CONFIG,
                "Requested position for button is in the forbidden zone.");
}

/*
      RemGrab

   Pick up a button. Using the downloaded sequences.
   Calculate proper distance from plate depth and button data.
   Return error if no button found.

*/

int RemGrab (struct fiber_dbase *fib, struct aPoint *targetCoord)
{
    int step;
    long Zdummy;                /* used only as an error flag here. */

    (void) PlateZ (fib, targetCoord, &step, SOFT_BOUNDRIES);

    /* fprintf( stderr, "grabbing from step #%d \n", step ); */

#ifndef HYDRA_II
    SetStroke (LONG);
#endif
// HYDRA_II

    switch (step)
    {

        case PLATEORD:
#ifdef HYDRA_II
            pick_n_drop (PICK_PLATE);
#else
            ExecProgram (Z_ord, GRABPLATE);
#endif
// HYDRA_II
            break;

        case PARKORD:
#ifdef HYDRA_II
            pick_n_drop (PICK_PARK);
#else
            ExecProgram (Z_ord, GRABPARK);
#endif
// HYDRA_II
            break;
    }

    GripPos = CLOSED;           /* download program can't set this... */

    if (verbose)
        log_fputs ("programmed grab called\n", stderr);

    /* send a buffered command so we'll know when the sequence is done */
    /* This command is *very* important!  It causes x,y motion */
    /* to be delayed until the z axis has finished moving */
    /* REMOVING THE NEXT 3 LINES WILL CAUSE HARDWARE DAMAGE! */

#ifndef HYDRA_II
    if (!faith)
        GetLoc (Z_ord, MOTOR, &Zdummy, &masterTimer);
#endif
// HYDRA_II

#ifdef STOPWATCH
    event ("downloaded grab");
#endif

    /* see if we have the button before relaxing */

#ifndef HYDRA_II_TEST_MODE
    if (!buttonPresent ())
    {
        (void) sprintf (log_buf, "ERROR:  Failure to grab button %d.\n",
                        fib->fiber_id);
        log_fputs (log_buf, stderr);

        /* set the fiber in trouble global for use by 
           recovery routines */

        if ((FiberInTrouble.fiber != NULL) &&
            (FiberInTrouble.state == CANT_GRAB))
        {

            (void) sprintf (log_buf,
                            "BIG TROUBLE:  Failure to grab button %d. after failure to grab\
		button %d\n", fib->fiber_id,
                            FiberInTrouble.fiber->fiber_id);
            log_fputs (log_buf, stderr);

            (void) sprintf (log_buf,
                            "button %d is now the \"fiber in trouble\".\n",
                            FiberInTrouble.fiber->fiber_id);
            log_fputs (log_buf, stderr);

            (void) sprintf (log_buf, "(will be default for \"thisis\")");
            log_fputs (log_buf, stderr);
        }

        FiberInTrouble.fiber = fib;
        FiberInTrouble.state = CANT_GRAB;

        labtyd (MISSED_BUTTON, "Gripper couldn't pick up button!");
    }
    else
#endif
// HYDRA_II_TEST_MODE
    {

        /* if appropriate, clear the fiber in trouble since
           we just succeeded in picking one up */

        if ((FiberInTrouble.fiber != NULL) &&
            (FiberInTrouble.state == CANT_GRAB) &&
            (FiberInTrouble.fiber == fib))
        {

            (void) sprintf (log_buf,
                            "GOOD NEWS:  button %d successfully picked up this time!\n",
                            fib->fiber_id);
            log_fputs (log_buf, stderr);

            FiberInTrouble.fiber = NULL;
            FiberInTrouble.state = NO_TROUBLE;
        }

        FiberInTransit.state = IN_TRANSIT;
    }

#ifdef STOPWATCH
    event ("button present test");
#endif

    /* we have to set the x axis cmd bit  before a relax */

#ifdef HYDRA_II
    mc_exe_cmd ("gripper relax");
#else
    SetStroke (SHORT);

    ExecProgram (Z_ord, RELAX);
#endif
// HYDRA_II
    GripPos = RELAXED;          /* download program can't set this... */

    /* send a buffered command so we'll know when it is done */
    /* REMOVING THE NEXT 3 LINES WILL CAUSE HARDWARE DAMAGE! */

      /**** or will it? *****/

/*
      if( !faith )
	 GetLoc( Z_ord, MOTOR, &Zdummy, &masterTimer );
*/

}

/*
      DropButt

   Put down a button.
   Calculate proper distance from plate depth and button data.
   Return error if not empty when done.

*/

int DropButt (struct fiber_dbase *fib, struct aPoint *targetCoord,
              int stepFlag)
{
    int dummy;
    long Zvalue;
    long grip_dist;

    Close ();

#ifndef HYDRA_II_TEST_MODE
    if (!buttonPresent ())
    {
        FiberInTransit.state = LOST;
        labtyd (SYS_CONFIG, "Gripper has no button to Drop!");
    }
#endif
// HYDRA_II_TEST_MODE

    if ((Zvalue = PlateZ (fib, targetCoord, &dummy, stepFlag)) > 0)
    {

        SetStroke (LONG);

        grip_dist = Zvalue + fib->GrabHeight;

        GoToZ (grip_dist, Z_GRAB_VEL, Z_GRAB_ACCEL, SYNC);

        Open ();

        GoToZ ((long) CRUISE_HEIGHT, Z_GRAB_VEL, Z_GRAB_ACCEL, SYNC);

        Close ();

        if (buttonPresent ())
        {
            (void) sprintf (log_buf, "ERROR:  Failure to drop button %d.\n",
                            fib->fiber_id);
            log_fputs (log_buf, stderr);

            /* set the fiber in trouble global for use by 
               recovery routines */

            (void) sprintf (log_buf,
                            "fiber %d is now the \"fiber in trouble\".\n",
                            fib->fiber_id);
            log_fputs (log_buf, stderr);

            (void) sprintf (log_buf, "(will be default for \"thisis\")");
            log_fputs (log_buf, stderr);

            FiberInTrouble.fiber = fib;
            FiberInTrouble.state = CANT_DROP;

            labtyd (SYS_CONFIG, "Couldn't drop button!");
        }
        else
        {

            /* if appropriate, clear the fiber in trouble since
               we just succeeded in picking one up */

            if ((FiberInTrouble.fiber != NULL) &&
                (FiberInTrouble.state == CANT_DROP) &&
                (FiberInTrouble.fiber == fib))
            {

                (void) sprintf (log_buf,
                                "GOOD NEWS:  button %d successfully dropped this time!\n",
                                fib->fiber_id);
                log_fputs (log_buf, stderr);

                FiberInTrouble.fiber = NULL;
                FiberInTrouble.state = NO_TROUBLE;
            }

            FiberInTransit.state = AT_NEW_LOC;
        }
    }

    else
        labtyd (SYS_CONFIG,
                "Requested position for button is in the forbidden zone.");
}

/*
      RemDrop

   Put down a button using the downloaded sequence.
   Button height is ignored, and assumed to be constant
   Return error if not empty when done.

*/

int RemDrop (struct fiber_dbase *fib, struct aPoint *targetCoord,
             int stepFlag)
{
    int step;
    long Zdummy;

#ifndef HYDRA_II
    Close ();
#endif
// HYDRA_II

#ifdef STOPWATCH
    event ("close");
#endif

#ifndef HYDRA_II_TEST_MODE
    if (!buttonPresent ())
    {
        FiberInTransit.state = LOST;
        labtyd (SYS_CONFIG, "Gripper has no button to Drop!");
    }
#endif
// HYDRA_II_TEST_MODE

#ifdef STOPWATCH
    event ("button present test");
#endif

    (void) PlateZ (fib, targetCoord, &step, stepFlag);

    /* fprintf( stderr, "dropping on step #%d \n", step ); */

#ifndef HYDRA_II
    SetStroke (LONG);
#endif
// HYDRA_II

    switch (step)
    {

        case PLATEORD:
#ifdef HYDRA_II
            pick_n_drop (DROP_PLATE);
#else
            ExecProgram (Z_ord, DROPPLATE);
#endif
// HYDRA_II
            break;

        case PARKORD:
#ifdef HYDRA_II
            pick_n_drop (DROP_PARK);
#else
            ExecProgram (Z_ord, DROPPARK);
#endif
// HYDRA_II
            break;

    }

    GripPos = CLOSED;           /* download program can't set this... */

    if (verbose)
        log_fputs ("programmed drop called\n", stderr);


#ifndef HYDRA_II
    /* send a buffered command so we'll know when it is done */
    /* This command is *very* important!  It causes x,y motion */
    /* to be delayed until the z axis has finished moving */
    /* REMOVING THE NEXT 3 LINES WILL CAUSE HARDWARE DAMAGE! */

    if (!faith)
        GetLoc (Z_ord, MOTOR, &Zdummy, &masterTimer);
#endif
// HYDRA_II

#ifdef STOPWATCH
    event ("download drop");
#endif

    if (buttonPresent ())
    {

        (void) sprintf (log_buf, "ERROR:  Failure to drop button %d.\n",
                        fib->fiber_id);
        log_fputs (log_buf, stderr);

        /* set the fiber in trouble global for use by 
           recovery routines */

        fputs ("made it to where expected\n", stderr);

        (void) sprintf (log_buf,
                        "fiber %d is now the \"fiber in trouble\".\n",
                        fib->fiber_id);
        log_fputs (log_buf, stderr);

        FiberInTrouble.fiber = fib;
        FiberInTrouble.state = CANT_DROP;

        LastXY = location_now;
        flushStatus ();

        labtyd (SYS_CONFIG, "Couldn't drop button!");
    }
    else
    {

        /* if appropriate, clear the fiber in trouble since
           we just succeeded in dropping it */

        if ((FiberInTrouble.fiber != NULL) && (FiberInTrouble.fiber == fib))
        {

            (void) sprintf (log_buf,
                            "GOOD NEWS:  button %d successfully dropped this time!\n",
                            fib->fiber_id);
            log_fputs (log_buf, stderr);

            FiberInTrouble.fiber = NULL;
            FiberInTrouble.state = NO_TROUBLE;
        }

        FiberInTransit.state = AT_NEW_LOC;
    }

#ifdef STOPWATCH
    event ("button present");
#endif

}

/*
      FaithDrop

   Put down a button in the simulator. Does the same things
   as RemDrop() but without accessing controllers. 

*/

int FaithDrop (struct fiber_dbase *fib, struct aPoint *targetCoord,
               int stepFlag)
{
    int step;

    if (PromptHw)
        log_fputs ("System thinks it's at new coord\n", stderr);

#ifndef HYDRA_II_TEST_MODE
    if (!buttonPresent ())
    {
        FiberInTransit.state = LOST;
        labtyd (SYS_CONFIG, "Gripper has no button to Drop!");
    }
#endif
// HYDRA_II_TEST_MODE

    (void) PlateZ (fib, targetCoord, &step, stepFlag);

    /* fprintf( stderr, "dropping on step #%d \n", step ); */

    GripPos = CLOSED;           /* download program can't set this... */

    if (verbose || PromptHw)
        log_fputs ("programmed drop called\n", stderr);

    if (PromptHw)
        log_fputs ("System has tried to drop the button.\n", stderr);

    if (PromptHw && buttonPresent ())
    {

        (void) sprintf (log_buf, "ERROR:  Failure to drop button %d.\n",
                        fib->fiber_id);
        log_fputs (log_buf, stderr);

        /* set the fiber in trouble global for use by 
           recovery routines */

        (void) sprintf (log_buf,
                        "fiber %d is now the \"fiber in trouble\".\n",
                        fib->fiber_id);
        log_fputs (log_buf, stderr);

        (void) sprintf (log_buf, "(will be default for \"thisis\")");
        log_fputs (log_buf, stderr);

        FiberInTrouble.fiber = fib;
        FiberInTrouble.state = CANT_DROP;

        LastXY = location_now;
        flushStatus ();

        labtyd (SYS_CONFIG, "Couldn't drop button!");
    }
    else
    {

        /* if appropriate, clear the fiber in trouble since
           we just succeeded in dropping it */

        if ((FiberInTrouble.fiber != NULL) && (FiberInTrouble.fiber == fib))
        {

            (void) sprintf (log_buf,
                            "GOOD NEWS:  button %d successfully dropped this time!\n",
                            fib->fiber_id);
            log_fputs (log_buf, stderr);

            FiberInTrouble.fiber = NULL;
            FiberInTrouble.state = NO_TROUBLE;
        }

        FiberInTransit.state = AT_NEW_LOC;
    }
}

/*
		releaseHere

   Release a button at the current location.  
*/

void releaseHere (void)
{
    struct aPoint here;
    struct fiber_dbase *fib;

    /* convert where we are now to a "point" (int to double) */

    here.x = location_now.x;
    here.y = location_now.y;

    /* see if we can figure out what (if any) button we have */

    if (FiberInTrouble.fiber != NULL)
    {

        fib = FiberInTrouble.fiber;

        if (xyValidForButton (fib, &here))
        {

            DisposeButt (fib, &here);
            recordNewLocation (fib, UPDATE_FILE);
        }

        else
        {
            log_fputs
                ("SORRY! This location unacceptable for button in gripper.\n",
                 stderr);
        }
    }

    else
    {                           /* unknown button in gripper */

        log_fputs ("WARNING: Unknown button in gripper.\n", stderr);
        log_fputs ("It must be identified explicitly with \"ThisIs\"\n",
                   stderr);
        log_fputs
            ("Proceed with identification process ONLY if you have been trained for it.",
             stderr);

    }
}

int blindRelease (void)
{
    long Zdummy;

#ifndef HYDRA_II_TEST_MODE
    if (!buttonPresent ())
    {
        log_fputs ("Sorry!  No button in found in gripper to be dropped.\n",
                   stderr);
        return -1;
    }
    else
#endif
// HYDRA_II_TEST_MODE
    {

        SetStroke (LONG);

        log_fputs ("seeing if system over highest step\n", stderr);

        ExecProgram (Z_ord, DROPPLATE);

        GetLoc (Z_ord, MOTOR, &Zdummy, &masterTimer);

        if (buttonPresent ())
        {

            ExecProgram (Z_ord, DROPPARK);
            GetLoc (Z_ord, MOTOR, &Zdummy, &masterTimer);

        }

        GripPos = CLOSED;       /* download program can't set this... */

        if (buttonPresent ())
        {
            log_fputs ("ERROR:  Could not seem to drop button any level.\n",
                       stderr);
            return (-1);
        }

        else
        {
            /* if we get here, we dropped the button */
            log_fputs ("Cool!  The button has been released.\n", stderr);
            return (0);
        }
    }
}

/*
         buttonPresent

   Check to see if a button is present in the gripper
   and return the result. 1 = button found, 0 = gripper empty.
*/

int buttonPresent (void)
{
#ifdef HYDRA_II_TEST_MODE
    return 0; 
#else
    int flags[4];
    int numberTrys = 0;

    if (faith)
        if (PromptHw)
        {                       /* allows user to create errors */
            if (getYesNo (stderr, stdin, "Did I find a button?", TRUE))
            {
                return (TRUE);
            }
            else
            {
                return (FALSE);
            }
        }
        else
        {
            return (TRUE);
        }

    /* first get the status bits to see what the state of the gripper is */

    GetTriggers (Z_ord, flags, &masterTimer);

    /* if the gripper is open or relaxed the test is useless
       so we have to close it first. */

    while ((flags[SHAFT] == PLUNGER_IN) && (numberTrys <= MAXTRYS))
    {

        numberTrys++;

        if (numberTrys > 1)
        {
            sprintf (log_buf, "\n%d trys to close\n", numberTrys);
            log_fputs (log_buf, stderr);
        }

        CloseAbsolute ();       /* force the circuit to close */

        usleep (400000);        /* give it time to close before testing */

        GetTriggers (Z_ord, flags, &masterTimer);
    }

    if (flags[SHAFT] == PLUNGER_OUT)
    {

        /* we succeeded in closing the jaws, therefore test can proceed */

        /* update the Gripper status */

        GripPos = CLOSED;

        return (flags[BUTTON] == TRUE);
    }
    else
    {

        /* if we get to here , then we know that the shaft never
           reached the switch, and the retry limit broke the
           loop. */

        log_fputs ("        ( From function 'buttonPresent()'   )\n", stderr);
        log_fputs ("ERROR:  Unable to close gripper for button test\n",
                   stderr);

        if (flags[BUTTON] == TRUE)
        {

            /* system thinks it has a button */
            log_fputs
                ("        Current indication of button in jaws may be false.\n",
                 stderr);
            log_fputs
                ("        informing calling function button in jaws, anyway...\n",
                 stderr);
            log_fputs ("        \nPlease run gripper diagnostics......\n",
                       stderr);
            return (TRUE);
        }
        else
        {
            /* system thinks jaws are empty */
            log_fputs
                ("        Current indication of jaws empty may be false.\n",
                 stderr);
            log_fputs ("        (But is *probably* correct)\n", stderr);
            log_fputs
                ("        informing calling function that jaws are empty, anyway...\n",
                 stderr);
            log_fputs ("        \nPlease run gripper diagnostics......\n",
                       stderr);
            return (FALSE);
        }
    }
/*NOTREACHED*/

#endif
// HYDRA_II_TEST_MODE
}

/*
		buttonAlreadyThere

   Check if a button is already at the destination
   specified in its new geometry structure
*/

buttonAlreadyThere (struct fiber_dbase * fib, int deadZone)
{
    if ((fabs (fib->CurPos.x - fib->TrialGeometry->button.x) <
         (double) deadZone)
        && (fabs (fib->CurPos.y - fib->TrialGeometry->button.y) <
            (double) deadZone))
    {

        return (TRUE);
    }
    else
    {
        return (FALSE);
    }
}

/*
		unresolved

   Check to see if an unresolved move exists and test for
   a button present if there is.  Return the True for 
   an unresolved move with A button present.
*/

int unresolved (void)
{
    /* this function is meaningless in the simulator, so.... */

    if (faith && !PromptHw)
        return (FALSE);

    /* normal path */

    if (buttonPresent ())
    {
        sprintf (log_buf, "There is a button in the jaws \n");
        log_fputs (log_buf, stderr);

        if (FiberInTrouble.fiber != NULL)
        {
            sprintf (log_buf, "It would appear to be button #%d\n",
                     FiberInTrouble.fiber->fiber_id);
            log_fputs (log_buf, stderr);

            sprintf (log_buf,
                     "Moves are are restricted to locations valid for this button.\n");
            log_fputs (log_buf, stderr);

            return (TRUE);
        }
        else
        {
            sprintf (log_buf, "DANGER!: This is an UNKNOWN button!!!\n");
            log_fputs (log_buf, stderr);

            labtyd (SYS_CONFIG,
                    "DO NOTHING UNTIL THIS BUTTON IS IDENTIFIED!");

            /* This is here to shut up lint */
            return (TRUE);
        }
    }

    else
    {
        if (FiberInTrouble.fiber != NULL)
        {
            sprintf (log_buf,
                     "WARNING: An unresolved move exists for button #%d.\n",
                     FiberInTrouble.fiber->fiber_id);
            log_fputs (log_buf, stderr);
        }
        return (FALSE);
    }
}
