
/************************************************************

			death.c

   This file contains all of the signal and error handlers.
   installSignals() must be called to activate the handlers.

   Functions in this file include:

   * installSignals()
     Death1()
     Death2()
     Death3()
     haltSystem()
   * labtyd(error, msg)

   ( * indicates function that is declared as an extern )

*************************************************************/
#include <stdio.h>
#include <sys/time.h>
#include <setjmp.h>
#include <signal.h>
#include <unistd.h>
#include <ncurses.h>

#include "errors.h"
#include "death.h"
#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "drives.h"
#include "reports.h"
#include "kaboom.h"
#include "parser.h"
#include "fiborg.h"
#include "log.h"
#include "tk.h"
#include "tkInterface.h"

extern int InCurses;

static char log_buf[256];

void Death1 (int sig, int code, struct sigcontext *scp);
void Death2 (int sig, int code, struct sigcontext *scp);
void Death3 (int sig, int code, struct sigcontext *scp);
void Death5 (int sig, int code, struct sigcontext *scp);
void Death6 (int sig, int code, struct sigcontext *scp);
void Death4 (int sig, int code, struct sigcontext *scp);
void Death7 (int sig, int code, struct sigcontext *scp);

/*************** structures ***************/

struct sigvec Death1Vec = {
    Death1,
    0,
    0
};

struct sigvec Death2Vec = {
    Death2,
    0,
    SV_INTERRUPT
};

struct sigvec Death3Vec = {
    Death3,
    0,
    SV_INTERRUPT
};

struct sigvec Death4Vec = {
    Death4,
    0,
    SV_INTERRUPT
};

struct sigvec Death5Vec = {
    Death5,
    0,
    SV_INTERRUPT
};

struct sigvec Death6Vec = {
    Death6,
    0,
    SV_INTERRUPT
};

struct sigvec Death7Vec = {
    Death7,
    0,
    SV_INTERRUPT
};

int stop (int axis_ord);
int mkill (int axis_ord);
int haltSystem (void);

extern int send_marker (char *theString);
extern int bowOut (void);
extern int Pause (unsigned int time);
extern int buttonPresent (void);
extern int setBlock (int flag);
extern void load_limits (void);
extern void setInputStreamMode (int mode);
int set_acceleration (int axis_ord, double arg);

/*
		installSignals

   Call sigvec and install the handlers for the signals that 
   require special attention.  All handlers are in this file.
*/
void installSignals (void)
{
    int error = 0;

    struct sigvec oldvec;       /* value is ignored, but sigvec needs it */

    error |= sigvec (SIGINT, &Death1Vec, &oldvec);
    error |= sigvec (SIGALRM, &Death2Vec, &oldvec);
    error |= sigvec (SIGHUP, &Death3Vec, &oldvec);
    error |= sigvec (SIGUSR2, &Death5Vec, &oldvec);
    error |= sigvec (SIGCHLD, &Death6Vec, &oldvec);
    error |= sigvec (SIGURG, &Death7Vec, &oldvec);

    if (error)
        labtyd (SYSTEM_CALL,
                "ERROR:  Signal handler installation failure.\n");
}

/*
	Handler for user ctrl-c.
*/

void Death1 (int sig, int code, struct sigcontext *scp)
{
    (void) fprintf (stderr,
                    "WARNING:  User interrupt. Exiting Script(s) after current command\n");
    supress = FALSE;            /* insure prompt is on */
    UserInterrupt = TRUE;
}

/*
	Handler for timer alarm.
*/

void Death2 (int sig, int code, struct sigcontext *scp)
{

    int busyStatus;

    int pauseFlags[10];         /* bigger than we need */

    int loopStatus[10];

    int driveFlagsX1[64];       /* bigger than we need */
    int driveFlagsX2[64];       /* bigger than we need */
    int driveFlagsY1[64];       /* bigger than we need */

    int axis;

    char *axisName;             /* character string name of axis */

    int driveFault = 0;         /*  flag that a drive has tripped */

/*  Here in case we later install a 2nd timout handler. */

/*
   struct sigvec oldvec;
*/

    /* tell the user what has happened */

    log_fputs ("ERROR: Timed out on indexer response.\n", stderr);
    log_fputs ("Beginning diagnostics.\n", stderr);

    /* create a marker in the communication log */

    send_marker ("ResponseTimeOut");

    /* reset the vector so that we can try to get back if
       a status request times out */

/*
   error |= sigvec( SIGALRM, &Death2Vec, &oldvec );
*/

    for (axis = X_ord; axis <= Y_ord; axis++)
    {

        axisName = ordToName (axis);

      /********** check for move in progress ***********/

        log_fputs ("Checking status. \n", stderr);

        GetBusy (axis, &busyStatus, &masterTimer);

        switch (busyStatus)
        {

            case READY:
                sprintf (log_buf, "%s axis not moving and not at limit\n",
                         axisName);
                log_fputs (log_buf, stderr);
                break;
            case READY_ATTENTION:
                sprintf (log_buf, "%s axis STOPPED AT LIMIT OR STALLED\n",
                         axisName);
                log_fputs (log_buf, stderr);
                break;
            case BUSY:
                sprintf (log_buf, "%s axis moving or busy; not at limit\n",
                         axisName);
                log_fputs (log_buf, stderr);

                sprintf (log_buf, "Attempting to halt motion...\n");
                log_fputs (log_buf, stderr);

                haltSystem ();
                break;
            case BUSY_ATTENTION:
                sprintf (log_buf, "%s axis AT LIMIT or STALLED and busy.\n",
                         axisName);
                log_fputs (log_buf, stderr);

                sprintf (log_buf, "Attempting to gain control... \n");
                log_fputs (log_buf, stderr);

                haltSystem ();
                break;
            case SEQINPROG:
                sprintf (log_buf,
                         "\n%s axis seq. in progress; not at limit.\n",
                         axisName);
                log_fputs (log_buf, stderr);
                break;
            case SEQINPROG_ATTENTION:
                sprintf (log_buf, "%s axis seq in progress; BUSY.\n",
                         axisName);
                log_fputs (log_buf, stderr);
                break;
            default:
                labtyd (STATUS_REPORT,
                        "Unknown indexer status Can't Continue.");
                break;
        }

      /********** check for stall  ***********/

        log_fputs ("Checking for stall.\n", stderr);

        GetLooped (axis, loopStatus, &masterTimer);

        if (loopStatus[STALL])
        {
            sprintf (log_buf, "%s axis STALLED.\n", axisName);
            log_fputs (log_buf, stderr);
        }

        GetPause (axis, pauseFlags, &masterTimer);

        /* we force the drive test for Y--Very kludgy.  It 
           should be hard-wired like X! */

        if (pauseFlags[STOP])
        {

            log_fputs ("WARNING:  Stop input is asserted\n", stderr);
            log_fputs ("Check panic button and rotary encoder circuit\n",
                       stderr);
        }

      /********** check for drive problems  ***********/

        if (pauseFlags[STOP] || (axis == Y_ord))
        {

            /* check the drives */

            switch (axis)
            {

                case X_ord:

                    driveFault |=
                        GetDriveStatus (XDRIVE1, driveFlagsX1, &masterTimer);

                    driveFault |=
                        GetDriveStatus (XDRIVE2, driveFlagsX2, &masterTimer);
                    break;

                case Y_ord:

                    driveFault |=
                        GetDriveStatus (YDRIVE1, driveFlagsY1, &masterTimer);
                    break;

                case Z_ord:

                    log_fputs ("Drive status not avail for Z\n", stderr);
                    break;

                default:
                    break;
            }

            if (driveFault)
                log_fputs ("Drive fault in X,Y system.\n", stderr);

        }
        else
        {                       /* We don't use the other bits at the moment */
        }

    }

    /* dump response buffer */

    /* report the current location of the system */

    printLocation ();
    gripStatus ();

    log_fputs ("System halted at (what it thinks is the) above location.\n\n",
               stderr);

    supress = FALSE;            /* insure that the prompt is on */

    log_fputs ("Returning to command parser.\n", stderr);

    longjmp (Env, 1);
}

/*
	Handler for hangup signal.
*/

void Death3 (int sig, int code, struct sigcontext *scp)
{
    log_fputs ("Fiborg: SIGHUP--terminating & exiting.\n", stderr);

/* quit(); *//* this function is in parser.c */
    bowOut ();

}

/*
	Handler for USR2 signal.
*/

void Death5 (int sig, int code, struct sigcontext *scp)
{
    log_fputs ("Fiborg: SIGUSR1--Communications error.\n", stderr);
    Ping ();
    log_fputs ("Clearing response buffer and returning to parser.\n", stderr);
    supress = FALSE;
}

/*
	Handler for SIGCHLD signal.
*/

void Death6 (int sig, int code, struct sigcontext *scp)
{

/*
   (void)sprintf (log_buf, "Fiborg: SIGCHLD--Child died.\n");
   (void)sprintf (log_buf, "Code = %d\n", code );
   sleep( 10 );
   bowOut();
*/
}

haltSystem (void)
{
    /*  set acceleration */

    if (set_acceleration (X_ord, HIGH_ACCEL))
        labtyd (COMMUNICATION, "Can't set acceleration for X.");

    if (set_acceleration (Y_ord, HIGH_ACCEL))
        labtyd (COMMUNICATION, "Can't set acceleration for Y.");

    /*  issue global stop  */

    if (stop (X_ord))
        labtyd (COMMUNICATION, "Can't stop X.");

    if (stop (Y_ord))
        labtyd (COMMUNICATION, "Can't stop Y.");

    Pause ((unsigned) 3);

    /*  issue a "kill" to dump the input buffers */

    if (mkill (X_ord))
        labtyd (COMMUNICATION, "Can't kill X.");

    if (mkill (Y_ord))
        labtyd (COMMUNICATION, "Can't kill Y.");

    Pause ((unsigned) 5);
}

/*
     Handler for status tests that are called from the 
     time out routine.  If, during a timeout, a second
     status request fails, it will come here and then
     return to the primary handler rather than return
     to the parser.
*/

void Death4 (int sig, int code, struct sigcontext *scp)
{
    log_fputs ("FATAL ERROR.  Timeout during timeout recovery.\n", stderr);
    log_fputs ("Software is unable to communicate with rack.\n\n", stderr);
    log_fputs ("The system is absolutely hung.  Restart the software.\n",
               stderr);
    log_fputs ("( or proceed with *great* caution... )\n", stderr);
}

/*
     Handler for socket trouble...
*/

void Death7 (int sig, int code, struct sigcontext *scp)
{
    log_fputs ("SIGURG\n", stderr);
    for (;;)
    {
    }
}

/*
      labtyd

   Life's A Bitch, Then You Die--labtyd

*/

void labtyd (int error, char *msg)
{
    (void) sprintf (log_buf, "%s\n", msg);
    log_fputs (log_buf, stderr);

    /* * *  error specific stuff to do * * */

    switch (error)
    {

        case MISSED_BUTTON:

            log_fputs ("PLEASE READ MESSAGES and ctrl-c to stop beeping.\n",
                       stderr);

            while (!UserInterrupt)
            {

                fputs ("", stderr);
                usleep (800000);

                /* keep tk alive... */
                while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
                {
                    /* next line is just for testing */
                    /* fputs( "Tick...\n", stderr ); */
                }
            }

            break;

        case STATUS_REPORT:

            Ping ();
            break;

        default:
            break;
    }

    /* * * if in a move, clean up the flexure info. * * */

    if (FiberInTransit.fiber != NULL)
    {

        switch (FiberInTransit.state)
        {

            case STATIC:
                break;

            case MOVE_PENDING:

                sprintf (log_buf,
                         "Restoring button %d geometry to original value.\n",
                         FiberInTransit.fiber->fiber_id);
                log_fputs (log_buf, stderr);

                restoreOldLocation ();
                break;

            case IN_TRANSIT:

                if (buttonPresent ())
                {
                    sprintf (log_buf,
                             "WARNING:  Button %d ( I think....)  is in the gripper\n",
                             FiberInTransit.fiber->fiber_id);
                    log_fputs (log_buf, stderr);
                }

                log_fputs ("Moves are restricted until button is released\n",
                           stderr);

                break;

            case AT_NEW_LOC:

                sprintf (log_buf,
                         "WARNING: New location of Button %d might not be recorded\n",
                         FiberInTransit.fiber->fiber_id);
                log_fputs (log_buf, stderr);

                log_fputs
                    ("Database may need to be updated with \"thisis\"\n",
                     stderr);

                log_fputs
                    ("Use caution until you are sure of database status\n",
                     stderr);

                break;

            case LOST:

                sprintf (log_buf,
                         "EXTREME ERROR: Button %d was lost during move\n",
                         FiberInTransit.fiber->fiber_id);
                log_fputs (log_buf, stderr);

                log_fputs
                    ("ALL ACTIVITY DANGEROUS UNTIL LOST BUTTON IS FOUND!!\n\n",
                     stderr);

                break;

            default:
                break;

        }
    }

    if (InCurses)
    {
        endwin ();
    }

    /* this prevents various commands from Finishing */

    UserInterrupt = TRUE;

    /* insure the GUI isn't blocked */

    setBlock (FALSE);

    /* insure the prompt is on */

    supress = FALSE;

    /* put back the original grip/drop mode */

    remote = remoteTemp;

    /* Load structure for x/y/z limits */

    load_limits ();

    /* turn commands from stdin back on for tk */

    setInputStreamMode (ON);

    /* jump to the parser */

    longjmp (Env, 1);

}
