
/*******************************************************************

		translate.c

	This file contains all routines that perform coordinate
	translation.  These include rotation and unit system
	conversion.

	Included functions are:

	*  initRotation()
   	*  addRotationPt()
	*  listRotationPts()
	*  listRotationPrs()
	*  clearRotationPt()
	*  clearRotationPr()
	*  calcRotation()
	   generatePairs()
	   enablePairs()
	   addRotationPair()
	*  calculatePairs()
	   normalize()
	   allocRotationPt()
	   freeRotationPt()
	   allocRotationPr()
	   freeRotationPr()

	( * indicates function is declared as an extern )

*******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "errors.h"
#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "detrot.h"
#include "log.h"

static char log_buf[256];

struct rotPoint *freeRotationPt (struct rotPoint *thePoint, int flag);
struct rotPair *freeRotationPr (struct rotPair *thePair, int flag);
struct rotPoint *allocRotationPt (void);
struct rotPair *allocRotationPr (void);
extern void rotate (point *, double);
void generatePairs (struct rotStats *rotationData);
void enablePairs (struct rotStats *rot);
extern double radToDeg (double);
void addRotationPair (struct rotPoint *point1, struct rotPoint *point2,
                      struct rotStats *rotationData);
double normalize (double angle);

/*************************/

/*
		initRotation
*/
void initRotation (struct rotStats *rot)
{
    rot->numberOfPairs = 0;

    rot->workingAngle = 0;

    rot->defaultAngle = 0;

    rot->sumAngles = 0;

    freeRotationPt (rot->nextPoint, RECURSE);

    freeRotationPr (rot->nextPair, RECURSE);

    /* create dummy records */

    if ((rot->nextPoint = allocRotationPt ()) == NULL)
        labtyd (SYSTEM_CALL, "Couldn't allocate rotation point");

    if ((rot->nextPair = allocRotationPr ()) == NULL)
        labtyd (SYSTEM_CALL, "Couldn't allocate rotation pair");
}

/*
		addRotationPt
*/

void addRotationPt (struct rotStats *rot, char *name,
                    struct plate_coords *starPt, struct plate_coords *measPt)
{
    struct rotPoint *currentPoint = rot->nextPoint;
    struct rotPoint *pt;

    while (currentPoint->nextPoint != NULL)
        currentPoint = currentPoint->nextPoint;

    if ((currentPoint->nextPoint = allocRotationPt ()) == NULL)
        labtyd (SYSTEM_CALL, "Couldn't allocate rotation point");

    pt = currentPoint->nextPoint;

    (void) strcpy (pt->pointName, name);

    pt->id = abs (currentPoint->id) + 1;

    pt->starLoc.x = starPt->x;  /* converted to doubles in the assignment */
    pt->starLoc.y = starPt->y;

    pt->measLoc.x = measPt->x;
    pt->measLoc.y = measPt->y;

    if (Rotflag)
        rotate (&pt->starLoc, -(RotData.aveAngle));

    /* regenerate the pair list */

    /* free the old list */

    rot->nextPair->nextPair =
        freeRotationPr (rot->nextPair->nextPair, RECURSE);

    /* make the new one */

    generatePairs (rot);

    enablePairs (rot);

    /* calculate the new angle */

    calcRotation (rot);

}

/*
		listRotationPts
*/

void listRotationPts (struct rotStats *rot)
{
    struct rotPoint *thePoint = rot->nextPoint->nextPoint;

    log_fputs ("\n\n", stderr);

    sprintf (log_buf, "%3s  %16s\n", "id#", "Object Name");
    log_fputs (log_buf, stderr);

    log_fputs ("\n", stderr);

    while (thePoint != NULL)
    {

        sprintf (log_buf, "%3d  %16s ",
                 abs (thePoint->id), thePoint->pointName);
        log_fputs (log_buf, stderr);

        sprintf (log_buf, "expected at % 7.0f, % 7.0f ",
                 thePoint->starLoc.x, thePoint->starLoc.y);
        log_fputs (log_buf, stderr);

        sprintf (log_buf, "found at % 7.0f, % 7.0f",
                 thePoint->measLoc.x, thePoint->measLoc.y);
        log_fputs (log_buf, stderr);

        if (thePoint->id < 0)
        {
            sprintf (log_buf, " %s\n", "*deleted*");
            log_fputs (log_buf, stderr);
        }
        else
            log_fputs ("\n", stderr);

        thePoint = thePoint->nextPoint;
    }
}

/*
		listRotationPrs
*/

void listRotationPrs (struct rotStats *rot)
{
    struct rotPair *thePair = rot->nextPair->nextPair;

    double deltaR;
    double weight;

    log_fputs ("\n\n", stderr);

    sprintf (log_buf, "%3s   %-30.30s   %s  %8s  %s\n", "id#",
             "Object  Pair", "Abs.Rot.", "Delta", "Weight");
    log_fputs (log_buf, stderr);

    log_fputs ("\n", stderr);

    while (thePair != NULL)
    {

        if (Rotflag)
            deltaR = thePair->rotAngle - rot->aveAngle;
        else
            deltaR = thePair->rotAngle;

        if (thePair->id < 0)
            weight = 0.0;
        else
            weight = thePair->rSep2;

        if (thePair->id >= 0)
        {
            sprintf (log_buf, "%3d   %30.30s   % 8.5f % 8.5f % 8.5f \n",
                     abs (thePair->id),
                     thePair->pairName,
                     radToDeg (thePair->rotAngle), radToDeg (deltaR), weight);
            log_fputs (log_buf, stderr);
        }

        thePair = thePair->nextPair;
    }

    sprintf (log_buf, "\nAbsolute Rotation based on these points: % f\n",
             radToDeg (rot->workingAngle));
    log_fputs (log_buf, stderr);

    if (Rotflag)
    {
        sprintf (log_buf, "Rotation angle currently in use:         % f\n",
                 radToDeg (rot->aveAngle));
        log_fputs (log_buf, stderr);

        sprintf (log_buf, "                         Difference:     % f\n",
                 radToDeg (rot->workingAngle - rot->aveAngle));
        log_fputs (log_buf, stderr);
    }
}

/*
		clearRotationPt
*/

void clearRotationPt (int id, struct rotStats *rot)
{
    struct rotPoint *preceedingPoint = rot->nextPoint;
    struct rotPoint *targetPoint = preceedingPoint->nextPoint;

    while ((targetPoint != NULL) && (abs (targetPoint->id) != id))
    {

        preceedingPoint = targetPoint;
        targetPoint = targetPoint->nextPoint;
    }

    if (targetPoint == NULL)
    {
        log_fputs ("Couldn't find requested point\n", stderr);
    }
    else
    {
        targetPoint->id = -(abs (targetPoint->id));
    }

    enablePairs (rot);

    calcRotation (rot);
}

/*
		clearRotationPr
*/

void clearRotationPr (int id, struct rotStats *rot)
{
    struct rotPair *preceedingPair = rot->nextPair;
    struct rotPair *targetPair = preceedingPair->nextPair;

    while ((targetPair != NULL) && (abs (targetPair->id) != id))
    {

        preceedingPair = targetPair;
        targetPair = targetPair->nextPair;
    }

    if (targetPair == NULL)
    {
        log_fputs ("Couldn't find requested pair\n", stderr);
    }
    else
    {
        targetPair->id = -(targetPair->id);
    }
}

/*
		calcRotation
   
   Calculate the rotation angle based on the points in
   the list in the rotStat structure supplied as the
   argument.  Print a summary of the results.

*/

double calcRotation (struct rotStats *rot)
{
    struct rotPair *dataPair = rot->nextPair;   /* set to head/dummy */

    rot->numberOfPairs = 0;

    rot->sumAngles = 0;

    rot->sumWeights = 0;

    rot->workingAngle = 0;

    /* evaluate angle for each  pair */

    calculatePairs (rot);

    while ((dataPair = dataPair->nextPair) != NULL)
    {

        if (dataPair->id > 0)
        {

            rot->sumAngles += (dataPair->rotAngle * dataPair->rSep2);
            rot->sumWeights += dataPair->rSep2;
            rot->numberOfPairs++;
        }
    }

    rot->workingAngle = rot->sumAngles / rot->sumWeights;

/*
   sprintf (log_buf, "New rotation based on %3d pairs = %+11.7f abs. ",
	    rot->numberOfPairs, radToDeg( rot->workingAngle ) );
   log_fputs (log_buf, stderr);

   sprintf (log_buf, "or %+11.7f rel.\n",
	    radToDeg( rot->workingAngle - rot->aveAngle ) );
   log_fputs (log_buf, stderr);

   log_fputs( "\nEnter 'applyrot' to use this angle in place of old one\n", stderr);
*/

    return (rot->aveAngle);
}

/*
		generatePairs

   Create all possible pairs of points from the point list.
   Save each pair in a rotPair structure and calculate a
   rotation angle based on the pair.  The end result is
   a linked list of pairs.
*/

void generatePairs (struct rotStats *rotationData)
{
    struct rotPoint *point1;
    struct rotPoint *point2;

    point1 = rotationData->nextPoint;

    while ((point1 = point1->nextPoint) != NULL)
    {

        point2 = point1;

        while ((point2 = point2->nextPoint) != NULL)
        {

            addRotationPair (point1, point2, rotationData);
        }
    }
}

/*
		enablePairs

   Descend the list of rotation pairs and enable/disable
   the appropriate pairs based on the status of the points
   in them.
*/

void enablePairs (struct rotStats *rot)
{
    /* We expect the first pointer to be set to the header record
       and so we will bump it past the header when we enter the loop. */

    struct rotPair *currentPair = rot->nextPair;

    struct rotPoint *point1;
    struct rotPoint *point2;

    while ((currentPair = currentPair->nextPair) != NULL)
    {

        point1 = currentPair->point1;
        point2 = currentPair->point2;

        /* check that the two points exist and are enabled */

        if ((point1 != NULL) && (point2 != NULL) &&
            (point1->id > 0) && (point2->id > 0))

            currentPair->id = abs (currentPair->id);

        else
            currentPair->id = -abs (currentPair->id);
    }
}

/*
		addRotationPair

   compute the rotation angle for the specified pair of points
   and link it into the list of rotation point pairs.
*/

void addRotationPair (struct rotPoint *point1, struct rotPoint *point2,
                      struct rotStats *rotationData)
{
    struct rotPair *currentPair = rotationData->nextPair;

    /* don't create points for which there is no data! */

    if ((point1 != NULL) && (point2 != NULL))
    {

        /* find the location for and create the next entry */

        while (currentPair->nextPair != NULL)
            currentPair = currentPair->nextPair;

        if ((currentPair->nextPair = allocRotationPr ()) == NULL)
            labtyd (SYSTEM_CALL, "Couldn't allocate rotation pair");

        /* link the two points we have used */

        currentPair->nextPair->point1 = point1;
        currentPair->nextPair->point2 = point2;

        /* create the name and ID number for the pair. */

        currentPair->nextPair->id = abs (currentPair->id) + 1;

        sprintf (currentPair->nextPair->pairName,
                 "p%02d '%-10.10s' & p%02d '%-10.10s'",
                 point1->id, point1->pointName,
                 point2->id, point2->pointName);
    }
}

/*
		calculatePairs

   Descend the list of rotation pairs and calculate the angles
   for them.
*/

void calculatePairs (struct rotStats *rot)
{
    /* We expect the first pointer to be set to the header record
       and so we will bump it past the header when we enter the loop. */

    struct rotPair *currentPair = rot->nextPair;

    struct rotPoint *point1;
    struct rotPoint *point2;

    double dxCalc;
    double dyCalc;

    double dxMeas;
    double dyMeas;

    double thetaCalc;
    double thetaMeas;

    /* don't create points for which there is no data! */

    while ((currentPair = currentPair->nextPair) != NULL)
    {

        point1 = currentPair->point1;
        point2 = currentPair->point2;

        /* check that the two points exist and that they
           should be allowed to enter into the calculation.
           Flag the pairs that are not valid as deleted. */

        if ((point1 != NULL) && (point2 != NULL) && (currentPair->id > 0))
        {

            /* find the two legs of each triangle--measured and calculated */

            dxCalc = point1->starLoc.x - point2->starLoc.x;
            dyCalc = point1->starLoc.y - point2->starLoc.y;

            dxMeas = point1->measLoc.x - point2->measLoc.x;
            dyMeas = point1->measLoc.y - point2->measLoc.y;

            currentPair->rSep2 =
                ((dxCalc * dxCalc) + (dyCalc * dyCalc)) / 1.0e9;

            /* Find the absolute angles of each line wrt x axis */

            thetaCalc = normalize (atan2 (dyCalc, dxCalc));
            thetaMeas = normalize (atan2 (dyMeas, dxMeas));

            /* Now find the rotation angle for this pair. */

            currentPair->rotAngle = normalize (thetaMeas - thetaCalc);

        }
    }
}

/*
		normalize

   Normalize an angle between +/- Pi to +/- Pi/2;
*/

double normalize (double angle)
{
    if (angle > M_PI_2)
        return (angle - M_PI);

    if (angle < -M_PI_2)
        return (angle + M_PI);

    else
        return (angle);
}

/*
		allocRotationPt

   Allocate a rotPoint structure.  Initialize the variables
   in the structure, and return a pointer to it.
*/

struct rotPoint *allocRotationPt (void)
{
    struct rotPoint *tmpPt;

    tmpPt = (struct rotPoint *) malloc ((unsigned) sizeof (struct rotPoint));

    /* bail out if we didn't get the space */

    if (tmpPt == NULL)
        return (tmpPt);

    /* otherwise, initialize the data */

    tmpPt->nextPoint = NULL;

    tmpPt->id = 0;

    *tmpPt->pointName = '\0';

    tmpPt->starLoc.x = 0;
    tmpPt->starLoc.y = 0;

    tmpPt->measLoc.x = 0;
    tmpPt->measLoc.y = 0;

    return (tmpPt);
}

/*
		freeRotationPt

   Free the specified rotation data point.  If the flag
   is set to RECURSE, then free all of the points beyond
   this one in the list as well and return null.  If the
   flag is set to SAVEPOINT, free only the specified point
   and return the pointer to the point beyond it.  
*/

struct rotPoint *freeRotationPt (struct rotPoint *thePoint, int flag)
{
    if (thePoint == NULL)
        return (NULL);

    else
    {

        struct rotPoint *tmpPt = thePoint->nextPoint;

        /* get rid of this point */

        free ((char *) thePoint);

        /* now figure out what to do with the next one in the link */

        if (flag == SAVEPOINT)
            return (tmpPt);

        else
            return (freeRotationPt (tmpPt, RECURSE));
    }
}

/*
		allocRotationPr

   Allocate a rotPair structure.  Initialize the variables
   in the structure, and return a pointer to it.
*/

struct rotPair *allocRotationPr (void)
{
    struct rotPair *tmpPt;

    tmpPt = (struct rotPair *) malloc ((unsigned) sizeof (struct rotPair));

    /* bail out if we didn't get the space */

    if (tmpPt == NULL)
        return (tmpPt);

    /* otherwise, initialize the data */

    tmpPt->nextPair = NULL;

    tmpPt->id = 0;

    *tmpPt->pairName = '\0';

    tmpPt->point1 = NULL;
    tmpPt->point2 = NULL;

    tmpPt->rotAngle = 0;

    return (tmpPt);
}

/*
		freeRotationPr

   Free the specified rotation data pair.  If the flag
   is set to RECURSE, then free all of the points beyond
   this one in the list as well and return null.  If the
   flag is set to SAVEPOINT, free only the specified point
   and return the pointer to the point beyond it.  
*/

struct rotPair *freeRotationPr (struct rotPair *thePair, int flag)
{
    if (thePair == NULL)
        return (NULL);

    else
    {

        struct rotPair *tmpPt = thePair->nextPair;

        /* get rid of this point */

        free ((char *) thePair);

        /* now figure out what to do with the next one in the link */

        if (flag == SAVEPOINT)
            return (tmpPt);

        else
            return (freeRotationPr (tmpPt, RECURSE));

    }
}
