
/*****************************************************

		drives.c

   This file contains all of the routines that deal
   with reseting and initializing the drives 
   attached to the X and Y axis.  

   these routines use #defs for the Unit IDs for each
   drive instead of looking up the ID in a table.

****************************************************/
#include <stdio.h>
#include <sys/time.h>
#include <string.h>

#include "errors.h"
#include "reports.h"
#include "fiborg.h"
#include "drives.h"
#include "log.h"

static char log_buf[256];

char *driveNames[] = { "NO SUCH DRIVE--software error",
    "NO SUCH DRIVE--software error",
    "X axis drive #1",
    "NO SUCH DRIVE--software error",
    "X axis drive #2",
    "NO SUCH DRIVE--software error",
    "Y axis drive #1"
};

char *driveErrors[] = { "Drive powered down as commanded by ST0",
    "Drive overcurrent",
    "Drive over-temp",
    "Drive PWM (Pulse Width Modulation) error",

    "Drive remote shutdown from indexer",
    "Drive max position Error exceeded",
    "Drive max average current exceeded",
    "Drive power dump overtemp",

    "Drive failed internal software CRC check",
    "Drive found indexer pulses at power-up",
    "Drive high voltage problem",
    "Drive enable circuit disabled",

    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",

    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",

    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",

    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",

    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined",
    "Drive error undefined"
};

int drive_status (int driveID);
extern int send_it (char *);

/*
		GetDriveStatus
   
   Obtain the current status from the drive specified in the
   first argument.  Put the result in the structure pointed to
   by the second argument.  If the report does not return in the
   specified amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/
GetDriveStatus (int unitID, int *respPtr, struct itimerval *maxTime)
{
    int num_found;
    char buff[64];              /* response line */
    char raw_resp[64];          /* response line */

    char *p1;                   /* working pointers for response manipulation */
    char *p2;

    int i = 0;

    int error = FALSE;

    /* turn on communication to the drive */

    if (enable_drive (unitID))
        labtyd (COMMUNICATION, "Could not disable communication to drive");

    /* send the request */

    if (drive_status (unitID))
        labtyd (COMMUNICATION, "Drive status request transmission failure");

    getString (buff, maxTime);

    if (disable_drive (unitID))
        labtyd (COMMUNICATION, "Could not disable communication to drive");

    num_found = sscanf (buff, " *%[^*]", raw_resp);

    if (verbose && (num_found > 0))
    {
        (void) sprintf (log_buf, "drive %d pause status is: %s\n", unitID,
                        raw_resp);
        log_fputs (log_buf, stderr);
    }

    if (num_found < 1)
        labtyd (STATUS_REPORT, "bad status report ");

    /*  process the raw string */

    /* set the pointers to the start of the string */

    p1 = raw_resp;
    p2 = raw_resp;

    /* copy all the numbers, but not the dashes or trailing characters */

    while (*p1 != '\0')
    {

        if (*p1 == '_')
        {
            p1++;
        }
        else
        {
            *p2 = *p1;
            p1++;
            p2++;
        }
    }

    *p2 = '\0';

    /* now copy the character "bits" into the numeric array */

    /* set the char pointer to the last character in the string */

    p1 = &raw_resp[(strlen (raw_resp) - 1)];

    while (p1 != raw_resp)
    {

        switch (*p1)
        {

            case '0':
                respPtr[i] = FALSE;
                break;

            case '1':
                respPtr[i] = TRUE;
                sprintf (log_buf, "%s %s\n",
                         idToDrive (unitID), driveErrors[i]);
                log_fputs (log_buf, stderr);
                error = TRUE;
                break;

            default:
                sprintf (log_buf, " Unknown char: %c in drive status.\n",
                         *p1);
                log_fputs (log_buf, stderr);
                labtyd (STATUS_REPORT, "Can't read drive status.");
                break;
        }

        p1--;
        i++;
    }

    if (!error)
    {
        sprintf (log_buf, "%s reports no errors.\n", idToDrive (unitID));
        log_fputs (log_buf, stderr);
    }

    return (error);

}

/***********************************************************

		drive_status

   Send the command to the appropriate drive to 
   cause it to return its status flags.

************************************************************/

drive_status (int driveID)
{
    char command[20];

    (void) sprintf (command, "%dDFS ", driveID);

    return (send_it (command));
}

/***********************************************************

		enable_drive

   Send the command to the specified drive to cause it to
   turn on communications.

************************************************************/

enable_drive (int driveID)
{
    char command[20];

    (void) sprintf (command, "%dE ", driveID);

    return (send_it (command));
}

/***********************************************************

		disable_drive

   Send the command to the specified drive to cause it to
   turn off communications.

************************************************************/

disable_drive (int driveID)
{
    char command[20];

    (void) sprintf (command, "%dF ", driveID);

    return (send_it (command));
}

/***********************************************************

		energize_drive

   Send the command to the specified drive to cause it to 
   energize or shut down, based on the argument.

************************************************************/

energize_drive (int driveID, int arg)
{
    char command[20];

    if (arg == 0 || arg == 1)
    {

        (void) sprintf (command, "%dST%d ", driveID, arg);

        return (send_it (command));
    }

    else
        return (MOTOR_RANGE);
}

/***********************************************************

		reset_drive

   Send the command to the specified drive to cause it to
   reset.

************************************************************/

reset_drive (int driveID)
{
    char command[20];

    (void) sprintf (command, "%dZ ", driveID);

    return (send_it (command));
}

/***********************************************************

		kill_drive

   Send the command to the specified drive to cause it
   to kill motion.

************************************************************/

kill_drive (int driveID)
{
    char command[20];

    (void) sprintf (command, "%dK ", driveID);

    return (send_it (command));
}

/*********************************************************

		idToDrive

   Given a drive ordinal, return the text description
   of it's use.

**********************************************************/

char *idToDrive (int id)
{
    return (driveNames[id]);
}
