
/*************************************************************

		fiborg.c

   This program translates high level commands on the input
   stream into coordinated, validated commands to the various
   components of the robotic fiber positioner.  The output
   is a character stream of Compumotor commands including
   the unit ID numbers required to dispatch the commands to
   the correct unit.

   The scope of this program is such that it allows  buttons
   to be picked up and moved to new locations.  Collisions
   are checked for and avoided.  The locations of the buttons
   is recorded and can be restored between invocations of the
   program.

   During a normal invocation of the program, the communication
   program is automatically launched and pipes are set up between
   it and this program.  These pipes allow communication in
   both directions.  With the proper argument at invocation,
   the program will not launch fibcomm, and instead send its output
   to the stdout. In this mode, all status requests are bypassed
   since there is no facility to supply them.

   The individual commands are described in the file parser.c.
   All of the *.c files have a comment block describing what 
   functions they contain and whether they are available as
   externs.

   Usage:

      fiborg [-defgqsv] [-v]

      Options:	(first argument)

	 d	"direct" mode.  All communication goes to stdout,
		and the communication process is not launched.
		This program operates on "faith" and does not
		need status reports back from the indexers/motors.

	 e	"explicit" mode.  Commands to operate the gripper
		and z axis are sent explicitly.  Normally, previously
		downloaded sequences are used, and the software 
		simply activates one of those sequences.  Sequences
		are available for picking up and putting down buttons,
		and for relaxing the gripper.  

	 f	"faith" mode.  The communication process launches
		normally, but the system does not request or wait
		for status responses.  This command should be used
		with EXTREME caution.  In no event should it be
		used with the fiber bundle!  

	 p	"prompt" mode.  Button presence tests will be
		deferred to the user for their result.  only
		useful in the simulator.

	 q	"quiet" mode.  Similar to simulate mode below.
		Goes one step further and does not even generate
		the commands for the motors.  Only the database
		is altered.

	 s	"simulate" mode.  The communication process is
		not started, and the moves are sent to stdout.  The 
		commands that move buttons simply update the
		internal database with the new, requested positions.

	 v	"verbose" mode.  Lots of "informative" messages are
		printed about the status of commands as they are
		executed.

	 w	"wiyn" mode.  Tries to make connection to wiyn via GWC

      Options:	(second argument)
		This  argument is passed intact to the communication
		program.  It has no bearing on the operation of this
		program.  If one of the parameters in the first
		argument blocks the launch of the communication process,
		then this argument is meaningless and never used.

	 v	"verbose" mode.  The echo of each command is printed
		to stderr when it is received back from the indexer.
		In addition, the raw status responses are printed.
		This does not effect the operation of the program
		at all.

*************************************************************/

#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <setjmp.h>
#include <string.h>
#include <signal.h>
#include <sys/socket.h>
#include <tcl.h>
#include <tk.h>

#ifdef LINUX
#include <ulimit.h>
#endif

#include <unistd.h>

#ifdef __CRUISE__
#include <ncurses.h>
extern WINDOW *win;             /* for curses */
#endif

#include "version.h"
#include "prompter.h"
#include "errors.h"
#include "death.h"
#include "miscMath.h"
#include "init.h"
#include "motor.h"
#include "flex.h"
#include "buttons.h"
#include "kaboom.h"
#include "nesslib.h"
#include "translate.h"
#include "detrot.h"
#include "concen.h"
#include "xref.h"
#include "archive.h"
#include "reports.h"
#include "statusIO.h"
#include "parser.h"
#include "stdBus.h"
#include "fiborg.h"
#include "tkInterface.h"
#include "guiInterface.h"
#include "log.h"

#ifdef HYDRA_II
#include "hydra_system.h"
#endif
// HYDRA_II

/* structures used by event handler */

hydraClientData StdinInfo;
hydraClientData GraphinInfo;

/* */

extern struct itimerval masterTimer;
FILE *devOut;                   /* where all the commands go */
static char log_buf[256];

void coldDeath (void);

int stdACindexers (int axis_ord, int arg);
int stdACdrives (int axis_ord, int arg);
int stdACcalib (int axis_ord, int arg);
int stdACpier (int axis_ord, int arg);

int tkMainHydra (int argc, char **argv);
int logCreate (char *name);

extern int bowOut (void);
extern int weAreNotAlone (void);
extern int announcePresence (void);
extern void flushStatus (void);
void getPowered (void);
extern void buttonRecover (char *args);
extern void led (char *args);
extern int setBlock (int flag);
extern int screenStatus (int *flags, int testMode);
extern void setStats (char *args);
extern char *tkfgets (char *s, int n, FILE * stream);
extern int getYesNo (FILE * devOut, FILE * devIn, char *prompt,
                     int defaultValue);
extern void coldstart (char *args);
void getTemperature (void);
extern void plate (char *args);
extern void execfile (char *args);

main (int argc, char **argv, char **envp)
{
    int i;                      /*  working counter for arg list */

    int pipeOut[2];             /*  Pipe to the communication program */
    int pipeIn[2];              /*  Pipe from the communication program */

    char *args;                 /*  Working pointer to arguments */

    char comArg1[64];           /* max number of chars in the first arg to fibcomm */
    char *comArgs[5];           /* becomes argv[] in main in fibcomm */
    char *comEnv[5];            /* becomes envp[] in main in fibcomm */

    char *fibPath;              /* pathname to binaries */

    char commFile[64];          /* complete path and filename */
    char *commName;             /* communication program name */

    char *archiveFileName = NULL;       /* file with button positions */

    char command[64];           /* storage for misc. user input */
    int noExit = 1;

    int stdBusResponse[12];

/*
   struct sockaddr graphSocketName;
   int addrLen = sizeof( graphSocketName.sa_data );

   int graphSocket;
   int graphConnectSocket;
*/

    CurrentLog = stderr;

#ifdef __CRUISE__

    win = initscr ();
    wclear (win);
    refresh ();
    endwin ();

    /* get rid of the handlers installed by curses. */

    signal (SIGCONT, SIG_DFL);
    signal (SIGTSTP, SIG_DFL);
    signal (SIGSTOP, SIG_DFL);

#endif

    /* the following is to make the window come up cleanly... */

    fputs ("\n\n\n", stderr);
    sleep (5);

    /* set up to bail in the event of catastrophe... */

    if (setjmp (Env))
    {
        log_fputs ("FATAL ERROR during initialization\n", stderr);
        log_fputs ("Exiting program.\n", stderr);

        signal (SIGHUP, SIG_IGN);

        if (killpg (getpgrp (), SIGHUP))
        {
            log_fputs ("WARNING: couldn't kill other processes\n", stderr);
        }
        else
            log_fputs ("Communication process (supposedly) killed\n", stderr);

        bowOut ();
    }

    /* create a socket in case the graphics decide to connect */

    guiSocketCreate ();

    nullFile = fopen ("/dev/null", "r+");

    graphCmds = nullFile;

    /* set up arg list for communication program */

    comArgs[0] = COMM_PROG;
    comArgs[1] = comArg1;
    comArgs[2] = NULL;

    comEnv[0] = NULL;

    /* if a second arg is present, make a copy to pass on to
       communication program. Else, make a dummy argument. */

    if (argc >= 3)
        (void) strcpy (comArg1, argv[2]);

    else
    {
        comArg1[0] = '-';
        comArg1[1] = '\0';
    }

    fprintf (stderr, "Starting fiborg version %s\n", VERSION);

    /*  Set up flags from arg list. */

    if (argc >= 2)
    {
        if (*argv[1] == '-')
        {

            args = argv[1];

            /* loop through arg list */

            for (i = 1; args[i] != '\0'; i++)
            {

                switch (args[i])
                {

                    case 'c':
                        progeny = TRUE; /*  we are a forked process */
                        break;

                    case 'd':
                        direct = TRUE;  /*  all communication goes to stdout. */
                        faith = TRUE;
                        break;

                    case 'e':
                        remote = FALSE; /* The program sends explicit Z commands */
                        break;

                    case 'f':
                        faith = TRUE;   /* operate on "faith".  Implied by "d". */
                        break;

                    case 'G':
                        graphic = TRUE; /* fork off the graphics program */
                        break;

                    case 'p':
                        PromptHw = TRUE;
                        break;

                    case 'q':
                        direct = TRUE;  /*  all communication goes to stdout. */
                        faith = TRUE;
                        quiet = TRUE;   /* suppress motor commands */
                        break;

                    case 's':
                        break;

                    case 'v':
                        verbose = TRUE; /* The program is to show all output. */
                        break;

                    case 'w':
                        noGWC = FALSE;  /* Program will try to connect to wiyn */
                        break;

                    default:
                        (void) fprintf (stderr,
                                        "WARNING: Unknown flag \"%c\" ignored.\n",
                                        args[i]);
                        break;
                }
            }
        }

        else
        {
            labtyd (1, "BAD ARGUMENTS -- Exiting...");
        }
    }

    /* get the log working */

    if (direct)
        strcpy (Prefix, SIM_PREFIX);
    else
        strcpy (Prefix, HYD_PREFIX);

    if (progeny)
        (void) fprintf (stderr, "Fiborg launched as child process.\n");

    else
    {
        logCreate ((char *) NULL);      /* make sure the log file is current */
    }

    logOpenLocal ();            /* open the log of system activity */

   /** be sure we are the only copy of the "real" program running **/

    if (!faith)
    {
        if (weAreNotAlone ())
        {

            fputs ("A copy of Hydra is already running!\n", stderr);

            fputs ("Continue? [y/n]>", stderr);

            noExit = 1;

            while (noExit)
            {

                fgets (command, 32, stdin);

                if ((*command == 'y') || (*command == 'y'))
                {
                    noExit = 0;
                }
                else
                {
                    if ((*command == 'n') || (*command == 'N'))
                    {
                        exit (1);
                    }
                    else
                    {
                        fputs ("ERROR: only a yes/no response allowed!\n",
                               stderr);
                        fputs ("Continue anyway? [y/n]>", stderr);
                    }
                }
            }
        }

        else
        {
            announcePresence ();
        }
    }

    /* restore the status */

    if (faith)
    {
        if (LoadStatusFile (SIMSTATUS, statusList) != 0)
        {
            fputs ("WARNING:  System status not restored from disk\n",
                   stderr);
        }
        else
            flushStatus ();     /* makes sure that defaults get written back out */
    }
    else
    {
        if (LoadStatusFile (STATUSFILE, statusList) != 0)
        {
            fputs ("WARNING:  System status not restored from disk\n",
                   stderr);
        }
        else
            flushStatus ();     /* makes sure that defaults get written back out */
    }

    /* get the temperature from the user */

    /* getTemperature(); */

/* Argument dependent initialization */

    if (!direct)
    {

        if (pipe (pipeOut) || pipe (pipeIn))
        {
            labtyd (1, "Could not create required pipes.");
        }

        else
        {

            /* fork off the communication process */

            commPid = fork ();

            if (commPid != 0)
            {                   /*  Continue as the parent process */

                /* assign stdout to the pipe to the communication process */

/*
	    if( close( fileno(stdout)))
	       labtyd( SYSTEM_CALL, "Couldn't close file descriptor");
	    if( dup( pipeOut[1] ) < 0 )
	       labtyd( SYSTEM_CALL, "Couldn't dup file descriptor");
*/

                if ((devOut = fdopen (pipeOut[1], "w")) == NULL)
                    labtyd (SYSTEM_CALL, "Couldn't open devOut descriptor");
                setbuf (devOut, (char *) NULL);

                /*
                   if( close( pipeOut[1] ))
                   labtyd( SYSTEM_CALL, "Couldn't close file descriptor");
                 */

                if (close (pipeOut[0]))
                    labtyd (SYSTEM_CALL, "Couldn't close file descriptor");

                /* create a stream for responses from the motors */

                responses = fdopen (pipeIn[0], "r");
                setlinebuf (responses);
                if (close (pipeIn[1]))
                    labtyd (SYSTEM_CALL, "Couldn't close file descriptor");
            }

            else
            {                   /* set up for and exec the child */

                /* start the communications process */

                /* assign stdin to the pipe from the parent */

                if (close (fileno (stdin)))
                    labtyd (SYSTEM_CALL, "Couldn't close file descriptor");
                if (dup (pipeOut[0]) < 0)
                    labtyd (SYSTEM_CALL, "Couldn't dup file descriptor");
                if (close (pipeOut[0]))
                    labtyd (SYSTEM_CALL, "Couldn't close file descriptor");

                /* assign stdout to the pipe to the parent */

                if (close (fileno (stdout)))
                    labtyd (SYSTEM_CALL, "Couldn't close file descriptor");
                if (dup (pipeIn[1]) < 0)
                    labtyd (SYSTEM_CALL, "Couldn't dup file descriptor");
                if (close (pipeIn[1]))
                    labtyd (SYSTEM_CALL, "Couldn't close file descriptor");

                (void) strcat (comArg1, CHILDOFFIBORG);

                if ((fibPath = getenv (FIBBIN)) != NULL)
                {
                    (void) strcpy (commFile, fibPath);
                    (void) strcat (commFile, "/");
                }

                if ((commName = getenv (COMM_PROG)) == NULL)
                {

                    log_fputs
                        ("Communication program name not in environment--exiting\n",
                         stderr);

                    signal (SIGHUP, SIG_IGN);

                    if (killpg (getpgrp (), SIGHUP))
                    {
                        log_fputs ("WARNING: couldn't kill other processes\n",
                                   stderr);
                    }

                    bowOut ();
                }

                (void) strcat (commFile, commName);

                /* launch the motor communication package */
                execve (commFile, comArgs, envp);
            }
        }
    }
    else
    {
        /* everything maps to stdout by default */
        devOut = stderr;
    }

#ifdef IGNORE
    if (graphic)
    {

        graphPid = fork ();

        if (graphPid == 0)
        {

            graphPath = getenv (FIBBIN);

            if (faith)
                graphProg = getenv (GRAPH_PROG);
            else
                graphProg = getenv (GRAPH_4M);

            sprintf (graphLaunchCmd, "%s/%s", graphPath, graphProg);

            fprintf (stderr, "launching graphics program:\"%s\"\n",
                     graphLaunchCmd);

            if (execve (graphLaunchCmd, argv, envp) != 0)
            {
                fputs ("ERROR: Couldn't launch graphic program\n", stderr);
                bowOut ();
            }
        }
        else
        {

            unlink (SOCK_NAME);

            graphSocketName.sa_family = AF_UNIX;

            strcpy (graphSocketName.sa_data, SOCK_NAME);

            /* create the socket to listen for the graphics program */

            graphSocket = socket (PF_UNIX, SOCK_STREAM, 0);

            if (graphSocket < 0)
                perror ("Fiborg graphic socket");

            if (bind
                (graphSocket, &graphSocketName,
                 sizeof (graphSocketName.sa_data)) != 0)
                perror ("Fiborg graphic bind");

            if (listen (graphSocket, 2) != 0)
                perror ("Fiborg graphic listen");

            /* wait for graphics to get going and then connect to it */

            fputs ("Waiting to connect to graphics window....\n", stderr);

            graphConnectSocket =
                accept (graphSocket, &graphSocketName, &addrLen);

            fputs ("CONNECTED.\n", stderr);

            if (graphConnectSocket < 0)
                perror ("Fiborg graphic accept");

            graphCmds = fdopen (graphConnectSocket, "r+");

            if (graphCmds == NULL)
                perror ("Fiborg graphic socket file null");
            else
                setbuf (graphCmds, 0);
        }
    }
#endif

    /* set up the structures used by Tk to call the parsers */

    StdinInfo.stream = stdin;
    StdinInfo.loopFlag = ONE_PASS;

    GraphinInfo.stream = nullFile;
    GraphinInfo.loopFlag = ONE_PASS;

    /* get temperature from user */
    /* getTemperature(); */

    /* get the cable name from the user */

/* setCable(""); *//* find out what cable we are using */

    /*  Do the major initialization of the system. */

    load_limits ();             /* Load structure for move restriction. */
    installSignals ();          /* Set up the signal handlers. */
    initRadiusList ();          /* quick collision test table */
    initRotation (&RotData);    /* table of field rotation points */

    softAngleFlag = TRUE;

    /* load the button eccentricities */

    eccenLoad (eccenOpen (ECCENTRIC_FILE), FiBase);

    if (graphic)
    {

        fprintf (graphCmds, "setcable %s \n", getNameForCable (CurrentCable));
        fflush (graphCmds);
    }

    /* be sure the database is set correctly and restored.
       First choose either the real locations or the simulator */

    if (faith)
        archiveFileName = getenv (SIMFILE);
    else
        archiveFileName = getenv (DATAFILE);

    sprintf (log_buf, "using fiber database: %s \n", archiveFileName);
    log_fputs (log_buf, stderr);

    /* bail out if environment is bad. */

    if (archiveFileName == NULL)
    {
        log_fputs
            ("Couldn't find name of archive in user environment--Exiting\n",
             stderr);

        bowOut ();

    }

    if ((positionArchive = openArchive (archiveFileName, READ_WRITE)) == NULL)
    {

        log_fputs ("Couldn't open archive of current configuration\n",
                   stderr);

        bowOut ();
    }

    log_fputs ("Initializing Database--please wait.\n", stderr);

    initDbase ();

    log_fputs ("Restoring button positions--please wait.\n", stderr);

    restoreFromArchive (positionArchive, FiBase, ALL);

#ifdef HYDRA_II
    // - make sure this also flattens the focal plate.
    // - note that for now we don't have button recovery
    //
    hydra_startup();
#else
    /* turn on the racks before checking for buttons... */
    getPowered ();

    /* make sure that any errors during initialization get to prompt */

    if (setjmp (Env) == 0)
    {

        if (FiberInTransit.id >= 0)
        {

            sprintf (log_buf,
                     "\nWARNING: Software last exited with button %d in gripper!!\n",
                     FiberInTransit.id);
            log_fputs (log_buf, stderr);

            fputs ("(or so it thinks...)\n", stderr);

            FiberInTransit.fiber = &FiBase[FiberInTransit.id];

            FiberInTrouble.fiber = FiberInTransit.fiber;
            FiberInTrouble.fiber->parkedStatus = EXTENDED;

            buttonRecover (0);

        }

    }
    else
    {
        log_fputs
            ("\nWARNING: Recovery process for button in jaws exited before completion!\n",
             stderr);
        log_fputs ("Proceed only if your have been properly trained.\n",
                   stderr);
        log_fputs ("GRIPPER MOTION IN ANY DIRECTION MAY BE DANGEROUS!\n",
                   stderr);
    }

    led ("illum 8");
#endif 
// HYDRA_II


    /* get tk started  --which includes allowing the graphics to connect */

    tkMainHydra (argc, argv);

   /*****  sync graphics to state of hardware *****/

    /* now allow graphics to send requests */

    setBlock (FALSE);

#ifdef HYDRA_II
    // update all the graphics here
#else
    if (!faith)
    {
        if (screenStatus (stdBusResponse, SCREEN_IN) == SCREEN_OK)
            if (graphic)
                fprintf (graphCmds, "showscreen in\n");
    }
#endif
// HYDRA_II

    setStats ("");

    /* main loop */

    while (1)
    {
        (void) Tk_DoOneEvent (0);
    }

}

/**************************************************************/

/* 
		getCold

   Test the system for a recent power-up and 
   force a coldstart if so.
*/

void getCold (char *args)
{
    int noExit = TRUE;
    char command[64];

    struct itimerval coldTimer;

    struct sigvec oldVec;
    struct sigvec ColdDeathVec;

    ColdDeathVec.sv_handler = coldDeath;
    ColdDeathVec.sv_mask = NULL;
    ColdDeathVec.sv_flags = NULL;

    coldTimer.it_value.tv_sec = COLD_TIME;
    coldTimer.it_value.tv_usec = 0;
    coldTimer.it_interval.tv_sec = 0;
    coldTimer.it_interval.tv_usec = 0;

    sigvec (SIGUSR2, &ColdDeathVec, &oldVec);

    if (faith && !PromptHw)
    {

        while (noExit)
        {
            log_fputs ("\nEnter \"coldstart\" to initialize system.\n> ",
                       stderr);

            tkfgets (command, 63, stdin);

            log_putsQ (command);

            /* a way out */

            if (strcmp (command, "quit\n") == 0)
            {
                quit (0);
                return;
            }

            if (strcmp (command, "coldstart\n") == 0)
                noExit = 0;
            else
                log_fputs
                    ("SORRY: All you can do at the moment is \"coldstart\"!\n",
                     stderr);
        }

        /* clear any gripout that was active */
        GripperParked = FALSE;
        flushStatus ();
    }
    else
    {                           /* deal with real hardware */

        if (faith && PromptHw)
        {                       /* allows user to create errors */
            if (getYesNo (stderr, stdin, "Has the rack cycled?", TRUE))
            {
                log_fputs
                    ("Warning:  The electronics rack is not initialized!\n\n",
                     stderr);
            }
            else
            {
                log_fputs ("FYI:  The electronics rack is initialized!\n\n",
                           stderr);
            }
        }
        else
        {
            if (SystemIsCold (&masterTimer))
                log_fputs
                    ("Warning:  The electronics rack is not initialized!\n\n",
                     stderr);
        }

        /* while( SystemIsCold( &masterTimer ) ){ */

        while (TRUE)
        {

            log_fputs
                ("Enter \"coldstart\" to initialize system (or \"quit\" to exit).\n > ",
                 stderr);

            fgets (command, 63, stdin);

            log_putsQ (command);

            /* a way out */

            if (strcmp (command, "quit\n") == 0)
            {
                quit (0);
                return;
            }

            /* the secret word */

            if (strcmp (command, "iamgod\n") == 0)
                return;

            if (strcmp (command, "coldstart\n") == 0)
                break;
            else
                log_fputs
                    ("SORRY: All you can do at the moment is \"coldstart\"!\n",
                     stderr);
        }

        sigvec (SIGUSR2, &oldVec, NULL);

        coldstart (0);
    }
}

/***********   initialization callback from TK *********************/

void syncToWorld (void)
{
    getTemperature ();

    if (setjmp (Env) == 0)
    {

        getCold ("");

        log_fputs ("System Initialization completed normally\n", stderr);
    }
    else
    {
        log_fputs
            ("WARNING: Due to errors, the system is NOT fully initialized!\n",
             stderr);
        log_fputs ("Proceed only if your have been properly trained.\n",
                   stderr);
        log_fputs ("SYSTEM MAY BE UNSTABLE!\n", stderr);
    }
}

/********************************/

void getTemperature (void)
{

    double *tempTemp;
    double oldTemp;
    double defaultTemp;
    double min = 0.0;
    double max = 90.0;
    int precision = 1;

    oldTemp = PhysicalEnvironment.temp;

    if (faith)
    {
        defaultTemp = oldTemp;
    }
    else
    {
        defaultTemp = (1.8 * getInstrumentTemperature ()) + 32.0;       /* must be degF. */

        if ((defaultTemp < 0.0) || (defaultTemp > 100.0))
        {

            defaultTemp = oldTemp;
        }
    }

    tempTemp = (double *) getUserInput (stderr, stdin, DOUBLE, SHOW_DEFAULT,
                                        "\nEnter instrument temperature [degrees F.]",
                                        &defaultTemp, &min, &max, &precision);

    PhysicalEnvironment.temp = *(double *) tempTemp;

    flushStatus ();             /* makes sure that defaults get written back out */
}

void getPowered (void)
{

    if (!faith &&
        getYesNo (stderr, stdin, "Do you wish to turn on the rack?", TRUE))
    {

        log_fputs ("Turning on indexers\n", stderr);
        stdACindexers (STD_ord, ON);
        sleep (1);
        log_fputs ("Turning on drives\n", stderr);
        stdACdrives (STD_ord, ON);
        sleep (1);
        log_fputs ("Turning on calibration rack\n", stderr);
        stdACcalib (STD_ord, ON);
        sleep (1);
        log_fputs ("Turning on pier power\n", stderr);
        stdACpier (STD_ord, ON);
        sleep (1);

        log_fputs ("Flattening focal plane.\n", stderr);
        plate ("init");
        sleep (1);

        /* now flush the daisy chain to get rid of garbage */

        send_it ("~f");

        execfile ("driveset");

    }

}

/**************************************************************/

/*
      send_it

   Send a command.  All commands call this routine.  Only error or
   user messages use stdio calls directly.
*/

send_it (char *theString)
{
    if (quiet)
        return (FALSE);

    else
    {

        if (verbose)
        {
            (void) sprintf (log_buf, "->%s\n", theString);
            log_fputs (log_buf, stderr);
        }

        if (fprintf (devOut, "%s\n", theString) == EOF)
        {
            return (TRUE);
        }
        else
            return (FALSE);
    }
}

void coldDeath (void)
{
    fputs ("\nFATAL ERROR:  Could not communicate with hardware!\n", stderr);
    fputs ("Check power and cabling and then re-start program.\n", stderr);
    fputs ("exiting in 5 seconds.\n", stderr);

    sleep (5);

    bowOut ();

}

/*
      send_marker

   Send a marker to the communication program.  
*/

send_marker (char *theString)
{
    if (quiet)
        return (FALSE);

    else
    {

        if (verbose)
        {
            (void) sprintf (log_buf, "->%s\n", theString);
            log_fputs (log_buf, stderr);
        }

        if (fprintf (devOut, "~M:%s\n", theString) == EOF)
        {
            return (TRUE);
        }
        else
            return (FALSE);
    }
}
