
/****************************************************************

			flex.c

Flexure equations for the fiber positioner.

****************************************************************/

#include <stdio.h>
#include <math.h>

#include "miscMath.h"
#include "nesslib.h"
#include "motor.h"
#include "translate.h"
#include "flex.h"
#include "fiborg.h"
#include "log.h"

void calcVecThermal (struct aPoint *thePoint, struct aPoint *thermVec,
                     struct envData *physEnv);
void calcVecDistort (struct aPoint *thePoint, struct aPoint *distortVec);
void calcVecFlex (struct aPoint *thePoint, struct aPoint *flexVec,
                  struct envData *physEnv);

/*
		coordCorrect

   Apply the set of temperature/distortion/flexure corrections
   to the specified coordinate.  
*/
void coordCorrect (struct aPoint *thePoint, struct envData *physEnv,
                   int direction)
                                /* flag add or subtract vectors */
{
    char log_buf[256];
    struct aPoint thermVec;
    struct aPoint distortVec;
    struct aPoint flexVec;

    /* calculate the vectors */

    calcVecThermal (thePoint, &thermVec, physEnv);
    calcVecDistort (thePoint, &distortVec);
    calcVecFlex (thePoint, &flexVec, physEnv);

    /* print them if required */

    if (verbose)
    {

        if (thermFlag)
        {
            sprintf (log_buf, " Thermal correction = %f x, %f y \n",
                     thermVec.x, thermVec.y);
            log_fputs (log_buf, stderr);
        }

        if (warpFlag)
        {
            sprintf (log_buf, " Distortion correction = %f x, %f y \n",
                     distortVec.x, distortVec.y);
            log_fputs (log_buf, stderr);
        }

        if (flexFlag)
        {
            sprintf (log_buf, " Grav. flex correction = %f x, %f y \n",
                     flexVec.x, flexVec.y);
            log_fputs (log_buf, stderr);
        }

    }

    /* apply the corrections in the proper order */

    if (direction == NORMAL)
    {

        /* we are going from sky coords to encoder coords. */

        if (thermFlag)
            applyVec (thePoint, &thermVec, direction);

        if (flexFlag)
            applyVec (thePoint, &flexVec, direction);

        if (warpFlag)
            applyVec (thePoint, &distortVec, direction);
    }

    else
    {                           /* direction is INVERT */

        /* we are going from encoder coords to sky coords. */

        if (warpFlag)
            applyVec (thePoint, &distortVec, direction);

        if (flexFlag)
            applyVec (thePoint, &flexVec, direction);

        if (thermFlag)
            applyVec (thePoint, &thermVec, direction);
    }
}

/*
		calcVecThermal

   Calculate the temperature correction vector for the
   the specified point.
*/

void calcVecThermal (struct aPoint *thePoint, struct aPoint *thermVec,
                     struct envData *physEnv)
{
    double dx;
    double dy;

    dx = THERM_COEF * (TZERO - physEnv->temp) * thePoint->x;
    dy = THERM_COEF * (TZERO - physEnv->temp) * thePoint->y;

    thermVec->x = dx;
    thermVec->y = dy;
}

/*
		calcVecDistort

  Calculate the correction vector to compensate for the 
  distortion in the stages andthe ball screws.
*/

void calcVecDistort (struct aPoint *thePoint, struct aPoint *distortVec)
{
    double y = thePoint->y;
    double x = thePoint->x;

    double dx;
    double dy;

/* Calculate the grid correction factor for x as a function of x and y */

    dx = (x * XSCALE_ADJUST) + (y * DXY_LINEAR) + (y * y * DXY_QUAD);

/* Calculate the final positional error adjustment for x */

    dx += x * (FDXX_LINEAR + x * FDXX_QUAD);

    distortVec->x = dx;

/* Calculate the grid correction factor for y as a function of x and y */

    dy = (y * YSCALE_ADJUST) + (y * y * DYY_QUAD) + (x * DYX_LINEAR);

/* Calculate the final positional error adjustment for y */

    dy += y * (FDYY_LINEAR + y * FDYY_QUAD);

    distortVec->y = dy;
}

/*
		calcVecFlex

  Calculate the flexure vector to compensate for the 
  gravity distortion of the system.

*/

void calcVecFlex (struct aPoint *thePoint, struct aPoint *flexVec,
                  struct envData *physEnv)
{
    double dx1;
    double dx2;
    double dy;

    double decR = degToRad (physEnv->dec);
    double haR = degToRad (physEnv->ha);
    double obsLatR = degToRad (KPNO_LAT);

    dx1 = Gx1 * (sin (haR) * cos (obsLatR));

    dx2 = Gx2 * ((sin (decR) * cos (haR) * cos (obsLatR)) -
                 (cos (decR) * sin (obsLatR)));

    dy = Gy * ((sin (decR) * cos (haR) * cos (obsLatR)) -
               (cos (decR) * sin (obsLatR)));

    flexVec->x = (dx1 + dx2) * THOU_TO_STEPS;
    flexVec->y = dy * THOU_TO_STEPS;
}
