
/************************************************************

		multAxis.c

      Multiple axis movement routines

*************************************************************/

#include <stdio.h>
#include <sys/time.h>
#include <setjmp.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>

#include "miscMath.h"
#include "errors.h"
#include "death.h"
#include "init.h"
#include "motor.h"
#include "flex.h"
#include "translate.h"
#include "detrot.h"
#include "reports.h"
#include "fiborg.h"
#include "log.h"
#include "compumotor.h"

#ifdef HYDRA_II
#include "hydra_common.h"
#include "hydra_sm.h"
#include "hydra_system.h"
#endif
// HYDRA_II

/****************** globals  ********************/
static char log_buf[256];
struct axis_characteristics axis[3];    /* contains physical limits */
struct plate_coords location_now;       /* Where the gripper is */
struct plate_coords Tags[MAXTAGS];

int RestrictedMoves;            /*  force additional error checking */
int GripPos;                    /* last known status of gripper */
int GripperParked = 0;
int ScreenStatus = 0;

/************* grip pos callback *************/
void dummyGripCallback (int x, int y);
void setGripPosCallback (void (*function) ( /* ??? */ ));
static void (*GripPosCallback) () = dummyGripCallback;

/*************local prototypes*****************/
static void printFlags (int flags[]);
char *axisNames[] = { "X", "Y", "Z" };

int GoToXY (long int x, long int y, double vel, double accel, int system);
int HomeZ (void);
int Pause (unsigned int time);
extern int buttonPresent (void);
extern void flushStatus (void);

/*
		setGripPosCallback
   
   A function to set the local pointer.
*/
void setGripPosCallback (void (*function) ( /* ??? */ ))
{
    GripPosCallback = function;
}

/*
		dummy Callback
   A function that automatically gets installed 
   and called when the gripper moves.
*/

void dummyGripCallback (int x, int y)
{
    /* fprintf( stderr, "%d, %d\n", x, y ); */
}

/*
		tagCoord

   Record a tag for the current location.
*/

void tagCoord (int tagID)
{
    Tags[tagID] = location_now;
}

/*
		defTag

   Set the specified tag to the supplied absolute coordinates.
*/

void defTag (int id, struct plate_coords *coord)
{
    struct aPoint target;

    target.x = coord->x;
    target.y = coord->y;

    if (Rotflag)
        rotate (&target, RotData.aveAngle);

    Tags[id].x = target.x;
    Tags[id].y = target.y;
}

/*
		printTag

   Print the contents of a tag.
*/

void printTag (int id)
{
    struct plate_coords coord;

    coord = Tags[id];

    sprintf (log_buf, "Tag #%2d is set to %ld, %ld\n", id, coord.x, coord.y);
    log_fputs (log_buf, stderr);
}

/*
      Bump

   Move the stages incrementally by the amounts indicated.
   Use the global, "location_now", to generate the 
   complete move.
*/

Bump (long int dx, long int dy, double vel, double accel)
{
    struct aPoint tmpCoords;

    int system;                 /* virtual or physical */

    tmpCoords.x = location_now.x;
    tmpCoords.y = location_now.y;

    if (Rotflag)
    {

        /* un-rotate to get physical coords */
        rotate (&tmpCoords, -(RotData.aveAngle));
        system = VIRTUAL;
    }

    else
        system = PHYSICAL;

    tmpCoords.x += dx;
    tmpCoords.y += dy;

    GoToXY ((long) tmpCoords.x, (long) tmpCoords.y, vel, accel, system);
    /* this will re-rotate if required */

}

GoToXY (long int x, long int y, double vel, double accel, int system)
{
    double ratio;               /*  ratio of mags of dist moved each axis */
    double theta;               /*  angle of move vector wrt X */
    double sinTheta;
    double cosTheta;
    double x_accel;
    double y_accel;
    double x_vel;
    double y_vel;
    long x_mag;
    long y_mag;
    int x_bigger = 0;           /*  flag for moving more in x than y */

    struct plate_coords reported_loc;
    struct aPoint target;

    double moveTime;
    struct itimerval moveTimer;

    int xLoopStatus[8];         /* twice what we need */
    int yLoopStatus[8];         /* twice what we need */

    /* clear the record of where the gripper incurred an error */

    LastXY.x = VOID_COORD;
    LastXY.y = VOID_COORD;

    flushStatus ();

#ifndef HYDRA_II
    /* be sure that the gripper is configured */
    if (GripperParked)
    {
        fputs ("ERROR:  Gripper is disabled!\n", stderr);
        fputs ("Use \"gripin\" to re-activate gripper\n", stderr);
        labtyd (GRIP_OUT, "Command not valid at this time");
    }
#endif
// HYDRA_II

    /*  rotate coordinates from virtual to physical system if required */

    target.x = x;
    target.y = y;

    if (Rotflag && (system == VIRTUAL))
        rotate (&target, RotData.aveAngle);

    /* Corrections to coordinates for imperfections in the mechanical 
       system. */

#ifndef HYDRA_II
    coordCorrect (&target, &PhysicalEnvironment, NORMAL);
#endif
// HYDRA_II

    x = target.x;
    y = target.y;


#ifdef HYDRA_II
    return move_xy (x, y, HYDRA_ONE);
#else
    if (!XYokToMove ())
        labtyd (SYS_CONFIG,
                "ERROR: Can't move. Gripper not at safe  height.");

    x_mag = abs ((x - location_now.x));
    y_mag = abs ((y - location_now.y));

    /* bail out if no move is to take place */

    if ((x_mag == 0) && (y_mag == 0))
        goto nadaToDo;

    /* insure positive values for velocity and acceleration */

    vel = fabs (vel);
    accel = fabs (accel);

    /* calculate angle of velocity and acceleration vectors */

    theta = (atan2 ((double) y_mag, (double) x_mag));

    sincos (theta, &sinTheta, &cosTheta);

    if (x_mag >= y_mag)
    {
        x_bigger = TRUE;
        if (x_mag > 0)
            ratio = (double) y_mag / (double) x_mag;
        else
            (ratio = 0);
    }

    else
    {
        x_bigger = FALSE;
        if (y_mag > 0)
            ratio = (double) x_mag / (double) y_mag;
        else
            (ratio = 0);

    }

    /* validate move location  */

    if (x > axis[X_ord].position.max ||
        x < axis[X_ord].position.min ||
        y > axis[Y_ord].position.max || y < axis[Y_ord].position.min)
    {

        return (XY_LIM);
    }

    /*  calculate & validate velocities and accelerations */

    if (x_bigger)
    {

        /* Compute the x component of the velocity vector */

        x_vel = vel * cosTheta;

        if (x_vel > axis[X_ord].velocity.max)
            x_vel = axis[X_ord].velocity.max;

        /* Compute the x component of the acceleration vector */

        x_accel = accel * cosTheta;

        if (x_accel > axis[X_ord].accel.max)
            x_accel = axis[X_ord].accel.max;

        /* Compute the y component of the velocity vector */

        y_vel = x_vel * ratio;

        if (y_vel > axis[Y_ord].velocity.max)
        {
            y_vel = axis[Y_ord].velocity.max;
            x_vel = y_vel / ratio;
        }

        if (y_vel < axis[Y_ord].velocity.min)
            y_vel = axis[Y_ord].velocity.min;

        /* Compute the y component of the acceleration vector */

        y_accel = x_accel * ratio;

        if (y_accel > axis[Y_ord].accel.max)
        {
            y_accel = axis[Y_ord].accel.max;
            x_accel = y_accel / ratio;
        }
        if (y_accel < axis[Y_ord].accel.min)
            y_accel = axis[Y_ord].accel.min;
    }

    else                        /* y is bigger */
    {

        /* Compute the y component of the velocity vector */

        y_vel = vel * sinTheta;

        if (y_vel > axis[Y_ord].velocity.max)
            y_vel = axis[Y_ord].velocity.max;

        y_accel = accel * sinTheta;

        if (y_accel > axis[Y_ord].accel.max)
            y_accel = axis[Y_ord].accel.max;

        /* Compute the y component of the acceleration vector */

        x_vel = y_vel * ratio;

        if (x_vel > axis[X_ord].velocity.max)
        {
            x_vel = axis[X_ord].velocity.max;
            y_vel = x_vel / ratio;
        }

        if (x_vel < axis[X_ord].velocity.min)
            x_vel = axis[X_ord].velocity.max;

        x_accel = y_accel * ratio;

        if (x_accel > axis[X_ord].accel.max)
        {
            x_accel = axis[X_ord].accel.max;
            y_accel = x_accel / ratio;
        }

        if (x_accel < axis[X_ord].accel.min)
            x_accel = axis[X_ord].accel.min;

    }                           /* end if x_bigger & else */

    /* initialize the timer */

    moveTimer = masterTimer;

    /* exit on first error that occurs */

    if (x_mag != 0)
    {

        if (set_acceleration (X_ord, x_accel))
            labtyd (COMMUNICATION,
                    " X axis acceleration configuration failed");

        if (set_velocity (X_ord, x_vel))
            labtyd (COMMUNICATION, " X axis velocity configuration failed");

        moveTime = calcMoveTime (x_mag, x_vel, x_accel);

    }

    if (set_distance (X_ord, x))
        labtyd (COMMUNICATION, " X axis distance configuration failed");

    if (y_mag != 0)
    {

        if (set_acceleration (Y_ord, y_accel))
            labtyd (COMMUNICATION,
                    " Y axis acceleration configuration failed");
        if (set_velocity (Y_ord, y_vel))
            labtyd (COMMUNICATION, " Y axis velocity configuration failed");

        moveTime = calcMoveTime (y_mag, y_vel, y_accel);
    }

    if (set_distance (Y_ord, y))
        labtyd (COMMUNICATION, " Y axis distance configuration failed");

    /* before beginning move, calculate the move time and set a timer */

    if (verbose)
    {
        sprintf (log_buf, "Estimated move time: %f\n", moveTime);
        log_fputs (log_buf, stderr);
    }

    secToTimer ((moveTime + TIME_CUSHION), &moveTimer);

    /* the following is commented out because there is too much
       delay activating two axis separately as fibcomm normal does it */

    /* new way of activating x/y moves */

    if ((x_mag != 0) || (y_mag != 0))
        send_it ("~G");         /* SPECIAL 2 AXIS "GO" IN FIBCOMM */

    MoveInProgress = TRUE;

    if (!faith)
    {

        GetLoc (X_ord, ENCODER, &reported_loc.x, &moveTimer);
        GetLoc (Y_ord, ENCODER, &reported_loc.y, &moveTimer);

        location_now.x = x;
        location_now.y = y;

        /* This will halt the software until the motors have stopped
           moving.  It will also update the globals "location_now"
           and "dLocNow"
         */

/*
      syncToIndexers();

*/

        MoveInProgress = FALSE;

        if ((reported_loc.x != x) || (reported_loc.y != y))
        {

            sprintf (log_buf,
                     "WARNING: software has position recorded as %ldx, %ldy\n",
                     location_now.x, location_now.y);
            log_fputs (log_buf, stderr);
            sprintf (log_buf,
                     "         motor controller is reporting     %ldx, %ldy\n",
                     reported_loc.x, reported_loc.y);
            log_fputs (log_buf, stderr);

        }

        GetLooped (X_ord, xLoopStatus, &masterTimer);
        GetLooped (Y_ord, yLoopStatus, &masterTimer);

        if (xLoopStatus[POSLOSS])
            log_fputs
                ("WARNING: X axis encoder/motor pulse Counts don't match.",
                 stderr);

        if (yLoopStatus[POSLOSS])
            log_fputs
                ("WARNING: Y axis encoder/motor pulse Counts don't match.",
                 stderr);
    }
    else
    {                           /* simulator */
        location_now.x = x;
        location_now.y = y;
    }

  nadaToDo:{
    }

    (*GripPosCallback) (location_now.x, location_now.y);

    return (XY_OK);
#endif
// HYDRA_II
}                               /* end function */

GoToZ (long int z, double vel, double accel, int sync)
{
    long reportedLoc;

    accel = fabs (accel);
    vel = fabs (vel);

    if (accel > axis[Z_ord].accel.max)
        accel = axis[Z_ord].accel.max;

    if (vel > axis[Z_ord].velocity.max)
        vel = axis[Z_ord].velocity.max;

    if (z > axis[Z_ord].position.max || z < axis[Z_ord].position.min)
        labtyd (ARG_RANGE, " Z axis position argument out of range");

    if (set_acceleration (Z_ord, accel))
        labtyd (COMMUNICATION, " Z axis acceleration configuration failed");
    if (set_velocity (Z_ord, vel))
        labtyd (COMMUNICATION, " Z axis velocity configuration failed");
    if (set_distance (Z_ord, z))
        labtyd (COMMUNICATION, " Z axis distance configuration failed");
    if (go (Z_ord))
        labtyd (COMMUNICATION, " Z move activation failed");

    if (!faith)
    {
        if (sync)
        {
            GetLoc (Z_ord, MOTOR, &reportedLoc, &masterTimer);
            location_now.z = reportedLoc;
        }
        else
        {
            location_now.z = z;
        }
    }
    else
        location_now.z = z;

    if (location_now.z < CRUISE_HEIGHT)
        RestrictedMoves = TRUE;
    else
        RestrictedMoves = FALSE;

}

/*
		gripOut

   Move the gripper to its "parked" position.
   Print message and exit if it is already there.
   Set the "GripperParked" flag to true. This flag is
   used to prevent attempts to move while the gripper
   is powered down.
*/

void gripOut (void)
{

    if (GripperParked == TRUE)
    {

        log_fputs ("Gripper is already parked\n", stderr);
        return;
    }

#ifdef HYDRA_II
    park();
#else
    GoToZ ((long) 0, ZHOMESPEED, HIHOMEACCEL, SYNC);

    GoToXY ((long) STOW_X, (long) STOW_Y, HIGH_VEL, HIGH_ACCEL, PHYSICAL);

    if (!faith)
    {

        sleep (2);

        encoder_pos_maint (X_ord, OFF);
        sleep (1);

        encoder_pos_maint (Y_ord, OFF);
        sleep (1);

        output2_on (X_ord);
        sleep (2);

        energize (X_ord, OFF);
        sleep (2);

        energize (Y_ord, OFF);
        sleep (2);

        energize (Z_ord, ON);   /* LOGIC IS REVERSED IN Z DRIVE */
        sleep (2);

        output2_on (Z_ord);
    }
#endif
// HYDRA_II

    fputs ("WARNING:  X,Y motion is now *disabled*!\n", stderr);
    fputs ("Use \"gripin\" to re-enable.\n", stderr);

    GripperParked = TRUE;
}

/*
		gripIn

   Move the gripper from its "parked" position back into operation.
   Print message and exit if it is not currently parked.
   Set the "GripperParked" flag to false.
*/

void gripIn (void)
{
#ifdef HYDRA_II
    move_xy (0, 0, HYDRA_ONE);
    GripperParked = FALSE;
#else
    log_fputs ("***Energizing X\n", stderr);
    energize (X_ord, ON);

    log_fputs ("***Energizing Y\n", stderr);
    energize (Y_ord, ON);

    sleep (2);

    log_fputs ("***Releasing brakes \n", stderr);
    output2_off (X_ord);

    sleep (2);

    log_fputs ("***Energizing Z\n", stderr);
    energize (Z_ord, OFF);      /* LOGIC IS REVERSED IN Z DRIVE */

    sleep (1);

    log_fputs ("***Enabling pos. maint in X\n", stderr);
    encoder_pos_maint (X_ord, ON);

    log_fputs ("***Enabling pos. maint in Y\n", stderr);
    encoder_pos_maint (Y_ord, ON);

    sleep (2);

    /* we have to change the flag before we move... */

    GripperParked = FALSE;

    /* magic numbers that reduce homing time */

    log_fputs ("***Moving to center \n", stderr);
    GoToXY ((long) -15000, (long) 26500, HIGH_VEL, HIGH_ACCEL, PHYSICAL);

    log_fputs ("***Homing X\n", stderr);
    HomeMotors (X_ord);

    log_fputs ("***Homing Y\n", stderr);
    HomeMotors (Y_ord);

    log_fputs ("***Homing Z\n", stderr);
    HomeZ ();

    printLocation ();

    fiborgStatusDump (stderr);
#endif
// HYDRA_II

}

/*
   Make sure gripper is up and no button in gripper before making
   an unconstrained move.
*/

OKtoMove (void)
{
    return (TRUE);              /* TEMPORARY until hardware can do it */
}

/*
		XYokToMove

   Test to see if the gripper is above the home position.
   If it is, return true.  If not, return false.
*/

XYokToMove (void)
{
    int statusBits[4];
    struct plate_coords reported_loc;
    struct itimerval moveTimer;

    if (!faith)
    {
        GetTriggers (Z_ord, &statusBits[0], &masterTimer);

        if (statusBits[ZBELOWHOME] == 0)
        {

            moveTimer = masterTimer;

            GetLoc (Z_ord, MOTOR, &reported_loc.z, &moveTimer);

            return (TRUE);

        }
        else
            return (FALSE);
    }
    else
    {
        return (TRUE);
    }
}

void Open (void)                /* open gripper */
{
#ifdef HYDRA_II
    if (GripPos != OPEN)
    {
        mc_exe_cmd ("gripper open");
        GripPos = OPEN;
    }
#else
    if (GripPos != OPEN)
    {

        if (GripPos == RELAXED)
            Close ();

        output2_on (Z_ord);
        ProgDelay (Z_ord, BITTIME);
        output1_on (Z_ord);
        ProgDelay (Z_ord, BITTIME);
        output2_off (Z_ord);
        ProgDelay (Z_ord, BITTIME);

        GripPos = OPEN;
    }

    Pause (3);
#endif
// HYDRA_II
}

void OpenWide (void)            /* open gripper */
{
#ifdef HYDRA_II
    mc_exe_cmd ("gripper openwide");
#else
    output2_on (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output1_on (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output2_off (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    Pause (2);
    output2_on (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output2_off (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    Pause (2);
#endif
// HYDRA_II
    GripPos = OPEN;

}

void OpenAbsolute (void)        /* open gripper without testing state first */
{
#ifdef HYDRA_II
    mc_exe_cmd ("gripper open");
#else
    output2_on (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output1_on (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output2_off (Z_ord);
    ProgDelay (Z_ord, BITTIME);
#endif
// HYDRA_II
}

void Close (void)               /* close gripper */
{
    if (GripPos != CLOSED)
    {
#ifdef HYDRA_II
        mc_exe_cmd ("gripper close");
#else

        SetStroke (LONG);       /* set for long stroke */

        output2_on (Z_ord);
        ProgDelay (Z_ord, BITTIME);
        output1_off (Z_ord);
        ProgDelay (Z_ord, BITTIME);
        output2_off (Z_ord);
        ProgDelay (Z_ord, BITTIME);

/*
      ProgDelay( Z_ord, CLOSETIME );
*/
#endif
// HYDRA_II

        GripPos = CLOSED;
    }
}

void CloseWide (void)           /* close gripper */
{
#ifdef HYDRA_II
    mc_exe_cmd ("gripper close");
#else
    SetStroke (LONG);           /* set for long stroke */

    output2_on (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output1_off (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output2_off (Z_ord);
    ProgDelay (Z_ord, 2.0);
    output2_on (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output1_off (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output2_off (Z_ord);
    ProgDelay (Z_ord, 2.0);
#endif
// HYDRA_II

    GripPos = CLOSED;
}

void CloseAbsolute (void)       /* close gripper, regardless of current position */
{
#ifdef HYDRA_II
    mc_exe_cmd ("gripper close");
#else
    SetStroke (LONG);           /* set for long stroke */

    output2_on (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output1_off (Z_ord);
    ProgDelay (Z_ord, BITTIME);
    output2_off (Z_ord);
    ProgDelay (Z_ord, BITTIME);
#endif
// HYDRA_II
}

void Relax (void)               /* Relax gripper */
{
#ifdef HYDRA_II
    mc_exe_cmd ("gripper relax");
    GripPos = RELAXED;
#else

    if (GripPos != RELAXED)
    {

        if (GripPos == OPEN)
            Close ();

        SetStroke (SHORT);      /* set for short stroke */

        output1_on (Z_ord);
        ProgDelay (Z_ord, BITTIME);
        output2_on (Z_ord);
        ProgDelay (Z_ord, BITTIME);
        output2_off (Z_ord);
        ProgDelay (Z_ord, BITTIME);

        GripPos = RELAXED;
    }
    Pause (1);
#endif
// HYDRA_II
}

void SetStroke (int magnitude)
{
    if (magnitude == SHORT)
    {
        if (output2_on (Y_ord)) /* set for short stroke */
            labtyd (COMMUNICATION, "Could not set to short stroke");
    }

    else
    {
        if (magnitude == LONG)
        {
            if (output2_off (Y_ord))    /* set for long stroke */
                labtyd (COMMUNICATION, "Could not set to short stroke");
        }
        else
            labtyd (ARG_RANGE, "Bad argument to SetStroke");
    }
}

void Hold (void)                /* power down gripper */
{
    (output2_on (Z_ord));
}

Brakes (int flag)               /*  turn stage brakes on or off  */
                 /*  NOTE!:  This also effects the z-home optocoupler  */
                 /* see the next function! */
            /* 1 = brakes are locked */
{
    int error;

    if (flag != 0)
        error = output2_on (X_ord);
    else
        error = output2_off (X_ord);

    return (error);
}

Zlight (int flag)               /*  activate/disable opto-coupler at z axis home position */
    /*  1 = z homing enabled  */
{
    return (Brakes (flag));
}

EraseProgram (int axis, int idNumber)
{
    erase_seq (axis, idNumber);
}

BeginProgram (int axis, int idNumber)
{
    enter_edit_seq (axis, idNumber);
}

EndProgram (int axis)
{
    exit_edit_save (axis);
}

ExecProgram (int axis, int idNumber)
{
    execute_sequence (axis, idNumber);
}

ProgDelay (int axis, double time)
{
    delay (axis, time);
}

Pause (unsigned int time)
{
    sleep (time);
}

void InitDrives (void)
{

/*
	sendString(RESET_DRIVES);
	sleep(4);
*/
}

/*
		WhamMotors

   Send a sequence of commands that will insure that we
   gain control of the indexer for the specified axis.
   Only hardware failure will prevent this routine from
   grabbing control.
*/

WhamMotors (int axis)
{
    int error = 0;

    error = enable_comm (axis); /* Insure communication is on */
    error |= mkill (axis);      /* Kill any motion in progress */
    error |= mreset (axis);     /* "Cold start" the axis */
    sleep (4);

    if (error)
        labtyd (COMMUNICATION, "Couldn't initialize axis");
}

/*
		Init Motors

   Send the commands required to put the specified axis into
   the proper mode.  The order of these commands *is* critical!
   They are explained as they occur in the code below.
*/

InitMotors (int axis)
{
    /* turn off error messages from the indexer since we
       have no mechanism from handling them */

    if (message_level (axis, 0))
        labtyd (COMMUNICATION, "Could not set message level on indexer.");

    /* Save the input buffer when a limit is encountered.
       This is required so that we can drive the stages to
       the limit in the process of homing and still use a
       buffered position report to indicate that the homing
       process has finished
     */

    if (save_buff_at_lim (axis, 1))
        labtyd (COMMUNICATION, "Could not set buffering on indexer.");

    /* put x and y but not z into the proper encoder modes.
       Z does not have an encoder! */

    if (axis != Z_ord)
    {

        if (set_encoder_resolution (axis, ENCODER_RESOLUTION))
            labtyd (COMMUNICATION, "Could not set encoder resolution.");

        if (encoder_step_mode (axis, 1))
            labtyd (COMMUNICATION, "Could not set indexer to encoder mode.");

        if (stop_on_stall (axis, 1))
            labtyd (COMMUNICATION, "Could not set stop on stall mode.");

        if (post_move_pos_loss_detect (axis, 1))
            labtyd (COMMUNICATION,
                    "Could not set position loss detect  mode.");

        if (encoder_pos_maint (axis, 1))
            labtyd (COMMUNICATION, "Could not set position maint. mode.");
    }

    /* Use absolute and not relative coordinates for moves */

    if (absolute_mode (axis))
        labtyd (COMMUNICATION, "Could not set indexer to absolute mode");

    /* Turn on the limits */

    if (enable_limits (axis, LIMITS_ON))
        labtyd (COMMUNICATION, "Could not enable limits");
}

/*
		HomeZ

   Home the Z axis.  
*/

HomeZ (void)
{

    /*  Cycle the gripper to insure we know its state. */

    Open ();
    Close ();

    /*  Make sure there is not a button in the gripper before moving! */

/*
   if( buttonPresent() )
     labtyd( SYS_CONFIG, "Can't initialize Z axis with button!!");
*/

    if (set_home_acceleration (Z_ord, HIHOMEACCEL))
        labtyd (COMMUNICATION, "Couldn't set z home acceleration.");

    /*  XYokToMove also tells us if we are above/below home on z so
       that we can always home in the correct direction. */

    if (XYokToMove ())
    {
        if (go_home (Z_ord, -(ZHOMESPEED)))
            labtyd (COMMUNICATION, "Could not home z.");
    }

    else
    {
        if (go_home (Z_ord, ZHOMESPEED))
            labtyd (COMMUNICATION, "Could not home z.");
    }

    /* Move off of the edge that marks home upward a small amount
       and reset the zero point so that we can use the opto-coupler
       as a binary "save to move" indicator while at z = 0. */

    GoToZ ((long) ZOFFSET, ZHOMESPEED, HIHOMEACCEL, NOSYNC);

    if (reset_absolute_counter (Z_ord))
        labtyd (COMMUNICATION, "Could not reset z home position.");

    location_now.z = 0;

    sleep (ZINITTIME);

}

HomeMotors (int axis)
{
    int pauseFlags[10];         /* bigger than we need */

    GetPause (axis, pauseFlags, &masterTimer);

    if (pauseFlags[STOP])
    {

        log_fputs ("WARNING:  Stop input is asserted\n", stderr);
        log_fputs ("Check panic button and rotary encoder circuit\n", stderr);
        labtyd (COMMUNICATION, "Could not home x/y.");
    }

    /*  set up the watchdog timer that will be used while waiting
       for the x & y axis to home. */

    masterTimer.it_value.tv_sec = HOMETIME;
    masterTimer.it_value.tv_usec = 0;
    masterTimer.it_interval.tv_sec = 0;
    masterTimer.it_interval.tv_usec = 0;

    /* unlock the brakes as the first. */

    if (Brakes (0))
        labtyd (COMMUNICATION, "Could not unlock brakes.");

    /* homing with a button present could destroy it, so check
       for a button in the gripper before moving. */

    if (buttonPresent ())
        labtyd (SYS_CONFIG,
                "Can not init. X or Y axis with button in gripper!!");

    if (set_home_acceleration (axis, HIHOMEACCEL))
        labtyd (COMMUNICATION, "Could not set home acceleration.");

    /* start the search for home */

    if (axis == X_ord)
    {

        if (go_home (axis, -HIHOMESPEED))
            labtyd (COMMUNICATION, "Could not home x axis.");
    }

    else
    {                           /* axis is Y */

        if (go_home (axis, HIHOMESPEED))
            labtyd (COMMUNICATION, "Could not home y axis.");
    }

    /* put a move to get off of home into the buffer.
       so that we can do a second home at a slow speed. */

    if (set_acceleration (axis, HIHOMEACCEL))
        labtyd (COMMUNICATION, "Could not set accel. for offset from home.");

    if (set_velocity (axis, HIHOMESPEED))
        labtyd (COMMUNICATION,
                "Could not set velocity for offset from home.");

    if (set_distance (axis, (long) NUDGEOFFHOME))
        labtyd (COMMUNICATION,
                "Could not set distance for offset from home.");

    if (go (axis))
        labtyd (COMMUNICATION,
                "Could not activate move for offset from home.");

    /* set up for a slow approach to home */

    if (set_home_acceleration (axis, LOWHOMEACCEL))
        labtyd (COMMUNICATION, "Could not set accel. for offset from home.");

    if (go_home (axis, -LOWHOMESPEED))
        labtyd (COMMUNICATION,
                "Could not set velocity for offset from home.");

    /* set up a position request that will not be answered
       until the system finishes homing. This will time
       out if the system hangs or can't home.  Set the offset
       distance to true home */

    if (axis == X_ord)
    {

        GetLoc (X_ord, ENCODER, &location_now.x, &masterTimer);

        /* if( set_distance( X_ord, (long)XOFFSET )) */
        if (set_distance (X_ord, (long) OffsetXY.x))
            labtyd (COMMUNICATION, "Could not set x distance to true home.");
    }

    else
    {                           /* axis == Y_ord */

        GetLoc (Y_ord, ENCODER, &location_now.y, &masterTimer);

        /* if( set_distance( Y_ord, (long)YOFFSET )) */
        if (set_distance (Y_ord, (long) OffsetXY.y))
            labtyd (COMMUNICATION, "Could not set y distance to true home.");
    }

    /* Set up for and make another move; this time to the "true"
       system origin.  (plate center)  */

    if (set_acceleration (axis, HIHOMEACCEL))
        labtyd (COMMUNICATION, "Could not set acceleration for home offset.");

    if (set_velocity (axis, HIHOMESPEED))
        labtyd (COMMUNICATION, "Could not set velocity for home offset.");

    if (go (axis))
        labtyd (COMMUNICATION, "Could not activate move for home offset.");

    /* As a safety measure, set the distance registers
       in the indexer to zero. */

    if (axis == X_ord)
        if (set_distance (X_ord, (long) 0))
            labtyd (COMMUNICATION, "Could not zero x distance value.");
        else if (set_distance (Y_ord, (long) 0))
            labtyd (COMMUNICATION, "Could not zero y distance value.");

    /* Reset the indexer counters to the new, "true" home. */

    if (reset_absolute_counter (axis))
        labtyd (COMMUNICATION, "Could not reset counter at true home.");

    if (reset_cumulative_counter (axis))
        labtyd (COMMUNICATION, "Could not reset cumulative counter.");

    /* now wait for everything to stop */

    if (axis == X_ord)
    {

        GetLoc (X_ord, ENCODER, &location_now.x, &masterTimer);
    }

    else
    {                           /* axis == Y_ord */

        GetLoc (Y_ord, ENCODER, &location_now.y, &masterTimer);
    }

    masterTimer.it_value.tv_sec = MOVETIME;
}

/*
		ordToName

   take the specified the axis ordinal and return a 
   point to a string giving the axis' name.
*/

char *ordToName (int axis)
{
    return (axisNames[axis]);
}

/*
	calcMoveTime

   Calculate the required time to complete a move given
   the specified distance, maximum velocity, and acceleration.
*/

double calcMoveTime (long int dist, double vel, double accel)
                /* revolutions per second */
                /* rev per sec per sec. */
{

    double velocity = vel * ENCODER_RESOLUTION; /* convert to steps/sec */
    double acceleration = accel * ENCODER_RESOLUTION;

    double dRamp;               /* Distance to ramp to full speed and back down */

    double tRamp;               /* Time to ramp to full speed */
    double tCruise;             /* Time at full speed */

    double tTotal;              /* Total time for move */

    dRamp = (velocity * velocity) / acceleration;

    tRamp = 2.0 * (velocity / acceleration);

    /* see what type of move we are making */

    if (dRamp > (double) dist)
    {

        /* The move will not hit full speed */

        tTotal = 2.0 * sqrt ((double) dist / acceleration);
    }

    else
    {

        /* The move will hit full speed, and run at max for some period. */

        tCruise = ((double) dist - dRamp) / velocity;

        tTotal = tRamp + tCruise;

    }

    return (tTotal);
}

/*
		secToTimer

   Take a time in seconds and convert it to the two values
   for an itimer structure.  Place the result in the structure
   pointed to by the second argument.
*/

void secToTimer (double moveTime, struct itimerval *moveTimer)
{
    moveTimer->it_value.tv_sec = (long) moveTime;       /* truncates the fraction */

    moveTimer->it_value.tv_usec = (long) (fmod (moveTime, 1.0) * SEC_TO_USEC);
}

void syncToIndexers (struct itimerval *maxTime)
{
    struct plate_coords encoderLoc;

    GetLoc (X_ord, ENCODER, &encoderLoc.x, maxTime);
    GetLoc (Y_ord, ENCODER, &encoderLoc.y, maxTime);

    location_now = encoderLoc;

    sprintf (log_buf, "Software internal coords reset to %ld, %ld \n",
             location_now.x, location_now.y);
    log_fputs (log_buf, stderr);
}

/*
         gripperTest

   Check to see if a button is present in the gripper
   and return the result. 1 = button found, 0 = gripper empty.
*/

void gripTest (void)
{
    int flagsClosed[4];
    int flagsOpen[4];

    int shaftTransitions;
    int buttonTransitions;

    int numberTrys = 0;

    if (faith)
    {
        log_fputs
            ("REALLY?  How do you expect me to test hardware from the simulator??\n",
             stderr);

        return;
    }

    flagsClosed[SHAFT] = PLUNGER_IN;

    log_fputs ("Beginning Gripper diagnostics.\n", stderr);

    /*
       if(  getYesNo( stderr, stdin, "Is there a button in the gripper", FALSE ) ){
       log_fputs( "Please remove the button before proceeding with diagnostics.\n", stderr );
       return;
       }
     */

    /* first put it in a known state */

    log_fputs ("Closing Gripper\n", stderr);

    while ((flagsClosed[SHAFT] == PLUNGER_IN) && (numberTrys <= CLOSE_TRYS))
    {

        numberTrys++;
        CloseAbsolute ();       /* force the circuit to close again */
        usleep (500000);        /* give it time to close before testing */

        GetTriggers (Z_ord, flagsClosed, &masterTimer);

        sprintf (log_buf, "After %d try(s) to close gripper\n", numberTrys);
        log_fputs (log_buf, stderr);
        printFlags (flagsClosed);
    }

    if ((flagsClosed[SHAFT] == PLUNGER_IN) && (numberTrys > CLOSE_TRYS))
    {

        log_fputs ("ERROR:  Unable to detect closing of gripper.\n", stderr);
        log_fputs ("        Please check plunger limit switch\n", stderr);
        log_fputs ("        and verify that plunger shaft is moving.\n",
                   stderr);

        if (flagsClosed[BUTTON] == TRUE)
        {
            log_fputs
                ("\nNOTE:  Indication of button presence is meaningless in this state!\n",
                 stderr);
        }
        else
        {
            log_fputs
                ("\nNOTE:  The correct indication of button absence suggests that gripper\n",
                 stderr);
            log_fputs ("       is closed, and plunger bit is in error.\n",
                       stderr);
        }
    }
    else
    {

        log_fputs ("\nGripper Correctly indicates closed position.\n",
                   stderr);

        if (flagsClosed[BUTTON] == FALSE)
        {
            log_fputs ("--and does not detect a button.\n\n", stderr);
        }
        else
        {
            log_fputs ("ERROR: Button detected! \n", stderr);
            log_fputs
                ("       If you are sure there is none, then there may be \n",
                 stderr);
            log_fputs ("       a problem with the button sense switch.\n",
                       stderr);
        }
    }

    numberTrys = 0;
    flagsOpen[SHAFT] = PLUNGER_OUT;

    log_fputs ("\nNow opening Gripper\n", stderr);

    while ((flagsOpen[SHAFT] == PLUNGER_OUT) && (numberTrys <= CLOSE_TRYS))
    {

        numberTrys++;
        OpenAbsolute ();        /* force the circuit to close again */
        usleep (400000);        /* give it time to close before testing */

        GetTriggers (Z_ord, flagsOpen, &masterTimer);

        sprintf (log_buf, "After %d try(s) to open gripper\n", numberTrys);
        log_fputs (log_buf, stderr);
        printFlags (flagsOpen);

    }

    if ((flagsOpen[SHAFT] == PLUNGER_OUT) && (numberTrys > CLOSE_TRYS))
    {

        log_fputs ("ERROR:  Unable to detect opening of gripper.\n", stderr);
        log_fputs ("        Please check plunger limit switch\n", stderr);
        log_fputs ("        and verify that plunger shaft is moving.\n",
                   stderr);

        if (flagsOpen[BUTTON] == FALSE)
        {
            log_fputs
                ("\nNOTE:  Indication of button absence is consistent with plunger,\n",
                 stderr);
            log_fputs
                ("       and suggests motor shaft is not moving correctly.\n",
                 stderr);
        }
        else
        {
            log_fputs
                ("\nNOTE:  The incorrect indication of button presence suggests that either\n",
                 stderr);
            log_fputs
                ("       gripper is open, and shaft sense bit is in error;\n",
                 stderr);
            log_fputs
                ("       or the motor is not moving and the button sense switch is wrong.\n",
                 stderr);
            log_fputs
                ("       (...or you you didn't make sure there was no button before starting.)\n",
                 stderr);
        }
    }
    else
    {

        log_fputs ("\nGripper Correctly indicates open position.\n", stderr);

        if (flagsOpen[BUTTON] == TRUE)
        {
            log_fputs
                ("--and detects a open circuit on button sense switch, \n",
                 stderr);
            log_fputs ("  which is normal in this state.\n\n", stderr);
        }
        else
        {
            log_fputs ("\nNOTE:  The indication of closed circuit on the \n",
                       stderr);
            log_fputs
                ("       button sense switch indicates a possible short.\n",
                 stderr);
        }
    }

    /* now look at the transitions */

    /* since the array elements can only be a zero or one, they can only
       add up to three possibilities. A "one" means that only we got two
       different readings for that bit, so the bit toggled */

    shaftTransitions = flagsClosed[SHAFT] + flagsOpen[SHAFT];
    buttonTransitions = flagsClosed[BUTTON] + flagsOpen[BUTTON];

    switch (shaftTransitions)
    {

        case 0:

            log_fputs
                ("Plunger switch never changed, says plunger stayed OUT.\n",
                 stderr);

            if (buttonTransitions == 1)
                log_fputs ("But Button sense switch DID change.\n", stderr);
            else
                log_fputs ("Button sense switch  never changed either.\n",
                           stderr);

            break;

        case 1:

            log_fputs ("Plunger switch changed.\n", stderr);

            if (buttonTransitions == 1)
            {
                log_fputs ("Button sense switch changed too.\n", stderr);
                log_fputs ("Gripper seems ok...\n", stderr);
            }
            else
                log_fputs ("But button sense switch didn't change.\n",
                           stderr);
            break;

        case 2:

            log_fputs
                ("Plunger switch never changed, says plunger stayed IN.\n",
                 stderr);

            if (buttonTransitions == 1)
                log_fputs ("But Button sense switch DID change.\n", stderr);
            else
                log_fputs ("Button sense switch  never changed either.\n",
                           stderr);

            break;

        default:
            log_fputs ("MAJOR FAILURE OF GRIPPER TEST!!!.\n", stderr);
            break;
    }
}

static void printFlags (int flags[])
{

    char plungerString[16];
    char buttonString[16];

    if (flags[SHAFT] == PLUNGER_IN)
        strcpy (plungerString, "IN");
    else
        strcpy (plungerString, "OUT");

    if (flags[BUTTON] == TRUE)
        strcpy (buttonString, "HAS BUTTON");
    else
        strcpy (buttonString, "IS EMPTY");

    sprintf (log_buf,
             "Flags are '%d%d'; so the plunger is %3s, and the gripper %s\n",
             flags[SHAFT], flags[BUTTON], plungerString, buttonString);
    log_fputs (log_buf, stderr);

}
