
/*******************************************************************

		reports.c

   This file contains all the functions for reading 
   status back from the Compumotor indexers.
   Included functions are:

   *  GetLoc()
   *  GetTriggers()
   *  GetBusy()
   *  GetPause()
   *  GetLimit()
   *  GetLooped()
      getString()

   ( * indicates that functions is declared as an extern )

*******************************************************************/

#include <stdio.h>
#include <sys/time.h>
#include <setjmp.h>
#include <signal.h>
#include <string.h>

#include "errors.h"
#include "death.h"
#include "miscMath.h"
#include "motor.h"
#include "kaboom.h"
#include "nesslib.h"
#include "coordConvert.h"
#include "flex.h"
#include "translate.h"
#include "detrot.h"
#include "xref.h"
#include "fiborg.h"
#include "reports.h"
#include "log.h"

static char log_buf[256];

/****** local globals   ********/
struct itimerval ClearTimer = { {0, 0}, {0, 0} };

extern int buttonPresent (void);
char *flagName (int flag);
extern char *tkfgets (char *s, int n, FILE * stream);
int encoder_step_mode (int axis_ord, int arg);
int absolute_pos (int axis_ord);
int cumulative_pos (int axis_ord);
int cumulative_encoder_pos (int axis_ord);
int trigger_status (int axis_ord);
int exec_status (int axis_ord);
int limit_status (int axis_ord);
int pause_status (int axis_ord);
int closed_loop_status (int axis_ord);
int encoder_functions (int axis_ord);

/*
		GetLoc
   
   Obtain the currently reported position from the
   axis specified in the first argument.  Put the
   result in the location given by the second argument.
   If the report does not return in the specified
   amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/
void bkpGetLoc (int axis, int mode, long int *posPtr,
                struct itimerval *maxTime)
{
    int num_found;
    char response[64];

/*
   bzero(response,sizeof(response));
*/

    if (faith)
    {

        switch (axis)
        {

            case X_ord:
                *posPtr = location_now.x;
                break;

            case Y_ord:
                *posPtr = location_now.y;
                break;

            case Z_ord:
                *posPtr = location_now.z;
                break;

            default:
                break;
        }
    }
    else
    {                           /* really go get the report */

        /* find out what mode we are in now. */
        /* if we want motor steps, switch to that mode */

        if ((mode == MOTOR) && (axis != Z_ord))
        {

            if (encoder_step_mode (axis, MOTOR))
                labtyd (COMMUNICATION,
                        "Encoder step mode transmission failure.");
        }

        /* send the request */

        if (absolute_pos (axis))
            labtyd (COMMUNICATION, "Position request transmission failure");

        getString (response, maxTime);

        /* put us back into encoder mode */

        if ((mode == MOTOR) && (axis != Z_ord))
        {

            if (encoder_step_mode (axis, ENCODER))
                labtyd (COMMUNICATION,
                        "Encoder step mode transmission failure.");
        }

        if (verbose)
        {
            (void) sprintf (log_buf, "<%s\n", response);
            log_fputs (log_buf, stderr);
        }

        num_found = sscanf (response, " %ld", posPtr);

        if (num_found < 1)
            labtyd (STATUS_REPORT, "Position report not a number.");
        else if (verbose)
        {
            (void) sprintf (log_buf, "axis %d reports %ld \n", axis, *posPtr);
            log_fputs (log_buf, stderr);
        }
    }
}

/*
		SystemIsCold

   Find out if the system has been left in incremental
   (as opposed to absolute) mode.  If so, it has just been
   powered up.  This is used at program launch time to
   warn the user to do a coldstart.
*/

SystemIsCold (struct itimerval *maxTime)
{
    int encoderStatus[ENCODER_MAX_STATUS_BIT + 1];

    /* Find out what mode we are in now. */

    GetEncoderFunctionStatus (X_ord, encoderStatus, maxTime);

    /* logic flips. If not in absolute mode, the system *is* cold */

    if (encoderStatus[ABSOLUTE_MODE] == FALSE)
        return (TRUE);
    else
        return (FALSE);

}

/*
		GetLoc
   
   Obtain the currently reported position from the
   axis specified in the first argument.  Put the
   result in the location given by the second argument.
   If the report does not return in the specified
   amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/

void GetLoc (int axis, int requestedMode, long int *posPtr,
             struct itimerval *maxTime)
{
    int num_found;
    char response[64];

    int encoderStatus[ENCODER_MAX_STATUS_BIT + 1];

    int currentMode;            /* motor or encoder mode */

    /* Check for "faith" mode and just report back where we
       should be and do bother to check if the mode is active. */

    if (faith)
    {

        switch (axis)
        {

            case X_ord:
                *posPtr = location_now.x;
                break;

            case Y_ord:
                *posPtr = location_now.y;
                break;

            case Z_ord:
                *posPtr = location_now.z;
                break;

            default:
                break;
        }
    }

    else
    {                           /* Really go get the report */

        /* Find out what mode we are in now. */

        GetEncoderFunctionStatus (axis, encoderStatus, maxTime);

        if (encoderStatus[ENCODER_STEP_MODE] == TRUE)
            currentMode = ENCODER;
        else
            currentMode = MOTOR;

        /* Compare the requested mode to the current one
           and switch if required before getting the
           current location. */

        switch (requestedMode)
        {

            case MOTOR:

                if (currentMode == ENCODER)
                    if (encoder_step_mode (axis, MOTOR))
                        labtyd (COMMUNICATION,
                                "Motor mode transmission failure.");

                if (absolute_pos (axis))
                    labtyd (COMMUNICATION,
                            "Position request transmission failure");

                break;

            case ENCODER:

                if ((currentMode == MOTOR) && (axis != Z_ord))
                    if (encoder_step_mode (axis, ENCODER))
                        labtyd (COMMUNICATION,
                                "Encoder mode transmission failure.");

                if (absolute_pos (axis))
                    labtyd (COMMUNICATION,
                            "Position request transmission failure");

                break;

            case CUMULATIVE:

                if ((currentMode == MOTOR) || (axis == Z_ord))
                {

                    if (cumulative_pos (axis))
                        labtyd (COMMUNICATION,
                                "Position request transmission failure");
                }

                else
                {               /* currentMode == ENCODER */

                    if (cumulative_encoder_pos (axis))
                        labtyd (COMMUNICATION,
                                "Position request transmission failure");
                }

                break;

            default:
                break;

        }

        /* get the response back from the indexer */

        getString (response, maxTime);

        /* put us back into the original mode mode */

        if ((requestedMode != currentMode) && (axis != Z_ord))
            if (encoder_step_mode (axis, currentMode))
                labtyd (COMMUNICATION,
                        "Encoder step mode transmission failure.");

        /* print some possible interesting information */

        if (verbose)
        {
            (void) sprintf (log_buf, "<%s\n", response);
            log_fputs (log_buf, stderr);
        }

        num_found = sscanf (response, " %ld", posPtr);

        if (num_found < 1)
        {
            Ping ();            /* clears the buffer */
            labtyd (STATUS_REPORT, "Position report not a number.");
        }
        else if (verbose)
        {
            (void) sprintf (log_buf, "axis %d reports %ld \n", axis, *posPtr);
            log_fputs (log_buf, stderr);
        }
    }
}

/*
		GetTriggers
   
   Obtain the current trigger bit status from the
   axis specified in the first argument.  Put the
   result in the structure pointed to by the second argument.
   If the report does not return in the specified
   amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/

void GetTriggers (int axis, int *response, struct itimerval *maxTime)
{
    int i;
    int numBits;
    char buff[64];              /* response line */

    /*  This is kludgy--should be replaced by an entry
       in the global limit data structure! */

    switch (axis)
    {

        case X_ord:
        case Y_ord:
            numBits = XNUMBITS;
            if (trigger_status (axis))
                labtyd (COMMUNICATION,
                        "Trigger request transmission failure");
            break;

        case Z_ord:
            numBits = ZNUMBITS;
            if (trigger_status (axis))
                labtyd (COMMUNICATION,
                        "Trigger request transmission failure");
            break;

        case STD_ord:
            numBits = STDNUMBITS;
            break;

        default:
            numBits = XNUMBITS;
            break;

    }

    getString (buff, maxTime);

    if (verbose)
    {
        (void) sprintf (log_buf, "axis %d reports triggers of %s \n", axis,
                        buff);
        log_fputs (log_buf, stderr);
    }

    for (i = 0; i < numBits; i++)

        switch (buff[i])
        {

            case '0':
                response[i] = FALSE;
                break;
            case '1':
                response[i] = TRUE;
                break;

            case '*':
                response[i] = FALSE;
                break;

            default:
                (void) sprintf (log_buf, "bit string received was: %s \n",
                                buff);
                log_fputs (log_buf, stderr);
                /* Ping(); */
                labtyd (2, "Other than zero or one in trigger bit");
                break;
        }
}

/*
		GetBusy
   
   Obtain the current busy/ready status from the
   axis specified in the first argument.  Put the
   result in the structure pointed to by the second argument.
   If the report does not return in the specified
   amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/

void GetBusy (int axis, int *respPtr, struct itimerval *maxTime)
{
    int num_found;
    char raw_resp;
    char buff[64];              /* response line */

    /* send the request */

    if (exec_status (axis))
        labtyd (COMMUNICATION, "Status request transmission failure");

    getString (buff, maxTime);

    num_found = sscanf (buff, " *%c", &raw_resp);

    if (verbose && (num_found > 0))
    {
        (void) sprintf (log_buf, "axis %d busy status is: %c\n", axis,
                        raw_resp);
        log_fputs (log_buf, stderr);
    }

    if (num_found < 1)
    {
        Ping ();
        labtyd (STATUS_REPORT, "bad status report ");
    }

    switch (raw_resp)
    {

        case 'R':
            *respPtr = READY;
            break;
        case 'S':
            *respPtr = READY_ATTENTION;
            break;
        case 'B':
            *respPtr = BUSY;
            break;
        case 'C':
            *respPtr = BUSY_ATTENTION;
            break;
        case 'F':
            *respPtr = SEQINPROG;
            break;
        case 'G':
            *respPtr = SEQINPROG_ATTENTION;
            break;
        default:
            *respPtr = BAD_REPORT;
            break;
    }

}

/*
		GetPause
   
   Obtain the current pause status from the
   axis specified in the first argument.  Put the
   result in the structure pointed to by the second argument.
   If the report does not return in the specified
   amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/

void GetPause (int axis, int *respPtr, struct itimerval *maxTime)
{
    int num_found;
    char raw_resp;
    char buff[64];              /* response line */

    /* send the request */

    if (pause_status (axis))
        labtyd (COMMUNICATION, "Pause status request transmission failure");

    getString (buff, maxTime);

    num_found = sscanf (buff, " *%c", &raw_resp);

    if (verbose && (num_found > 0))
    {
        (void) sprintf (log_buf, "%c pause status is: %c\n",
                        ordToName (axis), raw_resp);
        log_fputs (log_buf, stderr);
    }

    if (num_found < 1)
    {
        Ping ();
        labtyd (STATUS_REPORT, "bad pause status report ");
    }

    /* split off the bits of interest.  Straight line coded, but
       still clean enough to read.... */

    if (raw_resp & TRIGGER_BIT)
        respPtr[TRIGGER] = TRUE;
    else
        respPtr[TRIGGER] = FALSE;

    if (raw_resp & HOLD_BIT)
        respPtr[HOLD] = TRUE;
    else
        respPtr[HOLD] = FALSE;

    if (raw_resp & PAUSE_BIT)
        respPtr[PAUSE] = TRUE;
    else
        respPtr[PAUSE] = FALSE;

    if (raw_resp & SHUTDOWN_BIT)
        respPtr[SHUTDOWN] = TRUE;
    else
        respPtr[SHUTDOWN] = FALSE;

    if (raw_resp & STOP_BIT)
        respPtr[STOP] = TRUE;
    else
        respPtr[STOP] = FALSE;

}

/*
		GetLooped
   
   Obtain the current closed loop status from the
   axis specified in the first argument.  Put the
   result in the structure pointed to by the second argument.
   If the report does not return in the specified
   amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/

void GetLooped (int axis, int *respPtr, struct itimerval *maxTime)
{
    int num_found;
    char raw_resp;
    char buff[64];              /* response line */

    /* send the request */

    if (closed_loop_status (axis))
        labtyd (COMMUNICATION,
                "Closed loop status request transmission failure");

    getString (buff, maxTime);

    num_found = sscanf (buff, " *%c", &raw_resp);

    if (verbose && (num_found > 0))
    {
        (void) sprintf (log_buf, "%c closed loop status is: %c\n",
                        ordToName (axis), raw_resp);
        log_fputs (log_buf, stderr);
    }

    if (num_found < 1)
    {
        Ping ();
        labtyd (STATUS_REPORT, "bad status report ");
    }

    /* split off the bits of interest.  Straight line coded, but
       still clean enough to read.... */

    if (raw_resp & STALL_BIT)
        respPtr[STALL] = TRUE;
    else
        respPtr[STALL] = FALSE;

    if (raw_resp & NORMHOME_BIT)
        respPtr[NORMHOME] = TRUE;
    else
        respPtr[NORMHOME] = FALSE;

    if (raw_resp & POSLOSS_BIT)
        respPtr[POSLOSS] = TRUE;
    else
        respPtr[POSLOSS] = FALSE;

    if (raw_resp & STATPOSLOSS_BIT)
        respPtr[STATPOSLOSS] = TRUE;
    else
        respPtr[STATPOSLOSS] = FALSE;

}

/*
		GetLimit
   
   Obtain the current limit status from the
   axis specified in the first argument.  Put the
   result in the structure pointed to by the second argument.
   If the report does not return in the specified
   amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/

void GetLimit (int axis, int *respPtr, struct itimerval *maxTime)
{
    int num_found;
    char raw_resp;
    char buff[64];              /* response line */

    /* send the request */

    if (limit_status (axis))
        labtyd (COMMUNICATION, "Limit status request transmission failure");

    getString (buff, maxTime);

    num_found = sscanf (buff, " *%c", &raw_resp);

    if (verbose && (num_found > 0))
    {
        (void) sprintf (log_buf, "%c limit status is: %c\n",
                        ordToName (axis), raw_resp);
        log_fputs (log_buf, stderr);
    }

    if (num_found < 1)
    {
        Ping ();
        labtyd (STATUS_REPORT, "bad status report ");
    }

    /* split off the bits of interest.  Straight line coded, but
       still clean enough to read.... */

    if (raw_resp & MOVECWLIM_BIT)
        respPtr[MOVECWLIM] = TRUE;
    else
        respPtr[MOVECWLIM] = FALSE;

    if (raw_resp & MOVECCWLIM_BIT)
        respPtr[MOVECCWLIM] = TRUE;
    else
        respPtr[MOVECCWLIM] = FALSE;

    if (raw_resp & ATCWLIM_BIT)
        respPtr[ATCWLIM] = TRUE;
    else
        respPtr[ATCWLIM] = FALSE;

    if (raw_resp & ATCCWLIM_BIT)
        respPtr[ATCCWLIM] = TRUE;
    else
        respPtr[ATCCWLIM] = FALSE;
}

/*
		GetEncoderFunctionStatus

   Get the flags that report the status of various encoder
   modes.
*/

void GetEncoderFunctionStatus (int axis, int *response,
                               struct itimerval *maxTime)
{
    int i;
    char buff[64];              /* response line */

    /* send the request */

    if (encoder_functions (axis))
        labtyd (COMMUNICATION, "Encoder request transmission failure");

    getString (buff, maxTime);

    if (verbose)
    {
        (void) sprintf (log_buf, "axis %d reports encoder status: %s \n",
                        axis, buff);
        log_fputs (log_buf, stderr);
    }

    for (i = 0; i <= ENCODER_MAX_STATUS_BIT; i++)

        switch (buff[i])
        {

            case '0':
                response[i] = FALSE;
                break;
            case '1':
                response[i] = TRUE;
                break;
            default:
                (void) sprintf (log_buf, "bit string received was: %s \n",
                                buff);
                log_fputs (log_buf, stderr);
                Ping ();
                labtyd (2, "Other than zero or one in encoder status");
                break;
        }
}

/*
		printLocation

   Poll the indexers for their current locations
   and print the responses.
*/

void printLocation (void)
{
    struct plate_coords encoderLoc;     /* encoder position report */
    struct plate_coords motorLoc;       /* motor step position report */
    struct plate_coords cummLoc;        /* motor step position report */
    struct aPoint virtualLoc;   /* sky location */
    struct plate_coords skyLoc; /* sky location */

    GetLoc (X_ord, ENCODER, &encoderLoc.x, &masterTimer);
    GetLoc (Y_ord, ENCODER, &encoderLoc.y, &masterTimer);

    GetLoc (X_ord, MOTOR, &motorLoc.x, &masterTimer);
    GetLoc (Y_ord, MOTOR, &motorLoc.y, &masterTimer);

    GetLoc (X_ord, CUMULATIVE, &cummLoc.x, &masterTimer);
    GetLoc (Y_ord, CUMULATIVE, &cummLoc.y, &masterTimer);

    GetLoc (Z_ord, MOTOR, &encoderLoc.z, &masterTimer);

    /* Get the angle in the virtual system */

    virtualLoc.x = encoderLoc.x;
    virtualLoc.y = encoderLoc.y;

    /* un-apply the appropriate flexure corrections to get back to
       sky coordinates */

    coordCorrect (&virtualLoc, &PhysicalEnvironment, INVERT);

    (void) sprintf (log_buf,
                    "\n\ngripper is at plate:  x = %ld  y = %ld  z = %ld\n",
                    (long) virtualLoc.x, (long) virtualLoc.y, encoderLoc.z);
    log_fputs (log_buf, stderr);

    /* de-rotate if required */

    if (Rotflag)
        rotate (&virtualLoc, -(RotData.aveAngle));

    skyLoc.x = virtualLoc.x;
    skyLoc.y = virtualLoc.y;

    (void) sprintf (log_buf,
                    "                 sky:  x = %ld  y = %ld  z = %ld\n",
                    skyLoc.x, skyLoc.y, encoderLoc.z);
    log_fputs (log_buf, stderr);

    (void) sprintf (log_buf, "             encoder:  x = %ld  y = %ld\n\n",
                    encoderLoc.x, encoderLoc.y);
    log_fputs (log_buf, stderr);

    if (!faith)
    {

        (void) sprintf (log_buf, "              motor:  x = %ld  y = %ld\n\n",
                        motorLoc.x, motorLoc.y);
        log_fputs (log_buf, stderr);

        (void) sprintf (log_buf, "         cumulative:  x = %ld  y = %ld\n\n",
                        cummLoc.x, cummLoc.y);
        log_fputs (log_buf, stderr);

    }
}

/*
		gripStatus

   Print the status of the gripper
*/

void gripStatus (void)
{
    if (buttonPresent ())
    {
        if (FiberInTrouble.fiber != NULL)
        {
            sprintf (log_buf,
                     "Gripper is holding (what would appear to be ) button #%d \n",
                     FiberInTrouble.fiber->fiber_id);
            log_fputs (log_buf, stderr);
        }
        else
            log_fputs ("Gripper is holding an unknown button.\n\n", stderr);
    }
    else
    {
        log_fputs ("Gripper is empty.\n\n", stderr);
    }
}

void fiborgStatusDump (FILE * statusFile)
{
    sprintf (log_buf, "      ITEM                STATUS      VALUE \n");

#ifdef IGNORE

    sprintf (log_buf,
             "1   Rotation                %3s       %+6.4f deg.  \n",
             flagName (Rotflag), radToDeg (RotData.aveAngle));
    log_fputs (log_buf, statusFile);

#endif

    sprintf (log_buf,
             "2   Thermal Expansion       %3s       %4.0f deg. F\n",
             flagName (thermFlag), PhysicalEnvironment.temp);
    log_fputs (log_buf, statusFile);

    sprintf (log_buf,
             "3   Concentricity           %3s               \n",
             flagName (EccenFlag));
    log_fputs (log_buf, statusFile);

    sprintf (log_buf,
             "4   Stage/screw distortion  %3s                \n",
             flagName (warpFlag));
    log_fputs (log_buf, statusFile);

    sprintf (log_buf,
             "5   Gravitational flexure   %3s                \n",
             flagName (flexFlag));
    log_fputs (log_buf, statusFile);

#ifdef IGNORE

    sprintf (log_buf,
             "6   Declination                       %+6.3f deg.  \n",
             PhysicalEnvironment.dec);
    log_fputs (log_buf, statusFile);

    sprintf (log_buf,
             "7   Hour Angle                        %+5.3f hours \n",
             (PhysicalEnvironment.ha / 180.0 * 12.0));
    log_fputs (log_buf, statusFile);
#endif

}

/*
		flagName

   Return a pointer to a string that gives a name to an
   integer flag.
*/

char *flagName (int flag)
{
    static char name[32];

    switch (flag)
    {

        case TRUE:
            sprintf (name, "ON");
            break;

        case FALSE:
            sprintf (name, "OFF");
            break;

        default:
            sprintf (name, "??");
            break;

    }
    return (name);
}

void Ping (void)
{
    char garbage[128];

    *garbage = '\0';

    send_it (PING_COMMAND);

    while (strstr (garbage, PING_STRING) == NULL)
    {
        getString (garbage, &masterTimer);
        fprintf (stderr, "\"%s\" found during buffer dump\n", garbage);
    }
}

void getString (char *respPtr, struct itimerval *maxWait)
{
    if (setitimer (ITIMER_REAL, maxWait, (struct itimerval *) NULL))
        labtyd (SYSTEM_CALL, "Watchdog timer failed to set");

    if (tkfgets (respPtr, 128, responses) != respPtr)
        labtyd (FILE_SYS, "fgets returned EOF or other error.");

    if (setitimer (ITIMER_REAL, &ClearTimer, (struct itimerval *) NULL))
        labtyd (SYSTEM_CALL, "Watchdog timer failed to de-activate");
}
