
/***********************************************************************

		setup.c

   This file contains the functions that implement interactive
   user interface.  These are very device specific commands,
   and have be deliberately isolated from more general commands
   for ease of reading.  They are called from the main parsing
   loop in "parser.c"

***********************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <setjmp.h>

#include "miscMath.h"
#include "limits.h"
#include "motor.h"
#include "plate.h"
#include "buttons.h"
#include "kaboom.h"
#include "stdBus.h"
#include "nesslib.h"
#include "nessIO.h"
#include "flex.h"
#include "prompter.h"
#include "concen.h"
#include "xref.h"
#include "fiborg.h"
#include "log.h"

#define CONFIG_AT_TARGET       'a'
#define CONFIG_AT_ZENITH       'b'
#define CONFIG_AT_WHITE_SPOT   'c'
#define CONFIG_ELSEWHERE       'd'

char *positionMenuItems[] = {

    /* the prompt */
    "Enter telescope position at which the positioner will configure fibers",

    /* the choices (automatically labeled in order of appearance) */

    "Target field position",    /* choice a */
    "Zenith",                   /* choice b */
    "White spot",               /* choice c */
    "Other",                    /* choice d */
    NULL,
    "a"
};

void SetupDeath (int sig, int code, struct sigcontext *scp);

struct sigvec OldVec;

struct sigvec SetupDeathVec = {
    SetupDeath,
    0,
    SV_INTERRUPT
};

static char log_buf[256];

char ConfigurationType;
extern int LastViewOK;
extern int currentButton;

int LoadHydraHeader (FILE * dataFile, struct starinfo *theStars[],
                     struct exposinfo *exposureInfo);
int rotatorPrepObserve (double offsetAngle);
int rotatorNotInitialized (void);
int rotatorWaitForZero (void);
int rotatorStopAndZero (void);
int benchHydraCableMismatch (void);

extern int setBlock (int flag);
int cload (char *args);
extern int getYesNo (FILE * devOut, FILE * devIn, char *prompt,
                     int defaultValue);
extern void setCable (char *args);
extern void aload (char *args, int flag1);
extern void complamp (char *args);
extern void led (char *args);
extern int tvStatusGrip (int *flags);
extern void plate (char *args);
extern int vacuumStatus (int *flags, int testMode);
int setRotatorAngle (double value);
extern void configure (char *args);
extern void listDump (char *args);
extern void purge (char *args);
extern int goxy (char *args);
extern void qtz (char *args);
extern void gripin (char *args);
extern void screen (char *args);
void wiynView (void);
extern void gripout (char *args);
extern void sysStatusDump (struct exposinfo *pExposure, FILE * statusFile);
extern void view (char *args);
extern char *tkfgets (char *s, int n, FILE * stream);
extern int GoToXY (long int x, long int y, double vel, double accel,
                   int system);
extern int cruise (char *msg);
extern void load_limits (void);
extern void tweak (char *args);
extern int getYesNoQuit (FILE * devOut, FILE * devIn, char *prompt,
                         int defaultValue);

void wiynSetup (void)
{
    char theCommand[128];

    double expTime;
    double expTimeRadians;

    double siderialTime;
    double siderialTimeDefault;

    int unusedFiberMode;

    double doubleMin;
    double doubleMax;

    int precision;

    char *starFile;
    char *tmpName;
    char fileName[64];

    int responseGrip[12];       /* for TV warning */
    int responseVac[12];        /* for Vacuum warning */

    int i;

    if (sigvec (SIGINT, &SetupDeathVec, &OldVec))
        fputs ("Warning: interrupt handler for \"setup\" failed to install\n",
               stderr);

    setBlock (TRUE);

    if (!noGWC)
    {
        rotatorStopAndZero ();
        log_fputs ("\n Starting rotator towards Zero\n", stderr);
    }

    /* skip over whitespace */

  namePrompt:

    tmpName = getUserInput (stderr, stdin, CHAR, NO_DEFAULT,
                            "\nEnter name of  hydra file.", NULL, NULL, NULL,
                            NULL);

    if (tmpName == NULL)
    {
        log_fputs ("You must specify a star field!\n", stderr);
        goto namePrompt;
    }

    /* tmpName points to a reusable static, so get a copy quick! */

    strcpy (fileName, tmpName);

    /* get rid of the newline at the end... */

    fileName[strlen (fileName) - 1] = '\0';

    /* now link it to the pointer we'll use for the rest of this */

    starFile = fileName;

    if (cload (starFile) != 0)
    {
        setBlock (FALSE);
        goto namePrompt;
    }

    if (getCableIdentifier (Exposure.cable) != CurrentCable)
    {

        log_fputs
            ("\n WARNING:  File is not intended for the current fiber cable!\n",
             stderr);

        if (getYesNo
            (stderr, stdin,
             "Would you like to switch the current cable to match", TRUE))
        {

            setCable (Exposure.cable);
        }
        else
        {

            log_fputs ("\n Very well...  Exiting 'setup'.\n", stderr);
            setBlock (FALSE);
            return;
        }
    }

    if (benchHydraCableMismatch ())
    {
        log_fputs
            ("\n ATTENTION:  bench is is not configured for this fiber cable!\n",
             stderr);
        log_fputs ("               ( You might want to change that...)\n",
                   stderr);
    }

    /* prompt exposure time */

    doubleMin = 0.0;
    doubleMax = 12.0;

    precision = 2;

    expTime = *(double *) getUserInput (stderr, stdin, DOUBLE, SHOW_DEFAULT,
                                        "\nHow long do you plan to observe with this configuration? [hours]",
                                        &Exposure.exptime, &doubleMin,
                                        &doubleMax, &precision);

    expTimeRadians = (expTime / 12) * M_PI;

    /* prompt sidereal time */
    /* default has to be in radians... */

    siderialTimeDefault = (Exposure.st / 12) * M_PI;

    doubleMin = 0.0;
    doubleMax = (2 * M_PI);

    precision = 0;

    siderialTime =
        *(double *) getUserInput (stderr, stdin, TIME, SHOW_DEFAULT,
                                  "\nEnter the anticipated sidereal time at midpoint.",
                                  &siderialTimeDefault, &doubleMin,
                                  &doubleMax, &precision);

    /* ask what to do with unused fibers */

    unusedFiberMode = getYesNo (stderr, stdin, "\nPark unused fibers", TRUE);

    /* set the tolerance for position error to zero
       "update" may have changed it, and this is a
       new field, so the positioning should be perfect. */

    Sloppiness = 0;

   /***** configure *****/

    /* finish loading the file */
    /* set up the command using the recently obtained values */

    sprintf (theCommand, "%s -T%f -S%f", starFile, expTime,
             ((siderialTime / M_PI) * 12));

    aload (theCommand, 0);

    log_fputs ("Making sure comparison sources are off.\n", stderr);

#ifdef HYDRA_II
    mc_exe_cmd ("lamps on");
#else
    complamp ("off");
#endif
// HYDRA_II

    log_fputs ("Turning on gripper. LEDs\n", stderr);

#ifdef HYDRA_II
    mc_exe_cmd ("leds on");
#else
    led ("on");
#endif
// HYDRA_II

#ifndef HYDRA_II_TEST_MODE
    if (!faith && tvStatusGrip (responseGrip) != TV_HIVOLTAGE_ON)
    {
        log_fputs
            ("ATTENTION:  The gripper tv is off you might want to turn it on...\n",
             stderr);
    }

    log_fputs ("Flatting plate.\n", stderr);

    plate ("flat");

    if (!faith && vacuumStatus (responseVac, VAC_AMBIENT) != VAC_OK)
    {
        log_fputs
            ("ERROR:  Couldn't flatten  plate! --exiting 'setup field'\n",
             stderr);
        return;
    }
#endif
// HYDRA_II_TEST_MODE

    if (noGWC)
        setRotatorAngle (0.0);
    else
    {
        rotatorStopAndZero ();

        log_fputs ("WAITING: --for rotator to go to zero.\n", stderr);

        if (rotatorNotInitialized ())
        {

            log_fputs
                ("WARNING:  Instrument Rotator has not been initialized.\n",
                 stderr);
            log_fputs ("Please verify rotator position visually...\n",
                       stderr);

            while (!getYesNo (stderr, stdin, "Is the rotator at zero", TRUE))
            {
                fputs ("Please verify rotator position visually...\n",
                       stderr);
            }
        }
        else
        {
            if (rotatorWaitForZero () == 0)
            {
                log_fputs ("Rotator is in Position.\n", stderr);
            }
            else
            {
                log_fputs ("ERROR: Rotator did not zero.\n", stderr);
                setRotatorAngle (0.0);
            }
        }
    }

    /* put back the old handler so that we don't hurt fiber moves with a ctrl-c */

    if (sigvec (SIGINT, &OldVec, NULL))
        log_fputs ("Old interrupt handler failed to install\n", stderr);

    /* start moving fibers */

    if (unusedFiberMode == TRUE)
    {
        configure ("");
    }
    else
    {
        /* leave unused fibers in place */
        configure ("-p");
    }

    /* if this flag is set, there was a ctrl-c during the setup, and 
       therefore we shouldn't finish this function */

    if (UserInterrupt)
    {
        setBlock (FALSE);
        return;
    }

    /* since this is a "setup", clear the updated flags because all
       fibers should have been updated and hence the flag is
       meaningless */

    for (i = 0; i < MAXFIBER; i++)
    {
        FiBase[i].updated = FALSE;
    }

    /* list fibers that were assigned */

    listDump ("i");

    purge ("");

    setBlock (FALSE);

    /* Suggest possible Viewing and tweaking */

    /*
       if(   getYesNo( stderr, stdin, "Do you wish to view any fibers", FALSE ) ){
       wiynView();
       }
     */

    /* Go to the field center */

    goxy ("0 0 30 30 p");

    log_fputs ("Warping the focal plane plate.\n", stderr);
    log_fputs ("--please wait...\n", stderr);

    plate ("warp");

    if (!faith && vacuumStatus (responseVac, VAC_ON) != VAC_OK)
    {
        log_fputs ("ERROR:  Couldn't warp  plate!\n", stderr);
    }

#ifdef IGNORE

/* removed at sam's request */

    /* spin the rotator to align with the field */

    log_fputs ("Positioning the rotator.\n", stderr);

    if (noGWC)
        setRotatorAngle (FieldRotationalOffset / DEG2RAD);
    else
        rotatorPrepObserve (FieldRotationalOffset / DEG2RAD);

#endif

}

wiynPrepObserve (void)
{
    int responseVac[12];        /* for Vacuum warning */

    /* turn off complamps */

    log_fputs ("Making sure comparison sources are off.\n", stderr);

    complamp ("off");
    qtz ("off");

    if (GripperParked)
    {

        log_fputs ("ATTENTION:  Gripper was disabled--re-enabling...\n\n",
                   stderr);
        gripin (0);
    }

    /* warp the plate */

    log_fputs ("Making sure the focal plane plate is warped.\n", stderr);
    log_fputs ("--please wait...\n", stderr);

    plate ("warp");

    if (!faith && vacuumStatus (responseVac, VAC_ON) != VAC_OK)
    {
        log_fputs ("ERROR:  Couldn't warp  plate!\n", stderr);
    }

    /* get the rotator going */

    if (getYesNo (stderr, stdin,
                  "Do you wish to move the rotator into position",
                  TRUE) == TRUE)
    {

        /* spin the rotator to align with the field */

        log_fputs ("Positioning the rotator.\n", stderr);

        if (noGWC)
            setRotatorAngle (FieldRotationalOffset / DEG2RAD);
        else
            rotatorPrepObserve (FieldRotationalOffset / DEG2RAD);
    }

    /* Go to the field center */

    log_fputs ("Moving to field center.\n", stderr);

    goxy ("0 0 30 30 p");

    /* Make sure LED's are off */

    log_fputs ("Insuring LEDs are off.\n", stderr);

    led ("off");

    /* open screen */

    log_fputs ("Making sure screen is out of the way.\n", stderr);

    screen ("out");

    /* Inquire about viewing fibers */

    if (getYesNo (stderr, stdin, "Do you wish to view any fibers", FALSE))
    {
        wiynView ();
    }

    /* griper out of way */

    log_fputs ("Making sure Gripper is out of the way.\n", stderr);

    gripout (0);

    log_fputs ("\n\nYOU ARE NOW READY TO BEGIN OBSERVATIONS!\n\n", stderr);

}

void printViewMenu (void)
{

    log_fputs ("\nTo view buttons or objects, enter:\n\n", stderr);
    log_fputs ("	'o id# <return>' for an object\n", stderr);
    log_fputs ("	'f id# <return>' for a  fiber\n", stderr);
    log_fputs ("	's id# <return>' for a  slit position\n", stderr);
    log_fputs ("	'  c   <return>' for the field center\n", stderr);
    log_fputs ("	'  b   <return>' for a benign gripper position\n",
               stderr);
    log_fputs ("	'  ?   <return>' for this menu\n", stderr);
    log_fputs ("\nHitting <return> on an empty line will exit the loop\n\n",
               stderr);
}

int cload (char *args)
{
    FILE *starFile;

    starFile = openStarFile (args, "r");

    if (starFile == NULL)
    {
        log_fputs ("Hydra data file not found.\n", stderr);
        return (-1);
    }

    else
    {
        if (LoadHydraHeader (starFile, Stars, &Exposure) == FILE_OK)
        {
            strcpy (Exposure.fileName, args);
            sysStatusDump (&Exposure, stderr);
        }
        else
        {

            sprintf (log_buf,
                     "ERROR: File \"%s\"\nNot readable as Hydra file\n",
                     args);
            log_fputs (log_buf, stderr);
        }

        fclose (starFile);
    }
    return (0);
}

void viewAfops (void)
{
    int i;
    char viewString[16];
    int foundOne = FALSE;

    for (i = (MAXFIBER - 1); i >= 0; i--)
    {

        if (fiberIsAssignedFop (i, Stars, FiBase))
        {
            sprintf (viewString, "%d", i);
            view (viewString);
            foundOne = TRUE;
            break;
        }
    }

    if (!foundOne)
    {
        fprintf (stderr, "No assigned Fops found, viewing center\n");
        sprintf (viewString, "0 0");
        goxy (viewString);
    }
}

/*
	Special handler for user ctrl-c.
*/

void SetupDeath (int sig, int code, struct sigcontext *scp)
{

    if (sigvec (SIGINT, &OldVec, NULL))
        log_fputs ("Old interrupt handler failed to install\n", stderr);

    setBlock (FALSE);

    labtyd (0, "WARNING:  User interrupt. Abandoning \"setup\"\n");

}

void wiynView (void)
{
    int tweakMode;
    int exit = 0;

    char viewString[16];

    printViewMenu ();

    while (!exit)
    {

        tweakMode = 0;

        log_fputs ("view> ", stderr);

        /* get the user input */

        if (tkfgets (viewString, 15, stdin) == NULL)
            viewString[0] = '\0';

        log_putsQ (viewString);

        switch (viewString[0])
        {

            case 'b':
                goxy ("89000 -89000 30 30 p");  /* magic numbers-- so sue me */
                break;

            case 'c':
                goxy ("0 0 30 30 p");   /* magic numbers-- so sue me */
                break;

            case '\n':

                exit = TRUE;
                break;

            case '?':

                printViewMenu ();
                break;

            case 'f':
            case 'o':
            case 's':

                view (viewString);
                break;

            default:

                log_fputs ("\nBad command!  enter '?' for help\n", stderr);
                break;

        }
    }
}

/* the following is a test word for guiding at wiyn.  It's
   quick and dirty so forgive the magic numbers.... */

wiynGoGuide (void)
{

    /* start by moving to the center of the guide star acquisition area */

    GoToXY ((long) 60000, (long) 0, HIGH_VEL, HIGH_ACCEL, PHYSICAL);

    /* reset the limits */

    axis[X_ord].position.max = 80000;
    axis[X_ord].position.min = 40000;

    axis[Y_ord].position.max = 50000;
    axis[Y_ord].position.min = -50000;

    cruise ("Find a guide star and center in gripper");

    /* make sure limits are back to correct. */

    load_limits ();
}

foooCalib (void)
{
    int screenDeployMode;
    int viewMode;
    int tweakMode;
    int exit = 0;

    char viewString[16];

   /***** view loop *****/

    log_fputs
        ("\nWARNING:  Manual viewing and tweaking is VERY time consuming!\n",
         stderr);

    viewMode = getYesNo (stderr, stdin,
                         "Do you wish to view any fibers", FALSE);

    if (viewMode)
    {

        printViewMenu ();

        while (!exit)
        {

            tweakMode = 0;

            log_fputs ("view> ", stderr);

            /* get the user input */

            if (tkfgets (viewString, 15, stdin) == NULL)
                viewString[0] = '\0';

            log_putsQ (viewString);

            switch (viewString[0])
            {

                case 'b':
                    goxy ("90000 90000 30 30 p");       /* magic numbers-- so sue me */
                    break;

                case 'c':
                    goxy ("0 0 30 30 p");       /* magic numbers-- so sue me */
                    break;

                case '\n':

                    exit = TRUE;
                    break;

                case '?':

                    printViewMenu ();
                    break;

                case 'f':
                case 'o':
                case 's':

                    view (viewString);

                    if (LastViewOK && (currentButton != VOIDBUTTON))
                        tweakMode = getYesNo (stderr, stdin,
                                              "Do you wish to tweak this fiber",
                                              FALSE);

                    if (tweakMode)
                    {
                        tweak (viewString);
                        tweakMode = 0;
                    }

                    break;

                default:

                    log_fputs ("\nBad command!  enter '?' for help\n",
                               stderr);
                    break;

            }
        }
    }

    /* ask about calibration screen */

    screenDeployMode = getYesNo (stderr, stdin,
                                 "\nDo you wish to take a wavelength calibration? ",
                                 TRUE);

    if (screenDeployMode)
    {

        log_fputs ("\nTurn on the Hollow Cathode power.\n", stderr);

        log_fputs ("\nWhen the exposures are finished, power off the\n",
                   stderr);
        log_fputs ("Hollow Cathode lamps and type \"screenout\" to\n",
                   stderr);
        log_fputs ("stow the calibration screen.\n>", stderr);

        screen ("in");

        log_fputs ("\nBe sure the Hollow Cathode power is OFF!\n", stderr);
    }

}

setRotatorAngle (double value)
{

    int noExit = TRUE;

    sprintf (log_buf,
             "ATTENTION:  Please have OT set rotator to %f degrees\n", value);
    log_fputs (log_buf, stderr);

    while (noExit)
    {

        switch (getYesNoQuit
                (stderr, stdin, "\n  Is rotator in position?", FALSE))
        {

            case TRUE:
                return 0;

            case FALSE:
                break;

            case QUIT:
                labtyd (0, "Exiting command");
                break;

            default:
                break;
        }
    }
}
