
/***********************************************************************

		standard.c

   This file contains the functions that impliment the standard star command.
   It is called from the main parsing loop in "parser.c"

***********************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <setjmp.h>

#include "miscMath.h"
#include "limits.h"
#include "motor.h"
#include "plate.h"
#include "buttons.h"
#include "kaboom.h"
#include "stdBus.h"
#include "nesslib.h"
#include "nessIO.h"
#include "flex.h"
#include "prompter.h"
#include "concen.h"
#include "xref.h"
#include "tkInterface.h"
#include "fiborg.h"
#include "log.h"

#ifdef IGNORE
void StandardDeath ();          /*  special ctrl-C handler */

struct sigvec OldVec;

struct sigvec StdStarDeath = {
    StandardDeath,
    0,
    SV_INTERRUPT
};
#endif

#define STD_ACQUIRE  'a'
#define STD_GUIDE    'b'
#define STD_CONFIG   'c'
#define STD_EXIT     'd'

/* The index in the the array below for the default choice
   If you add a menu item, you'd better change this too! */

#define DEFAULT_INDEX 6

char *standardStarMenuItems[] = {

    /* the prompt */
    "---STANDARD STAR MODE MENU---",

    /* the choices (automatically labeled in order of appearance) */

    "Acquire Standard Star",    /* choice a */
    "Setup guiding for standard star",  /* choice b */
    "Select fiber for std. star observation",   /* choice c */
    "EXIT Standard Star mode",  /* choice d */
    NULL,
    "a"
};

static char DefaultChoice = 'a';

static int defaultSlitPosition = 50;

static void StandardStarConfig (int theFiber);
static int StandardSlitSelect (void);
static void StandardStarGuide (void);
static void StandardStarAcquire (void);

int rotatorStopAndZero (void);
int rotatorNotInitialized (void);
int rotatorWaitForZero (void);
int rotatorTrackFromZero (double offsetAngle);

extern void complamp (char *args);
extern void screen (char *args);
extern void gripin (char *args);
extern char getMenuSelection (char **menuItems, FILE * devIn, FILE * devOut);
extern int setRotatorAngle (double value);
extern int getYesNo (FILE * devOut, FILE * devIn, char *prompt,
                     int defaultValue);
extern void plate (char *args);
extern void configOne (char *args);
extern void view (char *args);
extern void led (char *args);
extern int wiynGoGuide (void);
extern void tag (char *args);
extern void goTag (char *args);

void wiynStdQuick (void)
{
    int theFiber;

    theFiber = StandardSlitSelect ();

    if (!noGWC)
    {
        rotatorStopAndZero ();
        log_fputs ("\n Starting rotator towards Zero\n", stderr);
    }

    log_fputs ("Making sure comparison sources are off.\n", stderr);

    complamp ("off");

    /* open screen */

    log_fputs ("Making sure screen is out of the way.\n", stderr);

    screen ("out");

    if (GripperParked)
    {

        log_fputs ("ATTENTION:  Gripper was disabled--re-enabling...\n\n",
                   stderr);
        gripin (0);
    }

    fputs ("************************************\n", stderr);

    StandardStarAcquire ();

    fputs ("************************************\n", stderr);

    StandardStarGuide ();

    fputs ("************************************\n", stderr);

    StandardStarConfig (theFiber);
}

void wiynStdStar (void)
{
    char userChoice;

    /* Get the default Item out of protected space */
    standardStarMenuItems[DEFAULT_INDEX] = &DefaultChoice;

    while ((userChoice =
            getMenuSelection (standardStarMenuItems, stdin,
                              stderr)) != STD_EXIT)
    {

        switch (userChoice)
        {

            case STD_ACQUIRE:

                StandardStarAcquire ();

                /* advance the default to the next menu item */
                DefaultChoice = 'b';
                break;

            case STD_GUIDE:

                StandardStarGuide ();

                /* advance the default to the next menu item */
                DefaultChoice = 'c';
                break;

            case STD_CONFIG:

                StandardStarConfig (StandardSlitSelect ());

                /* advance the default to the next menu item */
                DefaultChoice = 'd';
                break;

            default:
                DefaultChoice = 'a';
                break;

        }
    }

    /* put this back for next time we run... */
    DefaultChoice = 'a';
}

static void StandardStarAcquire (void)
{
    if (noGWC)
        setRotatorAngle (0.0);
    else
    {
        rotatorStopAndZero ();

        log_fputs ("WAITING: --for rotator to go to zero.\n", stderr);

        if (rotatorNotInitialized ())
        {

            log_fputs
                ("WARNING:  Instrument Rotator has not been initialized.\n",
                 stderr);
            log_fputs ("Please verify rotator position visually...\n",
                       stderr);

            while (!getYesNo (stderr, stdin, "Is the rotator at zero", TRUE))
            {
                fputs ("Please verify rotator position visually...\n",
                       stderr);
            }
        }
        else
        {
            if (rotatorWaitForZero () == 0)
            {
                log_fputs ("Rotator is in Position.\n", stderr);
            }
            else
            {
                log_fputs ("ERROR: Rotator did not zero.\n", stderr);
                setRotatorAngle (0.0);
            }
        }
    }

    log_fputs ("\nMoving FOPS 0 to center of field.\n", stderr);

    /*led( "on" ); */

    plate ("flat");

    /* the following line moves fiber (a fops) to the center */

    configOne ("0 0 0");

    log_fputs ("Warping the focal plane plate.\n", stderr);

    plate ("warp");

    view ("0");

    /* spin the rotator to align with the field */

    log_fputs ("Presetting and enabling the rotator.\n", stderr);

    if (noGWC)
    {
        log_fputs ("\n PLEASE HAVE THE O.T. TURN ON ROTATOR TRACKING\n",
                   stderr);

        while (!getYesNo
               (stderr, stdin, "\n  Is rotator tracking  on?", TRUE))
        {
            log_fputs ("\n PLEASE HAVE THE O.T. TURN ON ROTATOR TRACKING\n",
                       stderr);
        }
    }
    else
        rotatorTrackFromZero (0);

    /* center the star on the fops */

    log_fputs
        ("\n PLEASE HAVE THE O.T. ACQUIRE AND CENTER THE STAR (on Gripper TV).\n\n",
         stderr);

    while (!getYesNo (stderr, stdin, "\n  Is the star centered?", FALSE))
    {
        /* work takes place in control structure */
    }
}

static void StandardStarGuide (void)
{
    log_fputs
        ("\n Now acquire a guide star with the gripper on gripper TV.\n",
         stderr);

    led ("off");

    /* let the user cruise around and find a guide star */

    wiynGoGuide ();

    /* mark the position */

    tag ("17");

    /* center the star on the fops */

    log_fputs ("\n PLEASE HAVE THE O.T. RE-CENTER THE STAR (on FOPS TV).\n\n",
               stderr);
    log_fputs ("--and check the focus (on FOPS TV).\n\n", stderr);

    while (!getYesNo (stderr, stdin, "\n  Is the star centered?", FALSE))
    {
        /* work takes place in control structure */
    }
}

static int StandardSlitSelect (void)
{
    char log_buf[256];
    int slitPosition;

    xItem theList;

    int minPosition = 1;
    int maxPosition = getMaxSlitInCable (CurrentCable);
    int precision = 3;

    /* find out which slit position is to be used for the observation */

    theList.fiber = VOIDENTRY;

    while (theList.fiber == VOIDENTRY)
    {

        slitPosition =
            *(int *) getUserInput (stderr, stdin, INT, SHOW_DEFAULT,
                                   "\n\nWhich slit Position do you wish to use?",
                                   &defaultSlitPosition, &minPosition,
                                   &maxPosition, &precision);

        (void) xRefSlit (slitPosition, &theList, Stars, FiBase);

        if (theList.fiber == VOIDENTRY)
        {
            log_fputs ("INVALID SLIT POSITION:  Try again...", stderr);
        }
        else
        {
            sprintf (log_buf,
                     "Slit position #%d will use fiber #%d\n",
                     slitPosition, theList.fiber);
            log_fputs (log_buf, stderr);
        }
    }

    defaultSlitPosition = slitPosition;

    return (theList.fiber);
}

static void StandardStarConfig (int theFiber)
{

    char configString[32];

    /* led( "on" ); */

    plate ("flat");

    /* get the rotator ready for moving fibers */

    if (noGWC)
        setRotatorAngle (0.0);
    else
    {
        rotatorStopAndZero ();

        log_fputs ("WAITING: --for rotator to go to zero.\n", stderr);

        if (rotatorNotInitialized ())
        {

            log_fputs
                ("WARNING:  Instrument Rotator has not been initialized.\n",
                 stderr);
            log_fputs ("Please verify rotator position visually...\n",
                       stderr);

            while (!getYesNo (stderr, stdin, "Is the rotator at zero", TRUE))
            {
                fputs ("Please verify rotator position visually...\n",
                       stderr);
            }
        }
        else
        {
            if (rotatorWaitForZero () == 0)
            {
                log_fputs ("Rotator is in Position.\n", stderr);
            }
            else
            {
                log_fputs ("ERROR: Rotator did not zero.\n", stderr);
                setRotatorAngle (0.0);
            }
        }
    }

    /* setup the command to position the desired fiber in the center */

    sprintf (configString, "%d %d %d", theFiber, 0, 0);

    configOne (configString);

    log_fputs ("Warping the focal plane plate.\n", stderr);

    plate ("warp");

    /* spin the rotator to align with the field */

    log_fputs ("Presetting and enabling the rotator.\n", stderr);

    if (noGWC)
    {
        log_fputs ("\n PLEASE HAVE THE O.T. TURN ON ROTATOR TRACKING\n",
                   stderr);

        while (!getYesNo
               (stderr, stdin, "\n  Is rotator tracking  on?", TRUE))
        {
            log_fputs ("\n PLEASE HAVE THE O.T. TURN ON ROTATOR TRACKING\n",
                       stderr);
        }
    }
    else
        resumeTrackFromZero ();

    log_fputs
        ("\n PLEASE HAVE THE O.T. ACQUIRE AND CENTER THE STAR (on Gripper TV).\n\n",
         stderr);

    while (!getYesNo (stderr, stdin, "\n  Is the star still centered?", TRUE))
    {
        log_fputs ("\n PLEASE HAVE THE O.T. RE-CENTER THE STAR.\n\n", stderr);
    }

    led ("off");

    goTag ("17");

    log_fputs ("\n Gripper is now at guide position. Enable guiding.\n\n",
               stderr);

    log_fputs ("\n YOU ARE NOW READY TO BEGIN OBSERVATIONS.\n\n", stderr);
}
