
/***********************************************************************

		update.c

   user interface for "intelligent" updating of fiber positions
   based on refraction shifts.

***********************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>

#include "miscMath.h"
#include "motor.h"
#include "plate.h"
#include "buttons.h"
#include "kaboom.h"
#include "nesslib.h"
#include "nessIO.h"
#include "ness.h"
#include "flex.h"
#include "prompter.h"
#include "fiborg.h"
#include "log.h"

void updateDeath (int sig, int code, struct sigcontext *scp);   /*  special ctrl-C handler */
void autoUpdate (char *args);

struct sigvec OldVec;

struct sigvec updateDeathVec = {
    updateDeath,
    0,
    SV_INTERRUPT
};

double defaultThreshold = 0.5;

extern int setBlock (int flag);
extern int cload (char *args);
extern int batchAstrometry (char *starFileName, int direction);
extern void flushStatus (void);
extern void aload (char *args, int flag1);
extern void configure (char *args);
extern void listDump (char *args);
extern void gripout (char *args);
extern void purge (char *args);

/*****************************************************************/

/*
		autoCalc

   Figure out which fibers will be moved if "update" is executed

*/

void autoCalc (char *args)
{
    char theCommand[128];
    char flags[64];

    double expTime;
    double expTimeRadians;

    double siderialTime;
    double siderialTimeDefault;
    double thresholdDefault = 0.0;

    double threshold;

    double doubleMin;
    double doubleMax;

    int precision;

    char *starFile;

    starFile = args;

    if (sigvec (SIGINT, &updateDeathVec, &OldVec))
        log_fputs
            ("Warning: interrupt handler for \"setup\" failed to install\n",
             stderr);

    setBlock (TRUE);

    if (faith)                  /* simulator */
        strcpy (flags, "-suip");
    else
        strcpy (flags, "-uip");

    /* look and see what file we were using */

    starFile = Exposure.fileName;

    if (strlen (starFile) < 1)
    {
        log_fputs ("SORRY!  No file has been configured.\n", stderr);
        log_fputs ("--so how can you update it?\n", stderr);
        return;
    }

    if (cload (starFile) != 0)
    {
        setBlock (FALSE);
        return;
    }

    /* prompt exposure time */

    doubleMin = 0.0;
    doubleMax = 12.0;

    precision = 2;

    expTime = *(double *) getUserInput (stderr, stdin, DOUBLE, SHOW_DEFAULT,
                                        "\nHow long do you plan to observe with updated configuration? [hours]",
                                        &Exposure.exptime, &doubleMin,
                                        &doubleMax, &precision);

    expTimeRadians = (expTime / 12) * M_PI;

    /* prompt sidereal time */

    /* default has to be in radians... */

    siderialTimeDefault = (Exposure.st / 12) * M_PI;

    doubleMin = 0.0;
    doubleMax = (2 * M_PI);

    precision = 0;

    siderialTime =
        *(double *) getUserInput (stderr, stdin, TIME, SHOW_DEFAULT,
                                  "\nEnter the anticipated sidereal time at midpoint.",
                                  &siderialTimeDefault, &doubleMin,
                                  &doubleMax, &precision);

    /* prompt for threshold  */

    doubleMin = 0.0;
    doubleMax = 10.0;
    precision = 2;

    threshold = *(double *) getUserInput (stderr, stdin, DOUBLE, SHOW_DEFAULT,
                                          "\nEnter refraction error [arcsec] threshold \n above which fibers will be corrected.",
                                          &thresholdDefault, &doubleMin,
                                          &doubleMax, &precision);

    /* threshold of zero means prompt for an interval for iterations */

    if (threshold == 0)
    {

        thresholdDefault = 0.5;

        threshold =
            *(double *) getUserInput (stderr, stdin, DOUBLE, SHOW_DEFAULT,
                                      "\nEnter desired step interval. [arcsec]",
                                      &thresholdDefault, &doubleMin,
                                      &doubleMax, &precision);

    }
    else
    {
        strcat (flags, "o");
    }

   /***** configure *****/

    /* finish loading the file */
    /* set up the command using the recently obtained values */

    sprintf (theCommand, "%s -T%f -S%f",
             starFile, expTime, ((siderialTime / M_PI) * 12));

    fputs (theCommand, stderr);
    fputs ("\n", stderr);

    batchAstrometry (theCommand, SKY_TO_PLATE);

    /* generate the command. The string "theCommand" gets recycled... */

    sprintf (theCommand, "${%s}/%s  %s  %s %f",
             FIBBIN, PRE_CONFIG_SCRIPT, flags, starFile, threshold);

    fputs (theCommand, stderr);
    fputs ("\n", stderr);

    system (theCommand);

    setBlock (FALSE);
}

/*
		autoupdate

   Correct the positions of only those fibers for which refraction
   has created a positional error above a user specified threshold.

*/

void autoUpdate (char *args)
{
    char log_buf[256];
    char theCommand[128];

    double expTime;
    double expTimeRadians;

    double siderialTime;
    double siderialTimeDefault;
    double thresholdDefault = 0.1;

    double threshold;

    double doubleMin;
    double doubleMax;

    int precision;

    char *starFile;

    char argString[64];
    char configureString[64];

    int i;

    starFile = args;

    if (sigvec (SIGINT, &updateDeathVec, &OldVec))
        log_fputs
            ("Warning: interrupt handler for \"setup\" failed to install\n",
             stderr);

    setBlock (TRUE);

    *argString = '\0';

    sscanf (args, "%s", argString);     /* make our own copy... */

    if ((strlen (argString) > 0) && (argString[0] != '-'))
    {
        log_fputs ("ERROR:  bad args to \"update\".\n", stderr);
        return;
    }

    if (strlen (argString) == 0)
    {
        strcpy (argString, "-u");
    }
    else
    {
        strcat (argString, "u");
    }

    /* look and see what file we were using */

    starFile = Exposure.fileName;

    if (strlen (starFile) < 1)
    {
        log_fputs ("SORRY!  No file has been configured.\n", stderr);
        log_fputs ("--so how can you update it?\n", stderr);
        return;
    }

    if (cload (starFile) != 0)
    {
        setBlock (FALSE);
        return;
    }

    /* prompt exposure time */

    doubleMin = 0.0;
    doubleMax = 12.0;

    precision = 2;

    expTime = *(double *) getUserInput (stderr, stdin, DOUBLE, SHOW_DEFAULT,
                                        "\nHow long do you plan to observe with updated configuration? [hours]",
                                        &Exposure.exptime, &doubleMin,
                                        &doubleMax, &precision);

    expTimeRadians = (expTime / 12) * M_PI;

    /* prompt sidereal time */

    /* default has to be in radians... */

    siderialTimeDefault = (Exposure.st / 12) * M_PI;

    doubleMin = 0.0;
    doubleMax = (2 * M_PI);

    precision = 0;

    siderialTime =
        *(double *) getUserInput (stderr, stdin, TIME, SHOW_DEFAULT,
                                  "\nEnter the anticipated sidereal time at midpoint.",
                                  &siderialTimeDefault, &doubleMin,
                                  &doubleMax, &precision);

    /* prompt for threshold  */

    doubleMin = 0.0;
    doubleMax = 10.0;
    precision = 2;

    threshold = *(double *) getUserInput (stderr, stdin, DOUBLE, SHOW_DEFAULT,
                                          "\nEnter refraction error [arcsec] threshold \n above which fibers will be corrected.",
                                          &thresholdDefault, &doubleMin,
                                          &doubleMax, &precision);

    defaultThreshold = threshold;

    Sloppiness = (int) (threshold * AS2R * HYDRA_SCALE);

   /**** set up values for flexure ****/

    /* make the hour angle default the start of the exposure 
       Can't figure out why it's degrees, but it is! 
       so... make it radians. */

    PhysicalEnvironment.ha = (siderialTime -
                              (expTimeRadians / 2)) - Stars[0]->rawCoord.ra;

    while (PhysicalEnvironment.ha >= (M_PI / 2))
        PhysicalEnvironment.ha -= (M_PI);

    while (PhysicalEnvironment.ha <= -(M_PI / 2))
        PhysicalEnvironment.ha += (M_PI);

    /* put it back into degrees... */

    PhysicalEnvironment.ha *= (180 / M_PI);

    /* now do declination */

    PhysicalEnvironment.dec = Stars[0]->rawCoord.dec;

    PhysicalEnvironment.dec *= (180 / M_PI);

    sprintf (log_buf, "\nTelescope position is H.A. = %-6.2f\n",
             ((PhysicalEnvironment.ha / 180.0) * 12.0));
    log_fputs (log_buf, stderr);

    sprintf (log_buf, "                      DEC. = %-6.2f\n",
             PhysicalEnvironment.dec);
    log_fputs (log_buf, stderr);

    flushStatus ();

   /***** configure *****/

    /* finish loading the file */
    /* set up the command using the recently obtained values */

    sprintf (theCommand, "%s -T%f -S%f",
             starFile, expTime, ((siderialTime / M_PI) * 12));

    aload (theCommand, 0);

    /* this does the "dry run" to generate stats and allow user
       to abort the process */

    sprintf (configureString, "%sd %5.2f", argString, threshold);

    fprintf (stderr, "configure args (in update): %s\n", configureString);

    configure (configureString);

    /* wait for the go-ahead */

    (void) getUserInput (stderr, stdin, CHAR, NO_DEFAULT,
                         "\nHit return when ready to configure.",
                         NULL, NULL, NULL, NULL);

    for (i = 0; i < MAXFIBER; i++)
    {
        FiBase[i].updated = FALSE;
    }

    sprintf (configureString, "%s %5.2f", argString, threshold);

    fprintf (stderr, "configure args (in update): %s\n", configureString);

    /* move the fibers */

    configure (configureString);

    /* list fibers that were assigned */

    listDump ("i");

    /* check for post-setup activities */

    /* viewAndCalib(); */

    /* get out of the way.. */

    log_fputs ("\nMoving gripper to stow position.  --please wait.\n",
               stderr);
    gripout ("");

    purge ("");

    setBlock (FALSE);

}

/*
	Special handler for user ctrl-c.
*/

void updateDeath (int sig, int code, struct sigcontext *scp)
{

    if (sigvec (SIGINT, &OldVec, NULL))
        log_fputs ("Old interrupt handler failed to install\n", stderr);

    setBlock (FALSE);

    labtyd (0, "WARNING:  User interrupt. Abandoning \"update\"\n");

}
